"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UiFramework = exports.FrameworkVersionChangedEvent = exports.UiVisibilityChangedEvent = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const frontend_authorization_client_1 = require("@bentley/frontend-authorization-client");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const presentation_frontend_1 = require("@bentley/presentation-frontend");
const telemetry_client_1 = require("@bentley/telemetry-client");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_components_1 = require("@bentley/ui-components");
const ui_core_1 = require("@bentley/ui-core");
const BackstageManager_1 = require("./backstage/BackstageManager");
const DefaultIModelServices_1 = require("./clientservices/DefaultIModelServices");
const DefaultProjectServices_1 = require("./clientservices/DefaultProjectServices");
const ConfigurableUiManager_1 = require("./configurableui/ConfigurableUiManager");
const state_1 = require("./configurableui/state");
const SessionState_1 = require("./redux/SessionState");
const StateManager_1 = require("./redux/StateManager");
const HideIsolateEmphasizeManager_1 = require("./selection/HideIsolateEmphasizeManager");
const SyncUiEventDispatcher_1 = require("./syncui/SyncUiEventDispatcher");
const ThemeManager_1 = require("./theme/ThemeManager");
const keyinPaletteTools = require("./tools/KeyinPaletteTools");
const restoreLayoutTools = require("./tools/RestoreLayoutTool");
const openSettingTools = require("./tools/OpenSettingsTool");
const toolSettingTools = require("./tools/ToolSettingsTools");
const UiShowHideManager_1 = require("./utils/UiShowHideManager");
const WidgetManager_1 = require("./widgets/WidgetManager");
const ChildWindowManager_1 = require("./childwindow/ChildWindowManager");
/** UiVisibility Event class.
 * @public
 */
class UiVisibilityChangedEvent extends ui_core_1.UiEvent {
}
exports.UiVisibilityChangedEvent = UiVisibilityChangedEvent;
/** FrameworkVersion Changed Event class.
 * @internal
 */
class FrameworkVersionChangedEvent extends ui_core_1.UiEvent {
}
exports.FrameworkVersionChangedEvent = FrameworkVersionChangedEvent;
/**
 * Manages the Redux store, I18N service and iModel, Project and Login services for the ui-framework package.
 * @public
 */
class UiFramework {
    /** @beta */
    static get childWindowManager() {
        return UiFramework._PopupWindowManager;
    }
    /** Registers class that will be informed when the UserSettingsStorage location has been set or changed. This allows
     * classes to load any previously saved settings from the new storage location. Common storage locations are the browser's
     * local storage, or the iTwin Product Settings cloud storage available via the SettingsAdmin see `IModelApp.settingsAdmin`.
     * @alpha
     */
    static registerUserSettingsProvider(entry) {
        if (this._uiSettingsProviderRegistry.has(entry.providerId))
            return false;
        this._uiSettingsProviderRegistry.set(entry.providerId, entry);
        return true;
    }
    /**
     * Called by the application to initialize the UiFramework. Also initializes UiComponents, UiCore and UiAbstract.
     * @param store The single Redux store created by the host application. If this is `undefined` then it is assumed that the [[StateManager]] is being used to provide the Redux store.
     * @param i18n The internationalization service created by the application. Defaults to IModelApp.i18n.
     * @param frameworkStateKey The name of the key used by the app when adding the UiFramework state into the Redux store. If not defined "frameworkState" is assumed. This value is ignored if [[StateManager]] is being used. The StateManager use "frameworkState".
     */
    static async initialize(store, i18n, frameworkStateKey) {
        return this.initializeEx(store, i18n, frameworkStateKey);
    }
    /**
     * Called by the application to initialize the UiFramework. Also initializes UiComponents, UiCore and UiAbstract.
     * @param store The single Redux store created by the host application. If this is `undefined` then it is assumed that the [[StateManager]] is being used to provide the Redux store.
     * @param i18n The internationalization service created by the application. Defaults to IModelApp.i18n.
     * @param frameworkStateKey The name of the key used by the app when adding the UiFramework state into the Redux store. If not defined "frameworkState" is assumed. This value is ignored if [[StateManager]] is being used. The StateManager use "frameworkState".
     * @param projectServices Optional app defined projectServices. If not specified DefaultProjectServices will be used.
     * @param iModelServices Optional app defined iModelServices. If not specified DefaultIModelServices will be used.
     *
     * @internal
     */
    static async initializeEx(store, i18n, frameworkStateKey, projectServices, iModelServices) {
        if (UiFramework._initialized) {
            bentleyjs_core_1.Logger.logInfo(UiFramework.loggerCategory(UiFramework), `UiFramework.initialize already called`);
            return;
        }
        // if store is undefined then the StateManager class should have been initialized by parent app and the apps default set of reducer registered with it.
        UiFramework._store = store;
        UiFramework._i18n = i18n || imodeljs_frontend_1.IModelApp.i18n;
        // ignore setting _frameworkStateKeyInStore if not using store
        if (frameworkStateKey && store)
            UiFramework._frameworkStateKeyInStore = frameworkStateKey;
        // set up namespace and register all tools from package
        const frameworkNamespace = UiFramework._i18n.registerNamespace(UiFramework.i18nNamespace);
        [
            restoreLayoutTools,
            keyinPaletteTools,
            openSettingTools,
            toolSettingTools,
        ].forEach((tool) => imodeljs_frontend_1.IModelApp.tools.registerModule(tool, frameworkNamespace));
        const readFinishedPromise = frameworkNamespace.readFinished;
        UiFramework._projectServices = projectServices ? /* istanbul ignore next */ projectServices : new DefaultProjectServices_1.DefaultProjectServices();
        UiFramework._iModelServices = iModelServices ? /* istanbul ignore next */ iModelServices : new DefaultIModelServices_1.DefaultIModelServices();
        UiFramework._backstageManager = new BackstageManager_1.BackstageManager();
        UiFramework._hideIsolateEmphasizeActionHandler = new HideIsolateEmphasizeManager_1.HideIsolateEmphasizeManager(); // this allows user to override the default HideIsolateEmphasizeManager implementation.
        UiFramework._widgetManager = new WidgetManager_1.WidgetManager();
        UiFramework.onFrameworkVersionChangedEvent.addListener(UiFramework._handleFrameworkVersionChangedEvent);
        const oidcClient = imodeljs_frontend_1.IModelApp.authorizationClient;
        // istanbul ignore next
        if (frontend_authorization_client_1.isFrontendAuthorizationClient(oidcClient)) {
            const authorized = imodeljs_frontend_1.IModelApp.authorizationClient && imodeljs_frontend_1.IModelApp.authorizationClient.isAuthorized;
            if (authorized) {
                const accessToken = await oidcClient.getAccessToken();
                UiFramework.setUserInfo(accessToken !== undefined ? accessToken.getUserInfo() : undefined);
            }
            oidcClient.onUserStateChanged.addListener(UiFramework._handleUserStateChanged);
        }
        // Initialize ui-components, ui-core & ui-abstract
        await ui_components_1.UiComponents.initialize(UiFramework._i18n);
        UiFramework.settingsManager.onSettingsProvidersChanged.addListener(() => {
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventDispatcher_1.SyncUiEventId.SettingsProvidersChanged);
        });
        UiFramework._initialized = true;
        // initialize any standalone settings providers that don't need to have defaults set by iModelApp
        UiShowHideManager_1.UiShowHideSettingsProvider.initialize();
        ConfigurableUiManager_1.ConfigurableUiManager.initialize();
        return readFinishedPromise;
    }
    /** Un-registers the UiFramework internationalization service namespace */
    static terminate() {
        UiFramework._store = undefined;
        UiFramework._frameworkStateKeyInStore = "frameworkState";
        if (UiFramework._i18n)
            UiFramework._i18n.unregisterNamespace(UiFramework.i18nNamespace);
        UiFramework._i18n = undefined;
        UiFramework._projectServices = undefined;
        UiFramework._iModelServices = undefined;
        UiFramework._backstageManager = undefined;
        UiFramework._widgetManager = undefined;
        UiFramework._hideIsolateEmphasizeActionHandler = undefined;
        UiFramework._settingsManager = undefined;
        UiFramework.onFrameworkVersionChangedEvent.removeListener(UiFramework._handleFrameworkVersionChangedEvent);
        ui_components_1.UiComponents.terminate();
        UiFramework._initialized = false;
    }
    /** Determines if UiFramework has been initialized */
    static get initialized() { return UiFramework._initialized; }
    /** Property that returns the SettingManager used by AppUI-based applications.
     *  @beta */
    static get settingsManager() {
        if (undefined === UiFramework._settingsManager)
            UiFramework._settingsManager = new ui_core_1.SettingsManager();
        return UiFramework._settingsManager;
    }
    /** @beta */
    static get frameworkStateKey() {
        return UiFramework._frameworkStateKeyInStore;
    }
    /** The UiFramework state maintained by Redux
     * @beta
     */
    static get frameworkState() {
        try {
            // eslint-disable-next-line dot-notation
            return UiFramework.store.getState()[UiFramework.frameworkStateKey];
        }
        catch (_e) {
            return undefined;
        }
    }
    /** The Redux store */
    static get store() {
        if (UiFramework._store)
            return UiFramework._store;
        // istanbul ignore else
        if (!StateManager_1.StateManager.isInitialized(true))
            throw new ui_abstract_1.UiError(UiFramework.loggerCategory(this), UiFramework._complaint);
        // istanbul ignore next
        return StateManager_1.StateManager.store;
    }
    /** The internationalization service created by the app. */
    static get i18n() {
        if (!UiFramework._i18n)
            throw new ui_abstract_1.UiError(UiFramework.loggerCategory(this), UiFramework._complaint);
        return UiFramework._i18n;
    }
    /** The internationalization service namespace. */
    static get i18nNamespace() {
        return "UiFramework";
    }
    /** @beta */
    static get backstageManager() {
        // istanbul ignore next
        if (!UiFramework._backstageManager)
            throw new ui_abstract_1.UiError(UiFramework.loggerCategory(this), UiFramework._complaint);
        return UiFramework._backstageManager;
    }
    /** @alpha */
    static get hideIsolateEmphasizeActionHandler() {
        // istanbul ignore next
        if (!UiFramework._hideIsolateEmphasizeActionHandler)
            throw new ui_abstract_1.UiError(UiFramework.loggerCategory(this), UiFramework._complaint);
        return UiFramework._hideIsolateEmphasizeActionHandler;
    }
    /** @alpha */
    static setHideIsolateEmphasizeActionHandler(handler) {
        // istanbul ignore else
        if (handler)
            UiFramework._hideIsolateEmphasizeActionHandler = handler;
        else
            UiFramework._hideIsolateEmphasizeActionHandler = new HideIsolateEmphasizeManager_1.HideIsolateEmphasizeManager();
    }
    /** @alpha */
    static get widgetManager() {
        // istanbul ignore next
        if (!UiFramework._widgetManager)
            throw new ui_abstract_1.UiError(UiFramework.loggerCategory(this), UiFramework._complaint);
        return UiFramework._widgetManager;
    }
    /** Calls i18n.translateWithNamespace with the "UiFramework" namespace. Do NOT include the namespace in the key.
     * @internal
     */
    static translate(key) {
        return UiFramework.i18n.translateWithNamespace(UiFramework.i18nNamespace, key);
    }
    /** @internal */
    static get packageName() {
        return "ui-framework";
    }
    /** @internal */
    static loggerCategory(obj) {
        const className = ui_abstract_1.getClassName(obj);
        const category = UiFramework.packageName + (className ? `.${className}` : "");
        return category;
    }
    /** @internal */
    static get projectServices() {
        if (!UiFramework._projectServices)
            throw new ui_abstract_1.UiError(UiFramework.loggerCategory(this), UiFramework._complaint);
        return UiFramework._projectServices;
    }
    /** @internal */
    static get iModelServices() {
        if (!UiFramework._iModelServices)
            throw new ui_abstract_1.UiError(UiFramework.loggerCategory(this), UiFramework._complaint);
        return UiFramework._iModelServices;
    }
    static dispatchActionToStore(type, payload, immediateSync = false) {
        UiFramework.store.dispatch({ type, payload });
        if (immediateSync)
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchImmediateSyncUiEvent(type);
        else
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchSyncUiEvent(type);
    }
    static setAccudrawSnapMode(snapMode) {
        UiFramework.dispatchActionToStore(state_1.ConfigurableUiActionId.SetSnapMode, snapMode, true);
    }
    static getAccudrawSnapMode() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.configurableUiState.snapMode : /* istanbul ignore next */ imodeljs_frontend_1.SnapMode.NearestKeypoint;
    }
    static getActiveSelectionScope() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.sessionState.activeSelectionScope : /* istanbul ignore next */ "element";
    }
    static setActiveSelectionScope(selectionScopeId) {
        // istanbul ignore else
        if (UiFramework.frameworkState) {
            const foundIndex = UiFramework.frameworkState.sessionState.availableSelectionScopes.findIndex((selectionScope) => selectionScope.id === selectionScopeId);
            if (-1 !== foundIndex) {
                const scope = UiFramework.frameworkState.sessionState.availableSelectionScopes[foundIndex];
                UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetSelectionScope, scope.id);
                presentation_frontend_1.Presentation.selection.scopes.activeScope = scope.id;
            }
        }
    }
    /** @beta */
    static openCursorMenu(menuData) {
        UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.UpdateCursorMenu, menuData);
    }
    /** @beta */
    static closeCursorMenu() {
        UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.UpdateCursorMenu, undefined);
    }
    /** @beta */
    static getCursorMenuData() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.sessionState.cursorMenuData : /* istanbul ignore next */ undefined;
    }
    static getActiveIModelId() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.sessionState.iModelId : /* istanbul ignore next */ "";
    }
    static setActiveIModelId(iModelId) {
        UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetActiveIModelId, iModelId);
    }
    static setIModelConnection(iModelConnection, immediateSync = false) {
        const oldConnection = UiFramework.getIModelConnection();
        if (oldConnection !== iModelConnection) {
            iModelConnection && SyncUiEventDispatcher_1.SyncUiEventDispatcher.initializeConnectionEvents(iModelConnection);
            oldConnection && undefined === iModelConnection && SyncUiEventDispatcher_1.SyncUiEventDispatcher.clearConnectionEvents(oldConnection);
            UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetIModelConnection, iModelConnection, immediateSync);
        }
    }
    static getIModelConnection() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.sessionState.iModelConnection : /* istanbul ignore next */ undefined;
    }
    /** @beta */
    static async setUiSettingsStorage(storage, immediateSync = false) {
        var e_1, _a;
        if (UiFramework._uiSettingsStorage === storage)
            return;
        UiFramework._uiSettingsStorage = storage;
        // let any registered providers to load values from the new storage location
        const providerKeys = [...this._uiSettingsProviderRegistry.keys()];
        try {
            for (var providerKeys_1 = __asyncValues(providerKeys), providerKeys_1_1; providerKeys_1_1 = await providerKeys_1.next(), !providerKeys_1_1.done;) {
                const key = providerKeys_1_1.value;
                await this._uiSettingsProviderRegistry.get(key).loadUserSettings(storage);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (providerKeys_1_1 && !providerKeys_1_1.done && (_a = providerKeys_1.return)) await _a.call(providerKeys_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        // istanbul ignore next
        if (immediateSync)
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchImmediateSyncUiEvent(SyncUiEventDispatcher_1.SyncUiEventId.UiSettingsChanged);
        else
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventDispatcher_1.SyncUiEventId.UiSettingsChanged);
    }
    /** @beta */
    static getUiSettingsStorage() {
        return UiFramework._uiSettingsStorage;
    }
    /** @beta */
    static setUserInfo(userInfo, immediateSync = false) {
        UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetUserInfo, userInfo, immediateSync);
    }
    /** @beta */
    static getUserInfo() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.sessionState.userInfo : /* istanbul ignore next */ undefined;
    }
    static setDefaultIModelViewportControlId(iModelViewportControlId, immediateSync = false) {
        UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetDefaultIModelViewportControlId, iModelViewportControlId, immediateSync);
    }
    static getDefaultIModelViewportControlId() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.sessionState.defaultIModelViewportControlId : /* istanbul ignore next */ undefined;
    }
    static setDefaultViewId(viewId, immediateSync = false) {
        UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetDefaultViewId, viewId, immediateSync);
    }
    static getDefaultViewId() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.sessionState.defaultViewId : /* istanbul ignore next */ undefined;
    }
    static setDefaultViewState(viewState, immediateSync = false) {
        UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetDefaultViewState, viewState, immediateSync);
    }
    static getDefaultViewState() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.sessionState.defaultViewState : /* istanbul ignore next */ undefined;
    }
    /** @beta */
    static getAvailableSelectionScopes() {
        return UiFramework.frameworkState ?
            UiFramework.frameworkState.sessionState.availableSelectionScopes :
            /* istanbul ignore next */
            [{ id: "element", label: "Element" }];
    }
    static getIsUiVisible() {
        return UiShowHideManager_1.UiShowHideManager.isUiVisible;
    }
    static setIsUiVisible(visible) {
        if (UiShowHideManager_1.UiShowHideManager.isUiVisible !== visible) {
            UiShowHideManager_1.UiShowHideManager.isUiVisible = visible;
            UiFramework.onUiVisibilityChanged.emit({ visible });
        }
    }
    static setColorTheme(theme) {
        if (UiFramework.getColorTheme() === theme)
            return;
        UiFramework.dispatchActionToStore(state_1.ConfigurableUiActionId.SetTheme, theme, true);
    }
    static getColorTheme() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.configurableUiState.theme : /* istanbul ignore next */ ThemeManager_1.SYSTEM_PREFERRED_COLOR_THEME;
    }
    static setWidgetOpacity(opacity) {
        if (UiFramework.getWidgetOpacity() === opacity)
            return;
        UiFramework.dispatchActionToStore(state_1.ConfigurableUiActionId.SetWidgetOpacity, opacity, true);
    }
    static getWidgetOpacity() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.configurableUiState.widgetOpacity : /* istanbul ignore next */ ThemeManager_1.WIDGET_OPACITY_DEFAULT;
    }
    static isMobile() {
        return bentleyjs_core_1.ProcessDetector.isMobileBrowser;
    }
    /** Returns the Ui Version.
     * @beta
     */
    static get uiVersion() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.configurableUiState.frameworkVersion : this._uiVersion;
    }
    static setUiVersion(version) {
        if (UiFramework.uiVersion === version)
            return;
        UiFramework.dispatchActionToStore(state_1.ConfigurableUiActionId.SetFrameworkVersion, version === "1" ? "1" : "2", true);
    }
    static get useDragInteraction() {
        return UiFramework.frameworkState ? UiFramework.frameworkState.configurableUiState.useDragInteraction : false;
    }
    static setUseDragInteraction(useDragInteraction) {
        UiFramework.dispatchActionToStore(state_1.ConfigurableUiActionId.SetDragInteraction, useDragInteraction, true);
    }
    /** Send logging message to the telemetry system
     * @internal
     */
    // istanbul ignore next
    static async postTelemetry(eventName, eventId, contextId, iModeId, changeSetId, time, additionalProperties) {
        if (!imodeljs_frontend_1.IModelApp.authorizationClient || !imodeljs_frontend_1.IModelApp.authorizationClient.hasSignedIn)
            return;
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const telemetryEvent = new telemetry_client_1.TelemetryEvent(eventName, eventId, contextId, iModeId, changeSetId, time, additionalProperties);
        await imodeljs_frontend_1.IModelApp.telemetry.postTelemetry(requestContext, telemetryEvent);
    }
    /** Determines whether a ContextMenu is open
     * @alpha
     * */
    static get isContextMenuOpen() {
        const contextMenu = document.querySelector("div.core-context-menu-opened");
        return contextMenu !== null && contextMenu !== undefined;
    }
}
exports.UiFramework = UiFramework;
UiFramework._initialized = false;
UiFramework._complaint = "UiFramework not initialized";
UiFramework._frameworkStateKeyInStore = "frameworkState"; // default name
UiFramework._uiVersion = "";
UiFramework._uiSettingsStorage = new ui_core_1.LocalSettingsStorage(); // this provides a default storage location for settings
UiFramework._uiSettingsProviderRegistry = new Map();
UiFramework._PopupWindowManager = new ChildWindowManager_1.ChildWindowManager();
UiFramework.useDefaultPopoutUrl = false;
/** Get Show Ui event.
 * @public
 */
UiFramework.onUiVisibilityChanged = new UiVisibilityChangedEvent();
/** Get FrameworkVersion Changed event.
 * @internal
 */
UiFramework.onFrameworkVersionChangedEvent = new FrameworkVersionChangedEvent();
UiFramework._handleFrameworkVersionChangedEvent = (args) => {
    // Log Ui Version used
    bentleyjs_core_1.Logger.logInfo(UiFramework.loggerCategory(UiFramework), `Ui Version changed to ${args.version} `);
    // eslint-disable-next-line @typescript-eslint/no-floating-promises
    UiFramework.postTelemetry(`Ui Version changed to ${args.version} `, "F2772C81-962D-4755-807C-2D675A5FF399");
    UiFramework.setUiVersion(args.version);
};
// istanbul ignore next
UiFramework._handleUserStateChanged = (accessToken) => {
    UiFramework.setUserInfo(accessToken !== undefined ? accessToken.getUserInfo() : undefined);
    if (accessToken === undefined) {
        ConfigurableUiManager_1.ConfigurableUiManager.closeUi();
    }
};
//# sourceMappingURL=UiFramework.js.map