"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module AccuDraw
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccuDrawInputField = void 0;
require("./AccuDrawInputField.scss");
const classnames = require("classnames");
const React = require("react");
const ui_core_1 = require("@bentley/ui-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
const KeyboardShortcut_1 = require("../keyboardshortcut/KeyboardShortcut");
const FrameworkAccuDraw_1 = require("./FrameworkAccuDraw");
const ForwardRefAccuDrawInput = React.forwardRef(function ForwardRefAccuDrawInputField(props, ref) {
    const { className, style, id, label, iconSpec, labelClassName, labelStyle, labelCentered, field, isLocked, onValueChanged, valueChangedDelay, onEnterPressed, onEscPressed } = props, inputProps = __rest(props, ["className", "style", "id", "label", "iconSpec", "labelClassName", "labelStyle", "labelCentered", "field", "isLocked", "onValueChanged", "valueChangedDelay", "onEnterPressed", "onEscPressed"]);
    const [stringValue, setStringValue] = React.useState("");
    const timeoutId = React.useRef(0);
    const [needValueChanged, setNeedValueChanged] = React.useState(false);
    const [needSelection, setNeedSelection] = React.useState(false);
    const [isFocusField, setIsFocusField] = React.useState(false);
    const inputElementRef = React.useRef();
    const refs = ui_core_1.useRefs(inputElementRef, ref); // combine ref needed for target with the forwardRef needed by the Parent when parent is a Type Editor.
    React.useEffect(() => {
        const item = FrameworkAccuDraw_1.FrameworkAccuDraw.translateToItemField(field);
        const formattedValue = FrameworkAccuDraw_1.FrameworkAccuDraw.getFieldDisplayValue(item);
        setStringValue(formattedValue);
    }, [field]);
    const handleChange = React.useCallback((event) => {
        const value = event.currentTarget.value;
        // istanbul ignore next
        if (value === undefined)
            return;
        // istanbul ignore else
        if (stringValue !== value) {
            setStringValue(value);
            setNeedValueChanged(true);
        }
    }, [stringValue]);
    const unsetTimeout = () => {
        // istanbul ignore else
        if (timeoutId) {
            window.clearTimeout(timeoutId.current);
            timeoutId.current = 0;
        }
    };
    React.useEffect(() => {
        // After setStringValue & re-render
        // istanbul ignore else
        if (needValueChanged) {
            if (valueChangedDelay) {
                unsetTimeout();
                timeoutId.current = window.setTimeout(() => {
                    onValueChanged(stringValue);
                }, valueChangedDelay);
            }
            else {
                onValueChanged(stringValue);
            }
            setNeedValueChanged(false);
        }
    }, [onValueChanged, valueChangedDelay, stringValue, needValueChanged]);
    React.useEffect(() => {
        // On unmount
        return () => unsetTimeout();
    }, []);
    const handleKeyDown = React.useCallback((e) => {
        switch (e.key) {
            case ui_abstract_1.SpecialKey.Escape:
                onEscPressed && onEscPressed();
                return;
            case ui_abstract_1.SpecialKey.Enter:
                onEnterPressed && onEnterPressed();
                return;
        }
        if (ui_abstract_1.isLetter(e.key)) {
            e.preventDefault();
            KeyboardShortcut_1.KeyboardShortcutManager.processKey(e.key);
            return;
        }
    }, [onEscPressed, onEnterPressed]);
    React.useEffect(() => {
        const handleSetFieldValueToUi = (args) => {
            if (args.field === field && stringValue !== args.formattedValue) {
                setStringValue(args.formattedValue);
                // istanbul ignore else
                if (isFocusField)
                    setNeedSelection(true);
            }
        };
        return ui_abstract_1.AccuDrawUiAdmin.onAccuDrawSetFieldValueToUiEvent.addListener(handleSetFieldValueToUi);
    }, [field, isFocusField, stringValue]);
    React.useEffect(() => {
        if (needSelection) {
            // istanbul ignore else
            if (inputElementRef.current)
                inputElementRef.current.select();
            setNeedSelection(false);
        }
    }, [needSelection]);
    React.useEffect(() => {
        const handleSetFieldFocus = (args) => {
            setIsFocusField(args.field === field);
        };
        return ui_abstract_1.AccuDrawUiAdmin.onAccuDrawSetFieldFocusEvent.addListener(handleSetFieldFocus);
    }, [field]);
    const inputClassNames = classnames("uifw-accudraw-input-field", className);
    const labelClassNames = classnames("uifw-accudraw-input-label", labelCentered && "uifw-label-centered", labelClassName);
    return (React.createElement(React.Fragment, null,
        React.createElement("label", { htmlFor: id, className: labelClassNames, style: labelStyle },
            label,
            iconSpec && React.createElement(ui_core_1.Icon, { iconSpec: iconSpec })),
        React.createElement(ui_core_1.Input, Object.assign({}, inputProps, { ref: refs, id: id, value: stringValue, className: inputClassNames, style: style, autoComplete: "off", onChange: handleChange, onKeyDown: handleKeyDown })),
        React.createElement("span", { className: "uifw-accudraw-lock" }, isLocked && React.createElement(ui_core_1.Icon, { iconSpec: "icon-lock" }))));
});
/** Input field for AccuDraw
 * @internal
 */
exports.AccuDrawInputField = ForwardRefAccuDrawInput;
//# sourceMappingURL=AccuDrawInputField.js.map