"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module AccuDraw
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrameworkAccuDraw = exports.AccuDrawUiSettingsChangedEvent = void 0;
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_abstract_1 = require("@bentley/ui-abstract");
const UiFramework_1 = require("../UiFramework");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_core_1 = require("@bentley/ui-core");
// cspell:ignore dont
const itemToFieldMap = new Map([
    [imodeljs_frontend_1.ItemField.X_Item, ui_abstract_1.AccuDrawField.X],
    [imodeljs_frontend_1.ItemField.Y_Item, ui_abstract_1.AccuDrawField.Y],
    [imodeljs_frontend_1.ItemField.Z_Item, ui_abstract_1.AccuDrawField.Z],
    [imodeljs_frontend_1.ItemField.ANGLE_Item, ui_abstract_1.AccuDrawField.Angle],
    [imodeljs_frontend_1.ItemField.DIST_Item, ui_abstract_1.AccuDrawField.Distance],
]);
const fieldToItemMap = new Map([
    [ui_abstract_1.AccuDrawField.X, imodeljs_frontend_1.ItemField.X_Item],
    [ui_abstract_1.AccuDrawField.Y, imodeljs_frontend_1.ItemField.Y_Item],
    [ui_abstract_1.AccuDrawField.Z, imodeljs_frontend_1.ItemField.Z_Item],
    [ui_abstract_1.AccuDrawField.Angle, imodeljs_frontend_1.ItemField.ANGLE_Item],
    [ui_abstract_1.AccuDrawField.Distance, imodeljs_frontend_1.ItemField.DIST_Item],
]);
const compassModeToKeyMap = new Map([
    [imodeljs_frontend_1.CompassMode.Polar, "polar"],
    [imodeljs_frontend_1.CompassMode.Rectangular, "rectangular"],
]);
const rotationModeToKeyMap = new Map([
    [imodeljs_frontend_1.RotationMode.Top, "top"],
    [imodeljs_frontend_1.RotationMode.Front, "front"],
    [imodeljs_frontend_1.RotationMode.Side, "side"],
    [imodeljs_frontend_1.RotationMode.View, "view"],
    [imodeljs_frontend_1.RotationMode.ACS, "ACS"],
    [imodeljs_frontend_1.RotationMode.Context, "context"],
]);
/** @internal */
class AccuDrawUiSettingsChangedEvent extends bentleyjs_core_1.BeUiEvent {
}
exports.AccuDrawUiSettingsChangedEvent = AccuDrawUiSettingsChangedEvent;
/** @internal */
class FrameworkAccuDraw extends imodeljs_frontend_1.AccuDraw {
    constructor() {
        super();
        this.providerId = "FrameworkAccuDraw";
        this.handleSetFieldValueFromUiEvent = async (args) => {
            const item = FrameworkAccuDraw.translateToItemField(args.field);
            return imodeljs_frontend_1.IModelApp.accuDraw.processFieldInput(item, args.stringValue, false);
        };
        ui_abstract_1.AccuDrawUiAdmin.onAccuDrawSetFieldValueFromUiEvent.addListener(this.handleSetFieldValueFromUiEvent);
        UiFramework_1.UiFramework.registerUserSettingsProvider(this);
    }
    /** Determines if notifications should be displayed for AccuDraw changes */
    static get displayNotifications() { return FrameworkAccuDraw._displayNotifications; }
    static set displayNotifications(v) {
        FrameworkAccuDraw._displayNotifications = v;
        void UiFramework_1.UiFramework.getUiSettingsStorage().saveSetting(this._settingsNamespace, this._notificationsKey, v);
    }
    async loadUserSettings(storage) {
        const result = await storage.getSetting(FrameworkAccuDraw._settingsNamespace, FrameworkAccuDraw._notificationsKey);
        if (result.status === ui_core_1.UiSettingsStatus.Success)
            FrameworkAccuDraw._displayNotifications = result.setting;
    }
    /** AccuDraw User Interface settings */
    static get uiSettings() { return FrameworkAccuDraw._uiSettings; }
    static set uiSettings(v) {
        FrameworkAccuDraw._uiSettings = v;
        FrameworkAccuDraw.onAccuDrawUiSettingsChangedEvent.emit({});
    }
    /** @internal */
    static translateFromItemField(item) {
        let field = itemToFieldMap.get(item);
        // istanbul ignore if
        if (field === undefined)
            field = ui_abstract_1.AccuDrawField.X;
        return field;
    }
    /** @internal */
    static translateToItemField(field) {
        let item = fieldToItemMap.get(field);
        // istanbul ignore if
        if (item === undefined)
            item = imodeljs_frontend_1.ItemField.X_Item;
        return item;
    }
    /** @internal */
    onCompassModeChange() {
        const accuDrawMode = this.compassMode === imodeljs_frontend_1.CompassMode.Rectangular ? ui_abstract_1.AccuDrawMode.Rectangular : ui_abstract_1.AccuDrawMode.Polar;
        imodeljs_frontend_1.IModelApp.uiAdmin.accuDrawUi.setMode(accuDrawMode);
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawCompassModeChanged);
        this.outputCompassModeMessage();
    }
    /** @internal */
    onRotationModeChange() {
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawRotationChanged);
        this.outputRotationMessage();
    }
    /** @internal */
    onFieldLockChange(index) {
        const field = FrameworkAccuDraw.translateFromItemField(index);
        imodeljs_frontend_1.IModelApp.uiAdmin.accuDrawUi.setFieldLock(field, this.getFieldLock(index));
    }
    /** @internal */
    onFieldValueChange(index) {
        const field = FrameworkAccuDraw.translateFromItemField(index);
        const value = this.getValueByIndex(index);
        const formattedValue = FrameworkAccuDraw.getFieldDisplayValue(index);
        imodeljs_frontend_1.IModelApp.uiAdmin.accuDrawUi.setFieldValueToUi(field, value, formattedValue);
    }
    fieldValuesChanged() {
        this.onFieldValueChange(imodeljs_frontend_1.ItemField.X_Item);
        this.onFieldValueChange(imodeljs_frontend_1.ItemField.Y_Item);
        this.onFieldValueChange(imodeljs_frontend_1.ItemField.Z_Item);
        this.onFieldValueChange(imodeljs_frontend_1.ItemField.ANGLE_Item);
        this.onFieldValueChange(imodeljs_frontend_1.ItemField.DIST_Item);
    }
    /** @internal */
    setFocusItem(index) {
        const field = FrameworkAccuDraw.translateFromItemField(index);
        imodeljs_frontend_1.IModelApp.uiAdmin.accuDrawUi.setFieldFocus(field);
    }
    /** Implemented by sub-classes to update ui fields to show current deltas or coordinates when inactive.
     * Should also choose active x or y input field in rectangular mode based on cursor position when
     * axis isn't locked to support "smart lock".
     * @internal
     */
    onMotion(_ev) {
        if (!this.isEnabled || this.isDeactivated || UiFramework_1.UiFramework.isContextMenuOpen)
            return;
        this.fieldValuesChanged();
        if (!this.dontMoveFocus)
            this.setFocusItem(this.newFocus);
    }
    /** Determine if the AccuDraw UI has focus
     * @internal
     */
    get hasInputFocus() {
        let hasFocus = false;
        const el = document.querySelector("div.uifw-accudraw-field-container");
        if (el)
            hasFocus = el.contains(document.activeElement);
        return hasFocus;
    }
    /** Implement this method to set focus to the AccuDraw UI.
     * @internal
     */
    grabInputFocus() {
        imodeljs_frontend_1.IModelApp.uiAdmin.accuDrawUi.grabInputFocus();
    }
    static getFieldDisplayValue(index) {
        const value = imodeljs_frontend_1.IModelApp.accuDraw.getValueByIndex(index);
        let formattedValue = value.toString();
        const formatterSpec = imodeljs_frontend_1.IModelApp.quantityFormatter.findFormatterSpecByQuantityType(imodeljs_frontend_1.ItemField.ANGLE_Item === index ? imodeljs_frontend_1.QuantityType.Angle : imodeljs_frontend_1.QuantityType.Length);
        // istanbul ignore else
        if (formatterSpec)
            formattedValue = imodeljs_frontend_1.IModelApp.quantityFormatter.formatQuantity(value, formatterSpec);
        return formattedValue;
    }
    outputInfoMessage(message) {
        // istanbul ignore else
        if (FrameworkAccuDraw.displayNotifications)
            imodeljs_frontend_1.IModelApp.notifications.outputMessage(new imodeljs_frontend_1.NotifyMessageDetails(imodeljs_frontend_1.OutputMessagePriority.Info, message));
    }
    outputCompassModeMessage() {
        if (FrameworkAccuDraw.displayNotifications) {
            let modeKey = compassModeToKeyMap.get(imodeljs_frontend_1.IModelApp.accuDraw.compassMode);
            // istanbul ignore if
            if (modeKey === undefined)
                modeKey = "polar";
            const modeString = UiFramework_1.UiFramework.translate(`accuDraw.compassMode.${modeKey}`);
            const modeMessage = UiFramework_1.UiFramework.i18n.translateWithNamespace(UiFramework_1.UiFramework.i18nNamespace, "accuDraw.compassModeSet", { modeString });
            this.outputInfoMessage(modeMessage);
        }
    }
    outputRotationMessage() {
        if (FrameworkAccuDraw.displayNotifications) {
            let rotationKey = rotationModeToKeyMap.get(imodeljs_frontend_1.IModelApp.accuDraw.rotationMode);
            // istanbul ignore if
            if (rotationKey === undefined)
                rotationKey = "top";
            const rotationString = UiFramework_1.UiFramework.translate(`accuDraw.rotation.${rotationKey}`);
            const rotationMessage = UiFramework_1.UiFramework.i18n.translateWithNamespace(UiFramework_1.UiFramework.i18nNamespace, "accuDraw.rotationSet", { rotationString });
            this.outputInfoMessage(rotationMessage);
        }
    }
}
exports.FrameworkAccuDraw = FrameworkAccuDraw;
FrameworkAccuDraw._displayNotifications = false;
FrameworkAccuDraw._settingsNamespace = "AppUiSettings";
FrameworkAccuDraw._notificationsKey = "AccuDrawNotifications";
/** Determines if AccuDraw.rotationMode === RotationMode.Top */
FrameworkAccuDraw.isTopRotationConditional = new ui_abstract_1.ConditionalBooleanValue(() => imodeljs_frontend_1.IModelApp.accuDraw.rotationMode === imodeljs_frontend_1.RotationMode.Top, [SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawRotationChanged]);
/** Determines if AccuDraw.rotationMode === RotationMode.Front */
FrameworkAccuDraw.isFrontRotationConditional = new ui_abstract_1.ConditionalBooleanValue(() => imodeljs_frontend_1.IModelApp.accuDraw.rotationMode === imodeljs_frontend_1.RotationMode.Front, [SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawRotationChanged]);
/** Determines if AccuDraw.rotationMode === RotationMode.Side */
FrameworkAccuDraw.isSideRotationConditional = new ui_abstract_1.ConditionalBooleanValue(() => imodeljs_frontend_1.IModelApp.accuDraw.rotationMode === imodeljs_frontend_1.RotationMode.Side, [SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawRotationChanged]);
/** Determines if AccuDraw.rotationMode === RotationMode.View */
FrameworkAccuDraw.isViewRotationConditional = new ui_abstract_1.ConditionalBooleanValue(() => imodeljs_frontend_1.IModelApp.accuDraw.rotationMode === imodeljs_frontend_1.RotationMode.View, [SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawRotationChanged]);
/** Determines if AccuDraw.rotationMode === RotationMode.ACS */
FrameworkAccuDraw.isACSRotationConditional = new ui_abstract_1.ConditionalBooleanValue(() => imodeljs_frontend_1.IModelApp.accuDraw.rotationMode === imodeljs_frontend_1.RotationMode.ACS, [SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawRotationChanged]);
/** Determines if AccuDraw.rotationMode === RotationMode.Context */
FrameworkAccuDraw.isContextRotationConditional = new ui_abstract_1.ConditionalBooleanValue(() => imodeljs_frontend_1.IModelApp.accuDraw.rotationMode === imodeljs_frontend_1.RotationMode.Context, [SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawRotationChanged]);
/** Determines if AccuDraw.compassMode === CompassMode.Polar */
FrameworkAccuDraw.isPolarModeConditional = new ui_abstract_1.ConditionalBooleanValue(() => imodeljs_frontend_1.IModelApp.accuDraw.compassMode === imodeljs_frontend_1.CompassMode.Polar, [SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawCompassModeChanged]);
/** Determines if AccuDraw.compassMode === CompassMode.Rectangular */
FrameworkAccuDraw.isRectangularModeConditional = new ui_abstract_1.ConditionalBooleanValue(() => imodeljs_frontend_1.IModelApp.accuDraw.compassMode === imodeljs_frontend_1.CompassMode.Rectangular, [SyncUiEventDispatcher_1.SyncUiEventId.AccuDrawCompassModeChanged]);
/** AccuDraw Grab Input Focus event. */
FrameworkAccuDraw.onAccuDrawUiSettingsChangedEvent = new AccuDrawUiSettingsChangedEvent();
//# sourceMappingURL=FrameworkAccuDraw.js.map