"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Backstage
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.BackstageComposer = exports.useGroupedItems = void 0;
const React = require("react");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_ninezone_1 = require("@bentley/ui-ninezone"); // eslint-disable-line sort-imports
const SafeAreaContext_1 = require("../safearea/SafeAreaContext");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
const BackstageComposerItem_1 = require("./BackstageComposerItem");
const BackstageManager_1 = require("./BackstageManager");
const useDefaultBackstageItems_1 = require("./useDefaultBackstageItems");
const useUiItemsProviderBackstageItems_1 = require("./useUiItemsProviderBackstageItems");
// cSpell:ignore safearea
/** Private function to set up sync event monitoring of backstage items */
function useBackstageItemSyncEffect(itemsManager, syncIdsOfInterest) {
    const isInitialMount = React.useRef(true);
    React.useEffect(() => {
        const handleSyncUiEvent = (args) => {
            if (0 === syncIdsOfInterest.length)
                return;
            // istanbul ignore else
            if (syncIdsOfInterest.some((value) => args.eventIds.has(value.toLowerCase()))) {
                // process each item that has interest
                itemsManager.refreshAffectedItems(args.eventIds);
            }
        };
        if (isInitialMount.current) {
            isInitialMount.current = false;
            // initialize the display state of any items with syncIds/condition defined
            itemsManager.refreshAffectedItems(new Set(syncIdsOfInterest));
        }
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.onSyncUiEvent.addListener(handleSyncUiEvent);
        return () => {
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.onSyncUiEvent.removeListener(handleSyncUiEvent);
        };
    }, [itemsManager, itemsManager.items, syncIdsOfInterest]);
}
/** local function to combine items from Stage and from Extensions */
function combineItems(stageItems, addonItems) {
    const items = [];
    if (stageItems.length)
        items.push(...stageItems);
    if (addonItems.length)
        items.push(...addonItems);
    return items;
}
/** @internal */
const useGroupedItems = (items) => {
    return React.useMemo(() => {
        const grouped = items.reduce((acc, item) => {
            if (ui_abstract_1.ConditionalBooleanValue.getValue(item.isHidden))
                return acc;
            const groupIndex = acc.findIndex((group) => group[0].groupPriority === item.groupPriority);
            if (groupIndex >= 0)
                return [
                    ...acc.slice(0, groupIndex),
                    [
                        ...acc[groupIndex],
                        item,
                    ],
                    ...acc.slice(groupIndex + 1),
                ];
            return [
                ...acc,
                [
                    item,
                ],
            ];
        }, []);
        const sortedGroups = grouped.reduce((acc, group) => {
            const sortedGroup = [...group].sort((a, b) => a.itemPriority - b.itemPriority);
            return [
                ...acc,
                sortedGroup,
            ];
        }, []);
        return [...sortedGroups].sort((a, b) => a[0].groupPriority - b[0].groupPriority);
    }, [items]);
};
exports.useGroupedItems = useGroupedItems;
/** Backstage component composed from [[BackstageManager]] items.
 * @public
 */
function BackstageComposer(props) {
    const [defaultItemsManager, setDefaultItemsManager] = React.useState(new ui_abstract_1.BackstageItemsManager(props.items));
    const initialItems = React.useRef(props.items);
    React.useEffect(() => {
        if (initialItems.current !== props.items) {
            initialItems.current = props.items;
            setDefaultItemsManager(new ui_abstract_1.BackstageItemsManager(props.items));
        }
    }, [props.items]);
    const manager = BackstageManager_1.useBackstageManager();
    const isOpen = BackstageManager_1.useIsBackstageOpen(manager);
    const safeAreaInsets = React.useContext(SafeAreaContext_1.SafeAreaContext);
    const handleClose = React.useCallback(() => {
        manager.close();
    }, [manager]);
    const defaultItems = useDefaultBackstageItems_1.useDefaultBackstageItems(defaultItemsManager);
    const syncIdsOfInterest = React.useMemo(() => ui_abstract_1.BackstageItemsManager.getSyncIdsOfInterest(defaultItems), [defaultItems]);
    useBackstageItemSyncEffect(defaultItemsManager, syncIdsOfInterest);
    const [addonItemsManager] = React.useState(new ui_abstract_1.BackstageItemsManager());
    const addonItems = useUiItemsProviderBackstageItems_1.useUiItemsProviderBackstageItems(addonItemsManager);
    const addonSyncIdsOfInterest = React.useMemo(() => ui_abstract_1.BackstageItemsManager.getSyncIdsOfInterest(addonItems), [addonItems]);
    useBackstageItemSyncEffect(addonItemsManager, addonSyncIdsOfInterest);
    const combinedBackstageItems = React.useMemo(() => combineItems(defaultItems, addonItems), [defaultItems, addonItems]);
    const groups = exports.useGroupedItems(combinedBackstageItems);
    return (React.createElement(ui_ninezone_1.Backstage, { className: props.className, header: props.header, isOpen: isOpen, onClose: handleClose, safeAreaInsets: safeAreaInsets, showOverlay: props.showOverlay, style: props.style }, groups.map((group, groupIndex) => (group.map((item, itemIndex) => {
        const composerItem = (React.createElement(BackstageComposerItem_1.BackstageComposerItem, { item: item, key: item.id }));
        return itemIndex === 0 && groupIndex > 0 ? (React.createElement(React.Fragment, { key: groupIndex },
            React.createElement(ui_ninezone_1.BackstageSeparator, null),
            composerItem)) : composerItem;
    })))));
}
exports.BackstageComposer = BackstageComposer;
//# sourceMappingURL=BackstageComposer.js.map