"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Backstage
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.useBackstageManager = exports.useIsBackstageOpen = exports.BackstageManager = void 0;
const React = require("react");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const CommandItemDef_1 = require("../shared/CommandItemDef");
const UiFramework_1 = require("../UiFramework");
const Backstage_1 = require("./Backstage");
/** Controls backstage.
 * @beta
 */
class BackstageManager {
    constructor() {
        this._isOpen = false;
        /** Event raised when backstage is opened or closed. */
        this.onToggled = new bentleyjs_core_1.BeEvent();
    }
    get isOpen() {
        return this._isOpen;
    }
    setIsOpen(isOpen) {
        if (isOpen === this._isOpen)
            return;
        this._isOpen = isOpen;
        this.onToggled.raiseEvent({
            isOpen,
        });
        Backstage_1.Backstage.onBackstageEvent.emit({ isVisible: isOpen }); // eslint-disable-line deprecation/deprecation
    }
    open() {
        this.setIsOpen(true);
    }
    close() {
        this.setIsOpen(false);
    }
    toggle() {
        this.setIsOpen(!this.isOpen);
    }
    /** Get CommandItemDef that will toggle display of Backstage and allow iconSpec to be overridden */
    static getBackstageToggleCommand(overrideIconSpec) {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.openBackstage",
            iconSpec: overrideIconSpec ? overrideIconSpec : "icon-home",
            labelKey: "UiFramework:commands.openBackstage",
            execute: () => {
                UiFramework_1.UiFramework.backstageManager.toggle();
            },
        });
    }
}
exports.BackstageManager = BackstageManager;
/** Hook that returns isOpen flag of the backstage.
 * @beta
 */
const useIsBackstageOpen = (manager) => {
    const [isOpen, setIsOpen] = React.useState(manager.isOpen);
    React.useEffect(() => {
        const handleToggled = (args) => {
            setIsOpen(args.isOpen);
        };
        setIsOpen(manager.isOpen);
        manager.onToggled.addListener(handleToggled);
        return () => {
            manager.onToggled.removeListener(handleToggled);
        };
    }, [manager]);
    return isOpen;
};
exports.useIsBackstageOpen = useIsBackstageOpen;
/** Hook that returns backstage manager.
 * @beta
 */
const useBackstageManager = () => {
    const [manager] = React.useState(UiFramework_1.UiFramework.backstageManager);
    return manager;
};
exports.useBackstageManager = useBackstageManager;
//# sourceMappingURL=BackstageManager.js.map