"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChildWindowManager = void 0;
/** @packageDocumentation
 * @module ChildWindowManager
 */
require("./ChildWindowManager.scss");
const React = require("react");
const ReactDOM = require("react-dom");
const CopyStyles_1 = require("./CopyStyles");
const react_redux_1 = require("react-redux");
const UiFramework_1 = require("../UiFramework");
const StateManager_1 = require("../redux/StateManager");
const useUiSettings_1 = require("../uisettings/useUiSettings");
const PopupManager_1 = require("../popup/PopupManager");
const ModelessDialogManager_1 = require("../dialog/ModelessDialogManager");
const ModalDialogManager_1 = require("../dialog/ModalDialogManager");
const CursorMenu_1 = require("../cursor/cursormenu/CursorMenu");
const useFrameworkVersion_1 = require("../hooks/useFrameworkVersion");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const childHtml = `<!DOCTYPE html>
<html>

<head>
  <meta charset="utf-8" />
  <style>
    html,
    body {
      height: 100%;
      width: 100%;
      margin: 0;
      overflow: hidden;
    }

    #root {
      height: 100%;
    }
  </style>
</head>
<body>
  <noscript>You need to enable JavaScript to run this app.</noscript>
  <div id="root"></div>
</body>
</html>`;
/** Supports opening a child browser window from the main application window. The child window is managed by the main application
 * and is running in the same security context. The application must deliver the html file iTwinPopup.html along side its index.html.
 * See also: [Child Window Manager]($docs/learning/ui/framework/ChildWindows.md)
 * @beta */
class ChildWindowManager {
    constructor() {
        this._openChildWindows = [];
        this.closeChildWindow = (childWindowId, processWindowClose = true) => {
            const windowIndex = this.openChildWindows.findIndex((openWindow) => openWindow.childWindowId === childWindowId);
            if (-1 === windowIndex)
                return false;
            const childWindow = this.openChildWindows[windowIndex];
            this.openChildWindows.splice(windowIndex, 1);
            if (processWindowClose) {
                childWindow.window.close();
            }
            else {
                // call the following to convert popout to docked widget
                const frontStageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
                frontStageDef && frontStageDef.dockPopoutWidgetContainer(childWindowId);
            }
            return true;
        };
    }
    get openChildWindows() {
        return this._openChildWindows;
    }
    findChildWindow(childWindowId) {
        if (undefined === childWindowId)
            return undefined;
        return this.openChildWindows.find((openWindow) => openWindow.childWindowId === childWindowId);
    }
    findChildWindowId(contentWindow) {
        if (!contentWindow)
            return undefined;
        const childWindow = this.openChildWindows.find((openWindow) => openWindow.window === contentWindow);
        return childWindow === null || childWindow === void 0 ? void 0 : childWindow.childWindowId;
    }
    // istanbul ignore next
    renderChildWindowContents(childWindow, childWindowId, content) {
        const reactConnectionDiv = childWindow.document.getElementById("root");
        if (reactConnectionDiv) {
            // set openChildWindows now so components can use it when they mount
            this._openChildWindows.push({
                childWindowId,
                window: childWindow,
                parentWindow: window,
            });
            setTimeout(() => {
                CopyStyles_1.copyStyles(childWindow.document);
                childWindow.document.documentElement.setAttribute("data-theme", UiFramework_1.UiFramework.getColorTheme());
                setImmediate(() => {
                    ReactDOM.render(React.createElement(react_redux_1.Provider, { store: StateManager_1.StateManager.store },
                        React.createElement(useUiSettings_1.UiSettingsProvider, { settingsStorage: UiFramework_1.UiFramework.getUiSettingsStorage() },
                            React.createElement(useFrameworkVersion_1.FrameworkVersion, { version: "2" },
                                React.createElement("div", { className: "uifw-child-window-container-host" },
                                    React.createElement(PopupManager_1.PopupRenderer, null),
                                    React.createElement(ModalDialogManager_1.ModalDialogRenderer, null),
                                    React.createElement(ModelessDialogManager_1.ModelessDialogRenderer, null),
                                    React.createElement(CursorMenu_1.CursorPopupMenu, null),
                                    React.createElement("div", { className: "uifw-child-window-container nz-widget-widget" }, content))))), reactConnectionDiv);
                });
            });
            childWindow.onbeforeunload = () => {
                const frontStageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
                if (frontStageDef) {
                    void frontStageDef.saveChildWindowSizeAndPosition(childWindowId, childWindow).then(() => {
                        this.closeChildWindow(childWindowId, false);
                    });
                }
            };
        }
    }
    /** Close all child/pop-out windows. This typically is called when the frontstage is changed. */
    closeAllChildWindows() {
        // istanbul ignore next
        this.openChildWindows.forEach((openChildWindow) => openChildWindow.window.close());
        this._openChildWindows = [];
    }
    // istanbul ignore next
    adjustWidowLocation(location, center) {
        const outLocation = Object.assign({}, location);
        if (0 === location.top && 0 === location.left) {
            center = center !== null && center !== void 0 ? center : true;
            // Prepare position of the new window to be centered against the 'parent' window.
            if (center) {
                outLocation.left =
                    window.top.outerWidth / 2 + window.top.screenX - location.width / 2;
                outLocation.top =
                    window.top.outerHeight / 2 + window.top.screenY - location.height / 2;
            }
            else {
                if (undefined !== window.screenLeft && undefined !== window.screenTop) {
                    outLocation.top = window.screenTop + location.top;
                    outLocation.left = window.screenLeft + location.left;
                }
            }
        }
        return outLocation;
    }
    // istanbul ignore next
    openChildWindow(childWindowId, title, content, location, useDefaultPopoutUrl) {
        // first check to see if content is already open in child window
        if (this.openChildWindows.findIndex((openWindow) => openWindow.childWindowId === childWindowId) >= 0) {
            return false;
        }
        location = this.adjustWidowLocation(location);
        const url = useDefaultPopoutUrl ? "/iTwinPopup.html" : "";
        const childWindow = window.open(url, "", `width=${location.width},height=${location.height},left=${location.left},top=${location.top},menubar=no,resizable=yes,scrollbars=no,status=no,location=no`);
        if (!childWindow)
            return false;
        if (0 === url.length) {
            childWindow.document.write(childHtml);
            this.renderChildWindowContents(childWindow, childWindowId, content);
        }
        else {
            childWindow.addEventListener("load", () => {
                childWindow.document.title = title;
                this.renderChildWindowContents(childWindow, childWindowId, content);
            }, false);
        }
        window.addEventListener("beforeunload", () => {
            const frontStageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
            if (frontStageDef) {
                this.closeChildWindow(childWindowId, true);
            }
        }, false);
        return true;
    }
}
exports.ChildWindowManager = ChildWindowManager;
//# sourceMappingURL=ChildWindowManager.js.map