"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ClientServices
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultIModelServices = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const imodelhub_client_1 = require("@bentley/imodelhub-client");
// import GatewayProxyApi from "./gatewayProxy";
const imodeljs_common_1 = require("@bentley/imodeljs-common");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const UiFramework_1 = require("../UiFramework");
// istanbul ignore next
class IModelInfoImpl {
    constructor(name, description, wsgId, createdDate, projectInfo, status = "", thumbnail) {
        this.name = name;
        this.description = description;
        this.wsgId = wsgId;
        this.createdDate = createdDate;
        this.projectInfo = projectInfo;
        this.status = status;
        this.thumbnail = thumbnail;
    }
}
// istanbul ignore next
class VersionInfoImpl {
    constructor(name, description, createdDate, changeSetId, userCreated, smallThumbnail, largeThumbnail) {
        this.name = name;
        this.description = description;
        this.createdDate = createdDate;
        this.changeSetId = changeSetId;
        this.userCreated = userCreated;
        this.smallThumbnail = smallThumbnail;
        this.largeThumbnail = largeThumbnail;
    }
}
// istanbul ignore next
class ChangeSetInfoImpl {
    constructor(name, description, pushDate, changeSetId, userCreated, smallThumbnail, largeThumbnail) {
        this.name = name;
        this.description = description;
        this.pushDate = pushDate;
        this.changeSetId = changeSetId;
        this.userCreated = userCreated;
        this.smallThumbnail = smallThumbnail;
        this.largeThumbnail = largeThumbnail;
    }
}
// istanbul ignore next
class IModelUserInfoImpl {
    constructor(firstName, lastName, email, id = "") {
        this.firstName = firstName;
        this.lastName = lastName;
        this.email = email;
        this.id = id;
    }
}
/**
 * Provides default [[IModelServices]]
 * @internal
 */
// istanbul ignore next
class DefaultIModelServices {
    /** Initialize the iModelHub Api */
    constructor() {
        this._hubClient = new imodelhub_client_1.IModelHubClient();
    }
    /** Get all iModels in a project */
    async getIModels(projectInfo, top, skip) {
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const iModelInfos = [];
        const queryOptions = new imodelhub_client_1.IModelQuery();
        queryOptions.select("*").top(top).skip(skip);
        try {
            const iModels = await this._hubClient.iModels.get(requestContext, projectInfo.wsgId, queryOptions);
            for (const imodel of iModels) {
                const versions = await this._hubClient.versions.get(requestContext, imodel.id, new imodelhub_client_1.VersionQuery().select("Name,ChangeSetId").top(1));
                if (versions.length > 0) {
                    imodel.latestVersionName = versions[0].name;
                    imodel.latestVersionChangeSetId = versions[0].changeSetId;
                }
            }
            for (const thisIModel of iModels) {
                iModelInfos.push(this.createIModelInfo(thisIModel, projectInfo));
            }
        }
        catch (e) {
            alert(JSON.stringify(e));
            throw e;
        }
        return iModelInfos;
    }
    /** Open the specified version of the IModel */
    async openIModel(contextId, iModelId, openMode, changeSetId) {
        try {
            // GatewayProxyApi.setAccessToken(accessToken);
            const iModelConnection = await imodeljs_frontend_1.RemoteBriefcaseConnection.open(contextId, iModelId, openMode ? openMode : bentleyjs_core_1.OpenMode.Readonly, changeSetId ? imodeljs_common_1.IModelVersion.asOfChangeSet(changeSetId) : imodeljs_common_1.IModelVersion.latest()); // eslint-disable-line deprecation/deprecation
            return iModelConnection;
        }
        catch (e) {
            alert(JSON.stringify(e));
            throw e;
        }
    }
    /** Get the thumbnail for the iModel */
    async getThumbnail(contextId, iModelId) {
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        try {
            const pngImage = await this._hubClient.thumbnails.download(requestContext, iModelId, { contextId, size: "Small" });
            return pngImage;
        }
        catch (err) {
            // No image available
        }
        return undefined;
    }
    /** Get versions (top 5 for testing) for the iModel */
    async getVersions(iModelId) {
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const versionInfos = [];
        try {
            const versions = await this._hubClient.versions.get(requestContext, iModelId, new imodelhub_client_1.VersionQuery().select("*").top(5));
            for (const thisVersion of versions) {
                versionInfos.push(this.createVersionInfo(thisVersion));
            }
        }
        catch (e) {
            alert(JSON.stringify(e));
            throw e;
        }
        return versionInfos;
    }
    /** Get changesets (top 5 for testing) for the iModel */
    async getChangeSets(iModelId) {
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const changeSetInfos = [];
        try {
            const changesets = await this._hubClient.changeSets.get(requestContext, iModelId, new imodelhub_client_1.ChangeSetQuery().top(5).latest());
            for (const thisChangeSet of changesets) {
                changeSetInfos.push(this.createChangeSetInfo(thisChangeSet));
            }
        }
        catch (e) {
            alert(JSON.stringify(e));
            throw e;
        }
        return changeSetInfos;
    }
    /** Get users that have access to a particular iModel */
    async getUsers(iModelId) {
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const userInfos = [];
        try {
            const users = await this._hubClient.users.get(requestContext, iModelId, new imodelhub_client_1.UserInfoQuery().select("*"));
            for (const userInfo of users) {
                userInfos.push(this.createUserInfo(userInfo));
            }
        }
        catch (e) {
            alert(JSON.stringify(e));
            throw e;
        }
        return userInfos;
    }
    async getUser(iModelId, userId) {
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const userInfos = [];
        try {
            const users = await this._hubClient.users.get(requestContext, iModelId, new imodelhub_client_1.UserInfoQuery().byId(userId));
            for (const userInfo of users) {
                userInfos.push(this.createUserInfo(userInfo));
            }
        }
        catch (e) {
            alert(JSON.stringify(e));
            throw e;
        }
        return userInfos;
    }
    createIModelInfo(thisIModel, thisProjectInfo) {
        const createDate = new Date(thisIModel.createdDate);
        bentleyjs_core_1.Logger.logTrace(UiFramework_1.UiFramework.loggerCategory(this), `Working on iModel '${thisIModel.name}'`);
        const thisIModelInfo = new IModelInfoImpl(thisIModel.name, thisIModel.description, thisIModel.wsgId, createDate, thisProjectInfo, "", thisIModel.thumbnail);
        return thisIModelInfo;
    }
    createVersionInfo(thisVersion) {
        const createDate = new Date(thisVersion.createdDate);
        // eslint-disable-next-line deprecation/deprecation
        const thisVersionInfo = new VersionInfoImpl(thisVersion.name, thisVersion.description, createDate, thisVersion.changeSetId, thisVersion.userCreated, thisVersion.smallThumbnailId.toString(), thisVersion.largeThumbnailId.toString());
        return thisVersionInfo;
    }
    createChangeSetInfo(thisChangeSet) {
        const pushDate = new Date(thisChangeSet.pushDate);
        const thisChangeSetInfo = new ChangeSetInfoImpl(thisChangeSet.name, thisChangeSet.description, pushDate, thisChangeSet.changeSetId, thisChangeSet.userCreated, thisChangeSet.smallThumbnailId, thisChangeSet.largeThumbnailId);
        return thisChangeSetInfo;
    }
    createUserInfo(thisUser) {
        const thisUserInfo = new IModelUserInfoImpl(thisUser.firstName, thisUser.lastName, thisUser.email, thisUser.id);
        return thisUserInfo;
    }
}
exports.DefaultIModelServices = DefaultIModelServices;
//# sourceMappingURL=DefaultIModelServices.js.map