/** @packageDocumentation
 * @module ConfigurableUi
 */
import { BeUiEvent } from "@bentley/bentleyjs-core";
import { ContentGroupProps } from "../content/ContentGroup";
import { ContentLayoutProps } from "../content/ContentLayoutProps";
import { FrontstageDef } from "../frontstage/FrontstageDef";
import { FrontstageProvider } from "../frontstage/FrontstageProvider";
import { KeyboardShortcutProps } from "../keyboardshortcut/KeyboardShortcut";
import { TaskPropsList } from "../workflow/Task";
import { WorkflowProps, WorkflowPropsList } from "../workflow/Workflow";
import { ConfigurableUiControlConstructor, ConfigurableUiElement } from "./ConfigurableUiControl";
import { ActivityTracker } from "./ActivityTracker";
/** Ui Activity Event Args interface.
 * @internal
 */
export interface UiActivityEventArgs {
    event: Event;
}
/** Ui Activity Event class.
 * @internal
 */
export declare class UiActivityEvent extends BeUiEvent<UiActivityEventArgs> {
}
/** Ui Interval Event Args interface
 * @internal
 */
export interface UiIntervalEventArgs {
    idleTimeout?: number;
}
/** Ui Interval Event class.
 * @internal
 */
export declare class UiIntervalEvent extends BeUiEvent<UiIntervalEventArgs> {
}
/** Configurable Ui Manager maintains controls, Frontstages, Content Groups, Content Layouts, Tasks and Workflows.
 * @public
 */
export declare class ConfigurableUiManager {
    private static _registeredControls;
    private static _initialized;
    /** @internal */
    static readonly activityTracker: ActivityTracker;
    /** @internal */
    static readonly onUiActivityEvent: UiActivityEvent;
    /** @internal */
    static readonly onUiIntervalEvent: UiIntervalEvent;
    /** Initializes the ConfigurableUiManager and registers core controls. */
    static initialize(): void;
    /** Registers a control implementing the [[ConfigurableUiElement]] interface.
     * These controls can be a
     * [[ContentControl]],
     * [[NavigationAidControl]],
     * [[StatusBarWidgetControl]],
     * [[WidgetControl]] or
     * [ToolUiProvider]($ui-framework).
     * @param classId the class id of the control to register
     * @param constructor the constructor of the control to register
     */
    static registerControl(classId: string, constructor: ConfigurableUiControlConstructor): void;
    /** Determines if a control has been registered based on its classId.
     * @param classId   the class id of the control to test
     * @returns  true if the control is registered or false if not
     */
    static isControlRegistered(classId: string): boolean;
    /** Determines if a control has been registered.
     * @internal
     */
    static getConstructorClassId(constructor: ConfigurableUiControlConstructor): string | undefined;
    /** Unregisters a control that has been registered.
     * @param classId   the class id of the control to unregister
     */
    static unregisterControl(classId: string): void;
    /** Creates a control registered by calling registerControl.
     * @param classId   the class id of the control to create
     * @param uniqueId  a unique id for the control
     * @param options   options passed to the constructor of the control
     * @returns  the created control
     */
    static createControl(classId: string, uniqueId: string, options?: any): ConfigurableUiElement | undefined;
    /** Add a Frontstage via a provider into the [[FrontstageManager]].
     * @param frontstageProvider  Provider of the Frontstage to add
     */
    static addFrontstageProvider(frontstageProvider: FrontstageProvider): void;
    /** Finds a FrontstageDef, given its id.
     * @param id  the id of the FrontstageDef to find
     * @returns the FrontstageDef with the given id, or undefined if not found
     */
    static findFrontstageDef(id?: string): FrontstageDef | undefined;
    /** Loads one or more ContentGroups into the [[ContentGroupManager]].
     * @param groupPropsList  the list of ContentGroups to load
     */
    static loadContentGroups(groupPropsList: ContentGroupProps[]): void;
    /** Loads a [[ContentGroup]] into the [[ContentGroupManager]].
     * @param groupProps  the properties of the ContentGroup to load
     */
    static loadContentGroup(groupProps: ContentGroupProps): void;
    /** Loads one or more ContentLayouts into the [[ContentLayoutManager]].
     * @param layoutPropsList  the list of ContentLayouts to load
     */
    static loadContentLayouts(layoutPropsList: ContentLayoutProps[]): void;
    /** Loads a [[ContentLayout]] into the [[ContentLayoutManager]].
     * @param layoutProps  the properties of the ContentLayout to load
     */
    static loadContentLayout(layoutProps: ContentLayoutProps): void;
    /** Loads one or more Tasks into the [[TaskManager]].
     * @param taskPropsList  the list of Tasks to load
     */
    static loadTasks(taskPropsList: TaskPropsList): void;
    /** Loads a Workflow into the [[WorkflowManager]].
     * @param workflowProps  the Workflow to load
     */
    static loadWorkflow(workflowProps: WorkflowProps): void;
    /** Loads one or more Workflows into the [[WorkflowManager]].
     * @param workflowPropsList  the list of Workflows to load
     */
    static loadWorkflows(workflowPropsList: WorkflowPropsList): void;
    /** Loads one or more Keyboard Shortcuts into the [[KeyboardShortcutManager]].
     * @param shortcutList  the properties of the Keyboard Shortcuts to load
     */
    static loadKeyboardShortcuts(shortcutList: KeyboardShortcutProps[]): void;
    /** Gets the HTML wrapper element for Configurable UI */
    static getWrapperElement(): HTMLElement;
    /** @internal */
    static closeUi(): void;
}
//# sourceMappingURL=ConfigurableUiManager.d.ts.map