"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ConfigurableUi
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigurableUiManager = exports.UiIntervalEvent = exports.UiActivityEvent = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ContentGroup_1 = require("../content/ContentGroup");
const ContentLayoutManager_1 = require("../content/ContentLayoutManager");
const FrontstageDef_1 = require("../frontstage/FrontstageDef");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const KeyboardShortcut_1 = require("../keyboardshortcut/KeyboardShortcut");
const CubeNavigationAidControl_1 = require("../navigationaids/CubeNavigationAidControl");
const DrawingNavigationAidControl_1 = require("../navigationaids/DrawingNavigationAidControl");
const SheetNavigationAid_1 = require("../navigationaids/SheetNavigationAid");
const StandardRotationNavigationAid_1 = require("../navigationaids/StandardRotationNavigationAid");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
const UiFramework_1 = require("../UiFramework");
const Task_1 = require("../workflow/Task");
const Workflow_1 = require("../workflow/Workflow");
const ToolSettingsManager_1 = require("../zones/toolsettings/ToolSettingsManager");
const ConfigurableUiControl_1 = require("./ConfigurableUiControl");
const ModelessDialogManager_1 = require("../dialog/ModelessDialogManager");
const ModalDialogManager_1 = require("../dialog/ModalDialogManager");
const MessageManager_1 = require("../messages/MessageManager");
const PopupManager_1 = require("../popup/PopupManager");
const ActivityTracker_1 = require("./ActivityTracker");
/** Ui Activity Event class.
 * @internal
 */
class UiActivityEvent extends bentleyjs_core_1.BeUiEvent {
}
exports.UiActivityEvent = UiActivityEvent;
/** Ui Interval Event class.
 * @internal
 */
class UiIntervalEvent extends bentleyjs_core_1.BeUiEvent {
}
exports.UiIntervalEvent = UiIntervalEvent;
/** Configurable Ui Manager maintains controls, Frontstages, Content Groups, Content Layouts, Tasks and Workflows.
 * @public
 */
class ConfigurableUiManager {
    /** Initializes the ConfigurableUiManager and registers core controls. */
    static initialize() {
        if (this._initialized)
            return;
        // Register core controls
        ConfigurableUiManager.registerControl(StandardRotationNavigationAid_1.StandardRotationNavigationAidControl.navigationAidId, StandardRotationNavigationAid_1.StandardRotationNavigationAidControl);
        ConfigurableUiManager.registerControl(SheetNavigationAid_1.SheetNavigationAidControl.navigationAidId, SheetNavigationAid_1.SheetNavigationAidControl);
        ConfigurableUiManager.registerControl(DrawingNavigationAidControl_1.DrawingNavigationAidControl.navigationAidId, DrawingNavigationAidControl_1.DrawingNavigationAidControl);
        ConfigurableUiManager.registerControl(CubeNavigationAidControl_1.CubeNavigationAidControl.navigationAidId, CubeNavigationAidControl_1.CubeNavigationAidControl);
        // Initialize SyncUiEventDispatcher so it can register event callbacks.
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.initialize();
        // Initialize the FrontstageManager
        FrontstageManager_1.FrontstageManager.initialize();
        // Initialize the ToolSettingsManager that manages Tool Settings properties.
        ToolSettingsManager_1.ToolSettingsManager.initialize();
        // Initialize the modeless dialog manager.
        ModelessDialogManager_1.ModelessDialogManager.initialize();
        // Initialize the Keyboard Shortcut manager
        KeyboardShortcut_1.KeyboardShortcutManager.initialize();
        this._initialized = true;
    }
    /** Registers a control implementing the [[ConfigurableUiElement]] interface.
     * These controls can be a
     * [[ContentControl]],
     * [[NavigationAidControl]],
     * [[StatusBarWidgetControl]],
     * [[WidgetControl]] or
     * [ToolUiProvider]($ui-framework).
     * @param classId the class id of the control to register
     * @param constructor the constructor of the control to register
     */
    static registerControl(classId, constructor) {
        if (this._registeredControls.get(classId) !== undefined) {
            throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `registerControl: classId '${classId}' already registered`);
        }
        this._registeredControls.set(classId, constructor);
    }
    /** Determines if a control has been registered based on its classId.
     * @param classId   the class id of the control to test
     * @returns  true if the control is registered or false if not
     */
    static isControlRegistered(classId) {
        const constructor = this._registeredControls.get(classId);
        return constructor !== undefined;
    }
    /** Determines if a control has been registered.
     * @internal
     */
    static getConstructorClassId(constructor) {
        for (const [key, value] of this._registeredControls.entries()) {
            if (value === constructor)
                return key;
        }
        return undefined;
    }
    /** Unregisters a control that has been registered.
     * @param classId   the class id of the control to unregister
     */
    static unregisterControl(classId) {
        const constructor = this._registeredControls.get(classId);
        if (constructor)
            this._registeredControls.delete(classId);
    }
    /** Creates a control registered by calling registerControl.
     * @param classId   the class id of the control to create
     * @param uniqueId  a unique id for the control
     * @param options   options passed to the constructor of the control
     * @returns  the created control
     */
    static createControl(classId, uniqueId, options) {
        const info = new ConfigurableUiControl_1.ConfigurableCreateInfo(classId, uniqueId, uniqueId);
        const constructor = this._registeredControls.get(info.classId);
        if (!constructor) {
            throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `createControl: classId '${classId}' not registered`);
        }
        const control = new constructor(info, options);
        return control;
    }
    /** Add a Frontstage via a provider into the [[FrontstageManager]].
     * @param frontstageProvider  Provider of the Frontstage to add
     */
    static addFrontstageProvider(frontstageProvider) {
        FrontstageManager_1.FrontstageManager.addFrontstageProvider(frontstageProvider);
    }
    /** Finds a FrontstageDef, given its id.
     * @param id  the id of the FrontstageDef to find
     * @returns the FrontstageDef with the given id, or undefined if not found
     */
    static findFrontstageDef(id) {
        const frontstageDef = FrontstageManager_1.FrontstageManager.findFrontstageDef(id);
        if (frontstageDef && frontstageDef instanceof FrontstageDef_1.FrontstageDef)
            return frontstageDef;
        return undefined;
    }
    /** Loads one or more ContentGroups into the [[ContentGroupManager]].
     * @param groupPropsList  the list of ContentGroups to load
     */
    static loadContentGroups(groupPropsList) {
        ContentGroup_1.ContentGroupManager.loadGroups(groupPropsList);
    }
    /** Loads a [[ContentGroup]] into the [[ContentGroupManager]].
     * @param groupProps  the properties of the ContentGroup to load
     */
    static loadContentGroup(groupProps) {
        ContentGroup_1.ContentGroupManager.loadGroup(groupProps);
    }
    /** Loads one or more ContentLayouts into the [[ContentLayoutManager]].
     * @param layoutPropsList  the list of ContentLayouts to load
     */
    static loadContentLayouts(layoutPropsList) {
        ContentLayoutManager_1.ContentLayoutManager.loadLayouts(layoutPropsList);
    }
    /** Loads a [[ContentLayout]] into the [[ContentLayoutManager]].
     * @param layoutProps  the properties of the ContentLayout to load
     */
    static loadContentLayout(layoutProps) {
        ContentLayoutManager_1.ContentLayoutManager.loadLayout(layoutProps);
    }
    /** Loads one or more Tasks into the [[TaskManager]].
     * @param taskPropsList  the list of Tasks to load
     */
    static loadTasks(taskPropsList) {
        Task_1.TaskManager.loadTasks(taskPropsList);
    }
    /** Loads a Workflow into the [[WorkflowManager]].
     * @param workflowProps  the Workflow to load
     */
    static loadWorkflow(workflowProps) {
        Workflow_1.WorkflowManager.loadWorkflow(workflowProps);
    }
    /** Loads one or more Workflows into the [[WorkflowManager]].
     * @param workflowPropsList  the list of Workflows to load
     */
    static loadWorkflows(workflowPropsList) {
        Workflow_1.WorkflowManager.loadWorkflows(workflowPropsList);
    }
    /** Loads one or more Keyboard Shortcuts into the [[KeyboardShortcutManager]].
     * @param shortcutList  the properties of the Keyboard Shortcuts to load
     */
    static loadKeyboardShortcuts(shortcutList) {
        KeyboardShortcut_1.KeyboardShortcutManager.loadKeyboardShortcuts(shortcutList);
    }
    /** Gets the HTML wrapper element for Configurable UI */
    static getWrapperElement() {
        const wrapper = document.getElementById("uifw-configurableui-wrapper");
        const htmlElement = wrapper;
        return htmlElement;
    }
    /** @internal */
    static closeUi() {
        MessageManager_1.MessageManager.closeAllMessages();
        ModelessDialogManager_1.ModelessDialogManager.closeAll();
        ModalDialogManager_1.ModalDialogManager.closeAll();
        KeyboardShortcut_1.KeyboardShortcutManager.closeShortcutsMenu();
        UiFramework_1.UiFramework.closeCursorMenu();
        PopupManager_1.PopupManager.clearPopups();
    }
}
exports.ConfigurableUiManager = ConfigurableUiManager;
ConfigurableUiManager._registeredControls = new Map();
ConfigurableUiManager._initialized = false;
/** @internal */
ConfigurableUiManager.activityTracker = new ActivityTracker_1.ActivityTracker();
/** @internal */
ConfigurableUiManager.onUiActivityEvent = new UiActivityEvent();
/** @internal */
ConfigurableUiManager.onUiIntervalEvent = new UiIntervalEvent();
//# sourceMappingURL=ConfigurableUiManager.js.map