"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ContentView
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContentGroupManager = exports.ContentGroup = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ConfigurableUiControl_1 = require("../configurableui/ConfigurableUiControl");
const ConfigurableUiManager_1 = require("../configurableui/ConfigurableUiManager");
const UiFramework_1 = require("../UiFramework");
/** ContentGroup class. Content Groups define content displayed in content views that are laid out using a [[ContentLayout]].
 * @public
 */
class ContentGroup {
    constructor(groupProps) {
        this._contentControls = new Map();
        this._contentSetMap = new Map();
        if (groupProps.id !== undefined)
            this.groupId = groupProps.id;
        else {
            ContentGroup._sId++;
            this.groupId = `ContentGroup-${ContentGroup._sId}`;
        }
        this.contentPropsList = groupProps.contents;
    }
    /** Gets a [[ContentControl]] from the Content Group based on its [[ContentProps]]. */
    getContentControl(contentProps, index) {
        let id;
        if (contentProps.id !== undefined)
            id = contentProps.id;
        else
            id = `${this.groupId}-${index}`;
        let contentControl;
        if (!this._contentControls.get(id)) {
            let usedClassId = "";
            if (typeof contentProps.classId === "string") {
                if (!this._contentControls.get(id) && ConfigurableUiManager_1.ConfigurableUiManager.isControlRegistered(contentProps.classId)) {
                    contentControl = ConfigurableUiManager_1.ConfigurableUiManager.createControl(contentProps.classId, id, contentProps.applicationData);
                    usedClassId = contentProps.classId;
                }
            }
            else {
                const info = new ConfigurableUiControl_1.ConfigurableCreateInfo(contentProps.classId.name, id, id);
                contentControl = new contentProps.classId(info, contentProps.applicationData);
                usedClassId = contentProps.classId.name;
            }
            if (contentControl) {
                if (contentControl.getType() !== ConfigurableUiControl_1.ConfigurableUiControlType.Content && contentControl.getType() !== ConfigurableUiControl_1.ConfigurableUiControlType.Viewport) {
                    throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `getContentControl error: '${usedClassId}' is NOT a ContentControl or ViewportContentControl`);
                }
                contentControl.initialize();
                this._contentControls.set(id, contentControl);
            }
        }
        return this._contentControls.get(id);
    }
    /** Gets a [[ContentControl]] from the Content Group with a given ID. */
    getContentControlById(id) {
        return this._contentControls.get(id);
    }
    /** Gets the React nodes representing the Content Views in this Content Group. */
    getContentNodes() {
        const contentNodes = new Array();
        this._contentSetMap.clear();
        this.contentPropsList.forEach((contentProps, index) => {
            const control = this.getContentControl(contentProps, index);
            if (control) {
                contentNodes.push(control.reactNode);
                this._contentSetMap.set(control.controlId, control);
            }
        });
        return contentNodes;
    }
    /** Gets the [[ContentControl]] associated with a given React node representing a Content View. */
    getControlFromElement(node) {
        if (this._contentSetMap.size === 0)
            this.getContentNodes();
        if (node && node.key)
            return this._contentSetMap.get(node.key);
        bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), `getControlFromElement: no control found for element`);
        return undefined;
    }
    /** Refreshes the React nodes representing the Content Views in this Content Group.. */
    refreshContentNodes() {
        this._contentSetMap.clear();
    }
    /** Gets an array of the content controls representing the Content Views. */
    getContentControls() {
        const contentControls = new Array();
        this.contentPropsList.forEach((contentProps, index) => {
            const control = this.getContentControl(contentProps, index);
            if (control) {
                contentControls.push(control);
            }
        });
        return contentControls;
    }
    /** Called when Frontstage is deactivated. */
    onFrontstageDeactivated() {
        this.clearContentControls();
    }
    /** Called when Frontstage is ready. */
    onFrontstageReady() { }
    /** Clears the map of content controls. */
    clearContentControls() {
        this._contentControls.clear();
    }
    /** Creates [[ContentGroupProps]] for JSON purposes
     * @public
     */
    toJSON(contentCallback) {
        const contentGroupProps = {
            id: this.groupId,
            contents: this.contentPropsList,
        };
        contentGroupProps.contents.forEach((content, index) => {
            if (typeof content.classId !== "string") {
                const classId = ConfigurableUiManager_1.ConfigurableUiManager.getConstructorClassId(content.classId);
                if (classId !== undefined)
                    content.classId = classId;
                else
                    throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `toJSON: ContentControl at index ${index} is NOT registered with a string id`);
                if (contentCallback)
                    contentCallback(content);
            }
        });
        return contentGroupProps;
    }
    /** Gets Viewports from Viewport Content Controls
     * @internal
     */
    getViewports() {
        const contentControls = this.getContentControls();
        const viewports = new Array();
        contentControls.forEach((control) => {
            if (control.isViewport && control.viewport) {
                viewports.push(control.viewport);
            }
            else {
                viewports.push(undefined);
            }
        });
        return viewports;
    }
}
exports.ContentGroup = ContentGroup;
ContentGroup._sId = 0;
// -----------------------------------------------------------------------------
// ContentGroupManager class
// -----------------------------------------------------------------------------
/** ContentGroup Manager class.
 * @public
 */
class ContentGroupManager {
    static loadGroups(groupPropsList) {
        groupPropsList.map((groupProps, _index) => {
            this.loadGroup(groupProps);
        });
    }
    static loadGroup(groupProps) {
        const group = new ContentGroup(groupProps);
        if (groupProps.id)
            this.addGroup(groupProps.id, group);
        else
            throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `loadGroup: ContentGroupProps should contain an id`);
    }
    static findGroup(groupId) {
        return this._groups.get(groupId);
    }
    static addGroup(groupId, group) {
        this._groups.set(groupId, group);
    }
}
exports.ContentGroupManager = ContentGroupManager;
ContentGroupManager._groups = new Map();
//# sourceMappingURL=ContentGroup.js.map