"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ContentView
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContentLayout = exports.ContentLayoutActivatedEvent = exports.ContentLayoutDef = void 0;
// cSpell:ignore contentlayout
require("./ContentLayout.scss");
const classnames = require("classnames");
const React = require("react");
const react_split_pane_1 = require("react-split-pane");
const ui_core_1 = require("@bentley/ui-core");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const UiShowHideManager_1 = require("../utils/UiShowHideManager");
const ContentViewManager_1 = require("./ContentViewManager");
/** ContentWrapper React component.
 */
class ContentWrapper extends React.Component {
    constructor(props) {
        super(props);
        this._handleMouseDown = (_event) => {
            // event.preventDefault();
            ContentViewManager_1.ContentViewManager.setActiveContent(this.state.content);
        };
        this._handleActiveContentChanged = (args) => {
            const isActive = this.state.content === args.activeContent;
            if (this.state.isActive !== isActive) {
                this.setState({ isActive });
            }
        };
        this.state = {
            content: this.props.content,
            isActive: this.props.content === ContentViewManager_1.ContentViewManager.getActiveContent(),
        };
    }
    render() {
        const overlayClassName = classnames("uifw-contentlayout-overlay-div", this.state.isActive ? "uifw-contentlayout-overlay-active" : "uifw-contentlayout-overlay-inactive");
        return (React.createElement("div", { className: classnames("uifw-contentlayout-wrapper", this.props.className), style: this.props.style, onMouseDown: this._handleMouseDown, onMouseMove: UiShowHideManager_1.UiShowHideManager.handleContentMouseMove, role: "presentation" },
            this.state.content,
            React.createElement("div", { className: overlayClassName })));
    }
    componentDidMount() {
        ContentViewManager_1.ContentViewManager.onActiveContentChangedEvent.addListener(this._handleActiveContentChanged);
    }
    componentWillUnmount() {
        ContentViewManager_1.ContentViewManager.onActiveContentChangedEvent.removeListener(this._handleActiveContentChanged);
    }
    // istanbul ignore next
    componentDidUpdate(prevProps, _prevState) {
        if (this.props.content !== prevProps.content) {
            this.setState((_, props) => ({ content: props.content, isActive: props.content === ContentViewManager_1.ContentViewManager.getActiveContent() }));
        }
    }
}
/** Split Container class.
 */
class SplitContainer extends React.Component {
    constructor(props) {
        super(props);
        this._containerDiv = null;
        this._onSplitterChange = (size) => {
            let percentage = 0;
            // istanbul ignore else
            if (this._containerDiv && size > 0) {
                if (this.props.orientation === ui_core_1.Orientation.Horizontal) {
                    const height = this._containerDiv.getBoundingClientRect().height;
                    // istanbul ignore if
                    if (height > 0)
                        percentage = size / height;
                }
                else {
                    const width = this._containerDiv.getBoundingClientRect().width;
                    // istanbul ignore if
                    if (width > 0)
                        percentage = size / width;
                }
                // istanbul ignore else
                if (this.props.onSplitterChange)
                    this.props.onSplitterChange(size, percentage);
            }
        };
    }
    render() {
        const orientation = (this.props.orientation === ui_core_1.Orientation.Horizontal) ? "horizontal" : "vertical";
        const defaultSize = `${(this.props.percentage * 100).toString()}%`;
        const minSizeTopLeft = this.props.minSizeTopLeft;
        const minSizeBottomRight = -(this.props.minSizeBottomRight);
        return (React.createElement("div", { ref: (e) => { this._containerDiv = e; }, className: classnames("uifw-contentlayout-full-size", this.props.className), style: this.props.style },
            React.createElement(react_split_pane_1.default, { split: orientation, minSize: minSizeTopLeft, maxSize: minSizeBottomRight, defaultSize: defaultSize, onChange: this._onSplitterChange, allowResize: this.props.resizable },
                this.props.contentA,
                this.props.contentB)));
    }
}
/** Single Content Container class.
 */
class SingleContentContainer extends React.Component {
    render() {
        return (React.createElement("div", { className: classnames("uifw-contentlayout-full-size", this.props.className), style: this.props.style, "data-testid": "single-content-container", onMouseMove: UiShowHideManager_1.UiShowHideManager.handleContentMouseMove }, this.props.content));
    }
}
/** Minimum split size  */
const MIN_SPLIT_SIZE = 6;
/** Base Split class.
 */
class BaseSplit {
    constructor(props) {
        this.stateId = "";
        this.isLocked = false;
        this.defaultPercentage = props.percentage;
        if (props.id)
            this.stateId = props.id;
        if (props.lock)
            this.isLocked = props.lock;
    }
}
/** Horizontal Split class.
 */
class HorizontalSplit extends BaseSplit {
    constructor(props) {
        super(props);
        this._topIndex = -1;
        this._bottomIndex = -1;
        this._minSizeTop = MIN_SPLIT_SIZE;
        this._minSizeBottom = MIN_SPLIT_SIZE;
        this._handleSplitterChange = (_size, percentage) => {
            this._props.percentage = percentage;
        };
        this._props = props;
        if (typeof props.top === "number") {
            this._topIndex = props.top;
        }
        else {
            this._topSplit = ContentLayoutDef.createSplit(props.top);
        }
        if (typeof props.bottom === "number") {
            this._bottomIndex = props.bottom;
        }
        else {
            this._bottomSplit = ContentLayoutDef.createSplit(props.bottom);
        }
        if (props.minSizeTop)
            this._minSizeTop = Math.max(props.minSizeTop, MIN_SPLIT_SIZE);
        if (props.minSizeBottom)
            this._minSizeBottom = Math.max(props.minSizeBottom, MIN_SPLIT_SIZE);
    }
    createContentContainer(contentNodes, resizable) {
        if (this.isLocked)
            resizable = false;
        const topContent = (!this._topSplit) ?
            React.createElement(ContentWrapper, { content: contentNodes[this._topIndex] }) :
            this._topSplit.createContentContainer(contentNodes, resizable);
        const bottomContent = (!this._bottomSplit) ?
            React.createElement(ContentWrapper, { content: contentNodes[this._bottomIndex] }) :
            this._bottomSplit.createContentContainer(contentNodes, resizable);
        return (React.createElement(SplitContainer, { contentA: topContent, contentB: bottomContent, orientation: ui_core_1.Orientation.Horizontal, percentage: this.defaultPercentage, resizable: resizable, minSizeTopLeft: this._minSizeTop, minSizeBottomRight: this._minSizeBottom, splitterStateId: this.stateId, onSplitterChange: this._handleSplitterChange }));
    }
}
/** Vertical Split class.
 */
class VerticalSplit extends BaseSplit {
    constructor(props) {
        super(props);
        this._leftIndex = -1;
        this._rightIndex = -1;
        this._minSizeLeft = MIN_SPLIT_SIZE;
        this._minSizeRight = MIN_SPLIT_SIZE;
        this._handleSplitterChange = (_size, percentage) => {
            this._props.percentage = percentage;
        };
        this._props = props;
        if (typeof props.left === "number") {
            this._leftIndex = props.left;
        }
        else {
            this._leftSplit = ContentLayoutDef.createSplit(props.left);
        }
        if (typeof props.right === "number") {
            this._rightIndex = props.right;
        }
        else {
            this._rightSplit = ContentLayoutDef.createSplit(props.right);
        }
        if (props.minSizeLeft)
            this._minSizeLeft = Math.max(props.minSizeLeft, MIN_SPLIT_SIZE);
        if (props.minSizeRight)
            this._minSizeRight = Math.max(props.minSizeRight, MIN_SPLIT_SIZE);
    }
    createContentContainer(contentNodes, resizable) {
        if (this.isLocked)
            resizable = false;
        const leftContent = (!this._leftSplit) ?
            React.createElement(ContentWrapper, { content: contentNodes[this._leftIndex] }) :
            this._leftSplit.createContentContainer(contentNodes, resizable);
        const rightContent = (!this._rightSplit) ?
            React.createElement(ContentWrapper, { content: contentNodes[this._rightIndex] }) :
            this._rightSplit.createContentContainer(contentNodes, resizable);
        return (React.createElement(SplitContainer, { contentA: leftContent, contentB: rightContent, orientation: ui_core_1.Orientation.Vertical, percentage: this.defaultPercentage, resizable: resizable, minSizeTopLeft: this._minSizeLeft, minSizeBottomRight: this._minSizeRight, splitterStateId: this.stateId, onSplitterChange: this._handleSplitterChange }));
    }
}
/** Content Layout Definition class.
 * @public
 */
class ContentLayoutDef {
    constructor(layoutProps) {
        /** ID for this Content Layout */
        this.id = "";
        /** Localization key for a description. */
        this.descriptionKey = "";
        /** The priority for the layout. Determines its position in menus. Higher numbers appear first. */
        this.priority = 0;
        this._layoutProps = layoutProps;
        if (layoutProps.id)
            this.id = layoutProps.id;
        else {
            ContentLayoutDef._sId++;
            this.id = `ContentLayout-${ContentLayoutDef._sId}`;
        }
        if (layoutProps.descriptionKey !== undefined)
            this.descriptionKey = layoutProps.descriptionKey;
        if (layoutProps.priority !== undefined)
            this.priority = layoutProps.priority;
    }
    get rootSplit() { return this._rootSplit; }
    /** Creates [[ContentLayoutProps]] for JSON purposes
     * @public
     */
    toJSON() { return this._layoutProps; }
    /** Fill a layout container with React nodes for each content view
     */
    fillLayoutContainer(contentNodes, resizable) {
        this._rootSplit = ContentLayoutDef.createSplit(this._layoutProps);
        if (this.rootSplit) {
            return this.rootSplit.createContentContainer(contentNodes, resizable);
        }
        if (contentNodes.length > 0)
            return React.createElement(SingleContentContainer, { content: contentNodes[0] });
        return undefined;
    }
    /** Gets the indexes of content views used in this Content Layout
     */
    getUsedContentIndexes() {
        let allContentIndexes = [];
        if (!this._layoutProps.horizontalSplit && !this._layoutProps.verticalSplit)
            allContentIndexes.push(0);
        else {
            allContentIndexes = allContentIndexes.concat(this.getHorizontalSplitContentIndexes(this._layoutProps.horizontalSplit));
            allContentIndexes = allContentIndexes.concat(this.getVerticalSplitContentIndexes(this._layoutProps.verticalSplit));
        }
        const uniqueContentIndexes = [...new Set(allContentIndexes)];
        return uniqueContentIndexes;
    }
    getHorizontalSplitContentIndexes(splitProps) {
        let contentIndexes = [];
        if (!splitProps)
            return contentIndexes;
        if (typeof splitProps.top === "number")
            contentIndexes.push(splitProps.top);
        else {
            contentIndexes = contentIndexes.concat(this.getHorizontalSplitContentIndexes(splitProps.top.horizontalSplit));
            contentIndexes = contentIndexes.concat(this.getVerticalSplitContentIndexes(splitProps.top.verticalSplit));
        }
        if (typeof splitProps.bottom === "number")
            contentIndexes.push(splitProps.bottom);
        else {
            contentIndexes = contentIndexes.concat(this.getHorizontalSplitContentIndexes(splitProps.bottom.horizontalSplit));
            contentIndexes = contentIndexes.concat(this.getVerticalSplitContentIndexes(splitProps.bottom.verticalSplit));
        }
        return contentIndexes;
    }
    getVerticalSplitContentIndexes(splitProps) {
        let contentIndexes = [];
        if (!splitProps)
            return contentIndexes;
        if (typeof splitProps.left === "number")
            contentIndexes.push(splitProps.left);
        else {
            contentIndexes = contentIndexes.concat(this.getHorizontalSplitContentIndexes(splitProps.left.horizontalSplit));
            contentIndexes = contentIndexes.concat(this.getVerticalSplitContentIndexes(splitProps.left.verticalSplit));
        }
        if (typeof splitProps.right === "number")
            contentIndexes.push(splitProps.right);
        else {
            contentIndexes = contentIndexes.concat(this.getHorizontalSplitContentIndexes(splitProps.right.horizontalSplit));
            contentIndexes = contentIndexes.concat(this.getVerticalSplitContentIndexes(splitProps.right.verticalSplit));
        }
        return contentIndexes;
    }
    /** @internal */
    static createSplit(fragmentDef) {
        if (fragmentDef.horizontalSplit) {
            return new HorizontalSplit(fragmentDef.horizontalSplit);
        }
        else if (fragmentDef.verticalSplit) {
            return new VerticalSplit(fragmentDef.verticalSplit);
        }
        return undefined;
    }
}
exports.ContentLayoutDef = ContentLayoutDef;
ContentLayoutDef._sId = 0;
/** Content Layout Activated Event class.
 * @public
 */
class ContentLayoutActivatedEvent extends ui_core_1.UiEvent {
}
exports.ContentLayoutActivatedEvent = ContentLayoutActivatedEvent;
/** Content Layout React component.
 * @public
 */
class ContentLayout extends React.Component {
    constructor(props) {
        super(props);
        this._handleContentLayoutActivated = (args) => {
            const contentLayoutDef = args.contentLayout;
            const contentGroup = args.contentGroup;
            const contentNodes = contentGroup.getContentNodes();
            const contentContainer = contentLayoutDef.fillLayoutContainer(contentNodes, true);
            this.setState({
                contentLayoutDef: args.contentLayout,
                contentContainer,
            });
        };
        this._onMouseDown = (_event) => {
            ContentViewManager_1.ContentViewManager.setMouseDown(true);
        };
        this._onMouseUp = (_event) => {
            ContentViewManager_1.ContentViewManager.setMouseDown(false);
        };
        const contentLayoutDef = this.props.contentLayout;
        const contentGroup = this.props.contentGroup;
        const contentNodes = contentGroup.getContentNodes();
        const contentContainer = contentLayoutDef.fillLayoutContainer(contentNodes, true);
        this.state = {
            contentLayoutDef: this.props.contentLayout,
            contentContainer,
        };
    }
    componentDidMount() {
        FrontstageManager_1.FrontstageManager.onContentLayoutActivatedEvent.addListener(this._handleContentLayoutActivated);
    }
    componentWillUnmount() {
        FrontstageManager_1.FrontstageManager.onContentLayoutActivatedEvent.removeListener(this._handleContentLayoutActivated);
    }
    render() {
        if (this.state.contentContainer) {
            return (React.createElement("div", { id: "uifw-contentlayout-div", className: this.props.className, style: this.props.style, key: this.state.contentLayoutDef.id, onMouseDown: this._onMouseDown, onMouseUp: this._onMouseUp, role: "presentation" }, this.state.contentContainer));
        }
        return null;
    }
}
exports.ContentLayout = ContentLayout;
//# sourceMappingURL=ContentLayout.js.map