"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ContentView
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContentLayoutManager = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const UiFramework_1 = require("../UiFramework");
const ContentLayout_1 = require("./ContentLayout");
/** ContentLayout Manager class.
 * @public
 */
class ContentLayoutManager {
    /** Loads one or more Content Layouts.
     * @param layoutPropsList  the list of Content Layout properties to load
     */
    static loadLayouts(layoutPropsList) {
        layoutPropsList.map((layoutProps, _index) => {
            ContentLayoutManager.loadLayout(layoutProps);
        });
    }
    /** Loads a Content Layout.
     * @param layoutProps  the properties of the Content Layout to load
     */
    static loadLayout(layoutProps) {
        const layout = new ContentLayout_1.ContentLayoutDef(layoutProps);
        if (layoutProps.id)
            ContentLayoutManager.addLayout(layoutProps.id, layout);
        else
            throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `loadLayout: ContentLayoutProps should contain an 'id'`);
    }
    /** Finds a Content Layout with a given id.
     * @param layoutId  the id of the Content Layout to find
     * @returns the [[ContentLayoutDef]] if found, or undefined otherwise
     */
    static findLayout(layoutId) {
        return ContentLayoutManager._layoutDefs.get(layoutId);
    }
    /** Adds a Content Layout.
     * @param layoutId  the id of the Content Layout to add
     * @param layoutDef  the Content Layout definition to add
     */
    static addLayout(layoutId, layoutDef) {
        ContentLayoutManager._layoutDefs.set(layoutId, layoutDef);
    }
    /** Gets the active Content Layout */
    static get activeLayout() {
        let layoutDef;
        const activeFrontstageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
        if (activeFrontstageDef)
            layoutDef = activeFrontstageDef.contentLayoutDef;
        return layoutDef;
    }
    /** Gets the active Content Group */
    static get activeContentGroup() {
        let contentGroup;
        const activeFrontstageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
        if (activeFrontstageDef)
            contentGroup = activeFrontstageDef.contentGroup;
        return contentGroup;
    }
    /** Sets the active Content Layout, Content Group and Content Control.
     * @param contentLayoutDef  Content layout to make active
     * @param contentGroup  Content Group to make active
     */
    static async setActiveLayout(contentLayoutDef, contentGroup) {
        await FrontstageManager_1.FrontstageManager.setActiveLayout(contentLayoutDef, contentGroup);
    }
    /** Refreshes the active layout and content group.
     */
    static refreshActiveLayout() {
        // istanbul ignore else
        const activeFrontstageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
        if (activeFrontstageDef && activeFrontstageDef.contentLayoutDef && activeFrontstageDef.contentGroup) {
            FrontstageManager_1.FrontstageManager.onContentLayoutActivatedEvent.emit({
                contentLayout: activeFrontstageDef.contentLayoutDef,
                contentGroup: activeFrontstageDef.contentGroup,
            });
            activeFrontstageDef.contentGroup.refreshContentNodes();
        }
    }
}
exports.ContentLayoutManager = ContentLayoutManager;
ContentLayoutManager._layoutDefs = new Map();
//# sourceMappingURL=ContentLayoutManager.js.map