"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ContentView
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContentViewManager = exports.ActiveContentChangedEvent = exports.MouseDownChangedEvent = void 0;
const ui_core_1 = require("@bentley/ui-core");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const ViewUtilities_1 = require("../utils/ViewUtilities");
const ContentLayoutManager_1 = require("./ContentLayoutManager");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
/** Mouse Down Changed Event class.
 * @public
 */
class MouseDownChangedEvent extends ui_core_1.UiEvent {
}
exports.MouseDownChangedEvent = MouseDownChangedEvent;
/** Active Content Changed Event class.
 * @public
 */
class ActiveContentChangedEvent extends ui_core_1.UiEvent {
}
exports.ActiveContentChangedEvent = ActiveContentChangedEvent;
/** Content View Manager class.
 * @public
 */
class ContentViewManager {
    /** Determines if the mouse is down in a content view */
    static get isMouseDown() {
        return this._mouseDown;
    }
    /** Sets the mouse down status for a content view */
    static setMouseDown(mouseDown) {
        this._mouseDown = mouseDown;
        this.onMouseDownChangedEvent.emit({ mouseDown });
    }
    /** Gets the active content as a React.ReactNode. */
    static getActiveContent() {
        return this._activeContent;
    }
    /** Return the active ContentControl. */
    static getActiveContentControl() {
        let activeContentControl;
        const activeFrontstageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
        // istanbul ignore else
        if (this._activeContent && activeFrontstageDef) {
            const activeContentGroup = activeFrontstageDef.contentGroup;
            // istanbul ignore else
            if (activeContentGroup) {
                activeContentControl = activeContentGroup.getControlFromElement(this._activeContent);
            }
        }
        return activeContentControl;
    }
    /** Sets the active [[ContentControl]] */
    static setActiveContent(activeContent, forceEventProcessing = false) {
        // istanbul ignore else
        if (this._activeContent !== activeContent || forceEventProcessing) {
            const oldContent = this._activeContent;
            this._activeContent = activeContent;
            const activeFrontstageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
            // istanbul ignore else
            if (activeFrontstageDef) {
                const activeContentGroup = activeFrontstageDef.contentGroup;
                // istanbul ignore else
                if (activeContentGroup) {
                    const oldContentControl = oldContent ? activeContentGroup.getControlFromElement(oldContent) : undefined;
                    const activeContentControl = activeContentGroup.getControlFromElement(this._activeContent);
                    // Only call setActiveView if going to or coming from a non-viewport ContentControl
                    // istanbul ignore else
                    if (activeContentControl) {
                        const doSetActiveView = forceEventProcessing || (!activeContentControl.viewport ||
                            /* istanbul ignore next */ (oldContentControl && !oldContentControl.viewport));
                        // istanbul ignore else
                        if (doSetActiveView) {
                            activeFrontstageDef.setActiveView(activeContentControl, oldContentControl);
                            this.onActiveContentChangedEvent.emit({ activeContent, oldContent });
                        }
                        else {
                            if (activeContentControl.viewport && activeContentControl.viewport !== imodeljs_frontend_1.IModelApp.viewManager.selectedView) {
                                imodeljs_frontend_1.IModelApp.viewManager.setSelectedView(activeContentControl.viewport);
                            }
                        }
                    }
                }
            }
        }
    }
    /** Refreshes the active [[ContentControl]] */
    static refreshActiveContent(activeContent) {
        ContentLayoutManager_1.ContentLayoutManager.refreshActiveLayout();
        this.setActiveContent(activeContent, true);
    }
    /**
     * Determines if content displays a Sheet view.
     * @param content ContentControl to check
     */
    static isContentSheetView(content) {
        if (!content || !content.viewport)
            return false;
        return (ViewUtilities_1.ViewUtilities.isSheetView(content.viewport));
    }
    /**
     * Determines if content displays a Drawing view.
     * @param content ContentControl to check
     */
    static isContentDrawingView(content) {
        if (!content || !content.viewport)
            return false;
        return (ViewUtilities_1.ViewUtilities.isDrawingView(content.viewport));
    }
    /**
     * Determines if content displays a Spatial view.
     * @param content ContentControl to check
     */
    static isContentSpatialView(content) {
        if (!content || !content.viewport)
            return false;
        return (ViewUtilities_1.ViewUtilities.isSpatialView(content.viewport));
    }
    /**
     * Determines if content displays a Orthographic view.
     * @param content ContentControl to check
     */
    static isContentOrthographicView(content) {
        if (!content || !content.viewport)
            return false;
        return (ViewUtilities_1.ViewUtilities.isOrthographicView(content.viewport));
    }
    /**
     * Determines if content displays a 3d view.
     * @param content ContentControl to check
     */
    static isContent3dView(content) {
        if (!content || !content.viewport)
            return false;
        return (ViewUtilities_1.ViewUtilities.is3dView(content.viewport));
    }
    /**
     * Determines if viewport supports use of a camera.
     * @param content ContentControl to check
     */
    static contentSupportsCamera(content) {
        if (!content || !content.viewport)
            return false;
        return (ViewUtilities_1.ViewUtilities.viewSupportsCamera(content.viewport));
    }
}
exports.ContentViewManager = ContentViewManager;
ContentViewManager._mouseDown = false;
/** Gets the [[MouseDownChangedEvent]] */
ContentViewManager.onMouseDownChangedEvent = new MouseDownChangedEvent();
/** Gets the [[ActiveContentChangedEvent]] */
ContentViewManager.onActiveContentChangedEvent = new ActiveContentChangedEvent();
//# sourceMappingURL=ContentViewManager.js.map