"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ContentView
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelViewportControl = exports.IModelConnectedViewport = void 0;
const React = require("react");
const presentation_components_1 = require("@bentley/presentation-components");
const ui_components_1 = require("@bentley/ui-components");
const ui_core_1 = require("@bentley/ui-core");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const ConfigurableUiManager_1 = require("../configurableui/ConfigurableUiManager");
const connectIModel_1 = require("../redux/connectIModel");
const UiFramework_1 = require("../UiFramework");
const DefaultViewOverlay_1 = require("./DefaultViewOverlay");
const ViewportContentControl_1 = require("./ViewportContentControl");
const StandardRotationNavigationAid_1 = require("../navigationaids/StandardRotationNavigationAid");
const ui_abstract_1 = require("@bentley/ui-abstract");
// create a HOC viewport component that supports unified selection
// eslint-disable-next-line @typescript-eslint/naming-convention
const UnifiedSelectionViewport = presentation_components_1.viewWithUnifiedSelection(ui_components_1.ViewportComponent);
/** Viewport that is connected to the IModelConnection property in the Redux store. The application must set up the Redux store and include the FrameworkReducer.
 * @beta
 */
exports.IModelConnectedViewport = connectIModel_1.connectIModelConnectionAndViewState(null, null)(UnifiedSelectionViewport); // eslint-disable-line @typescript-eslint/naming-convention
/** iModel Viewport Control
 * @beta
 */
// istanbul ignore next
class IModelViewportControl extends ViewportContentControl_1.ViewportContentControl {
    constructor(info, _options) {
        var _a, _b;
        super(info, _options);
        this._options = _options;
        /**
         * This is passed as a function to the ViewportComponent as the `viewState` prop in getImodelViewportReactElement().
         * It is called by ViewportComponent on a mount to resolve the viewState.
         */
        this._determineViewState = () => {
            let viewState;
            if (this.viewport && !this._alwaysUseSuppliedViewState)
                viewState = this.viewport.view;
            else if (this._viewState) {
                if (typeof this._viewState === "function")
                    viewState = this._viewState();
                else
                    viewState = this._viewState;
            }
            else
                throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), "No ViewState could be determined");
            return viewState;
        };
        /** Get the default ViewOverlay unless parameter is set to not use it. May be override in an application specific sub-class  */
        this._getViewOverlay = (viewport) => {
            if (this._userSuppliedViewOverlay)
                return this._userSuppliedViewOverlay(viewport);
            if (this._disableDefaultViewOverlay)
                return null;
            return React.createElement(DefaultViewOverlay_1.DefaultViewOverlay, { viewport: viewport });
        };
        this._disableDefaultViewOverlay = (_a = _options.disableDefaultViewOverlay) !== null && _a !== void 0 ? _a : false;
        this._alwaysUseSuppliedViewState = (_b = _options.alwaysUseSuppliedViewState) !== null && _b !== void 0 ? _b : false;
        this._userSuppliedViewOverlay = _options.supplyViewOverlay;
        if (!_options.deferNodeInitialization)
            this.initializeReactNode();
    }
    static get id() {
        return "UiFramework.IModelViewportControl";
    }
    initializeReactNode() {
        const options = this._options;
        if (options.viewState)
            this._viewState = options.viewState;
        const iModelConnection = (typeof options.iModelConnection === "function") ? options.iModelConnection() : options.iModelConnection;
        if (this._viewState && iModelConnection) {
            /** Passing _determineViewState as a function reference; it is not called here. */
            this._reactNode = this.getImodelViewportReactElement(iModelConnection, this._determineViewState);
        }
        else {
            if (UiFramework_1.UiFramework.getIModelConnection() && UiFramework_1.UiFramework.getDefaultViewState()) {
                this._reactNode = this.getImodelConnectedViewportReactElement();
            }
            else {
                this._reactNode = this.getNoContentReactElement(options);
                this.setIsReady();
            }
        }
    }
    getReactNode() {
        if (!React.isValidElement(this._reactNode) && this._options.deferNodeInitialization)
            this.initializeReactNode();
        return this.getKeyedReactNode();
    }
    /** Get the React component that will contain the Viewport */
    getImodelConnectedViewportReactElement() {
        return React.createElement(exports.IModelConnectedViewport, { viewportRef: (v) => {
                var _a;
                this.viewport = v;
                // for convenience, if window defined bind viewport to window
                if (undefined !== window)
                    window.viewport = v;
                if (!FrontstageManager_1.FrontstageManager.isLoading)
                    (_a = FrontstageManager_1.FrontstageManager.activeFrontstageDef) === null || _a === void 0 ? void 0 : _a.setActiveViewFromViewport(v);
            }, getViewOverlay: this._getViewOverlay });
    }
    /** Get the React component that will contain the Viewport */
    getImodelViewportReactElement(iModelConnection, viewState) {
        return React.createElement(UnifiedSelectionViewport, { viewState: viewState, imodel: iModelConnection, viewportRef: (v) => {
                var _a;
                this.viewport = v;
                // for convenience, if window defined bind viewport to window
                if (undefined !== window)
                    window.viewport = v;
                if (!FrontstageManager_1.FrontstageManager.isLoading)
                    (_a = FrontstageManager_1.FrontstageManager.activeFrontstageDef) === null || _a === void 0 ? void 0 : _a.setActiveViewFromViewport(v);
            }, getViewOverlay: this._getViewOverlay });
    }
    /** Get the React component that will be shown when no iModel data is available */
    getNoContentReactElement(_options) {
        const noContent = UiFramework_1.UiFramework.translate("general.no-content");
        return React.createElement(ui_core_1.FillCentered, null,
            " ",
            noContent,
            " ");
    }
    /** Get the React.Element for a ViewSelector change. */
    getReactElementForViewSelectorChange(iModelConnection, _unusedViewDefinitionId, viewState, _name) {
        return this.getImodelViewportReactElement(iModelConnection, viewState);
    }
    /** Get the NavigationAidControl associated with this ContentControl */
    get navigationAidControl() {
        if (this.viewport)
            return super.navigationAidControl;
        else
            return StandardRotationNavigationAid_1.StandardRotationNavigationAidControl.navigationAidId;
    }
}
exports.IModelViewportControl = IModelViewportControl;
ConfigurableUiManager_1.ConfigurableUiManager.registerControl(IModelViewportControl.id, IModelViewportControl);
//# sourceMappingURL=IModelViewport.js.map