"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Dialog
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DialogRendererBase = exports.DialogManagerBase = exports.DialogChangedEvent = void 0;
const React = require("react");
const ui_core_1 = require("@bentley/ui-core");
/** Dialog Changed Event class.
 * @public
 */
class DialogChangedEvent extends ui_core_1.UiEvent {
}
exports.DialogChangedEvent = DialogChangedEvent;
/** Dialog Manager class.
 * @internal
 */
class DialogManagerBase {
    constructor(onDialogChangedEvent) {
        this._dialogs = new Array();
        this._onDialogChangedEvent = onDialogChangedEvent;
    }
    get dialogs() { return this._dialogs; }
    get onDialogChangedEvent() { return this._onDialogChangedEvent; }
    /**
     * Triggers opening a dialog.
     * @param dialog Dialog React component.
     * @param id The unique Id the identifies the dialog.
     * @param parentDocument Optional document required when displaying a dialog in a child popup window.
     */
    openDialog(dialog, id, parentDocument) {
        if (!id)
            id = `Dialog-${++DialogManagerBase._sId}`;
        // istanbul ignore next
        const owningDoc = parentDocument !== null && parentDocument !== void 0 ? parentDocument : document;
        this.pushDialog({ reactNode: dialog, id, parentDocument: owningDoc });
    }
    /** @internal */
    pushDialog(dialogInfo) {
        this._dialogs.push(dialogInfo);
        this.emitDialogChangedEvent();
    }
    closeDialog(dialog) {
        let targetDialog = dialog;
        if (!dialog)
            targetDialog = this.activeDialog;
        this.removeDialog(targetDialog);
        this.emitDialogChangedEvent();
    }
    /** @internal */
    closeAll() {
        this._dialogs = [];
        this.emitDialogChangedEvent();
    }
    /** @internal */
    removeDialog(dialog) {
        const index = this._dialogs.findIndex((dialogInfo) => {
            return dialog === dialogInfo.reactNode;
        });
        if (index >= 0)
            this._dialogs.splice(index, 1);
    }
    emitDialogChangedEvent() {
        this._onDialogChangedEvent.emit({ dialogCount: this.dialogCount, activeDialog: this.activeDialog });
    }
    update() {
        this.emitDialogChangedEvent();
    }
    get activeDialog() {
        if (this._dialogs.length > 0)
            return this._dialogs[this._dialogs.length - 1].reactNode;
        return undefined;
    }
    get dialogCount() {
        return this._dialogs.length;
    }
}
exports.DialogManagerBase = DialogManagerBase;
DialogManagerBase._sId = 0;
/** DialogRenderer React component.
 * @internal
 */
class DialogRendererBase extends React.PureComponent {
    constructor() {
        super(...arguments);
        /** @internal */
        this.state = {
            parentDocument: null,
        };
        this._handleRefSet = (popupDiv) => {
            var _a;
            this.setState({ parentDocument: (_a = popupDiv === null || popupDiv === void 0 ? void 0 : popupDiv.ownerDocument) !== null && _a !== void 0 ? _a : null });
        };
        this._handleDialogChangedEvent = (_args) => {
            this.forceUpdate();
        };
    }
    render() {
        if (this.props.dialogManager.dialogCount <= 0)
            return null;
        return (React.createElement("div", { className: "ui-framework-dialog-render-container", ref: this._handleRefSet }, this.state.parentDocument &&
            this.props.dialogManager.dialogs.filter((info) => info.parentDocument === this.state.parentDocument)
                .map((dialogInfo) => {
                return (React.createElement(React.Fragment, { key: dialogInfo.id }, dialogInfo.reactNode));
            })));
    }
    componentDidMount() {
        this.props.dialogManager.onDialogChangedEvent.addListener(this._handleDialogChangedEvent);
    }
    componentWillUnmount() {
        this.props.dialogManager.onDialogChangedEvent.removeListener(this._handleDialogChangedEvent);
    }
}
exports.DialogRendererBase = DialogRendererBase;
//# sourceMappingURL=DialogManagerBase.js.map