"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Dialog
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelessDialogRenderer = exports.ModelessDialogManager = exports.ModelessDialogChangedEvent = void 0;
const React = require("react");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_core_1 = require("@bentley/ui-core");
const UiFramework_1 = require("../UiFramework");
const DialogManagerBase_1 = require("./DialogManagerBase");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
// cSpell:ignore ZINDEX modeless
/** Modeless Dialog Changed Event class.
 * @public
 */
class ModelessDialogChangedEvent extends DialogManagerBase_1.DialogChangedEvent {
}
exports.ModelessDialogChangedEvent = ModelessDialogChangedEvent;
/** Used if the 'dialog' z-index CSS variable cannot be read */
const ZINDEX_DEFAULT = 12000;
/** Modeless Dialog Manager class displays and manages multiple modeless dialogs
 * @public
 */
class ModelessDialogManager {
    /** Get the array of modeless dialogs */
    static get dialogs() { return ModelessDialogManager.dialogManager.dialogs; }
    /** Initialize the modeless dialog manager */
    static initialize() {
        ModelessDialogManager._topZIndex = ModelessDialogManager.getDialogZIndexDefault();
    }
    static getDialogZIndexDefault() {
        const variable = "--uicore-z-index-dialog";
        const value = ui_core_1.getCssVariableAsNumber(variable);
        // istanbul ignore next
        if (!isNaN(value))
            return value;
        bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), `'${variable}' CSS variable not found`);
        return ZINDEX_DEFAULT;
    }
    /** Open a modeless dialog
     * @param dialog The Dialog to open
     * @param id The id of the Dialog to open
     */
    static openDialog(dialog, id, parentDocument = document) {
        const dialogInfo = ModelessDialogManager._dialogMap.get(id);
        if (dialogInfo) {
            const message = `Dialog with id of '${id}' already opened`;
            bentleyjs_core_1.Logger.logInfo(UiFramework_1.UiFramework.loggerCategory(this), `openDialog: ${message}`);
            imodeljs_frontend_1.IModelApp.notifications.outputMessage(new imodeljs_frontend_1.NotifyMessageDetails(imodeljs_frontend_1.OutputMessagePriority.Info, message, undefined, imodeljs_frontend_1.OutputMessageType.Toast));
            return;
        }
        ModelessDialogManager._dialogMap.set(id, { reactNode: dialog, zIndex: ++ModelessDialogManager._topZIndex, parentDocument });
        ModelessDialogManager._idArray.push(id);
        ModelessDialogManager.dialogManager.openDialog(dialog, id, parentDocument);
    }
    /** Close a modeless dialog
     * @param id The id of the Dialog to close.
     */
    static closeDialog(id) {
        const dialogInfo = ModelessDialogManager._dialogMap.get(id);
        if (dialogInfo) {
            ModelessDialogManager.dialogManager.removeDialog(dialogInfo.reactNode);
            ModelessDialogManager._dialogMap.delete(id);
            const index = ModelessDialogManager._idArray.indexOf(id);
            // istanbul ignore else
            if (index >= 0)
                ModelessDialogManager._idArray.splice(index, 1);
            if (ModelessDialogManager.activeDialog === undefined)
                ModelessDialogManager._topZIndex = ModelessDialogManager.getDialogZIndexDefault();
            this.update();
        }
        else {
            bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), `closeDialog: Could not find dialog with id of '${id}'`);
        }
    }
    /** @internal */
    static closeAll() {
        ModelessDialogManager.dialogManager.closeAll();
    }
    /** Update the dialogs */
    static update() {
        ModelessDialogManager.dialogManager.update();
    }
    /** Get the active modeless dialog */
    static get activeDialog() {
        if (ModelessDialogManager._idArray.length > 0) {
            const id = ModelessDialogManager._idArray[ModelessDialogManager._idArray.length - 1];
            const dialogInfo = ModelessDialogManager._dialogMap.get(id);
            // istanbul ignore else
            if (dialogInfo)
                return dialogInfo.reactNode;
        }
        return undefined;
    }
    /** Get the count of modeless dialogs */
    static get dialogCount() {
        return ModelessDialogManager.dialogManager.dialogCount;
    }
    /** Handle a pointer down event on a modeless dialog */
    static handlePointerDownEvent(_event, id, updateFunc) {
        const dialogInfo = ModelessDialogManager._dialogMap.get(id);
        if (dialogInfo && dialogInfo.reactNode !== ModelessDialogManager.activeDialog) {
            dialogInfo.zIndex = ++ModelessDialogManager._topZIndex;
            ModelessDialogManager._idArray.splice(ModelessDialogManager._idArray.indexOf(id), 1);
            ModelessDialogManager._idArray.push(id);
            updateFunc();
            this.update();
        }
    }
    /** Get the z-index for a modeless dialog */
    static getDialogZIndex(id) {
        let zIndex = ModelessDialogManager.getDialogZIndexDefault();
        const dialogInfo = ModelessDialogManager._dialogMap.get(id);
        // istanbul ignore else
        if (dialogInfo)
            zIndex = dialogInfo.zIndex;
        return zIndex;
    }
    static getDialogInfo(id) {
        return ModelessDialogManager._dialogMap.get(id);
    }
}
exports.ModelessDialogManager = ModelessDialogManager;
/** Modeless Dialog Changed Event */
ModelessDialogManager.onModelessDialogChangedEvent = new ModelessDialogChangedEvent();
/** @internal */
ModelessDialogManager.dialogManager = new DialogManagerBase_1.DialogManagerBase(ModelessDialogManager.onModelessDialogChangedEvent);
ModelessDialogManager._dialogMap = new Map();
ModelessDialogManager._idArray = new Array();
ModelessDialogManager._topZIndex = ZINDEX_DEFAULT;
/** ModelessDialogRenderer React component renders modeless dialogs.
 * @public
 */
class ModelessDialogRenderer extends React.PureComponent {
    constructor(props) {
        super(props);
    }
    render() {
        return (React.createElement(DialogManagerBase_1.DialogRendererBase, Object.assign({}, this.props, { dialogManager: ModelessDialogManager.dialogManager })));
    }
}
exports.ModelessDialogRenderer = ModelessDialogRenderer;
//# sourceMappingURL=ModelessDialogManager.js.map