"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Notification
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElementTooltip = exports.ElementTooltipChangedEvent = void 0;
const classnames = require("classnames");
const React = require("react");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const MessageSpan_1 = require("../messages/MessageSpan");
/** ElementTooltip Changed Event class.
 * @public
 */
class ElementTooltipChangedEvent extends ui_core_1.UiEvent {
}
exports.ElementTooltipChangedEvent = ElementTooltipChangedEvent;
/** ElementTooltip React component.
 * @public
 */
class ElementTooltip extends React.Component {
    constructor(props) {
        super(props);
        this._size = {
            height: 0,
            width: 0,
        };
        /** @internal */
        this.state = {
            message: "",
            isVisible: false,
            position: {
                x: 0,
                y: 0,
            },
        };
        this._handleElementTooltipChangedEvent = (args) => {
            this._element = args.el;
            this._position = args.pt;
            this.setState({
                isVisible: args.isTooltipVisible,
                message: args.message,
            });
            this.updatePosition();
        };
        // istanbul ignore next
        this._handleSizeChanged = (size) => {
            this._size = size;
            this.updatePosition();
        };
    }
    static get onElementTooltipChangedEvent() { return ElementTooltip._elementTooltipChangedEvent; }
    static get isTooltipVisible() { return ElementTooltip._isTooltipVisible; }
    static showTooltip(el, message, pt, options) {
        // istanbul ignore if
        if (ElementTooltip._isTooltipHalted)
            return;
        ElementTooltip._isTooltipVisible = true;
        ElementTooltip.onElementTooltipChangedEvent.emit({ isTooltipVisible: true, el, message, pt, options });
    }
    static hideTooltip() {
        ElementTooltip._isTooltipVisible = false;
        ElementTooltip.onElementTooltipChangedEvent.emit({ isTooltipVisible: false, message: "" });
    }
    // istanbul ignore next
    static get isTooltipHalted() { return ElementTooltip._isTooltipHalted; }
    // istanbul ignore next
    static set isTooltipHalted(halt) {
        ElementTooltip._isTooltipHalted = halt;
        if (halt && ElementTooltip._isTooltipVisible)
            ElementTooltip.hideTooltip();
    }
    render() {
        if (!this.state.isVisible)
            return null;
        const className = classnames("uifw-element-tooltip", this.props.className);
        const messageNode = React.createElement(MessageSpan_1.MessageDiv, { message: this.state.message });
        return (React.createElement("div", { className: "uifw-element-tooltip-container" },
            React.createElement(ui_ninezone_1.Tooltip, { className: className, style: this.props.style, position: this.state.position, onSizeChanged: this._handleSizeChanged }, messageNode)));
    }
    componentDidMount() {
        ElementTooltip.onElementTooltipChangedEvent.addListener(this._handleElementTooltipChangedEvent);
    }
    componentWillUnmount() {
        ElementTooltip.onElementTooltipChangedEvent.removeListener(this._handleElementTooltipChangedEvent);
    }
    updatePosition() {
        this.setState((prevState) => {
            if (!this._element)
                return null;
            // istanbul ignore next
            if (!this._position)
                return null;
            const containerBounds = ui_core_1.Rectangle.create(this._element.getBoundingClientRect());
            const relativeBounds = ui_core_1.Rectangle.createFromSize(this._size).offset(this._position);
            const adjustedPosition = ui_ninezone_1.offsetAndContainInContainer(relativeBounds, containerBounds.getSize(), { x: 8, y: 8 });
            const position = adjustedPosition.offset(containerBounds.topLeft());
            // istanbul ignore else
            if (position.equals(prevState.position))
                return null;
            // istanbul ignore next
            return {
                position,
            };
        });
    }
}
exports.ElementTooltip = ElementTooltip;
ElementTooltip._elementTooltipChangedEvent = new ElementTooltipChangedEvent();
//# sourceMappingURL=ElementTooltip.js.map