/** @packageDocumentation
 * @module Frontstage
 */
import * as React from "react";
import { StagePanelLocation } from "@bentley/ui-abstract";
import { CommonProps } from "@bentley/ui-core";
import { WidgetZoneId, ZoneManagerProps, ZonesManagerProps } from "@bentley/ui-ninezone";
import { ContentGroup } from "../content/ContentGroup";
import { ContentLayoutDef } from "../content/ContentLayout";
import { ToolItemDef } from "../shared/ToolItemDef";
import { StagePanelProps } from "../stagepanels/StagePanel";
import { StagePanelDef } from "../stagepanels/StagePanelDef";
import { ZoneLocation, ZoneProps } from "../zones/Zone";
import { ZoneDef } from "../zones/ZoneDef";
import { FrontstageRuntimeProps, ZoneDefProvider } from "./FrontstageComposer";
import { FrontstageDef } from "./FrontstageDef";
/** Properties for a [[Frontstage]] component.
 * @public
 */
export interface FrontstageProps extends CommonProps {
    /** Id for the Frontstage */
    id: string;
    /** Tool that is started once the Frontstage is activated */
    defaultTool: ToolItemDef;
    /** The default Content Layout used */
    defaultLayout: string | ContentLayoutDef;
    /** The Content Group providing the Content Views */
    contentGroup: string | ContentGroup;
    /** Id of the Content View to be activated initially */
    defaultContentId?: string;
    /** Indicated whether the StatusBar is in footer mode or widget mode. Defaults to true. */
    isInFooterMode?: boolean;
    /** Any application data to attach to this Frontstage. */
    applicationData?: any;
    /** Usage type for this Frontstage. */
    usage?: string;
    /** Frontstage version. Used to force saved layout reinitialization after changes to frontstage.
     * @note This value should be increased when changes are made to Frontstage.
     * Increasing the value will make sure to reinitialize App layout instead of restoring to old layout.
     * Version increase is required when widgets are added/removed.
     */
    version?: number;
    /** The Zone in the top-left corner. @deprecated Use 'contentManipulationTools' property. */
    topLeft?: React.ReactElement<ZoneProps>;
    /** The Zone along the top-center edge. @deprecated Use 'toolSettings' property. */
    topCenter?: React.ReactElement<ZoneProps>;
    /** The Zone in the top-right corner. @deprecated Use 'viewNavigationTools' property. */
    topRight?: React.ReactElement<ZoneProps>;
    /** The Zone along the center-left edge. @deprecated Place widgets in appropriate stage panel zone. */
    centerLeft?: React.ReactElement<ZoneProps>;
    /** The Zone along the center-right edge.  @deprecated  Place widgets in appropriate stage panel zone. */
    centerRight?: React.ReactElement<ZoneProps>;
    /** The Zone in the bottom-left corner.  @deprecated Place widgets in appropriate stage panel zone.  */
    bottomLeft?: React.ReactElement<ZoneProps>;
    /** The Zone along the bottom-center edge. @deprecated use statusBar property */
    bottomCenter?: React.ReactElement<ZoneProps>;
    /** The Zone in the bottom-right corner.  @deprecated Place widgets in appropriate stage panel zone. */
    bottomRight?: React.ReactElement<ZoneProps>;
    /** The Zone in the top-left corner that shows tools typically used to query and modify content. To be used in place of deprecated topLeft zone definition.  @beta */
    contentManipulationTools?: React.ReactElement<ZoneProps>;
    /** The Zone the that shows settings for the active tool. To be used in place of deprecated topCenter zone definition. @beta */
    toolSettings?: React.ReactElement<ZoneProps>;
    /** The Zone in the top-right corner that shows view navigation tools. To be used in place of deprecated topRight zone definition.  @beta */
    viewNavigationTools?: React.ReactElement<ZoneProps>;
    /** The status bar Zone shown as the application footer. To be used in place of deprecated bottomCenter zone definition.  @beta */
    statusBar?: React.ReactElement<ZoneProps>;
    /** The StagePanel on the top of the 9-zone area. @beta */
    topPanel?: React.ReactElement<StagePanelProps>;
    /** The StagePanel on the very top across the full width. @beta @deprecated Only topPanel is supported in UI 2.0 */
    topMostPanel?: React.ReactElement<StagePanelProps>;
    /** The StagePanel on the left. @beta  */
    leftPanel?: React.ReactElement<StagePanelProps>;
    /** The StagePanel on the right. @beta  */
    rightPanel?: React.ReactElement<StagePanelProps>;
    /** The StagePanel on the bottom of the 9-zone area. @beta  */
    bottomPanel?: React.ReactElement<StagePanelProps>;
    /** The StagePanel on the very bottom across the full width. @beta @deprecated Only bottomPanel is supported in UI 2.0  */
    bottomMostPanel?: React.ReactElement<StagePanelProps>;
    /** @internal */
    runtimeProps?: FrontstageRuntimeProps;
}
interface FrontstageState {
    isUiVisible: boolean;
    widgetIdToContent: Partial<{
        [id in WidgetZoneId]: HTMLDivElement | undefined;
    }>;
}
/** Frontstage React component.
 * A Frontstage is a full-screen configuration designed to enable the user to accomplish a task.
 * @public
 */
export declare class Frontstage extends React.Component<FrontstageProps, FrontstageState> {
    private static _zoneIds;
    private _contentRefs;
    private _zonesMeasurer;
    private _floatingZonesMeasurer;
    private _zonesStyle;
    private _zonesFooterModeStyle;
    /** @internal */
    constructor(props: FrontstageProps);
    /** React lifecycle method.
     * @internal
     */
    componentDidMount(): Promise<void>;
    componentDidUpdate(): void;
    /** React lifecycle method.
     * @internal
     */
    componentWillUnmount(): void;
    private _uiVisibilityChanged;
    private _handleWidgetsChanged;
    private _handleWidgetProvidersChanged;
    private _handleUiProviderRegisteredEvent;
    private updateWidgetDefs;
    /** Initializes a FrontstageDef from FrontstageProps */
    static initializeFrontstageDef(frontstageDef: FrontstageDef, props: FrontstageProps): void;
    /** @internal */
    static createZoneDef(zoneNode: React.ReactElement<ZoneProps> | undefined, zoneLocation: ZoneLocation, props: FrontstageProps): ZoneDef | undefined;
    private static getZoneElement;
    /** @internal */
    static createStagePanelDef(panelLocation: StagePanelLocation, props: FrontstageProps): StagePanelDef | undefined;
    private static getStagePanelElement;
    private _getContentRef;
    private doContentLayoutRender;
    private cloneStagePanelElement;
    private cloneZoneElements;
    private cloneWidgetContentElements;
    /** React render method
     * @internal
     */
    render(): React.ReactNode;
}
/** @internal */
export declare const getExtendedZone: (zoneId: WidgetZoneId, zones: ZonesManagerProps, defProvider: ZoneDefProvider) => ZoneManagerProps;
/** Hook that returns active frontstage id.
 * @beta
 */
export declare const useActiveFrontstageId: () => string;
/** @internal */
export declare function useActiveFrontstageDef(): FrontstageDef | undefined;
export {};
//# sourceMappingURL=Frontstage.d.ts.map