"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Frontstage
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.useActiveFrontstageDef = exports.useActiveFrontstageId = exports.getExtendedZone = exports.Frontstage = void 0;
const React = require("react");
const ReactDOM = require("react-dom");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const ContentLayout_1 = require("../content/ContentLayout");
const StagePanel_1 = require("../stagepanels/StagePanel");
const StagePanelDef_1 = require("../stagepanels/StagePanelDef");
const UiFramework_1 = require("../UiFramework");
const UiShowHideManager_1 = require("../utils/UiShowHideManager");
const ToolSettingsContent_1 = require("../widgets/ToolSettingsContent");
const Zone_1 = require("../zones/Zone");
const ZoneDef_1 = require("../zones/ZoneDef");
const FrontstageManager_1 = require("./FrontstageManager");
/** Frontstage React component.
 * A Frontstage is a full-screen configuration designed to enable the user to accomplish a task.
 * @public
 */
class Frontstage extends React.Component {
    /** @internal */
    constructor(props) {
        super(props);
        this._contentRefs = new Map();
        this._zonesMeasurer = React.createRef();
        this._floatingZonesMeasurer = React.createRef();
        this._zonesStyle = {
            pointerEvents: "none",
        };
        this._zonesFooterModeStyle = Object.assign(Object.assign({}, this._zonesStyle), { display: "flex", flexFlow: "column" });
        this._uiVisibilityChanged = (args) => {
            this.setState({ isUiVisible: args.visible });
        };
        this._handleWidgetsChanged = (_args) => {
            this.updateWidgetDefs();
        };
        this._handleWidgetProvidersChanged = (_args) => {
            this.updateWidgetDefs();
        };
        this._handleUiProviderRegisteredEvent = (_args) => {
            this.updateWidgetDefs();
        };
        this._getContentRef = (widget) => {
            const ref = this._contentRefs.get(widget);
            if (ref)
                return ref;
            const newRef = (el) => {
                this.setState((prevState) => ({
                    widgetIdToContent: Object.assign(Object.assign({}, prevState.widgetIdToContent), { [widget]: el === null ? undefined : el }),
                }));
            };
            this._contentRefs.set(widget, newRef);
            return newRef;
        };
        this.state = {
            isUiVisible: UiFramework_1.UiFramework.getIsUiVisible(),
            widgetIdToContent: {},
        };
    }
    /** React lifecycle method.
     * @internal
     */
    async componentDidMount() {
        UiFramework_1.UiFramework.onUiVisibilityChanged.addListener(this._uiVisibilityChanged);
        UiFramework_1.UiFramework.widgetManager.onWidgetsChanged.addListener(this._handleWidgetsChanged);
        UiFramework_1.UiFramework.widgetManager.onWidgetProvidersChanged.addListener(this._handleWidgetProvidersChanged);
        ui_abstract_1.UiItemsManager.onUiProviderRegisteredEvent.addListener(this._handleUiProviderRegisteredEvent);
    }
    componentDidUpdate() {
        if (!this._zonesMeasurer.current || !this._floatingZonesMeasurer.current || !this.props.runtimeProps)
            return;
        let floatingBounds = ui_core_1.Rectangle.create(this._floatingZonesMeasurer.current.getBoundingClientRect());
        const bounds = ui_core_1.Rectangle.create(this._zonesMeasurer.current.getBoundingClientRect());
        const offset = bounds.topLeft().getOffsetTo(floatingBounds.topLeft());
        floatingBounds = floatingBounds.setPosition(offset);
        this.props.runtimeProps.nineZoneChangeHandler.handleZonesBoundsChange(bounds);
        this.props.runtimeProps.nineZoneChangeHandler.handleFloatingZonesBoundsChange(floatingBounds);
    }
    /** React lifecycle method.
     * @internal
     */
    componentWillUnmount() {
        UiFramework_1.UiFramework.onUiVisibilityChanged.removeListener(this._uiVisibilityChanged);
        UiFramework_1.UiFramework.widgetManager.onWidgetsChanged.removeListener(this._handleWidgetsChanged);
        UiFramework_1.UiFramework.widgetManager.onWidgetProvidersChanged.removeListener(this._handleWidgetProvidersChanged);
        ui_abstract_1.UiItemsManager.onUiProviderRegisteredEvent.removeListener(this._handleUiProviderRegisteredEvent);
    }
    updateWidgetDefs() {
        if (!this.props.runtimeProps)
            return;
        const frontstageDef = this.props.runtimeProps.frontstageDef;
        frontstageDef.updateWidgetDefs();
        FrontstageManager_1.FrontstageManager.onWidgetDefsUpdatedEvent.emit();
        this.forceUpdate();
    }
    /** Initializes a FrontstageDef from FrontstageProps */
    static initializeFrontstageDef(frontstageDef, props) {
        frontstageDef.initializeFromProps(props);
    }
    /** @internal */
    static createZoneDef(zoneNode, zoneLocation, props) {
        if (zoneNode) {
            const zoneDef = new ZoneDef_1.ZoneDef();
            const zoneElement = Frontstage.getZoneElement(zoneLocation, props);
            zoneDef.zoneLocation = zoneLocation;
            // istanbul ignore else
            if (zoneElement && React.isValidElement(zoneElement)) {
                Zone_1.Zone.initializeZoneDef(zoneDef, zoneElement.props);
                return zoneDef;
            }
        }
        return undefined;
    }
    static getZoneElement(zoneId, props) {
        switch (zoneId) {
            case Zone_1.ZoneLocation.TopLeft:
                return props.contentManipulationTools ? props.contentManipulationTools : props.topLeft;
            case Zone_1.ZoneLocation.TopCenter:
                return props.toolSettings ? props.toolSettings : props.topCenter;
            case Zone_1.ZoneLocation.TopRight:
                return props.viewNavigationTools ? /* istanbul ignore next */ props.viewNavigationTools : props.topRight;
            case Zone_1.ZoneLocation.CenterLeft:
                return props.centerLeft;
            case Zone_1.ZoneLocation.CenterRight:
                return props.centerRight;
            case Zone_1.ZoneLocation.BottomLeft:
                return props.bottomLeft;
            case Zone_1.ZoneLocation.BottomCenter:
                return props.statusBar ? props.statusBar : props.bottomCenter;
            case Zone_1.ZoneLocation.BottomRight:
                return props.bottomRight;
        }
        // Zones can be undefined in a Frontstage
        // istanbul ignore next
        return undefined;
    }
    /** @internal */
    static createStagePanelDef(panelLocation, props) {
        const panelDef = new StagePanelDef_1.StagePanelDef();
        const panelElement = Frontstage.getStagePanelElement(panelLocation, props);
        panelDef.initializeFromProps(panelElement === null || panelElement === void 0 ? void 0 : panelElement.props, panelLocation);
        return panelDef;
    }
    static getStagePanelElement(location, props) {
        let panelElement;
        switch (location) {
            case ui_abstract_1.StagePanelLocation.Top:
                panelElement = props.topPanel;
                break;
            case ui_abstract_1.StagePanelLocation.TopMost:
                panelElement = props.topMostPanel;
                break;
            case ui_abstract_1.StagePanelLocation.Left:
                panelElement = props.leftPanel;
                break;
            case ui_abstract_1.StagePanelLocation.Right:
                panelElement = props.rightPanel;
                break;
            case ui_abstract_1.StagePanelLocation.Bottom:
                panelElement = props.bottomPanel;
                break;
            case ui_abstract_1.StagePanelLocation.BottomMost:
                panelElement = props.bottomMostPanel;
                break;
            // istanbul ignore next
            default:
                throw new RangeError();
        }
        // Panels can be undefined in a Frontstage
        return panelElement;
    }
    // This uses ConfigurableUi to render the content
    doContentLayoutRender() {
        let contentLayout;
        // istanbul ignore else
        if (this.props.runtimeProps && this.props.runtimeProps.frontstageDef) {
            const frontstageDef = this.props.runtimeProps.frontstageDef;
            // istanbul ignore else
            if (frontstageDef.contentLayoutDef && frontstageDef.contentGroup)
                contentLayout = (React.createElement(ContentLayout_1.ContentLayout, { contentLayout: frontstageDef.contentLayoutDef, contentGroup: frontstageDef.contentGroup, isInFooterMode: frontstageDef.isInFooterMode }));
            else
                bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), `FrontstageDef.contentLayoutDef and FrontstageDef.contentGroup are required for <ContentLayout> component`);
        }
        return contentLayout;
    }
    cloneStagePanelElement(panelDef, runtimeProps) {
        if (!this.state.isUiVisible && UiShowHideManager_1.UiShowHideManager.showHidePanels)
            return null;
        // istanbul ignore else
        if (panelDef) {
            const { location } = panelDef;
            const panelElement = Frontstage.getStagePanelElement(location, this.props);
            // istanbul ignore else
            if (panelElement && React.isValidElement(panelElement)) {
                const panelKey = StagePanel_1.getNestedStagePanelKey(panelDef.location);
                const panels = runtimeProps.nineZone.nested.panels[panelKey.id];
                const panel = ui_ninezone_1.StagePanelsManager.getPanel(panelKey.type, panels);
                const draggedWidget = runtimeProps.nineZone.zones.draggedWidget;
                const panelRuntimeProps = {
                    draggedWidgetId: draggedWidget ? /* istanbul ignore next */ draggedWidget.id : undefined,
                    getWidgetContentRef: this._getContentRef,
                    isInFooterMode: runtimeProps.nineZone.zones.isInFooterMode,
                    isTargeted: !!runtimeProps.nineZone.zones.target,
                    panel,
                    panelDef,
                    stagePanelChangeHandler: runtimeProps.stagePanelChangeHandler,
                    widgetChangeHandler: runtimeProps.widgetChangeHandler,
                    widgets: runtimeProps.nineZone.zones.widgets,
                    widgetTabs: runtimeProps.widgetTabs,
                    zoneDefProvider: runtimeProps.zoneDefProvider,
                };
                return React.cloneElement(panelElement, { runtimeProps: panelRuntimeProps });
            }
        }
        return null;
    }
    cloneZoneElements(zoneIds, runtimeProps) {
        return zoneIds.map((zoneId) => {
            const zoneElement = Frontstage.getZoneElement(zoneId, this.props);
            if (!zoneElement || !React.isValidElement(zoneElement))
                return null;
            const zoneDef = runtimeProps.zoneDefProvider.getZoneDef(zoneId);
            // istanbul ignore if
            if (!zoneDef)
                return null;
            const nestedPanelsManager = FrontstageManager_1.FrontstageManager.NineZoneManager.getNestedPanelsManager();
            const panelsManager = nestedPanelsManager.getPanelsManager("inner");
            const type = panelsManager.findWidget(zoneId, runtimeProps.nineZone.nested.panels.inner);
            // istanbul ignore if
            if (type !== undefined)
                return null;
            const zonesManager = FrontstageManager_1.FrontstageManager.NineZoneManager.getZonesManager();
            const zones = runtimeProps.nineZone.zones;
            const ghostOutline = zonesManager.getGhostOutlineBounds(zoneId, zones);
            const dropTarget = zonesManager.getDropTarget(zoneId, zones);
            const zone = exports.getExtendedZone(zoneId, zones, runtimeProps.zoneDefProvider);
            const widget = zone.widgets.length > 0 ? zones.widgets[zone.widgets[0]] : undefined;
            const openWidgetId = zone.widgets.find((wId) => zones.widgets[wId].tabIndex >= 0);
            const activeTabIndex = openWidgetId ? zones.widgets[openWidgetId].tabIndex : 0;
            const draggedWidget = runtimeProps.nineZone.zones.draggedWidget;
            const disabledResizeHandles = zonesManager.getDisabledResizeHandles(zoneId, zones);
            const zoneRuntimeProps = {
                activeTabIndex,
                disabledResizeHandles,
                draggedWidget: draggedWidget && /* istanbul ignore next */ draggedWidget.id === zoneId ? /* istanbul ignore next */ draggedWidget : undefined,
                dropTarget,
                getWidgetContentRef: this._getContentRef,
                ghostOutline,
                isHidden: (zoneDef.isStatusBar && this.props.isInFooterMode && /* istanbul ignore next */ (this.state.isUiVisible || !UiShowHideManager_1.UiShowHideManager.showHideFooter)) ?
                    /* istanbul ignore next */ false : !this.state.isUiVisible,
                isInFooterMode: runtimeProps.nineZone.zones.isInFooterMode,
                openWidgetId,
                targetChangeHandler: runtimeProps.targetChangeHandler,
                widgetChangeHandler: runtimeProps.widgetChangeHandler,
                widgetTabs: runtimeProps.widgetTabs,
                widget,
                zoneDef,
                zoneDefProvider: runtimeProps.zoneDefProvider,
                zone,
            };
            return React.cloneElement(zoneElement, { key: zoneId, runtimeProps: zoneRuntimeProps });
        });
    }
    cloneWidgetContentElements(zones, runtimeProps) {
        const widgets = zones.reduce((prev, zoneId) => {
            const zoneDef = runtimeProps.zoneDefProvider.getZoneDef(zoneId);
            // istanbul ignore if
            if (!zoneDef)
                return prev;
            const widgetDefs = zoneDef.widgetDefs.filter((widgetDef) => {
                if (zoneId === 2)
                    return widgetDef.isVisible;
                return widgetDef.isVisible && !widgetDef.isFloating;
            });
            for (let i = 0; i < widgetDefs.length; i++) {
                prev.push({
                    id: zoneId,
                    def: widgetDefs[i],
                    tabIndex: i,
                });
            }
            return prev;
        }, []);
        return widgets.map((widget) => {
            const nzWidget = runtimeProps.nineZone.zones.widgets[widget.id];
            return (React.createElement(WidgetContentRenderer, { anchor: nzWidget.horizontalAnchor, isHidden: nzWidget.tabIndex !== widget.tabIndex, key: `${widget.id}_${widget.tabIndex}`, renderTo: this.state.widgetIdToContent[widget.id], toolSettingsMode: Zone_1.isToolSettingsWidgetManagerProps(nzWidget) ? nzWidget.mode : undefined, widgetDef: widget.def }));
        });
    }
    /** React render method
     * @internal
     */
    render() {
        const { runtimeProps } = this.props;
        if (runtimeProps === undefined)
            return null;
        /** For Nine-zone area; includes ContentLayout */
        const ninezoneStyle = {
            position: "relative",
            height: "100%",
        };
        const frontstageDef = runtimeProps.frontstageDef;
        return (React.createElement("div", { style: ninezoneStyle, id: "uifw-ninezone-area", className: this.props.className },
            React.createElement(ui_ninezone_1.Zones, { style: runtimeProps.nineZone.zones.isInFooterMode ? this._zonesFooterModeStyle : this._zonesStyle },
                React.createElement(ui_ninezone_1.StagePanels, { bottomPanel: this.cloneStagePanelElement(frontstageDef.bottomMostPanel, runtimeProps), topPanel: this.cloneStagePanelElement(frontstageDef.topMostPanel, runtimeProps) },
                    React.createElement(ui_ninezone_1.StagePanels, { bottomPanel: this.cloneStagePanelElement(frontstageDef.bottomPanel, runtimeProps), leftPanel: this.cloneStagePanelElement(frontstageDef.leftPanel, runtimeProps), rightPanel: this.cloneStagePanelElement(frontstageDef.rightPanel, runtimeProps), topPanel: this.cloneStagePanelElement(frontstageDef.topPanel, runtimeProps) },
                        React.createElement("div", { id: "uifw-ninezone-zones-area", ref: this._zonesMeasurer, style: {
                                height: "100%",
                                position: "relative",
                            } },
                            this.doContentLayoutRender(),
                            this.cloneZoneElements(Frontstage._zoneIds, runtimeProps)))),
                this.cloneZoneElements([8], runtimeProps),
                React.createElement("div", { id: "uifw-ninezone-floating-zones-area", ref: this._floatingZonesMeasurer, style: {
                        height: "100%",
                        width: "100%",
                        position: "absolute",
                    } })),
            this.cloneWidgetContentElements(Frontstage._zoneIds, runtimeProps)));
    }
}
exports.Frontstage = Frontstage;
Frontstage._zoneIds = ui_ninezone_1.widgetZoneIds.filter((z) => z !== 8);
class WidgetContentRenderer extends React.PureComponent {
    constructor(props) {
        super(props);
        this._content = document.createElement("span");
        this._handleWidgetStateChangedEvent = (args) => {
            if (this.props.widgetDef !== args.widgetDef)
                return;
            this.forceUpdate();
        };
        this._handleToolActivatedEvent = () => {
            if (this.props.toolSettingsMode === undefined)
                return;
            // force update when tool is activated
            this.setState((prevState) => ({ widgetKey: prevState.widgetKey + 1 }));
        };
        this.state = {
            widgetKey: 0,
        };
    }
    componentDidMount() {
        FrontstageManager_1.FrontstageManager.onWidgetStateChangedEvent.addListener(this._handleWidgetStateChangedEvent);
        FrontstageManager_1.FrontstageManager.onToolActivatedEvent.addListener(this._handleToolActivatedEvent);
        this._content.style.display = this.props.isHidden ? "none" : "flex";
        this._content.style.flexDirection = "column";
        this._content.style.height = "100%";
        if (!this.props.renderTo)
            return;
        this.props.renderTo.appendChild(this._content);
    }
    componentDidUpdate(prevProps) {
        if (this.props.isHidden !== prevProps.isHidden) {
            this._content.style.display = this.props.isHidden ? "none" : "flex";
        }
        if (!this.props.renderTo || prevProps.renderTo === this.props.renderTo)
            return;
        this.props.widgetDef.saveTransientState();
        this.props.renderTo.appendChild(this._content);
        const shouldRemount = this.props.widgetDef.restoreTransientState() || this.props.widgetDef.isToolSettings;
        shouldRemount && this.setState((prevState) => ({ widgetKey: prevState.widgetKey + 1 }));
    }
    componentWillUnmount() {
        this._content.parentNode && this._content.parentNode.removeChild(this._content);
        FrontstageManager_1.FrontstageManager.onWidgetStateChangedEvent.removeListener(this._handleWidgetStateChangedEvent);
        FrontstageManager_1.FrontstageManager.onToolActivatedEvent.removeListener(this._handleToolActivatedEvent);
    }
    render() {
        if (this.props.toolSettingsMode !== undefined) {
            return ReactDOM.createPortal((React.createElement(ToolSettingsContent_1.ToolSettingsContent, { anchor: this.props.anchor, key: this.state.widgetKey, mode: this.props.toolSettingsMode }, FrontstageManager_1.FrontstageManager.activeToolSettingsProvider && FrontstageManager_1.FrontstageManager.activeToolSettingsProvider.toolSettingsNode)), this._content);
        }
        if (this.props.widgetDef.state === ui_abstract_1.WidgetState.Unloaded)
            return null;
        return ReactDOM.createPortal((React.createElement(React.Fragment, { key: this.state.widgetKey }, this.props.widgetDef.reactNode)), this._content);
    }
}
/** @internal */
const getExtendedZone = (zoneId, zones, defProvider) => {
    const zone = zones.zones[zoneId];
    if (zoneId === 1 || zoneId === 3) {
        let extendOverId = zoneId;
        const zonesManager = FrontstageManager_1.FrontstageManager.NineZoneManager.getZonesManager();
        let bottomZoneId = zonesManager.bottomZones.getInitial(extendOverId);
        while (bottomZoneId !== undefined) {
            const bottomZoneDef = defProvider.getZoneDef(bottomZoneId);
            if (bottomZoneDef && bottomZoneDef.widgetDefs.length !== 0)
                break;
            extendOverId = bottomZoneId;
            bottomZoneId = zonesManager.bottomZones.getInitial(bottomZoneId);
        }
        const bottom = zones.zones[extendOverId].bounds.bottom;
        if (bottom === zone.bounds.bottom)
            return zone;
        return Object.assign(Object.assign({}, zone), { bounds: Object.assign(Object.assign({}, zone.bounds), { bottom }) });
    }
    return zone;
};
exports.getExtendedZone = getExtendedZone;
/** Hook that returns active frontstage id.
 * @beta
 */
const useActiveFrontstageId = () => {
    const def = useActiveFrontstageDef();
    const id = React.useMemo(() => def ? /* istanbul ignore next */ def.id : "", [def]);
    return id;
};
exports.useActiveFrontstageId = useActiveFrontstageId;
/** @internal */
function useActiveFrontstageDef() {
    const [def, setDef] = React.useState(FrontstageManager_1.FrontstageManager.activeFrontstageDef);
    React.useEffect(() => {
        // istanbul ignore next
        const handleActivated = (args) => {
            setDef(args.activatedFrontstageDef);
        };
        FrontstageManager_1.FrontstageManager.onFrontstageActivatedEvent.addListener(handleActivated);
        return () => {
            FrontstageManager_1.FrontstageManager.onFrontstageActivatedEvent.removeListener(handleActivated);
        };
    }, []);
    return def;
}
exports.useActiveFrontstageDef = useActiveFrontstageDef;
//# sourceMappingURL=Frontstage.js.map