/** @packageDocumentation
 * @module Frontstage
 */
import { ScreenViewport } from "@bentley/imodeljs-frontend";
import { StagePanelLocation } from "@bentley/ui-abstract";
import { NineZoneManagerProps, NineZoneState } from "@bentley/ui-ninezone";
import { ContentControl } from "../content/ContentControl";
import { ContentGroup } from "../content/ContentGroup";
import { ContentLayoutDef } from "../content/ContentLayout";
import { ToolItemDef } from "../shared/ToolItemDef";
import { StagePanelDef } from "../stagepanels/StagePanelDef";
import { WidgetDef } from "../widgets/WidgetDef";
import { ZoneDef } from "../zones/ZoneDef";
import { FrontstageProps } from "./Frontstage";
import { FrontstageProvider } from "./FrontstageProvider";
import { TimeTracker } from "../configurableui/TimeTracker";
import { PointProps, SizeProps } from "@bentley/ui-core";
/** @internal */
export interface FrontstageEventArgs {
    frontstageDef: FrontstageDef;
}
/** @internal */
export interface FrontstageNineZoneStateChangedEventArgs extends FrontstageEventArgs {
    state: NineZoneState | undefined;
}
/** FrontstageDef class provides an API for a Frontstage.
 * @public
 */
export declare class FrontstageDef {
    private _id;
    private _defaultTool?;
    private _defaultLayoutId;
    private _defaultContentId;
    private _contentGroupId;
    private _isInFooterMode;
    private _isStageClosing;
    private _isApplicationClosing;
    private _applicationData?;
    private _usage?;
    private _version;
    private _topLeft?;
    private _topCenter?;
    private _topRight?;
    private _centerLeft?;
    private _centerRight?;
    private _bottomLeft?;
    private _bottomCenter?;
    private _bottomRight?;
    private _topPanel?;
    private _topMostPanel?;
    private _leftPanel?;
    private _rightPanel?;
    private _bottomPanel?;
    private _bottomMostPanel?;
    private _defaultLayout?;
    private _contentLayoutDef?;
    private _contentGroup?;
    private _frontstageProvider?;
    private _nineZone?;
    private _timeTracker;
    private _nineZoneState?;
    get id(): string;
    get defaultTool(): ToolItemDef | undefined;
    get defaultLayoutId(): string;
    get defaultContentId(): string;
    get contentGroupId(): string;
    get isInFooterMode(): boolean;
    get applicationData(): any | undefined;
    get usage(): string;
    get version(): number;
    get topLeft(): ZoneDef | undefined;
    get topCenter(): ZoneDef | undefined;
    get topRight(): ZoneDef | undefined;
    get centerLeft(): ZoneDef | undefined;
    get centerRight(): ZoneDef | undefined;
    get bottomLeft(): ZoneDef | undefined;
    get bottomCenter(): ZoneDef | undefined;
    get bottomRight(): ZoneDef | undefined;
    /** @beta */
    get topPanel(): StagePanelDef | undefined;
    /** @beta @deprecated Only topPanel is supported in UI 2.0 */
    get topMostPanel(): StagePanelDef | undefined;
    /** @beta */
    get leftPanel(): StagePanelDef | undefined;
    /** @beta */
    get rightPanel(): StagePanelDef | undefined;
    /** @beta */
    get bottomPanel(): StagePanelDef | undefined;
    /** @beta @deprecated Only bottomPanel is supported in UI 2.0  */
    get bottomMostPanel(): StagePanelDef | undefined;
    get defaultLayout(): ContentLayoutDef | undefined;
    get contentLayoutDef(): ContentLayoutDef | undefined;
    get contentGroup(): ContentGroup | undefined;
    get frontstageProvider(): FrontstageProvider | undefined;
    /** @internal */
    get nineZone(): NineZoneManagerProps | undefined;
    set nineZone(props: NineZoneManagerProps | undefined);
    /** @internal */
    get nineZoneState(): NineZoneState | undefined;
    set nineZoneState(state: NineZoneState | undefined);
    /** @internal */
    get timeTracker(): TimeTracker;
    /** Constructs the [[FrontstageDef]]  */
    constructor(props?: FrontstageProps);
    /** Handles when the Frontstage becomes activated */
    protected _onActivated(): void;
    /** Handles when the Frontstage becomes activated */
    onActivated(): void;
    /** Handles when the Frontstage becomes inactive */
    protected _onDeactivated(): void;
    /** Handles when the Frontstage becomes inactive */
    onDeactivated(): void;
    /** @internal */
    setIsApplicationClosing(value: boolean): void;
    /** Returns once the contained widgets and content controls are ready to use */
    waitUntilReady(): Promise<void>;
    /** Handles when the Frontstage becomes active */
    protected _onFrontstageReady(): void;
    /** Handles when the Frontstage becomes active */
    onFrontstageReady(): void;
    /** Starts the default tool for the Frontstage */
    startDefaultTool(): void;
    /** Sets the Content Layout and Content Group */
    setContentLayoutAndGroup(contentLayoutDef: ContentLayoutDef, contentGroup: ContentGroup): void;
    /** Sets the active view content control to the default or first */
    setActiveContent(): boolean;
    /** Sets the active view content control */
    setActiveView(newContent: ContentControl, oldContent?: ContentControl): void;
    /** Sets the active view content control based on the selected viewport. */
    setActiveViewFromViewport(viewport: ScreenViewport): boolean;
    /** Gets a [[ZoneDef]] based on a given zone id */
    getZoneDef(zoneId: number): ZoneDef | undefined;
    /** Gets a list of [[ZoneDef]]s */
    get zoneDefs(): ZoneDef[];
    /** Gets a [[StagePanelDef]] based on a given panel location
     * @beta
     */
    getStagePanelDef(location: StagePanelLocation): StagePanelDef | undefined;
    /** Gets a list of [[StagePanelDef]]s
     * @beta
     */
    get panelDefs(): StagePanelDef[];
    /** Finds a [[WidgetDef]] based on a given id */
    findWidgetDef(id: string): WidgetDef | undefined;
    /** Gets the list of [[WidgetControl]]s */
    private get _widgetControls();
    /** Gets the list of [[ContentControl]]s */
    get contentControls(): ContentControl[];
    /**
     * Initializes this [[FrontstageDef]] from a [[FrontstageProvider]]
     * @param frontstageProvider The FrontstageProvider to initialize from
     */
    initializeFromProvider(frontstageProvider: FrontstageProvider): void;
    /** Initializes a FrontstageDef from FrontstageProps
     * @internal
     */
    initializeFromProps(props: FrontstageProps): void;
    /** @internal */
    updateWidgetDefs(): void;
    /** @beta */
    restoreLayout(): void;
    isPopoutWidget(widgetId: string): boolean;
    isFloatingWidget(widgetId: string): boolean;
    /** Create a new floating panel that contains the widget specified by its Id. Supported only when in
     *  UI 2.0 or higher.
     * @param widgetId case sensitive Wigdet Id
     * @param point Position of top left corner of floating panel in pixels. If undefined {x:50, y:100} is used.
     * @param size defines the width and height of the floating panel. If undefined and widget has been floated before
     * the previous size is used, else {height:400, width:400} is used.
     * @beta
     */
    floatWidget(widgetId: string, point?: PointProps, size?: SizeProps): void;
    /** Opens window for specified PopoutWidget container. Used to reopen popout when running in Electron.
     * @internal */
    openPopoutWidgetContainer(state: NineZoneState, widgetContainerId: string): void;
    /** Create a new popout/child window that contains the widget specified by its Id. Supported only when in
     *  UI 2.0 or higher.
     * @param widgetId case sensitive Widget Id
     * @param point Position of top left corner of floating panel in pixels. If undefined {x:0, y:0} is used.
     * @param size defines the width and height of the floating panel. If undefined and widget has been floated before
     * the previous size is used, else {height:800, width:600} is used.
     * @beta
     */
    popoutWidget(widgetId: string, point?: PointProps, size?: SizeProps): void;
    get isStageClosing(): boolean;
    get isApplicationClosing(): boolean;
    /** @internal */
    saveChildWindowSizeAndPosition(childWindowId: string, childWindow: Window): Promise<void>;
    /** Method used to possibly change a Popout Wigdet back to a docked widget if the user was the one closing the popout's child
     * window (i.e. UiFramework.childWindowManager.isClosingChildWindow === false).
     *  @internal
     */
    dockPopoutWidgetContainer(widgetContainerId: string): void;
    /** Finds the container with the specified widget and re-docks all widgets
     * back to the panel zone location that was used when the floating container
     * was generated. Supported only when in UI 2.0 or higher.
     * @param widgetId  case sensitive Wigdet Id.
     * @beta
     */
    dockWidgetContainer(widgetId: string): void;
}
//# sourceMappingURL=FrontstageDef.d.ts.map