"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Frontstage
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrontstageDef = void 0;
// cSpell:ignore popout
const React = require("react");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const ContentGroup_1 = require("../content/ContentGroup");
const ContentLayoutManager_1 = require("../content/ContentLayoutManager");
const ContentViewManager_1 = require("../content/ContentViewManager");
const UiFramework_1 = require("../UiFramework");
const Zone_1 = require("../zones/Zone");
const Frontstage_1 = require("./Frontstage");
const FrontstageManager_1 = require("./FrontstageManager");
const TimeTracker_1 = require("../configurableui/TimeTracker");
const PopoutWidget_1 = require("../childwindow/PopoutWidget");
const timers_1 = require("timers");
const Frontstage_2 = require("../widget-panels/Frontstage");
/** FrontstageDef class provides an API for a Frontstage.
 * @public
 */
class FrontstageDef {
    /** Constructs the [[FrontstageDef]]  */
    constructor(props) {
        this._id = "";
        this._defaultLayoutId = "";
        this._defaultContentId = "";
        this._contentGroupId = "";
        this._isInFooterMode = true;
        this._isStageClosing = false;
        this._isApplicationClosing = false;
        this._version = 0;
        this._timeTracker = new TimeTracker_1.TimeTracker();
        if (props)
            this.initializeFromProps(props);
    }
    get id() { return this._id; }
    get defaultTool() { return this._defaultTool; }
    get defaultLayoutId() { return this._defaultLayoutId; }
    get defaultContentId() { return this._defaultContentId; }
    get contentGroupId() { return this._contentGroupId; }
    get isInFooterMode() { return this._isInFooterMode; }
    get applicationData() { return this._applicationData; }
    get usage() { return this._usage !== undefined ? this._usage : ui_abstract_1.StageUsage.General; }
    get version() { return this._version; }
    get topLeft() { return this._topLeft; }
    get topCenter() { return this._topCenter; }
    get topRight() { return this._topRight; }
    get centerLeft() { return this._centerLeft; }
    get centerRight() { return this._centerRight; }
    get bottomLeft() { return this._bottomLeft; }
    get bottomCenter() { return this._bottomCenter; }
    get bottomRight() { return this._bottomRight; }
    /** @beta */
    get topPanel() { return this._topPanel; }
    /** @beta @deprecated Only topPanel is supported in UI 2.0 */
    get topMostPanel() { return this._topMostPanel; }
    /** @beta */
    get leftPanel() { return this._leftPanel; }
    /** @beta */
    get rightPanel() { return this._rightPanel; }
    /** @beta */
    get bottomPanel() { return this._bottomPanel; }
    /** @beta @deprecated Only bottomPanel is supported in UI 2.0  */
    get bottomMostPanel() { return this._bottomMostPanel; }
    get defaultLayout() { return this._defaultLayout; }
    get contentLayoutDef() { return this._contentLayoutDef; }
    get contentGroup() { return this._contentGroup; }
    get frontstageProvider() { return this._frontstageProvider; }
    /** @internal */
    get nineZone() { return this._nineZone; }
    set nineZone(props) { this._nineZone = props; }
    /** @internal */
    get nineZoneState() { return this._nineZoneState; }
    set nineZoneState(state) {
        if (this._nineZoneState === state)
            return;
        this._nineZoneState = state;
        // istanbul ignore else
        if (!(this._isStageClosing || this._isApplicationClosing)) {
            FrontstageManager_1.FrontstageManager.onFrontstageNineZoneStateChangedEvent.emit({
                frontstageDef: this,
                state,
            });
        }
    }
    /** @internal */
    get timeTracker() { return this._timeTracker; }
    /** Handles when the Frontstage becomes activated */
    _onActivated() { }
    /** Handles when the Frontstage becomes activated */
    onActivated() {
        this.updateWidgetDefs();
        this._contentLayoutDef = this.defaultLayout;
        if (!this._contentLayoutDef) {
            this._contentLayoutDef = ContentLayoutManager_1.ContentLayoutManager.findLayout(this.defaultLayoutId);
            if (!this._contentLayoutDef)
                throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `onActivated: Content Layout '${this.defaultLayoutId}' not registered`);
        }
        if (!this._contentGroup) {
            this._contentGroup = ContentGroup_1.ContentGroupManager.findGroup(this.contentGroupId);
            if (!this._contentGroup)
                throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `onActivated: Content Group '${this.contentGroupId}' not registered`);
        }
        FrontstageManager_1.FrontstageManager.onContentLayoutActivatedEvent.emit({ contentLayout: this._contentLayoutDef, contentGroup: this._contentGroup });
        this._timeTracker.startTiming();
        this._onActivated();
    }
    /** Handles when the Frontstage becomes inactive */
    _onDeactivated() { }
    /** Handles when the Frontstage becomes inactive */
    onDeactivated() {
        for (const control of this._widgetControls) {
            control.onFrontstageDeactivated();
        }
        for (const control of this.contentControls) {
            control.onFrontstageDeactivated();
        }
        if (this.contentGroup)
            this.contentGroup.onFrontstageDeactivated();
        this._timeTracker.stopTiming();
        this._isStageClosing = true; // this keeps widgets in child windows from automatically re-docking
        UiFramework_1.UiFramework.childWindowManager.closeAllChildWindows();
        this._onDeactivated();
        this._isStageClosing = false;
    }
    /** @internal */
    setIsApplicationClosing(value) {
        this._isApplicationClosing = value;
    }
    /** Returns once the contained widgets and content controls are ready to use */
    async waitUntilReady() {
        // create an array of control-ready promises
        const controlReadyPromises = new Array();
        this._widgetControls.forEach((control) => {
            controlReadyPromises.push(control.isReady);
        });
        // istanbul ignore else
        if (this.contentLayoutDef) {
            const usedContentIndexes = this.contentLayoutDef.getUsedContentIndexes();
            this.contentControls.forEach((control, index) => {
                // istanbul ignore else
                if (usedContentIndexes.includes(index))
                    controlReadyPromises.push(control.isReady);
            });
        }
        await Promise.all(controlReadyPromises);
        // Frontstage ready
    }
    /** Handles when the Frontstage becomes active */
    _onFrontstageReady() { }
    /** Handles when the Frontstage becomes active */
    onFrontstageReady() {
        for (const control of this._widgetControls) {
            control.onFrontstageReady();
        }
        for (const control of this.contentControls) {
            control.onFrontstageReady();
        }
        // istanbul ignore else
        if (this.contentGroup)
            this.contentGroup.onFrontstageReady();
        this._onFrontstageReady();
    }
    /** Starts the default tool for the Frontstage */
    startDefaultTool() {
        // Start the default tool
        // istanbul ignore next
        if (this.defaultTool && imodeljs_frontend_1.IModelApp.toolAdmin && imodeljs_frontend_1.IModelApp.viewManager) {
            imodeljs_frontend_1.IModelApp.toolAdmin.defaultToolId = this.defaultTool.toolId;
            this.defaultTool.execute();
        }
    }
    /** Sets the Content Layout and Content Group */
    setContentLayoutAndGroup(contentLayoutDef, contentGroup) {
        this._contentLayoutDef = contentLayoutDef;
        this._contentGroup = contentGroup;
    }
    /** Sets the active view content control to the default or first */
    setActiveContent() {
        let contentControl;
        let activated = false;
        if (this.contentGroup && this.defaultContentId) {
            contentControl = this.contentGroup.getContentControlById(this.defaultContentId);
        }
        // istanbul ignore else
        if (!contentControl && this.contentControls.length >= 0) {
            contentControl = this.contentControls[0];
        }
        if (contentControl) {
            ContentViewManager_1.ContentViewManager.setActiveContent(contentControl.reactNode, true);
            if (contentControl.viewport)
                imodeljs_frontend_1.IModelApp.viewManager.setSelectedView(contentControl.viewport);
            activated = true;
        }
        return activated;
    }
    /** Sets the active view content control */
    setActiveView(newContent, oldContent) {
        if (oldContent)
            oldContent.onDeactivated();
        newContent.onActivated();
        FrontstageManager_1.FrontstageManager.onContentControlActivatedEvent.emit({ activeContentControl: newContent, oldContentControl: oldContent });
    }
    /** Sets the active view content control based on the selected viewport. */
    setActiveViewFromViewport(viewport) {
        const contentControl = this.contentControls.find((control) => control.viewport === viewport);
        // istanbul ignore else
        if (contentControl) {
            ContentViewManager_1.ContentViewManager.setActiveContent(contentControl.reactNode, true);
            return true;
        }
        // istanbul ignore next
        return false;
    }
    /** Gets a [[ZoneDef]] based on a given zone id */
    getZoneDef(zoneId) {
        let zoneDef;
        switch (zoneId) {
            case 1:
                zoneDef = this.topLeft;
                break;
            case 2:
                zoneDef = this.topCenter;
                break;
            case 3:
                zoneDef = this.topRight;
                break;
            case 4:
                zoneDef = this.centerLeft;
                break;
            case 6:
                zoneDef = this.centerRight;
                break;
            case 7:
                zoneDef = this.bottomLeft;
                break;
            case 8:
                zoneDef = this.bottomCenter;
                break;
            case 9:
                zoneDef = this.bottomRight;
                break;
            // istanbul ignore next
            default:
                throw new RangeError();
        }
        // Zones can be undefined in a Frontstage
        return zoneDef;
    }
    /** Gets a list of [[ZoneDef]]s */
    get zoneDefs() {
        const zones = [1, 2, 3, 4, 6, 7, 8, 9];
        const zoneDefs = [];
        zones.forEach((zoneId) => {
            const zoneDef = this.getZoneDef(zoneId);
            if (zoneDef)
                zoneDefs.push(zoneDef);
        });
        return zoneDefs;
    }
    /** Gets a [[StagePanelDef]] based on a given panel location
     * @beta
     */
    getStagePanelDef(location) {
        let panelDef;
        switch (location) {
            case ui_abstract_1.StagePanelLocation.Top:
                panelDef = this.topPanel;
                break;
            case ui_abstract_1.StagePanelLocation.TopMost:
                panelDef = this.topMostPanel; // eslint-disable-line deprecation/deprecation
                break;
            case ui_abstract_1.StagePanelLocation.Left:
                panelDef = this.leftPanel;
                break;
            case ui_abstract_1.StagePanelLocation.Right:
                panelDef = this.rightPanel;
                break;
            case ui_abstract_1.StagePanelLocation.Bottom:
                panelDef = this.bottomPanel;
                break;
            case ui_abstract_1.StagePanelLocation.BottomMost:
                panelDef = this.bottomMostPanel; // eslint-disable-line deprecation/deprecation
                break;
            // istanbul ignore next
            default:
                throw new RangeError();
        }
        // Panels can be undefined in a Frontstage
        return panelDef;
    }
    /** Gets a list of [[StagePanelDef]]s
     * @beta
     */
    get panelDefs() {
        const panels = [
            ui_abstract_1.StagePanelLocation.Left, ui_abstract_1.StagePanelLocation.Right,
            ui_abstract_1.StagePanelLocation.Top, ui_abstract_1.StagePanelLocation.TopMost,
            ui_abstract_1.StagePanelLocation.Bottom, ui_abstract_1.StagePanelLocation.BottomMost,
        ];
        const panelDefs = [];
        panels.forEach((location) => {
            const panelDef = this.getStagePanelDef(location);
            if (panelDef)
                panelDefs.push(panelDef);
        });
        return panelDefs;
    }
    /** Finds a [[WidgetDef]] based on a given id */
    findWidgetDef(id) {
        for (const zoneDef of this.zoneDefs) {
            const widgetDef = zoneDef.findWidgetDef(id);
            if (widgetDef)
                return widgetDef;
        }
        for (const panelDef of this.panelDefs) {
            const widgetDef = panelDef.findWidgetDef(id);
            if (widgetDef)
                return widgetDef;
        }
        // istanbul ignore next
        return undefined;
    }
    /** Gets the list of [[WidgetControl]]s */
    get _widgetControls() {
        const widgetControls = new Array();
        this.zoneDefs.forEach((zoneDef) => {
            zoneDef.widgetDefs.forEach((widgetDef) => {
                const widgetControl = widgetDef.widgetControl;
                if (widgetControl)
                    widgetControls.push(widgetControl);
            });
        });
        this.panelDefs.forEach((panelDef) => {
            panelDef.widgetDefs.forEach((widgetDef) => {
                const widgetControl = widgetDef.widgetControl;
                // istanbul ignore if
                if (widgetControl)
                    widgetControls.push(widgetControl);
            });
        });
        return widgetControls;
    }
    /** Gets the list of [[ContentControl]]s */
    get contentControls() {
        if (this.contentGroup) {
            return this.contentGroup.getContentControls();
        }
        return [];
    }
    /**
     * Initializes this [[FrontstageDef]] from a [[FrontstageProvider]]
     * @param frontstageProvider The FrontstageProvider to initialize from
     */
    initializeFromProvider(frontstageProvider) {
        // istanbul ignore else
        if (frontstageProvider.frontstage && React.isValidElement(frontstageProvider.frontstage)) {
            Frontstage_1.Frontstage.initializeFrontstageDef(this, frontstageProvider.frontstage.props);
            this._frontstageProvider = frontstageProvider;
        }
    }
    /** Initializes a FrontstageDef from FrontstageProps
     * @internal
     */
    initializeFromProps(props) {
        this._id = props.id;
        this._defaultTool = props.defaultTool;
        if (props.defaultContentId !== undefined)
            this._defaultContentId = props.defaultContentId;
        if (typeof props.defaultLayout === "string")
            this._defaultLayoutId = props.defaultLayout;
        else
            this._defaultLayout = props.defaultLayout;
        if (typeof props.contentGroup === "string")
            this._contentGroupId = props.contentGroup;
        else
            this._contentGroup = props.contentGroup;
        if (props.isInFooterMode !== undefined)
            this._isInFooterMode = props.isInFooterMode;
        if (props.applicationData !== undefined)
            this._applicationData = props.applicationData;
        this._usage = props.usage;
        this._version = props.version || 0;
        this._topLeft = Frontstage_1.Frontstage.createZoneDef(props.contentManipulationTools ? props.contentManipulationTools : props.topLeft, Zone_1.ZoneLocation.TopLeft, props);
        this._topCenter = Frontstage_1.Frontstage.createZoneDef(props.toolSettings ? props.toolSettings : props.topCenter, Zone_1.ZoneLocation.TopCenter, props);
        this._topRight = Frontstage_1.Frontstage.createZoneDef(props.viewNavigationTools ? /* istanbul ignore next */ props.viewNavigationTools : props.topRight, Zone_1.ZoneLocation.TopRight, props);
        this._centerLeft = Frontstage_1.Frontstage.createZoneDef(props.centerLeft, Zone_1.ZoneLocation.CenterLeft, props);
        this._centerRight = Frontstage_1.Frontstage.createZoneDef(props.centerRight, Zone_1.ZoneLocation.CenterRight, props);
        this._bottomLeft = Frontstage_1.Frontstage.createZoneDef(props.bottomLeft, Zone_1.ZoneLocation.BottomLeft, props);
        this._bottomCenter = Frontstage_1.Frontstage.createZoneDef(props.statusBar ? props.statusBar : props.bottomCenter, Zone_1.ZoneLocation.BottomCenter, props);
        this._bottomRight = Frontstage_1.Frontstage.createZoneDef(props.bottomRight, Zone_1.ZoneLocation.BottomRight, props);
        this._topPanel = Frontstage_1.Frontstage.createStagePanelDef(ui_abstract_1.StagePanelLocation.Top, props);
        this._topMostPanel = Frontstage_1.Frontstage.createStagePanelDef(ui_abstract_1.StagePanelLocation.TopMost, props);
        this._leftPanel = Frontstage_1.Frontstage.createStagePanelDef(ui_abstract_1.StagePanelLocation.Left, props);
        this._rightPanel = Frontstage_1.Frontstage.createStagePanelDef(ui_abstract_1.StagePanelLocation.Right, props);
        this._bottomPanel = Frontstage_1.Frontstage.createStagePanelDef(ui_abstract_1.StagePanelLocation.Bottom, props);
        this._bottomMostPanel = Frontstage_1.Frontstage.createStagePanelDef(ui_abstract_1.StagePanelLocation.BottomMost, props);
    }
    /** @internal */
    updateWidgetDefs() {
        // Tracks provided widgets to prevent duplicates.
        const widgetDefs = [];
        // Process panels before zones so in uiVersion="2" extension can explicitly target a widget for a StagePanelSection
        this.panelDefs.forEach((panelDef) => {
            panelDef.updateDynamicWidgetDefs(this.id, this.usage, panelDef.location, undefined, widgetDefs);
        });
        this.zoneDefs.forEach((zoneDef) => {
            zoneDef.updateDynamicWidgetDefs(this.id, this.usage, zoneDef.zoneLocation, undefined, widgetDefs);
        });
    }
    /** @beta */
    restoreLayout() {
        for (const zoneDef of this.zoneDefs) {
            for (const widgetDef of zoneDef.widgetDefs) {
                widgetDef.setWidgetState(widgetDef.defaultState);
            }
        }
        for (const panelDef of this.panelDefs) {
            panelDef.size = panelDef.defaultSize;
            panelDef.panelState = panelDef.defaultState;
            for (const widgetDef of panelDef.widgetDefs) {
                widgetDef.setWidgetState(widgetDef.defaultState);
            }
        }
        FrontstageManager_1.FrontstageManager.onFrontstageRestoreLayoutEvent.emit({ frontstageDef: this });
    }
    isPopoutWidget(widgetId) {
        // istanbul ignore else
        if (this.nineZoneState) {
            const location = ui_ninezone_1.findTab(this.nineZoneState, widgetId);
            // istanbul ignore else
            if (location)
                return ui_ninezone_1.isPopoutLocation(location);
        }
        return false;
    }
    isFloatingWidget(widgetId) {
        // istanbul ignore else
        if (this.nineZoneState) {
            const location = ui_ninezone_1.findTab(this.nineZoneState, widgetId);
            // istanbul ignore else
            if (location)
                return ui_ninezone_1.isFloatingLocation(location);
        }
        return false;
    }
    /** Create a new floating panel that contains the widget specified by its Id. Supported only when in
     *  UI 2.0 or higher.
     * @param widgetId case sensitive Wigdet Id
     * @param point Position of top left corner of floating panel in pixels. If undefined {x:50, y:100} is used.
     * @param size defines the width and height of the floating panel. If undefined and widget has been floated before
     * the previous size is used, else {height:400, width:400} is used.
     * @beta
     */
    floatWidget(widgetId, point, size) {
        // istanbul ignore else
        if (this.nineZoneState) {
            const location = ui_ninezone_1.findTab(this.nineZoneState, widgetId);
            if (location) {
                let popoutWidgetContainerId;
                if (ui_ninezone_1.isPopoutLocation(location)) {
                    popoutWidgetContainerId = location.popoutWidgetId;
                }
                const state = ui_ninezone_1.floatWidget(this.nineZoneState, widgetId, point, size);
                // istanbul ignore else
                if (state) {
                    this.nineZoneState = state;
                    timers_1.setImmediate(() => {
                        popoutWidgetContainerId && UiFramework_1.UiFramework.childWindowManager.closeChildWindow(popoutWidgetContainerId, true);
                    });
                }
            }
        }
    }
    /** Opens window for specified PopoutWidget container. Used to reopen popout when running in Electron.
     * @internal */
    openPopoutWidgetContainer(state, widgetContainerId) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        const location = ui_ninezone_1.findWidget(state, widgetContainerId);
        // istanbul ignore else
        if (location && ui_ninezone_1.isPopoutWidgetLocation(location) && 1 === state.widgets[widgetContainerId].tabs.length) {
            // NOTE: Popout Widget Container will only contain a single WidgetTab
            const widgetDef = this.findWidgetDef(state.widgets[widgetContainerId].tabs[0]);
            // istanbul ignore else
            if (widgetDef) {
                const tab = state.tabs[widgetDef.id];
                const popoutContent = (React.createElement(PopoutWidget_1.PopoutWidget, { widgetContainerId: widgetContainerId, widgetDef: widgetDef }));
                const position = {
                    width: (_b = (_a = tab.preferredPopoutWidgetSize) === null || _a === void 0 ? void 0 : _a.width) !== null && _b !== void 0 ? _b : 600,
                    height: (_d = (_c = tab.preferredPopoutWidgetSize) === null || _c === void 0 ? void 0 : _c.height) !== null && _d !== void 0 ? _d : 800,
                    left: (_f = (_e = tab.preferredPopoutWidgetSize) === null || _e === void 0 ? void 0 : _e.x) !== null && _f !== void 0 ? _f : 0,
                    top: (_h = (_g = tab.preferredPopoutWidgetSize) === null || _g === void 0 ? void 0 : _g.y) !== null && _h !== void 0 ? _h : 0,
                };
                UiFramework_1.UiFramework.childWindowManager.openChildWindow(widgetContainerId, widgetDef.label, popoutContent, position, UiFramework_1.UiFramework.useDefaultPopoutUrl);
            }
        }
    }
    /** Create a new popout/child window that contains the widget specified by its Id. Supported only when in
     *  UI 2.0 or higher.
     * @param widgetId case sensitive Widget Id
     * @param point Position of top left corner of floating panel in pixels. If undefined {x:0, y:0} is used.
     * @param size defines the width and height of the floating panel. If undefined and widget has been floated before
     * the previous size is used, else {height:800, width:600} is used.
     * @beta
     */
    popoutWidget(widgetId, point, size) {
        // istanbul ignore else
        if (this.nineZoneState) {
            let location = ui_ninezone_1.findTab(this.nineZoneState, widgetId);
            // istanbul ignore else
            if (location) {
                if (ui_ninezone_1.isPopoutLocation(location))
                    return;
                // get the state to apply that will pop-out the specified WidgetTab to child window.
                const state = ui_ninezone_1.popoutWidgetToChildWindow(this.nineZoneState, widgetId, point, size);
                // istanbul ignore else
                if (state) {
                    // now that the state is updated get the id of the container that houses the widgetTab/widgetId
                    location = ui_ninezone_1.findTab(state, widgetId);
                    // istanbul ignore else
                    if (location && ui_ninezone_1.isPopoutLocation(location)) {
                        const widgetDef = this.findWidgetDef(widgetId);
                        // istanbul ignore else
                        if (widgetDef) {
                            const widgetContainerId = location.widgetId;
                            const tab = state.tabs[widgetId];
                            this.nineZoneState = state;
                            timers_1.setImmediate(() => {
                                const popoutContent = (React.createElement(PopoutWidget_1.PopoutWidget, { widgetContainerId: widgetContainerId, widgetDef: widgetDef }));
                                const position = {
                                    width: tab.preferredPopoutWidgetSize.width,
                                    height: tab.preferredPopoutWidgetSize.height,
                                    left: tab.preferredPopoutWidgetSize.x,
                                    top: tab.preferredPopoutWidgetSize.y,
                                };
                                UiFramework_1.UiFramework.childWindowManager.openChildWindow(widgetContainerId, widgetDef.label, popoutContent, position, UiFramework_1.UiFramework.useDefaultPopoutUrl);
                            });
                        }
                    }
                }
            }
        }
    }
    get isStageClosing() {
        return this._isStageClosing;
    }
    get isApplicationClosing() {
        return this._isApplicationClosing;
    }
    /** @internal */
    async saveChildWindowSizeAndPosition(childWindowId, childWindow) {
        // istanbul ignore else
        if (this.nineZoneState) {
            const newState = await Frontstage_2.saveFrontstagePopoutWidgetSizeAndPosition(this.nineZoneState, this.id, this.version, childWindowId, childWindow);
            this._nineZoneState = newState; // set without triggering new render as only preferred floating position set
        }
    }
    /** Method used to possibly change a Popout Wigdet back to a docked widget if the user was the one closing the popout's child
     * window (i.e. UiFramework.childWindowManager.isClosingChildWindow === false).
     *  @internal
     */
    dockPopoutWidgetContainer(widgetContainerId) {
        if (this.nineZoneState) {
            const location = ui_ninezone_1.findWidget(this.nineZoneState, widgetContainerId);
            // Make sure the widgetContainerId is still in popout state. We don't want to set it to docked if the window is being closed because
            // an API call has moved the widget from a popout state to a floating state.
            // istanbul ignore else
            if (location && ui_ninezone_1.isPopoutWidgetLocation(location)) {
                const state = ui_ninezone_1.dockWidgetContainer(this.nineZoneState, widgetContainerId, true);
                state && (this.nineZoneState = state);
            }
        }
    }
    /** Finds the container with the specified widget and re-docks all widgets
     * back to the panel zone location that was used when the floating container
     * was generated. Supported only when in UI 2.0 or higher.
     * @param widgetId  case sensitive Wigdet Id.
     * @beta
     */
    dockWidgetContainer(widgetId) {
        // istanbul ignore else
        if (this.nineZoneState) {
            const location = ui_ninezone_1.findTab(this.nineZoneState, widgetId);
            if (location) {
                const widgetContainerId = location.widgetId;
                const state = ui_ninezone_1.dockWidgetContainer(this.nineZoneState, widgetContainerId, true);
                state && (this.nineZoneState = state);
                if (ui_ninezone_1.isPopoutLocation(location)) {
                    UiFramework_1.UiFramework.childWindowManager.closeChildWindow(location.widgetId, true);
                }
            }
        }
    }
}
exports.FrontstageDef = FrontstageDef;
//# sourceMappingURL=FrontstageDef.js.map