"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Frontstage
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrontstageManager = exports.ToolIconChangedEvent = exports.ToolActivatedEvent = exports.ModalFrontstageClosedEvent = exports.ModalFrontstageRequestedCloseEvent = exports.ModalFrontstageChangedEvent = exports.FrontstageReadyEvent = exports.FrontstageDeactivatedEvent = exports.FrontstageActivatedEvent = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const ContentControl_1 = require("../content/ContentControl");
const ContentLayout_1 = require("../content/ContentLayout");
const NavigationAidControl_1 = require("../navigationaids/NavigationAidControl");
const StagePanelDef_1 = require("../stagepanels/StagePanelDef");
const UiFramework_1 = require("../UiFramework");
const UiShowHideManager_1 = require("../utils/UiShowHideManager");
const WidgetDef_1 = require("../widgets/WidgetDef");
const ToolInformation_1 = require("../zones/toolsettings/ToolInformation");
const ToolSettingsManager_1 = require("../zones/toolsettings/ToolSettingsManager");
const FrontstageDef_1 = require("./FrontstageDef");
const TimeTracker_1 = require("../configurableui/TimeTracker");
/** Frontstage Activated Event class.
 * @public
 */
class FrontstageActivatedEvent extends ui_core_1.UiEvent {
}
exports.FrontstageActivatedEvent = FrontstageActivatedEvent;
/** Frontstage Deactivated Event class.
 * @public
 */
class FrontstageDeactivatedEvent extends ui_core_1.UiEvent {
}
exports.FrontstageDeactivatedEvent = FrontstageDeactivatedEvent;
/** Frontstage Ready Event class.
 * @public
 */
class FrontstageReadyEvent extends ui_core_1.UiEvent {
}
exports.FrontstageReadyEvent = FrontstageReadyEvent;
/** Modal Frontstage Stack Changed Event class.
 * @public
 */
class ModalFrontstageChangedEvent extends ui_core_1.UiEvent {
}
exports.ModalFrontstageChangedEvent = ModalFrontstageChangedEvent;
/** Modal Frontstage Requested Close Event class. Notifies the modal stage that the close button was
 * pressed and passes the function to actually close the modal stage. This allows stage to do any
 * saving of unsaved data prior to closing the stage. If the ModalFrontstageInfo sets notifyCloseRequest
 * to true it is up to the stage to register for this event and call the stageCloseFunc once it has saved
 * any unsaved data.
 * @alpha
 */
class ModalFrontstageRequestedCloseEvent extends ui_core_1.UiEvent {
}
exports.ModalFrontstageRequestedCloseEvent = ModalFrontstageRequestedCloseEvent;
/** Modal Frontstage Closed Event class.
 * @public
 */
class ModalFrontstageClosedEvent extends ui_core_1.UiEvent {
}
exports.ModalFrontstageClosedEvent = ModalFrontstageClosedEvent;
/** Tool Activated Event class.
 * @public
 */
class ToolActivatedEvent extends ui_core_1.UiEvent {
}
exports.ToolActivatedEvent = ToolActivatedEvent;
/** Tool Icon Changed Event class.
 * @public
 */
class ToolIconChangedEvent extends ui_core_1.UiEvent {
}
exports.ToolIconChangedEvent = ToolIconChangedEvent;
// -----------------------------------------------------------------------------
// FrontstageManager class
// -----------------------------------------------------------------------------
/** Frontstage Manager class.
 * @public
 */
class FrontstageManager {
    /** This should only be called within FrontstageManager and its tests.
     *  @internal
     */
    static ensureToolInformationIsSet(toolId) {
        // istanbul ignore else
        if (!FrontstageManager._toolInformationMap.get(toolId))
            FrontstageManager._toolInformationMap.set(toolId, new ToolInformation_1.ToolInformation(toolId));
    }
    // pass on SyncToolSettingsPropertiesEvent from ToolAdmin so they are treated as DialogItemSync events
    static handleSyncToolSettingsPropertiesEvent(args) {
        FrontstageManager.activeToolSettingsProvider && FrontstageManager.activeToolSettingsProvider.syncToolSettingsProperties(args);
    }
    // pass on ReloadToolSettingsEvent from ToolAdmin so they are treated by UiProviders
    static handleReloadToolSettingsEvent() {
        // istanbul ignore else
        if (FrontstageManager.activeToolSettingsProvider) {
            FrontstageManager.activeToolSettingsProvider.reloadPropertiesFromTool();
        }
    }
    /** Initializes the FrontstageManager */
    static initialize() {
        if (this._initialized)
            return;
        // istanbul ignore else
        if (imodeljs_frontend_1.IModelApp && imodeljs_frontend_1.IModelApp.toolAdmin) {
            imodeljs_frontend_1.IModelApp.toolAdmin.activeToolChanged.addListener((tool, _start) => {
                // make sure tool settings properties are cached before creating ToolInformation
                ToolSettingsManager_1.ToolSettingsManager.clearToolSettingsData();
                // istanbul ignore else
                if (tool instanceof imodeljs_frontend_1.InteractiveTool)
                    ToolSettingsManager_1.ToolSettingsManager.initializeDataForTool(tool);
                // if the tool data is not already cached then see if there is data to cache
                FrontstageManager.ensureToolInformationIsSet(tool.toolId);
                FrontstageManager.setActiveTool(tool);
            });
            ToolSettingsManager_1.ToolSettingsManager.onSyncToolSettingsProperties.addListener(FrontstageManager.handleSyncToolSettingsPropertiesEvent);
            ToolSettingsManager_1.ToolSettingsManager.onReloadToolSettingsProperties.addListener(FrontstageManager.handleReloadToolSettingsEvent);
        }
        // istanbul ignore else
        if (imodeljs_frontend_1.IModelApp && imodeljs_frontend_1.IModelApp.viewManager) {
            imodeljs_frontend_1.IModelApp.viewManager.onSelectedViewportChanged.addListener(FrontstageManager._handleSelectedViewportChanged);
        }
        this._initialized = true;
    }
    /** @internal */
    static get isInitialized() { return FrontstageManager._initialized; }
    static set isInitialized(v) { FrontstageManager._initialized = v; }
    /** Returns true if Frontstage is loading its controls. If false the Frontstage content and controls have been created. */
    static get isLoading() { return FrontstageManager._isLoading; }
    /** @internal */
    static get nineZoneSize() { return FrontstageManager._nineZoneSize; }
    static set nineZoneSize(size) {
        FrontstageManager._nineZoneSize = size;
    }
    /** Get Nine-zone State Manager. */
    static get NineZoneManager() {
        const id = FrontstageManager.activeFrontstageId;
        let manager = FrontstageManager._nineZoneManagers.get(id);
        if (!manager) {
            manager = new ui_ninezone_1.NineZoneManager();
            FrontstageManager._nineZoneManagers.set(id, manager);
        }
        return manager;
    }
    /** Clears the Frontstage map.
     */
    static clearFrontstageDefs() {
        FrontstageManager._frontstageDefs.clear();
    }
    /** Add a Frontstage via a definition.
     * @param frontstageDef  Definition of the Frontstage to add
     */
    static addFrontstageDef(frontstageDef) {
        FrontstageManager._frontstageDefs.set(frontstageDef.id, frontstageDef);
    }
    /** Add a Frontstage via a [[FrontstageProvider]].
     * @param frontstageProvider  FrontstageProvider representing the Frontstage to add
     */
    static addFrontstageProvider(frontstageProvider) {
        FrontstageManager.addFrontstageDef(frontstageProvider.initializeDef());
    }
    /** Find a loaded Frontstage with a given id. If the id is not provided, the active Frontstage is returned.
     * @param id  Id of the Frontstage to find
     * @returns  FrontstageDef with a given id if found, or undefined if not found.
     */
    static findFrontstageDef(id) {
        if (!id)
            return FrontstageManager.activeFrontstageDef;
        const frontstageDef = FrontstageManager._frontstageDefs.get(id);
        if (frontstageDef instanceof FrontstageDef_1.FrontstageDef)
            return frontstageDef;
        return undefined;
    }
    /** Gets the active FrontstageDef. If a Frontstage is not active, undefined is returned.
     * @return  Active FrontstageDef, or undefined if one is not active.
     */
    static get activeFrontstageDef() {
        return FrontstageManager._activeFrontstageDef;
    }
    /** Gets the Id of the active FrontstageDef. If a Frontstage is not active, blank is returned.
     * @return  Id of the active FrontstageDef, or blank if one is not active.
     */
    static get activeFrontstageId() {
        const activeFrontstage = FrontstageManager._activeFrontstageDef;
        return (activeFrontstage) ? activeFrontstage.id : "";
    }
    /** Sets the active FrontstageDef.
     * @param  frontstageId  Id of the Frontstage to set active.
     * @returns A Promise that is fulfilled when the [[Frontstage]] is ready.
     */
    static async setActiveFrontstage(frontstageId) {
        const frontstageDef = FrontstageManager.findFrontstageDef(frontstageId);
        if (!frontstageDef) {
            bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), `setActiveFrontstage: Could not find Frontstage with id of '${frontstageId}'`);
            return;
        }
        return FrontstageManager.setActiveFrontstageDef(frontstageDef);
    }
    /** Sets the active FrontstageDef.
     * @param  frontstageDef  FrontstageDef to set active.
     * @returns A Promise that is fulfilled when the [[FrontstageDef]] is ready.
     */
    static async setActiveFrontstageDef(frontstageDef) {
        if (FrontstageManager._activeFrontstageDef === frontstageDef)
            return;
        FrontstageManager._isLoading = true;
        const deactivatedFrontstageDef = FrontstageManager._activeFrontstageDef;
        if (deactivatedFrontstageDef) {
            deactivatedFrontstageDef.onDeactivated();
            const timeTracker = deactivatedFrontstageDef.timeTracker;
            FrontstageManager.onFrontstageDeactivatedEvent.emit({
                deactivatedFrontstageDef,
                activatedFrontstageDef: frontstageDef,
                totalTime: timeTracker.getTotalTimeSeconds(),
                engagementTime: timeTracker.getEngagementTimeSeconds(),
                idleTime: timeTracker.getIdleTimeSeconds(),
            });
        }
        FrontstageManager._activeFrontstageDef = frontstageDef;
        if (frontstageDef) {
            frontstageDef.onActivated();
            FrontstageManager.onFrontstageActivatedEvent.emit({ activatedFrontstageDef: frontstageDef, deactivatedFrontstageDef });
            await frontstageDef.waitUntilReady();
            FrontstageManager._isLoading = false;
            frontstageDef.onFrontstageReady();
            FrontstageManager.onFrontstageReadyEvent.emit({ frontstageDef });
            UiShowHideManager_1.UiShowHideManager.handleFrontstageReady();
            frontstageDef.startDefaultTool();
            frontstageDef.setActiveContent();
        }
        FrontstageManager._isLoading = false;
    }
    /** Deactivates the active FrontstageDef.
     */
    static async deactivateFrontstageDef() {
        await this.setActiveFrontstageDef(undefined);
    }
    /** Gets the Id of the active tool. If a tool is not active, blank is returned.
     * @return  Id of the active tool, or blank if one is not active.
     */
    static get activeToolId() {
        return FrontstageManager._activeToolId;
    }
    /** Sets the active tool id */
    static setActiveToolId(toolId) {
        FrontstageManager._activeToolId = toolId;
        const toolSettingsProvider = FrontstageManager.activeToolSettingsProvider;
        // ensure the toolSettingsProvider is initialized before emitting onToolActivatedEvent
        if (toolSettingsProvider)
            toolSettingsProvider.initialize();
        FrontstageManager.onToolActivatedEvent.emit({ toolId });
    }
    /** Sets the active tool */
    static setActiveTool(tool) {
        FrontstageManager.setActiveToolId(tool.toolId);
        FrontstageManager.onToolIconChangedEvent.emit({ iconSpec: tool.iconSpec });
    }
    /** Gets the active tool's [[ToolInformation]] */
    static get activeToolInformation() {
        return FrontstageManager._toolInformationMap.get(FrontstageManager.activeToolId);
    }
    /** Gets the Tool Setting React node of the active tool.
     * @return  Tool Setting React node of the active tool, or undefined if there is no active tool or Tool Settings for the active tool.
     * @internal
     */
    static get activeToolSettingsProvider() {
        const activeToolInformation = FrontstageManager.activeToolInformation;
        return (activeToolInformation) ? activeToolInformation.toolUiProvider : /* istanbul ignore next */ undefined;
    }
    /** Sets the active layout, content group and active content.
     * @param contentLayoutDef  Content layout to make active
     * @param contentGroup  Content Group to make active
     */
    static async setActiveLayout(contentLayoutDef, contentGroup) {
        const activeFrontstageDef = FrontstageManager.activeFrontstageDef;
        // istanbul ignore else
        if (activeFrontstageDef) {
            FrontstageManager._isLoading = false;
            activeFrontstageDef.setContentLayoutAndGroup(contentLayoutDef, contentGroup);
            FrontstageManager.onContentLayoutActivatedEvent.emit({ contentLayout: contentLayoutDef, contentGroup });
            await activeFrontstageDef.waitUntilReady();
            FrontstageManager._isLoading = false;
            activeFrontstageDef.setActiveContent();
        }
    }
    /** Opens a modal Frontstage. Modal Frontstages can be stacked.
     * @param modalFrontstage  Information about the modal Frontstage
     */
    static openModalFrontstage(modalFrontstage) {
        FrontstageManager.pushModalFrontstage(modalFrontstage);
    }
    static pushModalFrontstage(modalFrontstage) {
        const timeTracker = new TimeTracker_1.TimeTracker();
        timeTracker.startTiming();
        const frontstageItem = { modalFrontstage, timeTracker };
        FrontstageManager._modalFrontstages.push(frontstageItem);
        FrontstageManager.emitModalFrontstageChangedEvent();
    }
    /** Closes the top-most modal Frontstage.
     */
    static closeModalFrontstage() {
        // istanbul ignore else
        if (FrontstageManager._modalFrontstages.length > 0) {
            const topMostStageItem = FrontstageManager._modalFrontstages[FrontstageManager._modalFrontstages.length - 1];
            if (topMostStageItem.modalFrontstage.notifyCloseRequest)
                FrontstageManager.onCloseModalFrontstageRequestedEvent.emit({
                    modalFrontstage: topMostStageItem.modalFrontstage,
                    stageCloseFunc: FrontstageManager.popModalFrontstage,
                });
            else
                FrontstageManager.popModalFrontstage();
        }
    }
    static popModalFrontstage() {
        const frontstageItem = FrontstageManager._modalFrontstages.pop();
        // istanbul ignore else
        if (frontstageItem) {
            const modalFrontstage = frontstageItem.modalFrontstage;
            const timeTracker = frontstageItem.timeTracker;
            timeTracker.stopTiming();
            FrontstageManager.onModalFrontstageClosedEvent.emit({
                modalFrontstage,
                totalTime: timeTracker.getTotalTimeSeconds(),
                engagementTime: timeTracker.getEngagementTimeSeconds(),
                idleTime: timeTracker.getIdleTimeSeconds(),
            });
        }
        FrontstageManager.emitModalFrontstageChangedEvent();
        UiShowHideManager_1.UiShowHideManager.handleFrontstageReady();
    }
    static emitModalFrontstageChangedEvent() {
        FrontstageManager.onModalFrontstageChangedEvent.emit({ modalFrontstageCount: FrontstageManager.modalFrontstageCount });
    }
    /** Updates the top-most modal Frontstage.
     */
    static updateModalFrontstage() {
        FrontstageManager.emitModalFrontstageChangedEvent();
    }
    /** Gets the top-most modal Frontstage.
     * @returns Top-most modal Frontstage, or undefined if there is none.
     */
    static get activeModalFrontstage() {
        if (FrontstageManager._modalFrontstages.length > 0) {
            const frontstageItem = FrontstageManager._modalFrontstages[FrontstageManager._modalFrontstages.length - 1];
            const modalFrontstage = frontstageItem.modalFrontstage;
            return modalFrontstage;
        }
        else {
            return undefined;
        }
    }
    /** Gets the number of modal Frontstages.
     * @returns Modal Frontstage count
     */
    static get modalFrontstageCount() {
        return FrontstageManager._modalFrontstages.length;
    }
    /** Sets the active Navigation Aid via its Id.
     * @param navigationAidId  Id of the Navigation Aid to set as active
     * @param iModelConnection IModelConnection to query for view data
     */
    static setActiveNavigationAid(navigationAidId, iModelConnection) {
        FrontstageManager.onNavigationAidActivatedEvent.emit({ navigationAidId, iModelConnection });
    }
    /** Sets the state of the widget with the given id
     * @param widgetId  Id of the Widget for which to set the state
     * @param state     New state of the widget
     * @returns true if the widget state was set successfully, or false if not.
     */
    static setWidgetState(widgetId, state) {
        const widgetDef = FrontstageManager.findWidget(widgetId);
        if (widgetDef) {
            widgetDef.setWidgetState(state);
            return true;
        }
        else {
            bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), `setWidgetState: Could not find Widget with id of '${widgetId}'`);
        }
        return false;
    }
    /** Finds a widget with the given id in the active frontstage
     * @param widgetId  Id of the Widget to find
     * @returns The WidgetDef with the given id, or undefined if not found.
     */
    static findWidget(widgetId) {
        const activeFrontstageDef = FrontstageManager.activeFrontstageDef;
        // istanbul ignore else
        if (activeFrontstageDef)
            return activeFrontstageDef.findWidgetDef(widgetId);
        return undefined;
    }
    /** Opens a nested Frontstage. Nested Frontstages can be stacked.
     * @param nestedFrontstage  Information about the nested Frontstage
     */
    static async openNestedFrontstage(nestedFrontstage) {
        if (FrontstageManager.nestedFrontstageCount === 0)
            FrontstageManager._activePrimaryFrontstageDef = FrontstageManager._activeFrontstageDef;
        FrontstageManager.pushNestedFrontstage(nestedFrontstage);
        await FrontstageManager.setActiveFrontstageDef(nestedFrontstage);
    }
    static pushNestedFrontstage(nestedFrontstage) {
        FrontstageManager._nestedFrontstages.push(nestedFrontstage);
    }
    /** Closes the top-most nested Frontstage.
     */
    static async closeNestedFrontstage() {
        FrontstageManager.popNestedFrontstage();
        if (FrontstageManager.nestedFrontstageCount > 0) {
            await FrontstageManager.setActiveFrontstageDef(FrontstageManager.activeNestedFrontstage);
        }
        else {
            await FrontstageManager.setActiveFrontstageDef(FrontstageManager._activePrimaryFrontstageDef);
            FrontstageManager._activePrimaryFrontstageDef = undefined;
        }
    }
    static popNestedFrontstage() {
        FrontstageManager._nestedFrontstages.pop();
    }
    /** Gets the top-most nested Frontstage.
     * @returns Top-most nested Frontstage, or undefined if there is none.
     */
    static get activeNestedFrontstage() {
        // istanbul ignore else
        if (FrontstageManager._nestedFrontstages.length > 0)
            return FrontstageManager._nestedFrontstages[FrontstageManager._nestedFrontstages.length - 1];
        return undefined;
    }
    /** Gets the number of nested Frontstages.
     * @returns Nested Frontstage count
     */
    static get nestedFrontstageCount() {
        return FrontstageManager._nestedFrontstages.length;
    }
}
exports.FrontstageManager = FrontstageManager;
FrontstageManager._initialized = false;
FrontstageManager._isLoading = true;
FrontstageManager._activeToolId = "";
FrontstageManager._frontstageDefs = new Map();
FrontstageManager._modalFrontstages = new Array();
FrontstageManager._nineZoneManagers = new Map();
FrontstageManager._nineZoneSize = undefined;
FrontstageManager._nestedFrontstages = new Array();
FrontstageManager._toolInformationMap = new Map();
/** Handles a Viewport change & sets the active view accordingly */
FrontstageManager._handleSelectedViewportChanged = (args) => {
    // istanbul ignore else
    if (args.current && FrontstageManager.activeFrontstageDef && !FrontstageManager.isLoading) {
        FrontstageManager.activeFrontstageDef.setActiveViewFromViewport(args.current);
    }
};
/** Get Frontstage Deactivated event. */
FrontstageManager.onFrontstageDeactivatedEvent = new FrontstageDeactivatedEvent();
/** Get Frontstage Activated event. */
FrontstageManager.onFrontstageActivatedEvent = new FrontstageActivatedEvent();
/** Get Frontstage Activated event. */
FrontstageManager.onFrontstageReadyEvent = new FrontstageReadyEvent();
/** Get Modal Frontstage Changed event. */
FrontstageManager.onModalFrontstageChangedEvent = new ModalFrontstageChangedEvent();
/** Get Modal Frontstage Closed event. */
FrontstageManager.onModalFrontstageClosedEvent = new ModalFrontstageClosedEvent();
/** Get Modal Frontstage Requested Closed event.
 * @alpha
 */
FrontstageManager.onCloseModalFrontstageRequestedEvent = new ModalFrontstageRequestedCloseEvent();
/** Get Tool Activated event. */
FrontstageManager.onToolActivatedEvent = new ToolActivatedEvent();
/** Get ToolSetting Reload event. */
FrontstageManager.onToolSettingsReloadEvent = new ui_core_1.UiEvent();
/** Get Tool Panel Opened event.
 * @internal
 */
FrontstageManager.onToolPanelOpenedEvent = new ui_core_1.UiEvent();
/** Get Tool Icon Changed event. */
FrontstageManager.onToolIconChangedEvent = new ToolIconChangedEvent();
/** Get Content Layout Activated event. */
FrontstageManager.onContentLayoutActivatedEvent = new ContentLayout_1.ContentLayoutActivatedEvent();
/** Get Content Control Activated event. */
FrontstageManager.onContentControlActivatedEvent = new ContentControl_1.ContentControlActivatedEvent();
/** Get Navigation Aid Activated event. */
FrontstageManager.onNavigationAidActivatedEvent = new NavigationAidControl_1.NavigationAidActivatedEvent();
/** Get Widget State Changed event. */
FrontstageManager.onWidgetStateChangedEvent = new WidgetDef_1.WidgetStateChangedEvent();
/** @internal */
FrontstageManager.onWidgetLabelChangedEvent = new ui_core_1.UiEvent();
/** @internal */
FrontstageManager.onWidgetShowEvent = new ui_core_1.UiEvent();
/** @internal */
FrontstageManager.onWidgetExpandEvent = new ui_core_1.UiEvent();
/** @internal */
FrontstageManager.onWidgetDefsUpdatedEvent = new ui_core_1.UiEvent();
/** @internal */
FrontstageManager.onFrontstageNineZoneStateChangedEvent = new ui_core_1.UiEvent();
/** @internal */
FrontstageManager.onFrontstageRestoreLayoutEvent = new ui_core_1.UiEvent();
/** Get Widget State Changed event.
 * @alpha
 */
FrontstageManager.onPanelStateChangedEvent = new StagePanelDef_1.PanelStateChangedEvent();
/** @internal */
FrontstageManager.onPanelSizeChangedEvent = new StagePanelDef_1.PanelSizeChangedEvent();
//# sourceMappingURL=FrontstageManager.js.map