"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Frontstage
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SettingsModalFrontstage = void 0;
require("./ModalSettingsStage.scss");
const React = require("react");
const ui_abstract_1 = require("@bentley/ui-abstract");
const settings_svg_sprite_1 = require("@bentley/icons-generic/icons/settings.svg?sprite");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_core_1 = require("@bentley/ui-core");
const FrontstageManager_1 = require("./FrontstageManager");
const UiFramework_1 = require("../UiFramework");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
function ModalSettingsStage({ initialSettingsTabId }) {
    var _a, _b, _c, _d;
    const id = (_b = (_a = FrontstageManager_1.FrontstageManager.activeFrontstageDef) === null || _a === void 0 ? void 0 : _a.id) !== null && _b !== void 0 ? _b : "none";
    const stageUsage = (_d = (_c = FrontstageManager_1.FrontstageManager.activeFrontstageDef) === null || _c === void 0 ? void 0 : _c.usage) !== null && _d !== void 0 ? _d : ui_abstract_1.StageUsage.General;
    const tabEntries = UiFramework_1.UiFramework.settingsManager.getSettingEntries(id, stageUsage);
    const noSettingsAvailableLabel = React.useRef(UiFramework_1.UiFramework.translate("settings.noSettingsAvailable"));
    const currentSettingsTab = React.useCallback(() => {
        const categoryToFind = initialSettingsTabId ? initialSettingsTabId.toLowerCase() : tabEntries[0].tabId.toLowerCase();
        let foundTab = tabEntries.find((entry) => entry.tabId.toLowerCase() === categoryToFind);
        if (!foundTab)
            foundTab = tabEntries.find((entry) => entry.label.toLowerCase() === categoryToFind);
        return foundTab;
    }, [initialSettingsTabId, tabEntries]);
    React.useEffect(() => {
        const handleFrontstageCloseRequested = ({ modalFrontstage, stageCloseFunc }) => {
            // istanbul ignore else
            if (modalFrontstage instanceof SettingsModalFrontstage && stageCloseFunc) {
                UiFramework_1.UiFramework.settingsManager.closeSettingsContainer(stageCloseFunc);
            }
        };
        return FrontstageManager_1.FrontstageManager.onCloseModalFrontstageRequestedEvent.addListener(handleFrontstageCloseRequested);
    }, [tabEntries]);
    return (React.createElement("div", { className: "uifw-settings-container" }, tabEntries.length ?
        React.createElement(ui_core_1.SettingsContainer, { tabs: tabEntries, currentSettingsTab: currentSettingsTab(), settingsManager: UiFramework_1.UiFramework.settingsManager }) :
        React.createElement(ui_core_1.Centered, null, noSettingsAvailableLabel.current)));
}
/** Modal frontstage displaying and editing settings from registered settings providers. See [SettingsManager]($ui-core)
 * and [SettingsContainer]($ui-core).
 * @beta
 */
class SettingsModalFrontstage {
    constructor(initialSettingsTabId) {
        this.initialSettingsTabId = initialSettingsTabId;
        this.title = UiFramework_1.UiFramework.translate("settings.settingsStageLabel");
        this.notifyCloseRequest = true;
    }
    get content() {
        return (React.createElement(ModalSettingsStage, { initialSettingsTabId: this.initialSettingsTabId }));
    }
    static getBackstageActionItem(groupPriority, itemPriority) {
        return ui_abstract_1.BackstageItemUtilities.createActionItem(SettingsModalFrontstage.id, groupPriority, itemPriority, () => FrontstageManager_1.FrontstageManager.openModalFrontstage(new SettingsModalFrontstage()), UiFramework_1.UiFramework.translate("settings.settingsStageLabel"), undefined, ui_abstract_1.IconSpecUtilities.createSvgIconSpec(settings_svg_sprite_1.default), { isHidden: SettingsModalFrontstage.noSettingsAvailable() });
    }
    static showSettingsStage(initialSettingsTab) {
        if (UiFramework_1.UiFramework.settingsManager.providers.length) {
            // Check to see if it is already open
            // istanbul ignore else
            if (FrontstageManager_1.FrontstageManager.activeModalFrontstage && FrontstageManager_1.FrontstageManager.activeModalFrontstage instanceof SettingsModalFrontstage) {
                if (initialSettingsTab) {
                    UiFramework_1.UiFramework.settingsManager.activateSettingsTab(initialSettingsTab);
                    return;
                }
            }
            FrontstageManager_1.FrontstageManager.openModalFrontstage(new SettingsModalFrontstage(initialSettingsTab));
        }
        else {
            const briefMessage = UiFramework_1.UiFramework.translate("settings.noSettingsAvailable");
            const detailedMessage = UiFramework_1.UiFramework.translate("settings.noSettingsProvidersRegistered");
            const info = new imodeljs_frontend_1.NotifyMessageDetails(imodeljs_frontend_1.OutputMessagePriority.Info, briefMessage, detailedMessage, imodeljs_frontend_1.OutputMessageType.Toast);
            imodeljs_frontend_1.IModelApp.notifications.outputMessage(info);
            bentleyjs_core_1.Logger.logInfo(UiFramework_1.UiFramework.loggerCategory(this), detailedMessage);
        }
    }
}
exports.SettingsModalFrontstage = SettingsModalFrontstage;
SettingsModalFrontstage.id = "ui-framework.modalSettingsStage";
SettingsModalFrontstage.noSettingsAvailable = () => new ui_abstract_1.ConditionalBooleanValue(() => 0 === UiFramework_1.UiFramework.settingsManager.providers.length, [SyncUiEventDispatcher_1.SyncUiEventId.SettingsProvidersChanged]);
//# sourceMappingURL=ModalSettingsStage.js.map