"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module IModelComponents
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelConnectedModelsTree = exports.ModelsTree = exports.RULESET_MODELS_GROUPED_BY_CLASS = exports.RULESET_MODELS = void 0;
require("./ModelsTree.scss");
const React = require("react");
const presentation_components_1 = require("@bentley/presentation-components");
const presentation_frontend_1 = require("@bentley/presentation-frontend");
const ui_components_1 = require("@bentley/ui-components");
const ui_core_1 = require("@bentley/ui-core");
const connectIModel_1 = require("../../../ui-framework/redux/connectIModel");
const UiFramework_1 = require("../../../ui-framework/UiFramework");
const Common_1 = require("../Common");
const VisibilityTreeEventHandler_1 = require("../VisibilityTreeEventHandler");
const VisibilityTreeRenderer_1 = require("../VisibilityTreeRenderer");
const ModelsVisibilityHandler_1 = require("./ModelsVisibilityHandler");
const PAGING_SIZE = 20;
/** @internal */
exports.RULESET_MODELS = require("./Hierarchy.json"); // eslint-disable-line @typescript-eslint/no-var-requires
/** @internal */
exports.RULESET_MODELS_GROUPED_BY_CLASS = require("./Hierarchy.GroupedByClass.json"); // eslint-disable-line @typescript-eslint/no-var-requires
const RULESET_MODELS_SEARCH = require("./ModelsTreeSearch.json"); // eslint-disable-line @typescript-eslint/no-var-requires
/**
 * A tree component that shows a subject - model - category - element
 * hierarchy along with checkboxes that represent and allow changing
 * the display of those instances.
 * @public
 */
function ModelsTree(props) {
    const { nodeLoader, onItemsRendered } = useModelsTreeNodeLoader(props);
    const { filteredNodeLoader, isFiltering, nodeHighlightingProps } = VisibilityTreeRenderer_1.useVisibilityTreeFiltering(nodeLoader, props.filterInfo, props.onFilterApplied);
    const filterApplied = filteredNodeLoader !== nodeLoader;
    const { activeView, modelsVisibilityHandler, selectionPredicate } = props;
    const nodeSelectionPredicate = React.useCallback((key, node) => {
        return !selectionPredicate ? true : selectionPredicate(key, ModelsVisibilityHandler_1.ModelsVisibilityHandler.getNodeType(node, nodeLoader.dataProvider));
    }, [selectionPredicate, nodeLoader.dataProvider]);
    const visibilityHandler = useVisibilityHandler(nodeLoader.dataProvider.rulesetId, activeView, modelsVisibilityHandler, getFilteredDataProvider(filteredNodeLoader.dataProvider), props.enableHierarchyAutoUpdate);
    const eventHandler = ui_core_1.useDisposable(React.useCallback(() => new VisibilityTreeEventHandler_1.VisibilityTreeEventHandler({
        nodeLoader: filteredNodeLoader,
        visibilityHandler,
        collapsedChildrenDisposalEnabled: true,
        selectionPredicate: nodeSelectionPredicate,
    }), [filteredNodeLoader, visibilityHandler, nodeSelectionPredicate]));
    const visibleNodes = ui_components_1.useVisibleTreeNodes(filteredNodeLoader.modelSource);
    const treeRenderer = VisibilityTreeRenderer_1.useVisibilityTreeRenderer(true, false);
    const overlay = isFiltering ? React.createElement("div", { className: "filteredTreeOverlay" }) : undefined;
    // istanbul ignore next
    const noFilteredDataRenderer = React.useCallback(() => {
        return React.createElement(VisibilityTreeRenderer_1.VisibilityTreeNoFilteredData, { title: UiFramework_1.UiFramework.i18n.translate("UiFramework:modelTree.noModelFound"), message: UiFramework_1.UiFramework.i18n.translate("UiFramework:modelTree.noMatchingModelNames") });
    }, []);
    return (React.createElement("div", { className: "ui-fw-models-tree", ref: props.rootElementRef },
        React.createElement(ui_components_1.ControlledTree, { nodeLoader: filteredNodeLoader, visibleNodes: visibleNodes, selectionMode: props.selectionMode || ui_components_1.SelectionMode.None, treeEvents: eventHandler, treeRenderer: treeRenderer, nodeHighlightingProps: nodeHighlightingProps, noDataRenderer: filterApplied ? noFilteredDataRenderer : undefined, onItemsRendered: onItemsRendered }),
        overlay));
}
exports.ModelsTree = ModelsTree;
/**
 * ModelsTree that is connected to the IModelConnection property in the Redux store. The
 * application must set up the Redux store and include the FrameworkReducer.
 * @alpha
 */
exports.IModelConnectedModelsTree = connectIModel_1.connectIModelConnection(null, null)(ModelsTree); // eslint-disable-line @typescript-eslint/naming-convention
function useModelsTreeNodeLoader(props) {
    var _a, _b;
    // note: this is a temporary workaround for auto-update not working on ruleset variable changes - instead
    // of auto-updating we just re-create the node loader by re-creating the ruleset
    const rulesets = React.useMemo(() => {
        return {
            general: (!props.enableElementsClassGrouping) ? Object.assign({}, exports.RULESET_MODELS) : /* istanbul ignore next */ Object.assign({}, exports.RULESET_MODELS_GROUPED_BY_CLASS),
            search: Object.assign({}, RULESET_MODELS_SEARCH),
        };
    }, [props.filteredElementIds]); // eslint-disable-line react-hooks/exhaustive-deps
    // const rulesets = {
    //   general: (!props.enableElementsClassGrouping) ? RULESET_MODELS : /* istanbul ignore next */ RULESET_MODELS_GROUPED_BY_CLASS,
    //   search: RULESET_MODELS_SEARCH,
    // };
    const { nodeLoader, onItemsRendered } = presentation_components_1.usePresentationTreeNodeLoader({
        imodel: props.iModel,
        dataProvider: props.dataProvider,
        ruleset: rulesets.general,
        appendChildrenCountForGroupingNodes: (props.enableElementsClassGrouping === Common_1.ClassGroupingOption.YesWithCounts),
        pagingSize: PAGING_SIZE,
        enableHierarchyAutoUpdate: props.enableHierarchyAutoUpdate,
    });
    const { nodeLoader: searchNodeLoader, onItemsRendered: onSearchItemsRendered } = presentation_components_1.usePresentationTreeNodeLoader({
        imodel: props.iModel,
        dataProvider: props.dataProvider,
        ruleset: rulesets.search,
        pagingSize: PAGING_SIZE,
        enableHierarchyAutoUpdate: props.enableHierarchyAutoUpdate,
    });
    const activeNodeLoader = ((_a = props.filterInfo) === null || _a === void 0 ? void 0 : _a.filter) ? searchNodeLoader : nodeLoader;
    const activeItemsRenderedCallback = ((_b = props.filterInfo) === null || _b === void 0 ? void 0 : _b.filter) ? onSearchItemsRendered : onItemsRendered;
    const vars = presentation_frontend_1.Presentation.presentation.vars(activeNodeLoader.dataProvider.rulesetId);
    if (props.filteredElementIds) {
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        vars.setId64s("filtered-element-ids", props.filteredElementIds);
    }
    else {
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        vars.unset("filtered-element-ids");
    }
    return {
        nodeLoader: activeNodeLoader,
        onItemsRendered: activeItemsRenderedCallback,
    };
}
function useVisibilityHandler(rulesetId, activeView, visibilityHandler, filteredDataProvider, hierarchyAutoUpdateEnabled) {
    const defaultVisibilityHandler = ui_core_1.useOptionalDisposable(React.useCallback(() => {
        return visibilityHandler ? undefined : createVisibilityHandler(rulesetId, activeView, hierarchyAutoUpdateEnabled);
    }, [visibilityHandler, rulesetId, activeView, hierarchyAutoUpdateEnabled]));
    const handler = visibilityHandler !== null && visibilityHandler !== void 0 ? visibilityHandler : defaultVisibilityHandler;
    React.useEffect(() => {
        handler && handler.setFilteredDataProvider(filteredDataProvider);
    }, [handler, filteredDataProvider]);
    return handler;
}
const createVisibilityHandler = (rulesetId, activeView, hierarchyAutoUpdateEnabled) => {
    // istanbul ignore next
    return activeView ? new ModelsVisibilityHandler_1.ModelsVisibilityHandler({ rulesetId, viewport: activeView, hierarchyAutoUpdateEnabled }) : undefined;
};
const isFilteredDataProvider = (dataProvider) => {
    const filteredProvider = dataProvider;
    return filteredProvider.nodeMatchesFilter !== undefined && filteredProvider.getActiveMatch !== undefined && filteredProvider.countFilteringResults !== undefined;
};
const getFilteredDataProvider = (dataProvider) => {
    return isFilteredDataProvider(dataProvider) ? dataProvider : undefined;
};
//# sourceMappingURL=ModelsTree.js.map