"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module IModelComponents
 */
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelsVisibilityHandler = exports.ModelsTreeNodeType = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const presentation_common_1 = require("@bentley/presentation-common");
const presentation_components_1 = require("@bentley/presentation-components");
const presentation_frontend_1 = require("@bentley/presentation-frontend");
const UiFramework_1 = require("../../UiFramework");
/**
 * Visibility tree node types.
 * @beta
 */
var ModelsTreeNodeType;
(function (ModelsTreeNodeType) {
    ModelsTreeNodeType[ModelsTreeNodeType["Unknown"] = 0] = "Unknown";
    ModelsTreeNodeType[ModelsTreeNodeType["Subject"] = 1] = "Subject";
    ModelsTreeNodeType[ModelsTreeNodeType["Model"] = 2] = "Model";
    ModelsTreeNodeType[ModelsTreeNodeType["Category"] = 3] = "Category";
    ModelsTreeNodeType[ModelsTreeNodeType["Element"] = 4] = "Element";
    ModelsTreeNodeType[ModelsTreeNodeType["Grouping"] = 5] = "Grouping";
})(ModelsTreeNodeType = exports.ModelsTreeNodeType || (exports.ModelsTreeNodeType = {}));
/**
 * Visibility handler used by [[ModelsTree]] to control visibility of the tree items.
 * @alpha
 */
class ModelsVisibilityHandler {
    constructor(props) {
        this._listeners = new Array();
        this.onVisibilityChange = new bentleyjs_core_1.BeEvent();
        // eslint-disable-next-line @typescript-eslint/naming-convention
        this.onViewChanged = (_vp) => {
            this.onVisibilityChangeInternal();
        };
        // eslint-disable-next-line @typescript-eslint/naming-convention
        this.onElementAlwaysDrawnChanged = () => {
            this.onVisibilityChangeInternal();
        };
        // eslint-disable-next-line @typescript-eslint/naming-convention
        this.onElementNeverDrawnChanged = () => {
            this.onVisibilityChangeInternal();
        };
        this._props = props;
        this._subjectModelIdsCache = new SubjectModelIdsCache(this._props.viewport.iModel);
        this._elementIdsCache = new ElementIdsCache(this._props.viewport.iModel, this._props.rulesetId);
        this._listeners.push(this._props.viewport.onViewedCategoriesPerModelChanged.addListener(this.onViewChanged));
        this._listeners.push(this._props.viewport.onViewedCategoriesChanged.addListener(this.onViewChanged));
        this._listeners.push(this._props.viewport.onViewedModelsChanged.addListener(this.onViewChanged));
        this._listeners.push(this._props.viewport.onAlwaysDrawnChanged.addListener(this.onElementAlwaysDrawnChanged));
        this._listeners.push(this._props.viewport.onNeverDrawnChanged.addListener(this.onElementNeverDrawnChanged));
        if (this._props.hierarchyAutoUpdateEnabled) {
            this._listeners.push(presentation_frontend_1.Presentation.presentation.onIModelHierarchyChanged.addListener(/* istanbul ignore next */ () => this._elementIdsCache.clear()));
        }
    }
    dispose() {
        this._listeners.forEach((remove) => remove());
        clearTimeout(this._pendingVisibilityChange);
    }
    /** Sets data provider that is used to get filtered tree hierarchy. */
    setFilteredDataProvider(provider) { this._filteredDataProvider = provider; }
    static getNodeType(item, dataProvider) {
        if (presentation_common_1.NodeKey.isClassGroupingNodeKey(dataProvider.getNodeKey(item)))
            return ModelsTreeNodeType.Grouping;
        if (!item.extendedData)
            return ModelsTreeNodeType.Unknown;
        if (this.isSubjectNode(item))
            return ModelsTreeNodeType.Subject;
        if (this.isModelNode(item))
            return ModelsTreeNodeType.Model;
        if (this.isCategoryNode(item))
            return ModelsTreeNodeType.Category;
        return ModelsTreeNodeType.Element;
    }
    static isSubjectNode(node) {
        return node.extendedData && node.extendedData.isSubject;
    }
    static isModelNode(node) {
        return node.extendedData && node.extendedData.isModel;
    }
    static isCategoryNode(node) {
        return node.extendedData && node.extendedData.isCategory;
    }
    /** Returns visibility status of the tree node. */
    getVisibilityStatus(node, nodeKey) {
        if (presentation_common_1.NodeKey.isClassGroupingNodeKey(nodeKey))
            return this.getElementGroupingNodeDisplayStatus(node.id, nodeKey);
        if (!presentation_common_1.NodeKey.isInstancesNodeKey(nodeKey))
            return { state: "hidden", isDisabled: true };
        if (ModelsVisibilityHandler.isSubjectNode(node)) {
            // note: subject nodes may be merged to represent multiple subject instances
            return this.getSubjectNodeVisibility(nodeKey.instanceKeys.map((key) => key.id), node);
        }
        if (ModelsVisibilityHandler.isModelNode(node))
            return this.getModelDisplayStatus(nodeKey.instanceKeys[0].id);
        if (ModelsVisibilityHandler.isCategoryNode(node))
            return this.getCategoryDisplayStatus(nodeKey.instanceKeys[0].id, this.getCategoryParentModelId(node));
        return this.getElementDisplayStatus(nodeKey.instanceKeys[0].id, this.getElementModelId(node), this.getElementCategoryId(node));
    }
    /** Changes visibility of the items represented by the tree node. */
    async changeVisibility(node, nodeKey, on) {
        if (presentation_common_1.NodeKey.isClassGroupingNodeKey(nodeKey)) {
            await this.changeElementGroupingNodeState(nodeKey, on);
            return;
        }
        if (!presentation_common_1.NodeKey.isInstancesNodeKey(nodeKey))
            return;
        if (ModelsVisibilityHandler.isSubjectNode(node)) {
            await this.changeSubjectNodeState(nodeKey.instanceKeys.map((key) => key.id), node, on);
        }
        else if (ModelsVisibilityHandler.isModelNode(node)) {
            await this.changeModelState(nodeKey.instanceKeys[0].id, on);
        }
        else if (ModelsVisibilityHandler.isCategoryNode(node)) {
            this.changeCategoryState(nodeKey.instanceKeys[0].id, this.getCategoryParentModelId(node), on);
        }
        else {
            await this.changeElementState(nodeKey.instanceKeys[0].id, this.getElementModelId(node), this.getElementCategoryId(node), on);
        }
    }
    async getSubjectNodeVisibility(ids, node) {
        if (!this._props.viewport.view.isSpatialView())
            return { isDisabled: true, state: "hidden", tooltip: createTooltip("disabled", "subject.nonSpatialView") };
        if (this._filteredDataProvider)
            return this.getFilteredSubjectDisplayStatus(this._filteredDataProvider, ids, node);
        return this.getSubjectDisplayStatus(ids);
    }
    async getSubjectDisplayStatus(ids) {
        const modelIds = await this.getSubjectModelIds(ids);
        const isDisplayed = modelIds.some((modelId) => this.getModelDisplayStatus(modelId).state === "visible");
        if (isDisplayed)
            return { state: "visible", tooltip: createTooltip("visible", "subject.atLeastOneModelVisible") };
        return { state: "hidden", tooltip: createTooltip("hidden", "subject.allModelsHidden") };
    }
    async getFilteredSubjectDisplayStatus(provider, ids, node) {
        if (provider.nodeMatchesFilter(node))
            return this.getSubjectDisplayStatus(ids);
        const children = await provider.getNodes(node);
        const childrenDisplayStatuses = await Promise.all(children.map((childNode) => this.getVisibilityStatus(childNode, provider.getNodeKey(childNode))));
        if (childrenDisplayStatuses.some((status) => status.state === "visible"))
            return { state: "visible", tooltip: createTooltip("visible", "subject.atLeastOneModelVisible") };
        return { state: "hidden", tooltip: createTooltip("hidden", "subject.allModelsHidden") };
    }
    getModelDisplayStatus(id) {
        if (!this._props.viewport.view.isSpatialView())
            return { isDisabled: true, state: "hidden", tooltip: createTooltip("disabled", "model.nonSpatialView") };
        const isDisplayed = this._props.viewport.view.viewsModel(id);
        return { state: isDisplayed ? "visible" : "hidden", tooltip: createTooltip(isDisplayed ? "visible" : "hidden", undefined) };
    }
    getCategoryDisplayStatus(id, parentModelId) {
        if (parentModelId) {
            if (this.getModelDisplayStatus(parentModelId).state === "hidden")
                return { state: "hidden", isDisabled: true, tooltip: createTooltip("disabled", "category.modelNotDisplayed") };
            const override = this._props.viewport.perModelCategoryVisibility.getOverride(parentModelId, id);
            switch (override) {
                case imodeljs_frontend_1.PerModelCategoryVisibility.Override.Show:
                    return { state: "visible", tooltip: createTooltip("visible", "category.displayedThroughPerModelOverride") };
                case imodeljs_frontend_1.PerModelCategoryVisibility.Override.Hide:
                    return { state: "hidden", tooltip: createTooltip("hidden", "category.hiddenThroughPerModelOverride") };
            }
        }
        const isDisplayed = this._props.viewport.view.viewsCategory(id);
        return {
            state: isDisplayed ? "visible" : "hidden",
            tooltip: isDisplayed
                ? createTooltip("visible", "category.displayedThroughCategorySelector")
                : createTooltip("hidden", "category.hiddenThroughCategorySelector"),
        };
    }
    async getElementGroupingNodeDisplayStatus(_id, key) {
        const { modelId, categoryId, elementIds } = await this.getGroupedElementIds(key);
        if (!modelId || !this._props.viewport.view.viewsModel(modelId))
            return { isDisabled: true, state: "hidden", tooltip: createTooltip("disabled", "element.modelNotDisplayed") };
        const atLeastOneElementForceDisplayed = (this._props.viewport.alwaysDrawn !== undefined)
            && elementIds.some((elementId) => this._props.viewport.alwaysDrawn.has(elementId));
        if (atLeastOneElementForceDisplayed)
            return { state: "visible", tooltip: createTooltip("visible", "element.displayedThroughAlwaysDrawnList") };
        if (this._props.viewport.alwaysDrawn !== undefined && this._props.viewport.alwaysDrawn.size !== 0 && this._props.viewport.isAlwaysDrawnExclusive)
            return { state: "hidden", tooltip: createTooltip("hidden", "element.hiddenDueToOtherElementsExclusivelyAlwaysDrawn") };
        const allElementsForceHidden = (this._props.viewport.neverDrawn !== undefined)
            && elementIds.every((elementId) => this._props.viewport.neverDrawn.has(elementId));
        if (allElementsForceHidden)
            return { state: "hidden", tooltip: createTooltip("visible", "element.hiddenThroughNeverDrawnList") };
        if (categoryId && this.getCategoryDisplayStatus(categoryId, modelId).state === "visible")
            return { state: "visible", tooltip: createTooltip("visible", undefined) };
        return { state: "hidden", tooltip: createTooltip("hidden", "element.hiddenThroughCategory") };
    }
    getElementDisplayStatus(elementId, modelId, categoryId) {
        if (!modelId || !this._props.viewport.view.viewsModel(modelId))
            return { isDisabled: true, state: "hidden", tooltip: createTooltip("disabled", "element.modelNotDisplayed") };
        if (this._props.viewport.neverDrawn !== undefined && this._props.viewport.neverDrawn.has(elementId))
            return { state: "hidden", tooltip: createTooltip("hidden", "element.hiddenThroughNeverDrawnList") };
        if (this._props.viewport.alwaysDrawn !== undefined) {
            if (this._props.viewport.alwaysDrawn.has(elementId))
                return { state: "visible", tooltip: createTooltip("visible", "element.displayedThroughAlwaysDrawnList") };
            if (this._props.viewport.alwaysDrawn.size !== 0 && this._props.viewport.isAlwaysDrawnExclusive)
                return { state: "hidden", tooltip: createTooltip("hidden", "element.hiddenDueToOtherElementsExclusivelyAlwaysDrawn") };
        }
        if (categoryId && this.getCategoryDisplayStatus(categoryId, modelId).state === "visible")
            return { state: "visible", tooltip: createTooltip("visible", undefined) };
        return { state: "hidden", tooltip: createTooltip("hidden", "element.hiddenThroughCategory") };
    }
    async changeSubjectNodeState(ids, node, on) {
        if (!this._props.viewport.view.isSpatialView())
            return;
        if (this._filteredDataProvider)
            return this.changeFilteredSubjectState(this._filteredDataProvider, ids, node, on);
        return this.changeSubjectState(ids, on);
    }
    async changeFilteredSubjectState(provider, ids, node, on) {
        if (provider.nodeMatchesFilter(node))
            return this.changeSubjectState(ids, on);
        const children = await provider.getNodes(node);
        return Promise.all(children.map(async (childNode) => this.changeVisibility(childNode, provider.getNodeKey(childNode), on)));
    }
    async changeSubjectState(ids, on) {
        const modelIds = await this.getSubjectModelIds(ids);
        return this.changeModelsVisibility(modelIds, on);
    }
    async changeModelState(id, on) {
        if (!this._props.viewport.view.isSpatialView())
            return;
        return this.changeModelsVisibility([id], on);
    }
    async changeModelsVisibility(ids, visible) {
        if (visible)
            return this._props.viewport.addViewedModels(ids);
        else
            this._props.viewport.changeModelDisplay(ids, false);
    }
    changeCategoryState(categoryId, parentModelId, on) {
        if (parentModelId) {
            const isDisplayedInSelector = this._props.viewport.view.viewsCategory(categoryId);
            const ovr = (on === isDisplayedInSelector) ? imodeljs_frontend_1.PerModelCategoryVisibility.Override.None
                : on ? imodeljs_frontend_1.PerModelCategoryVisibility.Override.Show : imodeljs_frontend_1.PerModelCategoryVisibility.Override.Hide;
            this._props.viewport.perModelCategoryVisibility.setOverride(parentModelId, categoryId, ovr);
            if (ovr === imodeljs_frontend_1.PerModelCategoryVisibility.Override.None && on) {
                // we took off the override which means the category is displayed in selector, but
                // doesn't mean all its subcategories are displayed - this call ensures that
                this._props.viewport.changeCategoryDisplay([categoryId], true, true);
            }
            return;
        }
        this._props.viewport.changeCategoryDisplay([categoryId], on, on ? true : false);
    }
    async changeElementGroupingNodeState(key, on) {
        const { modelId, categoryId, elementIds } = await this.getGroupedElementIds(key);
        this.changeElementsState(modelId, categoryId, elementIds, on);
    }
    async changeElementState(id, modelId, categoryId, on) {
        this.changeElementsState(modelId, categoryId, [id, ...await this.getAssemblyElementIds(id)], on);
    }
    changeElementsState(modelId, categoryId, elementIds, on) {
        const isDisplayedByDefault = modelId && this.getModelDisplayStatus(modelId).state === "visible"
            && categoryId && this.getCategoryDisplayStatus(categoryId, modelId).state === "visible";
        const isHiddenDueToExclusiveAlwaysDrawnElements = this._props.viewport.isAlwaysDrawnExclusive && this._props.viewport.alwaysDrawn && 0 !== this._props.viewport.alwaysDrawn.size;
        const currNeverDrawn = new Set(this._props.viewport.neverDrawn ? this._props.viewport.neverDrawn : []);
        const currAlwaysDrawn = new Set(this._props.viewport.alwaysDrawn ?
            this._props.viewport.alwaysDrawn : /* istanbul ignore next */ []);
        elementIds.forEach((elementId) => {
            if (on) {
                currNeverDrawn.delete(elementId);
                if (!isDisplayedByDefault || isHiddenDueToExclusiveAlwaysDrawnElements)
                    currAlwaysDrawn.add(elementId);
            }
            else {
                currAlwaysDrawn.delete(elementId);
                if (isDisplayedByDefault && !isHiddenDueToExclusiveAlwaysDrawnElements)
                    currNeverDrawn.add(elementId);
            }
        });
        this._props.viewport.setNeverDrawn(currNeverDrawn);
        this._props.viewport.setAlwaysDrawn(currAlwaysDrawn, this._props.viewport.isAlwaysDrawnExclusive);
    }
    onVisibilityChangeInternal() {
        if (this._pendingVisibilityChange)
            return;
        this._pendingVisibilityChange = setTimeout(() => {
            this.onVisibilityChange.raiseEvent();
            this._pendingVisibilityChange = undefined;
        }, 0);
    }
    getCategoryParentModelId(categoryNode) {
        return categoryNode.extendedData ? categoryNode.extendedData.modelId : /* istanbul ignore next */ undefined;
    }
    getElementModelId(elementNode) {
        return elementNode.extendedData ? elementNode.extendedData.modelId : /* istanbul ignore next */ undefined;
    }
    getElementCategoryId(elementNode) {
        return elementNode.extendedData ? elementNode.extendedData.categoryId : /* istanbul ignore next */ undefined;
    }
    async getSubjectModelIds(subjectIds) {
        return (await Promise.all(subjectIds.map(async (id) => this._subjectModelIdsCache.getSubjectModelIds(id))))
            .reduce((allModelIds, curr) => [...allModelIds, ...curr], []);
    }
    // istanbul ignore next
    async getAssemblyElementIds(assemblyId) {
        return this._elementIdsCache.getAssemblyElementIds(assemblyId);
    }
    // istanbul ignore next
    async getGroupedElementIds(groupingNodeKey) {
        return this._elementIdsCache.getGroupedElementIds(groupingNodeKey);
    }
}
exports.ModelsVisibilityHandler = ModelsVisibilityHandler;
class SubjectModelIdsCache {
    constructor(imodel) {
        this._imodel = imodel;
    }
    async initSubjectModels() {
        var e_1, _a, e_2, _b;
        var _c;
        const querySubjects = () => {
            const subjectsQuery = `
        SELECT ECInstanceId id, Parent.Id parentId, json_extract(JsonProperties, '$.Subject.Model.TargetPartition') targetPartitionId
        FROM bis.Subject
      `;
            return this._imodel.query(subjectsQuery);
        };
        const queryModels = () => {
            const modelsQuery = `
        SELECT p.ECInstanceId id, p.Parent.Id parentId, json_extract(p.JsonProperties, '$.PhysicalPartition.Model.Content') content
        FROM bis.InformationPartitionElement p
        INNER JOIN bis.GeometricModel3d m ON m.ModeledElement.Id = p.ECInstanceId
        WHERE NOT m.IsPrivate
      `;
            return this._imodel.query(modelsQuery);
        };
        function pushToMap(map, key, value) {
            let list = map.get(key);
            if (!list) {
                list = [];
                map.set(key, list);
            }
            list.push(value);
        }
        this._subjectsHierarchy = new Map();
        const targetPartitionSubjects = new Map();
        try {
            for (var _d = __asyncValues(querySubjects()), _e; _e = await _d.next(), !_e.done;) {
                const subject = _e.value;
                // istanbul ignore else
                if (subject.parentId)
                    pushToMap(this._subjectsHierarchy, subject.parentId, subject.id);
                // istanbul ignore if
                if (subject.targetPartitionId)
                    pushToMap(targetPartitionSubjects, subject.targetPartitionId, subject.id);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d.return)) await _a.call(_d);
            }
            finally { if (e_1) throw e_1.error; }
        }
        this._subjectModels = new Map();
        try {
            for (var _f = __asyncValues(queryModels()), _g; _g = await _f.next(), !_g.done;) {
                const model = _g.value;
                // istanbul ignore next
                const subjectIds = (_c = targetPartitionSubjects.get(model.id)) !== null && _c !== void 0 ? _c : [];
                // istanbul ignore else
                if (!subjectIds.includes(model.parentId))
                    subjectIds.push(model.parentId);
                const v = { id: model.id, isHidden: (model.content !== undefined) };
                subjectIds.forEach((subjectId) => {
                    pushToMap(this._subjectModels, subjectId, v);
                });
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_g && !_g.done && (_b = _f.return)) await _b.call(_f);
            }
            finally { if (e_2) throw e_2.error; }
        }
    }
    async initCache() {
        if (!this._init) {
            this._init = this.initSubjectModels().then(() => { });
        }
        return this._init;
    }
    appendSubjectModelsRecursively(modelIds, subjectId) {
        const subjectModelIds = this._subjectModels.get(subjectId);
        if (subjectModelIds)
            modelIds.push(...subjectModelIds.map((info) => info.id));
        const childSubjectIds = this._subjectsHierarchy.get(subjectId);
        if (childSubjectIds)
            childSubjectIds.forEach((cs) => this.appendSubjectModelsRecursively(modelIds, cs));
    }
    async getSubjectModelIds(subjectId) {
        await this.initCache();
        const modelIds = new Array();
        this.appendSubjectModelsRecursively(modelIds, subjectId);
        return modelIds;
    }
}
// istanbul ignore next
class RulesetDrivenIdsProvider extends presentation_components_1.ContentDataProvider {
    constructor(imodel, rulesetId, displayType, inputKeys) {
        super({ imodel, ruleset: rulesetId, displayType });
        this.keys = new presentation_common_1.KeySet(inputKeys);
    }
    shouldConfigureContentDescriptor() { return false; }
    getDescriptorOverrides() {
        return {
            displayType: this.displayType,
            contentFlags: presentation_common_1.ContentFlags.KeysOnly,
            hiddenFieldNames: [],
        };
    }
    async getResultIds() {
        const content = await this.getContent();
        if (!content)
            return [];
        const result = [];
        content.contentSet.forEach((item) => {
            result.push(...item.primaryKeys.map((k) => k.id));
        });
        return result;
    }
}
// istanbul ignore next
class ElementIdsCache {
    constructor(_imodel, _rulesetId) {
        this._imodel = _imodel;
        this._rulesetId = _rulesetId;
        this._assemblyElementIdsCache = new Map();
        this._groupedElementIdsCache = new Map();
    }
    clear() {
        this._assemblyElementIdsCache.clear();
        this._groupedElementIdsCache.clear();
    }
    async getAssemblyElementIds(assemblyId) {
        const ids = this._assemblyElementIdsCache.get(assemblyId);
        if (ids)
            return ids;
        const idsProvider = new AssemblyElementIdsProvider(this._imodel, this._rulesetId, assemblyId);
        const newIds = await idsProvider.getElementIds();
        this._assemblyElementIdsCache.set(assemblyId, newIds);
        return newIds;
    }
    async getGroupedElementIds(groupingNodeKey) {
        const keyString = JSON.stringify(groupingNodeKey);
        const ids = this._groupedElementIdsCache.get(keyString);
        if (ids)
            return ids;
        const idsProvider = new GroupedElementIdsProvider(this._imodel, this._rulesetId, groupingNodeKey);
        const newIds = await idsProvider.getElementIds();
        this._groupedElementIdsCache.set(keyString, newIds);
        return newIds;
    }
}
// istanbul ignore next
class AssemblyElementIdsProvider extends RulesetDrivenIdsProvider {
    constructor(imodel, rulesetId, assemblyId) {
        super(imodel, rulesetId, "AssemblyElementsRequest", [{ className: "BisCore:Element", id: assemblyId }]);
    }
    async getElementIds() {
        return this.getResultIds();
    }
}
// istanbul ignore next
class GroupedElementIdsProvider extends RulesetDrivenIdsProvider {
    constructor(imodel, rulesetId, groupingNodeKey) {
        super(imodel, rulesetId, "AssemblyElementsRequest", [groupingNodeKey]);
    }
    async getElementIds() {
        var e_3, _a;
        const elementIds = await this.getResultIds();
        let modelId, categoryId;
        const query = `SELECT Model.Id AS modelId, Category.Id AS categoryId FROM bis.GeometricElement3d WHERE ECInstanceId = ? LIMIT 1`;
        try {
            for (var _b = __asyncValues(this.imodel.query(query, [elementIds[0]])), _c; _c = await _b.next(), !_c.done;) {
                const modelAndCategoryIds = _c.value;
                modelId = modelAndCategoryIds.modelId;
                categoryId = modelAndCategoryIds.categoryId;
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return { modelId, categoryId, elementIds };
    }
}
const createTooltip = (status, tooltipStringId) => {
    const statusStringId = `UiFramework:modelTree.status.${status}`;
    const statusString = UiFramework_1.UiFramework.i18n.translate(statusStringId);
    if (!tooltipStringId)
        return statusString;
    tooltipStringId = `UiFramework:modelTree.tooltips.${tooltipStringId}`;
    const tooltipString = UiFramework_1.UiFramework.i18n.translate(tooltipStringId);
    return `${statusString}: ${tooltipString}`;
};
//# sourceMappingURL=ModelsVisibilityHandler.js.map