"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module KeyboardShortcut
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyboardShortcutManager = exports.KeyboardShortcutContainer = exports.KeyboardShortcut = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const CursorInformation_1 = require("../cursor/CursorInformation");
const ItemDefBase_1 = require("../shared/ItemDefBase");
const UiFramework_1 = require("../UiFramework");
const KeyboardShortcutMenu_1 = require("./KeyboardShortcutMenu");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
/** Keyboard Shortcut used to execute an action
 * @public
 */
class KeyboardShortcut extends ItemDefBase_1.ItemDefBase {
    constructor(props) {
        super(props);
        this._isAltKeyRequired = false;
        this._isCtrlKeyRequired = false;
        this._isShiftKeyRequired = false;
        this._isFunctionKey = false;
        this._isSpecialKey = false;
        this._key = props.key;
        if (this._key in ui_abstract_1.FunctionKey)
            this._isFunctionKey = true;
        if (this._key in ui_abstract_1.SpecialKey)
            this._isSpecialKey = true;
        this._shortcuts = new KeyboardShortcutContainer();
        if (props.item) {
            this._item = props.item;
            // Copy over icon, label & tooltip from the item
            if (!this.iconSpec)
                this.iconSpec = this._item.iconSpec;
            if (!this.label)
                this.setLabel(this._item.label);
            if (!this.tooltip)
                this.setTooltip(this._item.tooltip);
            if (this.isDisabled === undefined)
                this.isDisabled = this._item.isDisabled;
            if (this.isHidden === undefined)
                this.isHidden = this._item.isHidden;
        }
        else if (props.shortcuts) {
            props.shortcuts.forEach((childProps) => {
                const shortcut = new KeyboardShortcut(childProps);
                this._shortcuts.registerKey(shortcut.keyMapKey, shortcut);
            });
        }
        else {
            throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `Either 'item' or 'shortcuts' must be specified for '${props.key}' key.`);
        }
        if (props.isAltKeyRequired !== undefined)
            this._isAltKeyRequired = props.isAltKeyRequired;
        if (props.isCtrlKeyRequired !== undefined)
            this._isCtrlKeyRequired = props.isCtrlKeyRequired;
        if (props.isShiftKeyRequired !== undefined)
            this._isShiftKeyRequired = props.isShiftKeyRequired;
    }
    /** Returns the id for this shortcut */
    get id() { return this.keyMapKey; }
    /** Returns the shortcut container */
    get shortcutContainer() {
        return this._shortcuts;
    }
    /** Finds a shortcut with a given key in the shortcut's container */
    getShortcut(mapKey) {
        return this._shortcuts.findKey(mapKey);
    }
    /** Returns the shortcut's key map key used as the id */
    get keyMapKey() {
        const keyMapKey = KeyboardShortcutContainer.generateKeyMapKey(this.key, this._isAltKeyRequired, this._isCtrlKeyRequired, this._isShiftKeyRequired);
        return keyMapKey;
    }
    /** Returns the [[ActionButtonItemDef]] associated with this shortcut */
    get item() {
        return this._item;
    }
    /** Called when the [[ActionButtonItemDef]] associated with this shortcut is invoked */
    itemPicked() {
        if (this._shortcuts.areKeyboardShortcutsAvailable()) {
            this._shortcuts.showShortcutsMenu();
        }
        else {
            setTimeout(() => {
                // istanbul ignore else
                if (this._item)
                    this._item.execute();
            });
        }
    }
    /** Gets the keyboard key */
    get key() { return this._key; }
    /** Gets whether the Alt key required. */
    get isAltKeyRequired() { return this._isAltKeyRequired; }
    /** Gets whether the Ctrl key required. */
    get isCtrlKeyRequired() { return this._isCtrlKeyRequired; }
    /** Gets whether the Shift key required. */
    get isShiftKeyRequired() { return this._isShiftKeyRequired; }
    /** Gets whether this is a Function key. */
    get isFunctionKey() { return this._isFunctionKey; }
    /** Gets whether this is a Special key. */
    get isSpecialKey() { return this._isSpecialKey; }
}
exports.KeyboardShortcut = KeyboardShortcut;
/** Keyboard Shortcut Container
 * @public
 */
class KeyboardShortcutContainer {
    constructor() {
        this._keyMap = new Map();
        this._keyArray = new Array();
    }
    /** Registers a Keyboard Shortcut associated with a given key in the managed list */
    registerKey(keyMapKey, inShortcut) {
        let shortcut;
        if ((shortcut = this.findKey(keyMapKey)) === undefined) {
            shortcut = inShortcut;
            this._keyArray.push(shortcut);
        }
        else {
            const index = this._keyArray.findIndex((value) => {
                return value.keyMapKey === keyMapKey;
            });
            // istanbul ignore else
            if (index >= 0) {
                shortcut = inShortcut;
                this._keyArray[index] = shortcut;
            }
        }
        // istanbul ignore else
        if (shortcut)
            this._keyMap.set(keyMapKey, shortcut);
        return shortcut;
    }
    /** Finds a Keyboard Shortcut associated with a given key */
    findKey(keyMapKey) {
        return this._keyMap.get(keyMapKey);
    }
    /** Determines if any Keyboard Shortcuts are available in this container */
    areKeyboardShortcutsAvailable() {
        return this._keyMap.size !== 0;
    }
    /** Empties any Keyboard Shortcuts from this container */
    emptyData() {
        this._keyMap.clear();
        this._keyArray.length = 0;
    }
    getAvailableKeyboardShortcuts() {
        return this._keyArray.slice();
    }
    /** Generates a key used for storing and finding the Keyboard Shortcuts in this container */
    static generateKeyMapKey(keyboardKey, isAltKeyRequired, isCtrlKeyRequired, isShiftKeyRequired) {
        let keyMapKey = keyboardKey;
        if (isAltKeyRequired)
            keyMapKey = `Alt+${keyMapKey}`;
        if (isShiftKeyRequired)
            keyMapKey = `Shift+${keyMapKey}`;
        if (isCtrlKeyRequired)
            keyMapKey = `Ctrl+${keyMapKey}`;
        return keyMapKey;
    }
    /** Displays a menu for the Keyboard Shortcuts in this container */
    showShortcutsMenu() {
        const offset = 8;
        KeyboardShortcutMenu_1.KeyboardShortcutMenu.onKeyboardShortcutMenuEvent.emit({
            menuVisible: true,
            menuX: KeyboardShortcutManager.cursorX - offset,
            menuY: KeyboardShortcutManager.cursorY - offset,
            shortcuts: this.getAvailableKeyboardShortcuts(),
        });
    }
}
exports.KeyboardShortcutContainer = KeyboardShortcutContainer;
/** Keyboard Shortcut Manager
 * @public
 */
class KeyboardShortcutManager {
    /** Initialize the Keyboard Shortcut manager */
    static initialize() {
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.onSyncUiEvent.addListener(KeyboardShortcutManager._handleSyncUiEvent);
    }
    /** Loads Keyboard Shortcuts into the managed list */
    static loadKeyboardShortcuts(shortcutList) {
        shortcutList.forEach((shortcutProps) => {
            this.loadKeyboardShortcut(shortcutProps);
        });
    }
    /** Loads a Keyboard Shortcut into the managed list */
    static loadKeyboardShortcut(shortcutProps) {
        const shortcut = new KeyboardShortcut(shortcutProps);
        this._shortcuts.registerKey(shortcut.keyMapKey, shortcut);
    }
    /** Processes a keystroke and invokes a matching Keyboard Shortcut */
    static processKey(keyboardKey, isAltKeyPressed = false, isCtrlKeyPressed = false, isShiftKeyPressed = false) {
        const keyMapKey = KeyboardShortcutContainer.generateKeyMapKey(keyboardKey, isAltKeyPressed, isCtrlKeyPressed, isShiftKeyPressed);
        const shortcut = this.getShortcut(keyMapKey);
        if (shortcut) {
            shortcut.itemPicked();
            return true;
        }
        return false;
    }
    /** Returns the managed list of Keyboard Shortcuts */
    static get shortcutContainer() {
        return this._shortcuts;
    }
    /** Returns a Keyboard Shortcut from the managed lists */
    static getShortcut(keyMapKey) {
        return this._shortcuts.findKey(keyMapKey);
    }
    /** Determines if focus is set to Home */
    static get isFocusOnHome() {
        const element = document.activeElement;
        return element && element === document.body;
    }
    /** Sets focus to Home */
    static setFocusToHome() {
        const element = document.activeElement;
        if (element && element !== document.body) {
            element.blur();
            document.body.focus();
        }
    }
    /** Displays the Keyboard Shortcuts menu at the cursor */
    static displayShortcutsMenu() {
        if (this._shortcuts.areKeyboardShortcutsAvailable()) {
            this._shortcuts.showShortcutsMenu();
        }
    }
    /** Closes the Keyboard Shortcuts menu */
    static closeShortcutsMenu() {
        KeyboardShortcutMenu_1.KeyboardShortcutMenu.onKeyboardShortcutMenuEvent.emit({
            menuVisible: false,
            menuX: 0,
            menuY: 0,
            shortcuts: undefined,
        });
    }
    /** Returns the cursor X position, which is mouseEvent.pageX. */
    static get cursorX() { return CursorInformation_1.CursorInformation.cursorX; }
    /** Returns the cursor Y position, which is mouseEvent.pageY. */
    static get cursorY() { return CursorInformation_1.CursorInformation.cursorY; }
}
exports.KeyboardShortcutManager = KeyboardShortcutManager;
KeyboardShortcutManager._shortcuts = new KeyboardShortcutContainer();
KeyboardShortcutManager._handleSyncUiEvent = (args) => {
    const updateBooleanValue = (booleanValue) => {
        if (SyncUiEventDispatcher_1.SyncUiEventDispatcher.hasEventOfInterest(args.eventIds, booleanValue.syncEventIds))
            booleanValue.refresh();
    };
    const handleForSyncIds = (shortcut) => {
        if (shortcut.isDisabled instanceof ui_abstract_1.ConditionalBooleanValue)
            updateBooleanValue(shortcut.isDisabled);
        if (shortcut.isHidden instanceof ui_abstract_1.ConditionalBooleanValue)
            updateBooleanValue(shortcut.isHidden);
    };
    KeyboardShortcutManager._traverseShortcuts(KeyboardShortcutManager._shortcuts.getAvailableKeyboardShortcuts(), handleForSyncIds);
};
KeyboardShortcutManager._traverseShortcuts = (shortcuts, callback) => {
    shortcuts.forEach((shortcut) => {
        callback(shortcut);
        if (shortcut.shortcutContainer.areKeyboardShortcutsAvailable()) {
            const childShortcuts = shortcut.shortcutContainer.getAvailableKeyboardShortcuts();
            KeyboardShortcutManager._traverseShortcuts(childShortcuts, callback);
        }
    });
};
//# sourceMappingURL=KeyboardShortcut.js.map