"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module KeyboardShortcut
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyboardShortcutMenu = exports.KeyboardShortcutMenuEvent = void 0;
const React = require("react");
const ui_core_1 = require("@bentley/ui-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
/** KeyboardShortcut Menu Event class.
 * @public
 */
class KeyboardShortcutMenuEvent extends ui_core_1.UiEvent {
}
exports.KeyboardShortcutMenuEvent = KeyboardShortcutMenuEvent;
/** React component that displays a context menu at the cursor containing keyboard shortcuts.
 * @public
 */
class KeyboardShortcutMenu extends React.PureComponent {
    constructor() {
        super(...arguments);
        /** @internal */
        this.state = {
            menuVisible: false,
            menuX: 0,
            menuY: 0,
        };
        this._handleKeyboardShortcutMenuEvent = (state) => {
            this.setState(state);
        };
        this._hideContextMenu = () => {
            this.setState({ menuVisible: false, shortcuts: undefined });
        };
        this._itemPicked = (shortcut) => {
            this._hideContextMenu();
            shortcut.itemPicked();
        };
    }
    componentDidMount() {
        KeyboardShortcutMenu.onKeyboardShortcutMenuEvent.addListener(this._handleKeyboardShortcutMenuEvent);
    }
    componentWillUnmount() {
        KeyboardShortcutMenu.onKeyboardShortcutMenuEvent.removeListener(this._handleKeyboardShortcutMenuEvent);
    }
    render() {
        const { shortcuts, menuX, menuY, menuVisible } = this.state;
        const onClose = this._hideContextMenu;
        if (menuVisible) {
            const items = this.getShortcutMenuItems(shortcuts);
            return (React.createElement(ui_core_1.GlobalContextMenu, { className: this.props.className, style: this.props.style, identifier: "keyboard-shortcuts", x: menuX, y: menuY, opened: menuVisible, onEsc: onClose, onOutsideClick: onClose, edgeLimit: false, autoflip: true, ignoreNextKeyUp: true }, items));
        }
        return null;
    }
    getShortcutMenuItems(shortcuts) {
        const items = [];
        // istanbul ignore else
        if (shortcuts) {
            shortcuts.forEach((shortcut, index) => {
                const item = this.getShortcutMenuItem(shortcut, index);
                // istanbul ignore else
                if (item)
                    items.push(item);
            });
        }
        return items;
    }
    getShortcutMenuItem(shortcut, index) {
        const shortcutKey = shortcut.key;
        const isHidden = ui_abstract_1.ConditionalBooleanValue.getValue(shortcut.isHidden);
        // Only pure characters go into the context menu
        if (shortcutKey !== shortcut.keyMapKey || shortcut.isFunctionKey || shortcut.isSpecialKey || isHidden)
            return null;
        let node = null;
        let label = shortcut.label;
        const iconSpec = shortcut.iconSpec;
        label = `~${shortcutKey.toLocaleUpperCase()} ${label}`;
        if (shortcut.shortcutContainer.areKeyboardShortcutsAvailable()) {
            const shortcuts = shortcut.shortcutContainer.getAvailableKeyboardShortcuts();
            const items = this.getShortcutMenuItems(shortcuts);
            node = (React.createElement(ui_core_1.ContextSubMenu, { key: index, icon: iconSpec, label: label, disabled: shortcut.isDisabled }, items));
        }
        else {
            const sel = () => this._itemPicked(shortcut);
            node = (React.createElement(ui_core_1.ContextMenuItem, { key: index, onSelect: sel, icon: iconSpec, disabled: shortcut.isDisabled }, label));
        }
        return node;
    }
}
exports.KeyboardShortcutMenu = KeyboardShortcutMenu;
/** Get KeyboardShortcut Menu Event. */
KeyboardShortcutMenu.onKeyboardShortcutMenuEvent = new KeyboardShortcutMenuEvent();
//# sourceMappingURL=KeyboardShortcutMenu.js.map