"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Item
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyinBrowser = void 0;
require("./KeyinBrowser.scss");
// cSpell:ignore Modeless keyins keyinbrowser testid
const React = require("react");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_core_1 = require("@bentley/ui-core");
const UiFramework_1 = require("../UiFramework");
const ui_abstract_1 = require("@bentley/ui-abstract");
/**
 * Component used to allow users to select, provide arguments, and execute a key-in.
 * @beta
 */
class KeyinBrowser extends React.PureComponent {
    /** @internal */
    constructor(props) {
        super(props);
        this._toolIdLabel = UiFramework_1.UiFramework.translate("keyinbrowser.keyin");
        this._argsLabel = UiFramework_1.UiFramework.translate("keyinbrowser.args");
        this._argsTip = UiFramework_1.UiFramework.translate("keyinbrowser.argsTip");
        this._executeLabel = UiFramework_1.UiFramework.translate("keyinbrowser.execute");
        this._suggestPlaceholder = UiFramework_1.UiFramework.translate("keyinbrowser.placeholder");
        this._toolIdKey = "keyinbrowser:keyin";
        this._isMounted = false;
        this._onClick = () => {
            this._execute();
        };
        this._outputMessage = (msg) => {
            const details = new imodeljs_frontend_1.NotifyMessageDetails(imodeljs_frontend_1.OutputMessagePriority.Error, msg, undefined, imodeljs_frontend_1.OutputMessageType.Alert);
            imodeljs_frontend_1.IModelApp.notifications.outputMessage(details);
        };
        // istanbul ignore next
        this._onKeyinSelected = (selected) => {
            const currentToolId = selected.value;
            const currentArgs = this.getToolArgs(currentToolId);
            // istanbul ignore else
            if (this._isMounted)
                this.setState({ currentToolId, currentArgs });
        };
        this._onArgumentsChange = (event) => {
            // istanbul ignore else
            if (this._isMounted)
                this.setState({ currentArgs: event.target.value });
        };
        this._onKeyDown = (event) => {
            if (ui_abstract_1.SpecialKey.Enter === event.key) {
                event.stopPropagation();
                this._execute();
                return;
            }
            // istanbul ignore else
            if (ui_abstract_1.SpecialKey.Escape === event.key) {
                // istanbul ignore else
                if (this.props.onCancel)
                    this.props.onCancel();
                return;
            }
        };
        this._onInputFocus = (event) => {
            // istanbul ignore else
            if (event.target) {
                event.target.select();
            }
        };
        this._onAutoSuggestEnter = (event) => {
            event.stopPropagation();
            const inputValue = event.target.value;
            // istanbul ignore else
            if (this._processInputValue(inputValue)) {
                setTimeout(() => {
                    this._execute();
                });
            }
        };
        this._onAutoSuggestTab = (event) => {
            event.stopPropagation();
            const inputValue = event.target.value;
            this._processInputValue(inputValue);
        };
        this._onAutoSuggestEscape = (event) => {
            event.stopPropagation();
            // istanbul ignore else
            if (this.props.onCancel)
                this.props.onCancel();
        };
        /** Calculate suggestions for any given input value. */
        this._getSuggestions = async (value) => {
            const inputValue = value.trim().toLowerCase();
            const inputLength = inputValue.length;
            return Promise.resolve(inputLength === 0 ?
                /* istanbul ignore next */ [] :
                this.state.keyins.filter((data) => {
                    return data.label.toLowerCase().includes(inputValue) || data.englishKeyin.toLowerCase().includes(inputValue);
                }));
        };
        const currentToolId = this.getSavedToolId();
        const currentArgs = this.getToolArgs(currentToolId);
        this.state = {
            keyins: [],
            currentToolId,
            currentArgs,
        };
    }
    /** @internal */
    componentDidMount() {
        this._isMounted = true;
        let keyins = this.getToolKeyinMap();
        this.setState({ keyins });
        setTimeout(() => {
            keyins = keyins.sort((a, b) => a.label.localeCompare(b.label));
            // istanbul ignore else
            if (this._isMounted)
                this.setState({ keyins });
        });
    }
    /** @internal */
    componentWillUnmount() {
        this._isMounted = false;
    }
    getArgsKey(toolId) {
        if (toolId && toolId.length > 0)
            return `keyinbrowser:${toolId}`;
        return undefined;
    }
    getSavedToolId() {
        const toolId = window.localStorage.getItem(this._toolIdKey);
        return toolId ? toolId : undefined;
    }
    getToolArgs(toolId) {
        const argsKey = this.getArgsKey(toolId);
        if (argsKey && argsKey.length > 0) {
            const args = window.localStorage.getItem(argsKey);
            // istanbul ignore if
            if (args && /* istanbul ignore next */ args.length > 0 && /* istanbul ignore next */ args[0] === "[") {
                return JSON.parse(args).join("|");
            }
        }
        return "";
    }
    getToolKeyinMap() {
        const keyins = [];
        imodeljs_frontend_1.IModelApp.tools.getToolList()
            .forEach((tool) => keyins.push({ value: tool.toolId, label: tool.keyin, englishKeyin: tool.englishKeyin }));
        return keyins;
    }
    getArgsArray() {
        // istanbul ignore else
        if (this.state.currentArgs.length > 0) {
            return this.state.currentArgs.split("|");
        }
        return [];
    }
    _execute() {
        let toolId;
        let args = [];
        let runStatus = false;
        // istanbul ignore else
        if (this.state.currentToolId && this.state.currentToolId.length > 0) {
            const foundTool = imodeljs_frontend_1.IModelApp.tools.find(this.state.currentToolId);
            // istanbul ignore else
            if (foundTool) {
                toolId = foundTool.toolId;
                args = this.getArgsArray();
                const maxArgs = foundTool.maxArgs;
                if (args.length < foundTool.minArgs || (undefined !== maxArgs && args.length > maxArgs)) {
                    this._outputMessage(UiFramework_1.UiFramework.translate("keyinbrowser.incorrectArgs"));
                    return;
                }
                let key = "keyinbrowser:keyin";
                window.localStorage.setItem(key, toolId);
                // istanbul ignore if
                if (args && args.length > 0) {
                    key = `keyinbrowser:${toolId}`;
                    const objectAsString = JSON.stringify(args);
                    window.localStorage.setItem(key, objectAsString);
                }
                const tool = new foundTool();
                runStatus = false;
                try {
                    runStatus = args.length > 0 ? /* istanbul ignore next */ tool.parseAndRun(...args) : tool.run();
                    !runStatus && this._outputMessage(UiFramework_1.UiFramework.translate("keyinbrowser.failedToRun"));
                }
                catch (e) {
                    // istanbul ignore next
                    this._outputMessage(`${UiFramework_1.UiFramework.translate("keyinbrowser.exceptionOccurred")}: ${e}`);
                }
            }
        }
        // istanbul ignore else
        if (this.props.onExecute)
            this.props.onExecute({ toolId, args, runStatus });
    }
    _processInputValue(inputValue) {
        let currentKeyin = "";
        let foundTool;
        if (this.state.currentToolId && this.state.currentToolId.length > 0) {
            foundTool = imodeljs_frontend_1.IModelApp.tools.find(this.state.currentToolId);
            // istanbul ignore else
            if (foundTool)
                currentKeyin = foundTool.keyin;
        }
        // istanbul ignore next
        if (inputValue !== currentKeyin) {
            const inputValueLower = inputValue.trim().toLowerCase();
            foundTool = imodeljs_frontend_1.IModelApp.tools.getToolList()
                .find((tool) => tool.keyin.toLowerCase() === inputValueLower || tool.englishKeyin.toLowerCase() === inputValueLower);
            if (!foundTool) {
                this._outputMessage(UiFramework_1.UiFramework.translate("keyinbrowser.couldNotFindTool"));
                return false;
            }
            else {
                const currentToolId = foundTool.toolId;
                // istanbul ignore else
                if (this._isMounted)
                    this.setState({ currentToolId });
            }
        }
        return true;
    }
    /** @internal */
    render() {
        return (React.createElement("div", { className: "uif-keyinbrowser-div" },
            React.createElement(ui_core_1.InputLabel, { label: this._toolIdLabel },
                React.createElement(ui_core_1.AutoSuggest, { value: this.state.currentToolId, style: { width: "250px" }, placeholder: this._suggestPlaceholder, options: this.state.keyins, onSuggestionSelected: this._onKeyinSelected, onPressEnter: this._onAutoSuggestEnter, onPressTab: this._onAutoSuggestTab, onPressEscape: this._onAutoSuggestEscape, onInputFocus: this._onInputFocus, getSuggestions: this._getSuggestions, "data-testid": "uif-keyin-autosuggest" })),
            React.createElement(ui_core_1.LabeledInput, { label: this._argsLabel, title: this._argsTip, value: this.state.currentArgs, "data-testid": "uif-keyin-arguments", id: "uif-keyin-arguments", onKeyDown: this._onKeyDown, onChange: this._onArgumentsChange, onFocus: this._onInputFocus }),
            React.createElement(ui_core_1.Button, { "data-testid": "uif-keyin-browser-execute", onClick: this._onClick }, this._executeLabel)));
    }
}
exports.KeyinBrowser = KeyinBrowser;
//# sourceMappingURL=KeyinBrowser.js.map