"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Notification
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PointerMessage = exports.PointerMessageChangedEvent = void 0;
require("./Pointer.scss");
const classnames = require("classnames");
const React = require("react");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const MessageManager_1 = require("./MessageManager");
const MessageSpan_1 = require("./MessageSpan");
/** Pointer Message Changed Event emitted by the [[PointerMessage]] component
 * @public
 */
class PointerMessageChangedEvent extends ui_core_1.UiEvent {
}
exports.PointerMessageChangedEvent = PointerMessageChangedEvent;
/** Pointer Message Position Changed Event emitted by the [[PointerMessage]] component
 * @internal
 */
class PointerMessagePositionChangedEvent extends ui_core_1.UiEvent {
}
/** Pointer message pops up near pointer when attempting an invalid interaction.
 * @public
 */
class PointerMessage extends React.Component {
    constructor() {
        super(...arguments);
        this.state = {
            message: "",
            isVisible: false,
            position: {
                x: 0,
                y: 0,
            },
        };
        this._size = {
            height: 0,
            width: 0,
        };
        // istanbul ignore next
        this._handleSizeChanged = (size) => {
            this._size = size;
            this.updatePosition();
        };
        this._handlePointerMessageChangedEvent = (args) => {
            this._relativePosition = args.relativePosition;
            this._viewport = args.viewport;
            this._position = args.pt;
            this.setState(() => ({
                isVisible: args.isVisible,
                message: args.message,
                detailedMessage: args.detailedMessage,
                messageDetails: args.messageDetails,
            }));
            this.updatePosition();
        };
        this._handlePointerMessagePositionChangedEvent = (args) => {
            this._relativePosition = args.relativePosition;
            this._position = args.pt;
            this.updatePosition();
        };
    }
    static get onPointerMessageChangedEvent() { return PointerMessage._pointerMessageChangedEvent; }
    static showMessage(message) {
        PointerMessage.onPointerMessageChangedEvent.emit({
            isVisible: true,
            priority: message.priority,
            message: message.briefMessage,
            detailedMessage: message.detailedMessage,
            relativePosition: message.relativePosition,
            viewport: message.viewport,
            pt: message.displayPoint,
            messageDetails: message,
        });
    }
    static updateMessage(displayPoint, relativePosition) {
        PointerMessage._onPointerMessagePositionChangedEvent.emit({
            pt: displayPoint,
            relativePosition,
        });
    }
    static hideMessage() {
        PointerMessage.onPointerMessageChangedEvent.emit({
            isVisible: false,
            priority: imodeljs_frontend_1.OutputMessagePriority.None,
            message: "",
        });
    }
    render() {
        if (!this.state.isVisible)
            return null;
        const className = classnames("uifw-pointer-message", this.props.className);
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion -- false positive
        const severity = MessageManager_1.MessageManager.getSeverity(this.state.messageDetails);
        return (React.createElement(ui_ninezone_1.Tooltip, { className: className, onSizeChanged: this._handleSizeChanged, position: this.state.position, style: this.props.style },
            React.createElement("div", { className: "uifw-pointer-message-content" },
                (severity !== ui_core_1.MessageSeverity.None) &&
                    React.createElement("span", { className: "uifw-pointer-message-icon" },
                        React.createElement("i", { className: `icon ${ui_core_1.MessageContainer.getIconClassName(severity, false)}` })),
                React.createElement("span", { className: "uifw-pointer-message-text" },
                    React.createElement(MessageSpan_1.MessageSpan, { className: "uifw-pointer-message-brief", message: this.state.message }),
                    this.state.detailedMessage &&
                        React.createElement(MessageSpan_1.MessageDiv, { className: "uifw-pointer-message-detailed", message: this.state.detailedMessage })))));
    }
    componentDidMount() {
        PointerMessage.onPointerMessageChangedEvent.addListener(this._handlePointerMessageChangedEvent);
        PointerMessage._onPointerMessagePositionChangedEvent.addListener(this._handlePointerMessagePositionChangedEvent);
    }
    componentWillUnmount() {
        PointerMessage.onPointerMessageChangedEvent.removeListener(this._handlePointerMessageChangedEvent);
        PointerMessage._onPointerMessagePositionChangedEvent.removeListener(this._handlePointerMessagePositionChangedEvent);
    }
    updatePosition() {
        const adjustmentOffset = 20;
        let offset;
        switch (this._relativePosition) {
            case ui_abstract_1.RelativePosition.Top:
                offset = { x: 0, y: -adjustmentOffset };
                break;
            case ui_abstract_1.RelativePosition.TopRight:
                offset = { x: adjustmentOffset, y: -adjustmentOffset };
                break;
            case ui_abstract_1.RelativePosition.Right:
                offset = { x: adjustmentOffset, y: 0 };
                break;
            case ui_abstract_1.RelativePosition.BottomRight:
                offset = { x: adjustmentOffset, y: adjustmentOffset };
                break;
            case ui_abstract_1.RelativePosition.Bottom:
                offset = { x: 0, y: adjustmentOffset };
                break;
            case ui_abstract_1.RelativePosition.BottomLeft:
                offset = { x: -adjustmentOffset, y: adjustmentOffset };
                break;
            case ui_abstract_1.RelativePosition.Left:
                offset = { x: -adjustmentOffset, y: 0 };
                break;
            case ui_abstract_1.RelativePosition.TopLeft:
                offset = { x: -adjustmentOffset, y: -adjustmentOffset };
                break;
        }
        this.setState((prevState) => {
            if (!this._viewport)
                return null;
            // istanbul ignore if
            if (!this._position)
                return null;
            const containerBounds = ui_core_1.Rectangle.create(this._viewport.getBoundingClientRect());
            const relativeBounds = ui_core_1.Rectangle.createFromSize(this._size).offset(this._position);
            const viewportOffset = new ui_core_1.Point().getOffsetTo(containerBounds.topLeft());
            const adjustedPosition = ui_ninezone_1.offsetAndContainInContainer(relativeBounds, containerBounds.getSize(), offset);
            const position = adjustedPosition.offset(viewportOffset);
            // istanbul ignore else
            if (position.equals(prevState.position))
                return null;
            // istanbul ignore next
            return {
                position,
            };
        });
    }
}
exports.PointerMessage = PointerMessage;
PointerMessage._pointerMessageChangedEvent = new PointerMessageChangedEvent();
PointerMessage._onPointerMessagePositionChangedEvent = new PointerMessagePositionChangedEvent();
//# sourceMappingURL=Pointer.js.map