"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module OIDC
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignIn = void 0;
const React = require("react");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const frontend_authorization_client_1 = require("@bentley/frontend-authorization-client");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_components_1 = require("@bentley/ui-components");
const UiFramework_1 = require("../UiFramework");
/**
 * SignIn React component.
 * `IModelApp.authorizationClient.signIn` is called when the "Sign In" button is pressed,
 * then `props.onSignedIn` is called after sign-in has completed.
 * @public
 */
class SignIn extends React.PureComponent {
    constructor(props) {
        super(props);
        this._onStartSignin = async () => {
            // istanbul ignore next
            if (this._oidcClient)
                this._oidcClient.signIn(new bentleyjs_core_1.ClientRequestContext()); // eslint-disable-line @typescript-eslint/no-floating-promises
            // istanbul ignore else
            if (this.props.onStartSignIn)
                this.props.onStartSignIn();
        };
    }
    componentDidMount() {
        const oidcClient = imodeljs_frontend_1.IModelApp.authorizationClient;
        // istanbul ignore if
        if (frontend_authorization_client_1.isFrontendAuthorizationClient(oidcClient))
            this._oidcClient = oidcClient;
        // istanbul ignore next
        const isAuthorized = this._oidcClient && this._oidcClient.isAuthorized;
        // istanbul ignore if
        if (isAuthorized)
            this._oidcClient.onUserStateChanged.addListener(this._onUserStateChanged);
    }
    // istanbul ignore next
    _onUserStateChanged() {
        // istanbul ignore next
        if (this._oidcClient && this._oidcClient.isAuthorized && this.props.onSignedIn)
            this.props.onSignedIn();
    }
    componentWillUnmount() {
        // istanbul ignore next
        if (this._oidcClient)
            this._oidcClient.onUserStateChanged.removeListener(this._onUserStateChanged);
    }
    render() {
        /*
         * Note: In the case of electron, the signin happens in a disconnected web browser. We therefore show
         * a message to direc the user to the browser. Also, since we cannot capture the errors in the browser,
         * to clear the state of the signin UI, we instead allow signin button to be clicked multiple times.
         * See https://authguidance.com/2018/01/11/desktop-apps-overview/ for the pattern
         */
        let disableSignInOnClick = true;
        let signingInMessage;
        // istanbul ignore next
        if (bentleyjs_core_1.ProcessDetector.isElectronAppFrontend) {
            disableSignInOnClick = false;
            const signingInMessageStringId = `UiFramework:signIn.signingInMessage`;
            signingInMessage = UiFramework_1.UiFramework.i18n.translate(signingInMessageStringId);
        }
        return React.createElement(ui_components_1.SignIn, { className: this.props.className, style: this.props.style, onSignIn: this._onStartSignin, onRegister: this.props.onRegister, onOffline: this.props.onOffline, disableSignInOnClick: disableSignInOnClick, signingInMessage: signingInMessage });
    }
}
exports.SignIn = SignIn;
//# sourceMappingURL=SignIn.js.map