"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Picker
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelSelectorDataProvider = exports.Groups = void 0;
const _ = require("lodash");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const presentation_components_1 = require("@bentley/presentation-components");
/**
 * Group types available for [[ModelSelectorWidget]] picker
 * @internal
 */
var Groups;
(function (Groups) {
    Groups[Groups["Models"] = 0] = "Models";
    Groups[Groups["Categories"] = 1] = "Categories";
})(Groups = exports.Groups || (exports.Groups = {}));
/**
 * Implementation of a PresentationTreeDataProvider that manages model and category
 * data in [[CategoryModelTree]]
 * @internal
 */
class ModelSelectorDataProvider {
    /** @internal */
    constructor(imodel, rulesetId) {
        /** Listener for tree node changes */
        this.onTreeNodeChanged = new bentleyjs_core_1.BeEvent();
        /**
         * Returns filtered node paths.
         * @param filter Filter.
         * @returns Filtered NodePaths
         */
        this.getFilteredNodePaths = async (filter) => {
            return this._baseProvider.getFilteredNodePaths(filter);
        };
        /**
         * Provides count for number of nodes under parent node
         * @param parentNode Node to count children for
         */
        this.getNodesCount = _.memoize(async (parentNode) => {
            return this._baseProvider.getNodesCount(parentNode);
        });
        /**
         * Modifies and returns nodes to be displayed.
         * @param parentNode The parent node for all nodes to be returned
         * @param pageOptions Paging options
         * @returns TreeNodeItems to be displayed
         */
        this.getNodes = async (parentNode, pageOptions) => {
            return this._baseProvider.getNodes(parentNode, pageOptions);
        };
        this._baseProvider = new presentation_components_1.PresentationTreeDataProvider({ imodel, ruleset: rulesetId });
        this._baseProvider.pagingSize = 5;
    }
    // istanbul ignore next
    dispose() {
        this._baseProvider.dispose();
    }
    /** Id of the ruleset used by this data provider */
    get rulesetId() {
        return this._baseProvider.rulesetId;
    }
    /** [[IModelConnection]] used by this data provider */
    get imodel() {
        return this._baseProvider.imodel;
    }
    /**
     * Returns a [[NodeKey]] from given [[TreeNodeItem]].
     * **Warning:** the `node` must be created by this data provider.
     */
    // istanbul ignore next
    getNodeKey(node) {
        return this._baseProvider.getNodeKey(node);
    }
    // istanbul ignore next
    async loadHierarchy() { }
}
exports.ModelSelectorDataProvider = ModelSelectorDataProvider;
//# sourceMappingURL=ModelSelectorDefinitions.js.map