"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Picker
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelConnectedViewSelector = exports.ViewSelector = exports.ViewSelectorChangedEvent = void 0;
const React = require("react");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_core_1 = require("@bentley/ui-core");
const ContentViewManager_1 = require("../content/ContentViewManager");
const connectIModel_1 = require("../redux/connectIModel");
const UiFramework_1 = require("../UiFramework");
const ViewUtilities_1 = require("../utils/ViewUtilities");
const ListPicker_1 = require("./ListPicker");
/** ViewSelector Changed Event class.
 * @beta
 */
class ViewSelectorChangedEvent extends ui_core_1.UiEvent {
}
exports.ViewSelectorChangedEvent = ViewSelectorChangedEvent;
/** ViewSelector Show Update Event class.
 */
class ViewSelectorShowUpdateEvent extends ui_core_1.UiEvent {
}
/** View Selector React component
 * @beta
 */
class ViewSelector extends React.Component {
    /** Creates a ViewSelector */
    constructor(props) {
        super(props);
        this._isMounted = false;
        this._handleViewSelectorShowUpdateEvent = (args) => {
            // istanbul ignore next
            if (!this._isMounted)
                return;
            this.setState(args, async () => this.loadViews());
        };
        // enable/disable the models
        // istanbul ignore next
        this._setEnabled = async (item, _enabled) => {
            const activeContentControl = ContentViewManager_1.ContentViewManager.getActiveContentControl();
            if (!activeContentControl || !activeContentControl.supportsViewSelectorChange) {
                bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), `No active ContentControl for ViewSelector change`);
                return;
            }
            // Enable the item temporarily to let user see that their click was registered
            // while we query for view state and change the current view which may take a bit
            if (_enabled && item.type !== ListPicker_1.ListItemType.Container) {
                // This itemMapper simply looks through all the list items and their nested children and enables the one
                // that we have registered to enable
                // Also disable all other items
                const itemMapper = (tempItem) => {
                    if (tempItem.type === ListPicker_1.ListItemType.Container) {
                        return Object.assign(Object.assign({}, tempItem), { children: tempItem.children.map(itemMapper) });
                    }
                    else if (tempItem.key === item.key) {
                        return Object.assign(Object.assign({}, tempItem), { enabled: true });
                    }
                    else {
                        return Object.assign(Object.assign({}, tempItem), { enabled: false });
                    }
                };
                // Create the new array with the current item enabled
                const itemsWithEnabled = this.state.items.map(itemMapper);
                // istanbul ignore next
                if (!this._isMounted)
                    return;
                // Update the state so that we show the user it was enabled while we work in the background
                this.setState({ items: itemsWithEnabled });
            }
            // Load the view state using the viewSpec's ID
            const viewState = await this.props.imodel.views.load(item.key);
            // Let activeContentControl process the ViewSelector change
            await activeContentControl.processViewSelectorChange(this.props.imodel, item.key, viewState, item.name);
            // Emit a change event
            ViewSelector.onViewSelectorChangedEvent.emit({
                iModelConnection: this.props.imodel,
                viewDefinitionId: item.key,
                viewState,
                name: item.name,
            });
            // Set state to show enabled the view that got selected
            this.updateState(item.key); // eslint-disable-line @typescript-eslint/no-floating-promises
        };
        // Hook on the category selector being expanded so that we may initialize if needed
        // istanbul ignore next
        this._onExpanded = (expand) => {
            if (expand)
                this.updateState(imodeljs_frontend_1.IModelApp.viewManager.selectedView ? imodeljs_frontend_1.IModelApp.viewManager.selectedView.view.id : undefined); // eslint-disable-line @typescript-eslint/no-floating-promises
        };
        this.state = {
            items: new Array(),
            title: UiFramework_1.UiFramework.translate("savedViews.views"),
            initialized: false,
            showSpatials: props.showSpatials,
            showDrawings: props.showDrawings,
            showSheets: props.showSheets,
            showUnknown: props.showUnknown,
        };
    }
    /** Updates the ViewSelector show settings.
     */
    static updateShowSettings(showSpatials, showDrawings, showSheets, showUnknown) {
        ViewSelector._onViewSelectorShowUpdateEvent.emit({ showSpatials, showDrawings, showSheets, showUnknown });
    }
    async componentDidMount() {
        this._isMounted = true;
        if (this.props.listenForShowUpdates)
            this._removeShowUpdateListener = ViewSelector._onViewSelectorShowUpdateEvent.addListener(this._handleViewSelectorShowUpdateEvent);
        await this.loadViews();
    }
    async componentDidUpdate(prevProps) {
        if (this.props.imodel !== prevProps.imodel) {
            await this.loadViews();
        }
    }
    componentWillUnmount() {
        this._isMounted = false;
        // istanbul ignore else
        if (this._removeShowUpdateListener)
            this._removeShowUpdateListener();
    }
    setStateContainers(views3d, views2d, sheets, unknown) {
        const views3dContainer = {
            key: "views3dContainer",
            name: UiFramework_1.UiFramework.translate("savedViews.spatialViews"),
            enabled: false,
            type: ListPicker_1.ListItemType.Container,
            children: views3d,
        };
        const views2dContainer = {
            key: "views2dContainer",
            name: UiFramework_1.UiFramework.translate("savedViews.drawings"),
            enabled: false,
            type: ListPicker_1.ListItemType.Container,
            children: views2d,
        };
        const sheetContainer = {
            key: "sheetContainer",
            name: UiFramework_1.UiFramework.translate("savedViews.sheets"),
            enabled: false,
            type: ListPicker_1.ListItemType.Container,
            children: sheets,
        };
        const containers = [views3dContainer, views2dContainer, sheetContainer];
        // istanbul ignore if
        if (unknown && unknown.length > 0) {
            // This should never show, but just in case we missed a type of view state
            const unknownContainer = {
                key: "unknownContainer",
                name: UiFramework_1.UiFramework.translate("savedViews.others"),
                enabled: false,
                type: ListPicker_1.ListItemType.Container,
                children: unknown,
            };
            if (unknown.length !== 0)
                containers.push(unknownContainer);
        }
        // istanbul ignore next
        if (!this._isMounted)
            return;
        this.setState({
            items: containers,
            title: UiFramework_1.UiFramework.translate("savedViews.views"),
            initialized: true,
        });
    }
    /**
     * Query the views and set the initial state with the iModel's views.
     */
    async loadViews() {
        // Query views and add them to state
        const views3d = [];
        const views2d = [];
        const sheets = [];
        const unknown = [];
        // istanbul ignore if
        if (this.props.imodel && this.props.imodel.views.getViewList) {
            const query = { wantPrivate: false };
            const specs = await this.props.imodel.views.getViewList(query);
            specs.forEach((spec) => {
                const viewItem = {
                    key: spec.id,
                    name: spec.name,
                    enabled: false,
                    type: ListPicker_1.ListItemType.Item,
                };
                const className = ViewUtilities_1.ViewUtilities.getBisBaseClass(spec.class);
                if (ViewUtilities_1.ViewUtilities.isSpatial(className) && this.state.showSpatials)
                    views3d.push(viewItem);
                else if (ViewUtilities_1.ViewUtilities.isDrawing(className) && this.state.showDrawings)
                    views2d.push(viewItem);
                else if (ViewUtilities_1.ViewUtilities.isSheet(className) && this.state.showSheets)
                    sheets.push(viewItem);
                else if (this.state.showUnknown)
                    unknown.push(viewItem);
            });
        }
        this.setStateContainers(views3d, views2d, sheets, unknown);
    }
    /**
     * Update state of the entries in the widget.
     * @param viewId Identifier for the relevant view
     */
    // istanbul ignore next
    async updateState(viewId) {
        // Wait for initialization finished
        if (!this.state.initialized)
            return;
        // Query views and add them to state
        const views3d = this.state.items[0].children;
        const views2d = this.state.items[1].children;
        const sheets = this.state.items[2].children;
        const unknown = this.state.items.length > 3 ? this.state.items[3].children : [];
        const updateChildren = (item) => {
            if (item.key === viewId)
                return Object.assign(Object.assign({}, item), { enabled: true });
            else
                return Object.assign(Object.assign({}, item), { enabled: false });
        };
        this.setStateContainers(views3d.map(updateChildren), views2d.map(updateChildren), sheets.map(updateChildren), unknown.map(updateChildren));
    }
    /**
     *  Renders ViewSelector component
     */
    render() {
        if (!this.state.initialized)
            return null;
        const _a = this.props, { imodel } = _a, props = __rest(_a, ["imodel"]); // eslint-disable-line @typescript-eslint/no-unused-vars
        return (React.createElement(ListPicker_1.ListPicker, Object.assign({}, props, { title: this.state.title, setEnabled: this._setEnabled, items: this.state.items, iconSpec: "icon-saved-view", onExpanded: this._onExpanded })));
    }
}
exports.ViewSelector = ViewSelector;
ViewSelector._onViewSelectorShowUpdateEvent = new ViewSelectorShowUpdateEvent();
ViewSelector.defaultProps = {
    showSpatials: true,
    showDrawings: true,
    showSheets: true,
    showUnknown: true,
};
/** Gets the [[ViewSelectorChangedEvent]] */
ViewSelector.onViewSelectorChangedEvent = new ViewSelectorChangedEvent();
/** ViewSelector that is connected to the IModelConnection property in the Redux store. The application must set up the Redux store and include the FrameworkReducer.
 * @beta
 */
exports.IModelConnectedViewSelector = connectIModel_1.connectIModelConnection(null, null)(ViewSelector); // eslint-disable-line @typescript-eslint/naming-convention
//# sourceMappingURL=ViewSelector.js.map