"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Popup
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyinPalettePanel = exports.clearKeyinPaletteHistory = void 0;
// cSpell:ignore focusvalue
const React = require("react");
require("./KeyinPalettePanel.scss");
const ui_core_1 = require("@bentley/ui-core");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const UiFramework_1 = require("../UiFramework");
const ui_abstract_1 = require("@bentley/ui-abstract");
const KeyinPaletteTools_1 = require("../tools/KeyinPaletteTools");
const useUiSettings_1 = require("../uisettings/useUiSettings");
const KEYIN_PALETTE_NAMESPACE = "KeyinPalettePanel";
const KEYIN_HISTORY_KEY = "historyArray";
/** @internal */
function clearKeyinPaletteHistory() {
    const uiSettingsStorage = UiFramework_1.UiFramework.getUiSettingsStorage();
    // istanbul ignore else
    if (uiSettingsStorage) {
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        uiSettingsStorage.deleteSetting(KEYIN_PALETTE_NAMESPACE, KEYIN_HISTORY_KEY);
    }
}
exports.clearKeyinPaletteHistory = clearKeyinPaletteHistory;
/**
 * @internal
 */
function KeyinPalettePanel({ keyins, onKeyinExecuted, historyLength: allowedHistoryLength = 6 }) {
    const [currentKeyin, setCurrentKeyin] = React.useState("");
    const placeholderLabel = React.useRef(UiFramework_1.UiFramework.translate("keyinbrowser.placeholder"));
    const inputRef = React.useRef(null);
    const keyinSeparator = "--#--";
    const [historyKeyins, setHistoryKeyins] = React.useState([]);
    const uiSettingsStorage = useUiSettings_1.useUiSettingsStorageContext();
    React.useEffect(() => {
        async function fetchState() {
            const settingsResult = await uiSettingsStorage.getSetting(KEYIN_PALETTE_NAMESPACE, KEYIN_HISTORY_KEY);
            // istanbul ignore else
            if (ui_core_1.UiSettingsStatus.Success === settingsResult.status) {
                const filteredHistory = settingsResult.setting.filter((keyin) => {
                    const result = imodeljs_frontend_1.IModelApp.tools.parseKeyin(keyin);
                    return result.ok;
                });
                setHistoryKeyins(filteredHistory);
            }
            else {
                setHistoryKeyins([]);
            }
        }
        fetchState(); // eslint-disable-line @typescript-eslint/no-floating-promises
    }, [uiSettingsStorage]);
    // istanbul ignore next
    const storeHistoryKeyins = React.useCallback(async (value) => {
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        const result = await uiSettingsStorage.saveSetting(KEYIN_PALETTE_NAMESPACE, KEYIN_HISTORY_KEY, value);
        if (result.status !== ui_core_1.UiSettingsStatus.Success) {
            const briefMessage = UiFramework_1.UiFramework.translate("keyinbrowser.couldNotSaveHistory");
            const errorDetails = new imodeljs_frontend_1.NotifyMessageDetails(imodeljs_frontend_1.OutputMessagePriority.Error, briefMessage);
            imodeljs_frontend_1.IModelApp.notifications.outputMessage(errorDetails);
        }
    }, [uiSettingsStorage]);
    const allKeyins = React.useMemo(() => {
        const availableKeyins = [];
        historyKeyins.forEach((value) => {
            availableKeyins.push({ value, isHistory: true });
        });
        availableKeyins.push(...keyins.sort((a, b) => a.value.toLowerCase().localeCompare(b.value.toLowerCase())));
        return availableKeyins;
    }, [historyKeyins, keyins]);
    const submitKeyin = React.useCallback(async (value) => {
        let message;
        try {
            switch (imodeljs_frontend_1.IModelApp.tools.parseAndRun(value)) {
                case imodeljs_frontend_1.ParseAndRunResult.ToolNotFound:
                    message = `UiFramework.translate("keyinbrowser.couldNotFindTool")} ${value}`;
                    break;
                // istanbul ignore next
                case imodeljs_frontend_1.ParseAndRunResult.BadArgumentCount:
                    message = UiFramework_1.UiFramework.translate("keyinbrowser.incorrectArgs");
                    break;
                // istanbul ignore next
                case imodeljs_frontend_1.ParseAndRunResult.FailedToRun:
                    message = UiFramework_1.UiFramework.translate("keyinbrowser.failedToRun");
                    break;
            }
        }
        catch (ex) {
            // istanbul ignore next
            message = `${UiFramework_1.UiFramework.translate("keyinbrowser.exceptionOccurred")}: ${ex}`;
        }
        // istanbul ignore else
        if (undefined !== message) {
            const briefMessage = UiFramework_1.UiFramework.translate("keyinbrowser.exceptionOccurred");
            const errorDetails = new imodeljs_frontend_1.NotifyMessageDetails(imodeljs_frontend_1.OutputMessagePriority.Error, briefMessage, message, imodeljs_frontend_1.OutputMessageType.Sticky);
            imodeljs_frontend_1.IModelApp.notifications.outputMessage(errorDetails);
        }
        else {
            // istanbul ignore next
            if (value.length < 400 && value !== KeyinPaletteTools_1.ClearKeyinPaletteHistoryTool.keyin && value !== KeyinPaletteTools_1.ClearKeyinPaletteHistoryTool.englishKeyin) {
                const newHistoryEntries = [value];
                for (const entry of historyKeyins) {
                    if (entry !== value) {
                        newHistoryEntries.push(entry);
                        if (newHistoryEntries.length >= allowedHistoryLength)
                            break;
                    }
                }
                await storeHistoryKeyins(newHistoryEntries);
            }
            // close the popup by processing the supplied onKeyinExecuted function
            if (onKeyinExecuted)
                onKeyinExecuted(value);
        }
    }, [storeHistoryKeyins, onKeyinExecuted, historyKeyins, allowedHistoryLength]);
    const selectKeyin = React.useCallback(() => {
        // istanbul ignore else
        if (inputRef.current) {
            inputRef.current.focus();
            inputRef.current.setSelectionRange(inputRef.current.value.length, inputRef.current.value.length);
        }
    }, [inputRef]);
    const updateKeyin = React.useCallback((value) => {
        setCurrentKeyin(value);
        selectKeyin();
    }, [selectKeyin]);
    const getKeyinFromListboxValue = (value) => {
        // istanbul ignore else
        if (value) {
            const indexSeparator = value.search(keyinSeparator);
            // istanbul ignore else
            if (indexSeparator)
                return value.substr(0, indexSeparator);
        }
        // istanbul ignore next
        return undefined;
    };
    const handleKeypressOnKeyinsList = React.useCallback(async (event) => {
        var _a, _b;
        const key = event.key;
        // built into listbox a "Space" key will trigger the selection of a entry so if an Enter key is pressed
        // assume user wants to execute the entry.
        // istanbul ignore else
        if (key === ui_abstract_1.SpecialKey.Enter) {
            event.preventDefault();
            // istanbul ignore next
            const keyinToSend = getKeyinFromListboxValue((_b = (_a = event.currentTarget) === null || _a === void 0 ? void 0 : _a.dataset) === null || _b === void 0 ? void 0 : _b.focusvalue);
            // istanbul ignore else
            if (keyinToSend) {
                if (event.ctrlKey)
                    updateKeyin(keyinToSend);
                else
                    await submitKeyin(keyinToSend);
            }
        }
    }, [submitKeyin, updateKeyin]);
    const onListboxValueChange = React.useCallback(async (value, isControlOrCommandPressed) => {
        const keyin = getKeyinFromListboxValue(value);
        // istanbul ignore else
        if (keyin) {
            if (isControlOrCommandPressed)
                updateKeyin(keyin);
            else
                await submitKeyin(keyin);
        }
    }, [submitKeyin, updateKeyin]);
    const onInputValueChange = React.useCallback((event) => {
        setCurrentKeyin(event.target.value);
    }, []);
    const filteredKeyins = React.useMemo(() => {
        const filteredHistory = [];
        if (undefined === currentKeyin || 0 === currentKeyin.length) {
            return allKeyins;
        }
        else {
            const newKeyinSet = [];
            allKeyins.forEach((value) => {
                const matches = ui_abstract_1.matchesWords(currentKeyin, value.value);
                if (matches && matches.length) {
                    // istanbul ignore else
                    if (value.isHistory) {
                        filteredHistory.push(value);
                        newKeyinSet.push(Object.assign(Object.assign({}, value), { matches }));
                    }
                    else {
                        // only add entry if no match in filtered history
                        // istanbul ignore else
                        if (-1 === filteredHistory.findIndex((historyEntry) => historyEntry.value === value.value))
                            newKeyinSet.push(Object.assign(Object.assign({}, value), { matches }));
                    }
                }
            });
            return newKeyinSet;
        }
    }, [allKeyins, currentKeyin]);
    const onInputValueKeyDown = React.useCallback(async (event) => {
        if (ui_abstract_1.SpecialKey.Enter === event.key) {
            event.preventDefault();
            event.stopPropagation();
            // istanbul ignore else
            if (1 === filteredKeyins.length) {
                if (event.ctrlKey)
                    updateKeyin(filteredKeyins[0].value);
                else
                    await submitKeyin(filteredKeyins[0].value);
            }
            else {
                if (currentKeyin)
                    await submitKeyin(currentKeyin);
            }
        }
        else if (ui_abstract_1.SpecialKey.Tab === event.key && 1 === filteredKeyins.length) {
            event.preventDefault();
            event.stopPropagation();
            updateKeyin(filteredKeyins[0].value);
        }
        else {
            if (event.key === ui_abstract_1.SpecialKey.ArrowDown && filteredKeyins.length > 0) {
                event.preventDefault();
                event.stopPropagation();
                // istanbul ignore else
                if (inputRef.current) {
                    const nextElement = inputRef.current.nextElementSibling;
                    nextElement && nextElement.focus();
                }
            }
        }
    }, [filteredKeyins, submitKeyin, currentKeyin, updateKeyin]);
    const lastHistoryIndex = filteredKeyins.findIndex((entry) => (true !== entry.isHistory)) - 1;
    return React.createElement("div", { className: "uifw-command-palette-panel" },
        React.createElement("input", { ref: inputRef, type: "text", onKeyDown: onInputValueKeyDown, className: "uicore-inputs-input uifw-command-palette-input", "data-testid": "command-palette-input", onChange: onInputValueChange, placeholder: placeholderLabel.current, value: currentKeyin }),
        (filteredKeyins.length > 0) &&
            React.createElement(ui_core_1.Listbox, { id: "uifw-command-sources", className: "map-manager-source-list", onKeyPress: handleKeypressOnKeyinsList, onListboxValueChange: onListboxValueChange }, filteredKeyins.map((entry, index) => {
                const value = `${entry.value}${keyinSeparator}${entry.isHistory ? "history" : "registry"}`;
                const itemClass = `uifw-command-palette-value-entry${index === lastHistoryIndex ? " uifw-history-bottom-border" : ""}`;
                return React.createElement(ui_core_1.ListboxItem, { key: `${entry.value}-${index}`, className: itemClass, value: value },
                    React.createElement(ui_core_1.FilteredText, { value: entry.value, matches: entry.matches }));
            })));
}
exports.KeyinPalettePanel = KeyinPalettePanel;
//# sourceMappingURL=KeyinPalettePanel.js.map