"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module State
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionStateReducer = exports.sessionStateMapDispatchToProps = exports.SessionStateActions = exports.SessionStateActionId = void 0;
const redux_ts_1 = require("./redux-ts");
/** Action Ids used by Redux and to send sync UI components. Typically used to refresh visibility or enable state of control.
 *  Since these are also used as sync ids they should be in lowercase.
 * @public
 */
var SessionStateActionId;
(function (SessionStateActionId) {
    SessionStateActionId["SetNumItemsSelected"] = "sessionstate:set-num-items-selected";
    SessionStateActionId["SetAvailableSelectionScopes"] = "sessionstate:set-available-selection-scopes";
    SessionStateActionId["SetSelectionScope"] = "sessionstate:set-selection-scope";
    SessionStateActionId["SetActiveIModelId"] = "sessionstate:set-active-imodelid";
    SessionStateActionId["SetIModelConnection"] = "sessionstate:set-imodel-connection";
    SessionStateActionId["SetUserInfo"] = "sessionstate:set-user-info";
    SessionStateActionId["SetDefaultIModelViewportControlId"] = "sessionstate:set-default-viewportid";
    SessionStateActionId["SetDefaultViewId"] = "sessionstate:set-default-viewid";
    SessionStateActionId["SetDefaultViewState"] = "sessionstate:set-default-view-state";
    SessionStateActionId["UpdateCursorMenu"] = "sessionstate:update-cursor-menu";
})(SessionStateActionId = exports.SessionStateActionId || (exports.SessionStateActionId = {}));
const defaultSelectionScope = { id: "element", label: "Element" };
/** used on first call of SessionStateReducer */
const initialState = {
    /** number of selected items in Presentation Selection */
    numItemsSelected: 0,
    /** initialize to only support "Element" scope, this will be overwritten when iModelConnection is established */
    availableSelectionScopes: [defaultSelectionScope],
    /** initialize to active selection scope to "Element", this will be overwritten when iModelConnection is established */
    activeSelectionScope: defaultSelectionScope.id,
    /** set to iModelId if an iModel is active else it is an empty string, so initialize to empty string */
    iModelId: "",
    defaultIModelViewportControlId: undefined,
    defaultViewId: undefined,
    defaultViewState: undefined,
    iModelConnection: undefined,
    userInfo: undefined,
    cursorMenuData: undefined,
};
/** An object with a function that creates each SessionStateReducer that can be handled by our reducer.
 * @public
 */
exports.SessionStateActions = {
    setUserInfo: 
    // istanbul ignore next
    (userInfo) => redux_ts_1.createAction(SessionStateActionId.SetUserInfo, userInfo),
    setActiveIModelId: 
    // istanbul ignore next
    (iModelId) => redux_ts_1.createAction(SessionStateActionId.SetActiveIModelId, iModelId),
    setAvailableSelectionScopes: 
    // istanbul ignore next
    (availableSelectionScopes) => redux_ts_1.createAction(SessionStateActionId.SetAvailableSelectionScopes, availableSelectionScopes),
    setDefaultIModelViewportControlId: (iModelViewportControlId) => redux_ts_1.createAction(SessionStateActionId.SetDefaultIModelViewportControlId, iModelViewportControlId),
    setDefaultViewId: (viewId) => redux_ts_1.createAction(SessionStateActionId.SetDefaultViewId, viewId),
    setDefaultViewState: 
    // istanbul ignore next
    (viewState) => redux_ts_1.createAction(SessionStateActionId.SetDefaultViewState, viewState),
    setNumItemsSelected: (numSelected) => redux_ts_1.createAction(SessionStateActionId.SetNumItemsSelected, numSelected),
    setIModelConnection: 
    // istanbul ignore next
    (iModelConnection) => redux_ts_1.createAction(SessionStateActionId.SetIModelConnection, iModelConnection),
    setSelectionScope: 
    // istanbul ignore next
    (activeSelectionScope) => redux_ts_1.createAction(SessionStateActionId.SetSelectionScope, activeSelectionScope),
    updateCursorMenu: 
    // istanbul ignore next
    (cursorMenuData) => redux_ts_1.createAction(SessionStateActionId.UpdateCursorMenu, cursorMenuData),
};
/** Object that contains available actions that modify SessionState. Parent control's props should
 * extend from SessionStateActionsProps before using this in Redux 'connect' function.
 * @beta
 */
// ...SessionStateActionsProps
exports.sessionStateMapDispatchToProps = Object.assign({}, exports.SessionStateActions);
/** Handles actions to update SessionState.
 * @public
 */
function SessionStateReducer(state = initialState, action) {
    switch (action.type) {
        case SessionStateActionId.SetNumItemsSelected: {
            // istanbul ignore else
            if (undefined !== action.payload)
                return Object.assign(Object.assign({}, state), { numItemsSelected: action.payload });
            else
                return Object.assign(Object.assign({}, state), { numItemsSelected: 0 });
        }
        case SessionStateActionId.SetAvailableSelectionScopes: {
            const payloadArray = [];
            action.payload.forEach((scope) => payloadArray.push(scope));
            // istanbul ignore else
            if (undefined !== action.payload)
                return Object.assign(Object.assign({}, state), { availableSelectionScopes: payloadArray });
            else
                return Object.assign(Object.assign({}, state), { availableSelectionScopes: [defaultSelectionScope] });
        }
        case SessionStateActionId.SetSelectionScope: {
            // istanbul ignore else
            if (undefined !== action.payload)
                return Object.assign(Object.assign({}, state), { activeSelectionScope: action.payload });
            else
                return Object.assign(Object.assign({}, state), { activeSelectionScope: defaultSelectionScope.id });
        }
        case SessionStateActionId.SetActiveIModelId: {
            // istanbul ignore else
            if (undefined !== action.payload)
                return Object.assign(Object.assign({}, state), { iModelId: action.payload });
            else
                return Object.assign(Object.assign({}, state), { iModelId: "" });
        }
        case SessionStateActionId.SetDefaultIModelViewportControlId: {
            return Object.assign(Object.assign({}, state), { defaultIModelViewportControlId: action.payload });
        }
        case SessionStateActionId.SetDefaultViewId: {
            return Object.assign(Object.assign({}, state), { defaultViewId: action.payload });
        }
        case SessionStateActionId.SetDefaultViewState: {
            return Object.assign(Object.assign({}, state), { defaultViewState: action.payload });
        }
        case SessionStateActionId.SetIModelConnection: {
            return Object.assign(Object.assign({}, state), { iModelConnection: action.payload });
        }
        case SessionStateActionId.SetUserInfo: {
            return Object.assign(Object.assign({}, state), { userInfo: action.payload });
        }
        case SessionStateActionId.UpdateCursorMenu: {
            return Object.assign(Object.assign({}, state), { cursorMenuData: action.payload });
        }
    }
    return state;
}
exports.SessionStateReducer = SessionStateReducer;
//# sourceMappingURL=SessionState.js.map