"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Tools
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectionContextToolDefinitions = exports.selectionContextStateFunc = exports.featureOverridesActiveStateFunc = exports.getIsHiddenIfSelectionNotActive = exports.getIsHiddenIfFeatureOverridesActive = exports.areNoFeatureOverridesActive = exports.isNoSelectionActive = exports.getSelectionContextSyncEventIds = exports.getFeatureOverrideSyncEventIds = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const ContentViewManager_1 = require("../content/ContentViewManager");
const SessionState_1 = require("../redux/SessionState");
const CommandItemDef_1 = require("../shared/CommandItemDef");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
const GroupItem_1 = require("../toolbar/GroupItem");
const UiFramework_1 = require("../UiFramework");
const HideIsolateEmphasizeManager_1 = require("./HideIsolateEmphasizeManager");
/** return SyncEventIds that trigger selection state function refresh.
 * @beta
 */
function getFeatureOverrideSyncEventIds() {
    return [SyncUiEventDispatcher_1.SyncUiEventId.ActiveContentChanged, SyncUiEventDispatcher_1.SyncUiEventId.ActiveViewportChanged, HideIsolateEmphasizeManager_1.HideIsolateEmphasizeActionHandler.hideIsolateEmphasizeUiSyncId];
}
exports.getFeatureOverrideSyncEventIds = getFeatureOverrideSyncEventIds;
/** return SyncEventIds that trigger selection state function refresh.
 * @beta
 */
function getSelectionContextSyncEventIds() {
    return [SyncUiEventDispatcher_1.SyncUiEventId.SelectionSetChanged, SyncUiEventDispatcher_1.SyncUiEventId.ActiveContentChanged, SyncUiEventDispatcher_1.SyncUiEventId.ActiveViewportChanged, SessionState_1.SessionStateActionId.SetNumItemsSelected];
}
exports.getSelectionContextSyncEventIds = getSelectionContextSyncEventIds;
/** return SyncEventIds that trigger selection state function refresh.
 * @beta
 */
function isNoSelectionActive() {
    const activeContentControl = ContentViewManager_1.ContentViewManager.getActiveContentControl();
    let selectionCount = 0;
    // istanbul ignore if
    if (!UiFramework_1.UiFramework.frameworkStateKey)
        selectionCount = UiFramework_1.UiFramework.store.getState()[UiFramework_1.UiFramework.frameworkStateKey].frameworkState.sessionState.numItemsSelected;
    // istanbul ignore if
    if (activeContentControl /* istanbul ignore next */ && activeContentControl.viewport
        && /* istanbul ignore next */ (activeContentControl.viewport.view.iModel.selectionSet.size > 0 || selectionCount > 0))
        return false;
    return true;
}
exports.isNoSelectionActive = isNoSelectionActive;
/** return ConditionalBooleanValue object used to show items if selection set is active.
 * @beta
 */
function areNoFeatureOverridesActive() {
    const activeContentControl = ContentViewManager_1.ContentViewManager.getActiveContentControl();
    // istanbul ignore next
    if (activeContentControl && activeContentControl.viewport)
        return !UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.areFeatureOverridesActive(activeContentControl.viewport);
    return true;
}
exports.areNoFeatureOverridesActive = areNoFeatureOverridesActive;
/** return ConditionalBooleanValue object used to show item if feature overrides are active.
 * @beta
 */
function getIsHiddenIfFeatureOverridesActive() {
    return new ui_abstract_1.ConditionalBooleanValue(areNoFeatureOverridesActive, getFeatureOverrideSyncEventIds());
}
exports.getIsHiddenIfFeatureOverridesActive = getIsHiddenIfFeatureOverridesActive;
/** return ConditionalBooleanValue object used to show items if selection set is active.
 * @beta
 */
function getIsHiddenIfSelectionNotActive() {
    return new ui_abstract_1.ConditionalBooleanValue(isNoSelectionActive, getSelectionContextSyncEventIds());
}
exports.getIsHiddenIfSelectionNotActive = getIsHiddenIfSelectionNotActive;
/** return state with isVisible set to true is SectionSet is active.
 * @beta
 */
// istanbul ignore next
function featureOverridesActiveStateFunc(state) {
    const activeContentControl = ContentViewManager_1.ContentViewManager.getActiveContentControl();
    let isVisible = false;
    // istanbul ignore next
    if (activeContentControl && activeContentControl.viewport)
        isVisible = UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.areFeatureOverridesActive(activeContentControl.viewport);
    return Object.assign(Object.assign({}, state), { isVisible });
}
exports.featureOverridesActiveStateFunc = featureOverridesActiveStateFunc;
/** return state with isVisible set to true is SectionSet is active.
 * @beta
 */
// istanbul ignore next
function selectionContextStateFunc(state) {
    const activeContentControl = ContentViewManager_1.ContentViewManager.getActiveContentControl();
    let isVisible = false;
    let selectionCount = 0;
    if (!UiFramework_1.UiFramework.frameworkStateKey)
        selectionCount = UiFramework_1.UiFramework.store.getState()[UiFramework_1.UiFramework.frameworkStateKey].frameworkState.sessionState.numItemsSelected;
    if (activeContentControl && activeContentControl.viewport && (activeContentControl.viewport.view.iModel.selectionSet.size > 0 || selectionCount > 0))
        isVisible = true;
    return Object.assign(Object.assign({}, state), { isVisible });
}
exports.selectionContextStateFunc = selectionContextStateFunc;
/** Utility Class that provides definitions for tools dependent on current selection. These definitions can be used to populate toolbars.
 * @public
 */
// istanbul ignore next
class SelectionContextToolDefinitions {
    static get isolateModelsInSelectionItemDef() {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.IsolateModel",
            iconSpec: "icon-model-isolate",
            labelKey: "UiFramework:tools.isolateModels",
            execute: async () => UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processIsolateSelectedElementsModel(),
        });
    }
    static get isolateCategoriesInSelectionItemDef() {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.IsolateCategory",
            iconSpec: "icon-layers-isolate",
            labelKey: "UiFramework:tools.isolateCategories",
            execute: async () => UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processIsolateSelectedElementsCategory(),
        });
    }
    static get isolateElementsItemDef() {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.IsolateSelected",
            iconSpec: "icon-asset-isolate",
            labelKey: "UiFramework:tools.isolateSelected",
            stateSyncIds: getSelectionContextSyncEventIds(),
            stateFunc: selectionContextStateFunc,
            isHidden: getIsHiddenIfSelectionNotActive(),
            execute: async () => UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processIsolateSelected(),
        });
    }
    static get isolateSelectionToolGroup() {
        return new GroupItem_1.GroupItemDef({
            groupId: "UiFramework.IsolateSelectionGroup",
            labelKey: "UiFramework:tools.isolate",
            iconSpec: "icon-isolate",
            stateSyncIds: getSelectionContextSyncEventIds(),
            stateFunc: selectionContextStateFunc,
            isHidden: getIsHiddenIfSelectionNotActive(),
            items: [this.isolateElementsItemDef, this.isolateCategoriesInSelectionItemDef, this.isolateModelsInSelectionItemDef],
            itemsInColumn: 3,
        });
    }
    static get hideModelsInSelectionItemDef() {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.HideModel",
            iconSpec: "icon-model-hide",
            labelKey: "UiFramework:tools.hideModels",
            execute: async () => UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processHideSelectedElementsModel(),
        });
    }
    static get hideCategoriesInSelectionItemDef() {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.HideCategory",
            iconSpec: "icon-layers-hide",
            labelKey: "UiFramework:tools.hideCategories",
            execute: async () => UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processHideSelectedElementsCategory(),
        });
    }
    static get hideElementsItemDef() {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.HideSelected",
            iconSpec: "icon-asset-classification-hide",
            labelKey: "UiFramework:tools.hideSelected",
            isHidden: getIsHiddenIfSelectionNotActive(),
            stateSyncIds: getSelectionContextSyncEventIds(),
            stateFunc: selectionContextStateFunc,
            execute: async () => UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processHideSelected(),
        });
    }
    static get hideSectionToolGroup() {
        return new GroupItem_1.GroupItemDef({
            groupId: "UiFramework.HideSelectionGroup",
            labelKey: "UiFramework:tools.hide",
            iconSpec: "icon-visibility-hide-2",
            isHidden: getIsHiddenIfSelectionNotActive(),
            stateSyncIds: getSelectionContextSyncEventIds(),
            stateFunc: selectionContextStateFunc,
            items: [this.hideElementsItemDef, this.hideCategoriesInSelectionItemDef, this.hideModelsInSelectionItemDef],
            itemsInColumn: 3,
        });
    }
    static get emphasizeElementsItemDef() {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.EmphasizeSelected",
            iconSpec: "icon-visibility-semi-transparent",
            labelKey: "UiFramework:tools.emphasizeSelected",
            isHidden: getIsHiddenIfSelectionNotActive(),
            stateSyncIds: getSelectionContextSyncEventIds(),
            stateFunc: selectionContextStateFunc,
            execute: async () => UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processEmphasizeSelected(),
        });
    }
    static get clearHideIsolateEmphasizeElementsItemDef() {
        return new CommandItemDef_1.CommandItemDef({
            commandId: "UiFramework.ClearHideIsolateEmphasize",
            iconSpec: "icon-visibility",
            labelKey: "UiFramework:tools.clearVisibility",
            isHidden: getIsHiddenIfFeatureOverridesActive(),
            stateSyncIds: getFeatureOverrideSyncEventIds(),
            stateFunc: featureOverridesActiveStateFunc,
            execute: async () => {
                await UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processClearEmphasize();
                await UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processClearOverrideModels();
                await UiFramework_1.UiFramework.hideIsolateEmphasizeActionHandler.processClearOverrideCategories();
            },
        });
    }
}
exports.SelectionContextToolDefinitions = SelectionContextToolDefinitions;
//# sourceMappingURL=SelectionContextItemDef.js.map