"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Settings
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuantityFormatSettingsPage = exports.getQuantityFormatsSettingsManagerEntry = void 0;
require("./QuantityFormat.scss");
const React = require("react");
const geometry_core_1 = require("@bentley/geometry-core");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_components_1 = require("@bentley/ui-components");
const ui_core_1 = require("@bentley/ui-core");
const ModalDialogManager_1 = require("../../dialog/ModalDialogManager");
const UiFramework_1 = require("../../UiFramework");
const presentation_common_1 = require("@bentley/presentation-common");
const UnitSystemSelector_1 = require("./UnitSystemSelector");
const presentation_frontend_1 = require("@bentley/presentation-frontend");
function formatAreEqual(obj1, obj2) {
    const compare = new geometry_core_1.DeepCompare();
    return compare.compare(obj1, obj2);
}
/**
 * Return a SettingsTabEntry that can be used to define the available settings that can be set for an application.
 * @param itemPriority - Used to define the order of the entry in the Settings Stage
 * @param opts - Options to initialize the settings page that allows users to set Quantity formatting overrides.
 * @beta
 */
function getQuantityFormatsSettingsManagerEntry(itemPriority, opts) {
    const { availableUnitSystems, initialQuantityType } = Object.assign({}, opts);
    return {
        itemPriority, tabId: "uifw:Quantity",
        label: UiFramework_1.UiFramework.translate("settings.quantity-formatting.label"),
        subLabel: UiFramework_1.UiFramework.translate("settings.quantity-formatting.subLabel"),
        page: React.createElement(QuantityFormatSettingsPage, { initialQuantityType: initialQuantityType !== null && initialQuantityType !== void 0 ? initialQuantityType : imodeljs_frontend_1.QuantityType.Length, availableUnitSystems: availableUnitSystems !== null && availableUnitSystems !== void 0 ? availableUnitSystems : new Set(["metric", "imperial", "usCustomary", "usSurvey"]) }),
        isDisabled: false,
        icon: "icon-measure",
        tooltip: UiFramework_1.UiFramework.translate("settings.quantity-formatting.tooltip"),
        pageWillHandleCloseRequest: true,
    };
}
exports.getQuantityFormatsSettingsManagerEntry = getQuantityFormatsSettingsManagerEntry;
/** UI Component shown in settings page to set the active Presentation Unit System and to set format overrides.
 * @beta
 */
function QuantityFormatSettingsPage({ initialQuantityType, availableUnitSystems }) {
    const [activeUnitSystemKey, setActiveUnitSystemKey] = React.useState(imodeljs_frontend_1.IModelApp.quantityFormatter.activeUnitSystem);
    const [activeQuantityType, setActiveQuantityType] = React.useState(imodeljs_frontend_1.getQuantityTypeKey(initialQuantityType));
    const [activeFormatterSpec, setActiveFormatterSpec] = React.useState(imodeljs_frontend_1.IModelApp.quantityFormatter.findFormatterSpecByQuantityType(imodeljs_frontend_1.getQuantityTypeKey(activeQuantityType)));
    const [saveEnabled, setSaveEnabled] = React.useState(false);
    const [clearEnabled, setClearEnabled] = React.useState(imodeljs_frontend_1.IModelApp.quantityFormatter.hasActiveOverride(initialQuantityType, true));
    const newQuantityTypeRef = React.useRef();
    const formatSectionLabel = React.useRef(UiFramework_1.UiFramework.translate("settings.quantity-formatting.formatSectionLabel"));
    const setButtonLabel = React.useRef(UiFramework_1.UiFramework.translate("settings.quantity-formatting.setButtonLabel"));
    const clearButtonLabel = React.useRef(UiFramework_1.UiFramework.translate("settings.quantity-formatting.clearButtonLabel"));
    React.useEffect(() => {
        const handleUnitSystemChanged = (() => {
            // istanbul ignore else
            if (activeUnitSystemKey !== imodeljs_frontend_1.IModelApp.quantityFormatter.activeUnitSystem) {
                setActiveUnitSystemKey(imodeljs_frontend_1.IModelApp.quantityFormatter.activeUnitSystem);
                setActiveFormatterSpec(imodeljs_frontend_1.IModelApp.quantityFormatter.findFormatterSpecByQuantityType(activeQuantityType));
                setSaveEnabled(false);
                setClearEnabled(imodeljs_frontend_1.IModelApp.quantityFormatter.hasActiveOverride(activeQuantityType, true));
            }
        });
        imodeljs_frontend_1.IModelApp.quantityFormatter.onActiveFormattingUnitSystemChanged.addListener(handleUnitSystemChanged);
        return () => {
            imodeljs_frontend_1.IModelApp.quantityFormatter.onActiveFormattingUnitSystemChanged.removeListener(handleUnitSystemChanged);
        };
    }, [activeQuantityType, activeUnitSystemKey]);
    React.useEffect(() => {
        const handleFormatChanged = ((args) => {
            if (!newQuantityTypeRef.current) {
                const quantityKey = imodeljs_frontend_1.IModelApp.quantityFormatter.getQuantityTypeKey(activeQuantityType);
                // istanbul ignore else
                if (args.quantityType === quantityKey) {
                    setActiveFormatterSpec(imodeljs_frontend_1.IModelApp.quantityFormatter.findFormatterSpecByQuantityType(activeQuantityType));
                    setSaveEnabled(false);
                    setClearEnabled(imodeljs_frontend_1.IModelApp.quantityFormatter.hasActiveOverride(activeQuantityType, true));
                }
            }
            newQuantityTypeRef.current = undefined;
        });
        imodeljs_frontend_1.IModelApp.quantityFormatter.onQuantityFormatsChanged.addListener(handleFormatChanged);
        return () => {
            imodeljs_frontend_1.IModelApp.quantityFormatter.onQuantityFormatsChanged.removeListener(handleFormatChanged);
        };
    }, [activeQuantityType]);
    const saveChanges = React.useCallback((afterSaveFunction, args) => {
        // istanbul ignore else
        if (activeFormatterSpec) {
            const formatProps = activeFormatterSpec.format.toJSON();
            const formatPropsInUse = imodeljs_frontend_1.IModelApp.quantityFormatter.findFormatterSpecByQuantityType(activeQuantityType).format.toJSON();
            if (formatPropsInUse && !formatAreEqual(formatProps, formatPropsInUse)) {
                ModalDialogManager_1.ModalDialogManager.openDialog(React.createElement(SaveFormatModalDialog, { formatProps: formatProps, quantityType: activeQuantityType, onDialogCloseArgs: args, onDialogClose: afterSaveFunction }), "saveQuantityFormat");
                return;
            }
        }
        afterSaveFunction(args);
    }, [activeFormatterSpec, activeQuantityType]);
    ui_core_1.useSaveBeforeActivatingNewSettingsTab(UiFramework_1.UiFramework.settingsManager, saveChanges);
    ui_core_1.useSaveBeforeClosingSettingsContainer(UiFramework_1.UiFramework.settingsManager, saveChanges);
    const processListboxValueChange = React.useCallback((newQuantityType) => {
        const volumeFormatterSpec = imodeljs_frontend_1.IModelApp.quantityFormatter.findFormatterSpecByQuantityType(newQuantityType);
        setActiveFormatterSpec(volumeFormatterSpec);
        setActiveQuantityType(newQuantityType);
        setSaveEnabled(false);
        setClearEnabled(imodeljs_frontend_1.IModelApp.quantityFormatter.hasActiveOverride(newQuantityType, true));
    }, []);
    const onListboxValueChange = React.useCallback((newQuantityType) => {
        // istanbul ignore else
        if (activeFormatterSpec) {
            const formatProps = activeFormatterSpec.format.toJSON();
            const formatPropsInUse = imodeljs_frontend_1.IModelApp.quantityFormatter.findFormatterSpecByQuantityType(activeQuantityType).format.toJSON();
            if (formatPropsInUse && !formatAreEqual(formatProps, formatPropsInUse)) {
                newQuantityTypeRef.current = newQuantityType;
                ModalDialogManager_1.ModalDialogManager.openDialog(React.createElement(SaveFormatModalDialog, { formatProps: formatProps, quantityType: activeQuantityType, onDialogCloseArgs: newQuantityType, onDialogClose: processListboxValueChange }), "saveQuantityFormat");
                return;
            }
        }
        processListboxValueChange(newQuantityType);
    }, [activeFormatterSpec, activeQuantityType, processListboxValueChange]);
    const handleOnFormatChanged = React.useCallback(async (formatProps) => {
        // istanbul ignore else
        if (activeFormatterSpec) {
            const newSpec = await imodeljs_frontend_1.IModelApp.quantityFormatter.generateFormatterSpecByType(activeQuantityType, formatProps);
            const formatPropsInUse = imodeljs_frontend_1.IModelApp.quantityFormatter.getFormatPropsByQuantityType(activeQuantityType);
            // istanbul ignore else
            if (formatPropsInUse)
                setSaveEnabled(!formatAreEqual(formatProps, formatPropsInUse));
            setActiveFormatterSpec(newSpec);
        }
    }, [activeFormatterSpec, activeQuantityType]);
    const handleOnFormatSave = React.useCallback(async () => {
        // istanbul ignore else
        if (activeFormatterSpec) {
            const format = activeFormatterSpec.format.toJSON();
            await imodeljs_frontend_1.IModelApp.quantityFormatter.setOverrideFormat(activeQuantityType, format);
            setClearEnabled(true);
        }
    }, [activeFormatterSpec, activeQuantityType]);
    const handleOnFormatReset = React.useCallback(async () => {
        await imodeljs_frontend_1.IModelApp.quantityFormatter.clearOverrideFormats(activeQuantityType);
        setClearEnabled(false);
    }, [activeQuantityType]);
    const processNewUnitSystem = React.useCallback(async (unitSystem) => {
        switch (unitSystem) {
            case "imperial":
                presentation_frontend_1.Presentation.presentation.activeUnitSystem = presentation_common_1.PresentationUnitSystem.BritishImperial;
                await imodeljs_frontend_1.IModelApp.quantityFormatter.setActiveUnitSystem(unitSystem);
                break;
            case "metric":
                presentation_frontend_1.Presentation.presentation.activeUnitSystem = presentation_common_1.PresentationUnitSystem.Metric;
                await imodeljs_frontend_1.IModelApp.quantityFormatter.setActiveUnitSystem(unitSystem);
                break;
            case "usSurvey":
                presentation_frontend_1.Presentation.presentation.activeUnitSystem = presentation_common_1.PresentationUnitSystem.UsSurvey;
                await imodeljs_frontend_1.IModelApp.quantityFormatter.setActiveUnitSystem(unitSystem);
                break;
            case "usCustomary":
                presentation_frontend_1.Presentation.presentation.activeUnitSystem = presentation_common_1.PresentationUnitSystem.UsCustomary;
                await imodeljs_frontend_1.IModelApp.quantityFormatter.setActiveUnitSystem(unitSystem);
                break;
        }
    }, []);
    const handleUnitSystemSelected = React.useCallback(async (unitSystem) => {
        if (unitSystem === activeUnitSystemKey)
            return;
        saveChanges(processNewUnitSystem, unitSystem);
    }, [activeUnitSystemKey, processNewUnitSystem, saveChanges]);
    return (React.createElement("div", { className: "quantity-formatting-container" },
        React.createElement(UnitSystemSelector_1.UnitSystemSelector, { selectedUnitSystemKey: activeUnitSystemKey, availableUnitSystems: availableUnitSystems, onUnitSystemSelected: handleUnitSystemSelected }),
        React.createElement("span", { className: "uifw-quantity-format-section-label" }, formatSectionLabel.current),
        React.createElement("div", { className: "uifw-quantity-types-container" },
            React.createElement("div", { className: "left-panel" },
                React.createElement(ui_core_1.Listbox, { id: "uifw-quantity-types-list", className: "uifw-quantity-types", onListboxValueChange: onListboxValueChange, selectedValue: activeQuantityType }, [...imodeljs_frontend_1.IModelApp.quantityFormatter.quantityTypesRegistry.keys()].map((key) => {
                    const entry = imodeljs_frontend_1.IModelApp.quantityFormatter.quantityTypesRegistry.get(key);
                    const description = entry.description;
                    const label = entry.label;
                    return (React.createElement(ui_core_1.ListboxItem, { key: entry.key, className: "quantity-type-list-entry", value: entry.key },
                        React.createElement("span", { className: "map-source-list-entry-name", title: description }, label)));
                }))),
            React.createElement("div", { className: "right-panel" }, activeFormatterSpec &&
                React.createElement(React.Fragment, null,
                    React.createElement("div", { className: "uifw-quantity-types-right-top" },
                        React.createElement("div", { className: "uifw-quantity-types-right-top-sample" },
                            React.createElement(ui_components_1.FormatSample, { formatSpec: activeFormatterSpec, initialMagnitude: 1234.56, hideLabels: true }))),
                    React.createElement("div", { className: "uifw-quantity-types-formats" },
                        React.createElement(ui_components_1.QuantityFormatPanel, { onFormatChange: handleOnFormatChanged, quantityType: activeQuantityType })),
                    React.createElement("div", { className: "components-button-panel" },
                        React.createElement(ui_core_1.Button, { buttonType: ui_core_1.ButtonType.Blue, onClick: handleOnFormatSave, disabled: !saveEnabled }, setButtonLabel.current),
                        React.createElement(ui_core_1.Button, { buttonType: ui_core_1.ButtonType.Hollow, onClick: handleOnFormatReset, disabled: !clearEnabled }, clearButtonLabel.current)))))));
}
exports.QuantityFormatSettingsPage = QuantityFormatSettingsPage;
function SaveFormatModalDialog({ formatProps, quantityType, onDialogCloseArgs, onDialogClose }) {
    const [isOpen, setIsOpen] = React.useState(true);
    const handleClose = React.useCallback(() => {
        setIsOpen(false);
        ModalDialogManager_1.ModalDialogManager.closeDialog();
        onDialogClose && onDialogClose(onDialogCloseArgs);
    }, [onDialogClose, onDialogCloseArgs]);
    const handleOK = React.useCallback(() => {
        void imodeljs_frontend_1.IModelApp.quantityFormatter.setOverrideFormat(quantityType, formatProps); // eslint-disable-line @typescript-eslint/no-floating-promises
        handleClose();
    }, [formatProps, handleClose, quantityType]);
    const handleCancel = React.useCallback(() => {
        handleClose();
    }, [handleClose]);
    return (React.createElement(ui_core_1.Dialog, { title: "Save Format Changes", opened: isOpen, resizable: false, movable: false, modal: true, buttonCluster: [
            { type: ui_abstract_1.DialogButtonType.Yes, onClick: handleOK },
            { type: ui_abstract_1.DialogButtonType.No, onClick: handleCancel },
        ], onEscape: handleCancel, onClose: handleCancel, onOutsideClick: handleCancel, minHeight: 150, maxHeight: 400, maxWidth: 400, minWidth: 200 },
        React.createElement("div", { className: "modal-dialog2" }, "Do you want to save changes to format before changing to another type?")));
}
//# sourceMappingURL=QuantityFormat.js.map