"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Settings
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUiSettingsManagerEntry = exports.UiSettingsPage = void 0;
// cSpell:ignore configurableui checkmark
const window_settings_svg_sprite_1 = require("@bentley/icons-generic/icons/window-settings.svg?sprite");
require("./UiSettingsPage.scss");
const React = require("react");
const ui_core_1 = require("@bentley/ui-core");
const UiFramework_1 = require("../../UiFramework");
const ThemeManager_1 = require("../../theme/ThemeManager");
const UiShowHideManager_1 = require("../../utils/UiShowHideManager");
const SyncUiEventDispatcher_1 = require("../../syncui/SyncUiEventDispatcher");
const ui_abstract_1 = require("@bentley/ui-abstract");
/** UiSettingsPage displaying the active UI settings. This page lets users set the following settings.
 *
 * - theme - Dark, Light, or based on OS preference.
 * - auto hide - Starts a timer and blanks out ui components that overlay content if there is no mouse movement for a period of time.
 * - drag interaction - If set, toolbar group buttons require a press and drag or a long press to open. In this mode a child action
 * item is shown as the group button and is activated when button is clicked. If a different child item is selected, it becomes the
 * active group button item.
 * - use proximity - Changes the opacity of toolbar from transparent to opaque as the mouse moves closer.
 * - snap widget opacity - triggers an abrupt change from transparent to opaque for tool and navigation widgets, instead of a gradual change based on mouse location.
 * - widget opacity - determines how transparent floating widgets in V2 and all widgets in V1 become when the mouse in not in them.
 * - UI version - if allowed by props, the UI version can be toggled between V1 and V2.
 *
 * @beta
 */
function UiSettingsPage({ allowSettingUiFrameworkVersion }) {
    const themeTitle = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.themeTitle"));
    const themeDescription = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.themeDescription"));
    const autoHideTitle = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.autoHideTitle"));
    const autoHideDescription = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.autoHideDescription"));
    const dragInteractionTitle = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.dragInteractionTitle"));
    const dragInteractionDescription = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.dragInteractionDescription"));
    const useNewUiTitle = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.newUiTitle"));
    const useNewUiDescription = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.newUiDescription"));
    const useProximityOpacityTitle = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.useProximityOpacityTitle"));
    const useProximityOpacityDescription = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.useProximityOpacityDescription"));
    const snapWidgetOpacityTitle = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.snapWidgetOpacityTitle"));
    const snapWidgetOpacityDescription = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.snapWidgetOpacityDescription"));
    const darkLabel = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.dark"));
    const lightLabel = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.light"));
    const systemPreferredLabel = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.systemPreferred"));
    const widgetOpacityTitle = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.widgetOpacityTitle"));
    const widgetOpacityDescription = React.useRef(UiFramework_1.UiFramework.translate("settings.uiSettingsPage.widgetOpacityDescription"));
    const [theme, setTheme] = React.useState(() => UiFramework_1.UiFramework.getColorTheme());
    const [uiVersion, setUiVersion] = React.useState(() => UiFramework_1.UiFramework.uiVersion);
    const [useDragInteraction, setUseDragInteraction] = React.useState(() => UiFramework_1.UiFramework.useDragInteraction);
    const [widgetOpacity, setWidgetOpacity] = React.useState(() => UiFramework_1.UiFramework.getWidgetOpacity());
    const [autoHideUi, setAutoHideUi] = React.useState(() => UiShowHideManager_1.UiShowHideManager.autoHideUi);
    const [useProximityOpacity, setUseProximityOpacity] = React.useState(() => UiShowHideManager_1.UiShowHideManager.useProximityOpacity);
    const [snapWidgetOpacity, setSnapWidgetOpacity] = React.useState(() => UiShowHideManager_1.UiShowHideManager.snapWidgetOpacity);
    React.useEffect(() => {
        const syncIdsOfInterest = ["configurableui:set_theme", "configurableui:set_widget_opacity",
            "configurableui:set-drag-interaction", "configurableui:set-framework-version", SyncUiEventDispatcher_1.SyncUiEventId.ShowHideManagerSettingChange];
        const handleSyncUiEvent = (args) => {
            // istanbul ignore else
            if (syncIdsOfInterest.some((value) => args.eventIds.has(value))) {
                if (UiFramework_1.UiFramework.getColorTheme() !== theme)
                    setTheme(UiFramework_1.UiFramework.getColorTheme());
                if (UiShowHideManager_1.UiShowHideManager.autoHideUi !== autoHideUi)
                    setAutoHideUi(UiShowHideManager_1.UiShowHideManager.autoHideUi);
                if (UiFramework_1.UiFramework.uiVersion !== uiVersion)
                    setUiVersion(UiFramework_1.UiFramework.uiVersion);
                if (UiFramework_1.UiFramework.useDragInteraction !== useDragInteraction)
                    setUseDragInteraction(UiFramework_1.UiFramework.useDragInteraction);
                if (UiFramework_1.UiFramework.getWidgetOpacity() !== widgetOpacity)
                    setWidgetOpacity(UiFramework_1.UiFramework.getWidgetOpacity());
                if (UiShowHideManager_1.UiShowHideManager.autoHideUi !== autoHideUi)
                    setAutoHideUi(UiShowHideManager_1.UiShowHideManager.autoHideUi);
                if (UiShowHideManager_1.UiShowHideManager.useProximityOpacity !== useProximityOpacity)
                    setUseProximityOpacity(UiShowHideManager_1.UiShowHideManager.useProximityOpacity);
                if (UiShowHideManager_1.UiShowHideManager.snapWidgetOpacity !== snapWidgetOpacity)
                    setSnapWidgetOpacity(UiShowHideManager_1.UiShowHideManager.snapWidgetOpacity);
            }
        };
        return SyncUiEventDispatcher_1.SyncUiEventDispatcher.onSyncUiEvent.addListener(handleSyncUiEvent);
    }, [autoHideUi, snapWidgetOpacity, theme, uiVersion, useDragInteraction, useProximityOpacity, widgetOpacity]);
    const defaultThemeOption = { label: systemPreferredLabel.current, value: ThemeManager_1.SYSTEM_PREFERRED_COLOR_THEME };
    const themeOptions = [
        defaultThemeOption,
        { label: lightLabel.current, value: ThemeManager_1.ColorTheme.Light },
        { label: darkLabel.current, value: ThemeManager_1.ColorTheme.Dark },
    ];
    const onThemeChange = React.useCallback((e) => {
        e.preventDefault();
        UiFramework_1.UiFramework.setColorTheme(e.target.value);
    }, []);
    const onAutoHideChange = React.useCallback(async () => {
        UiShowHideManager_1.UiShowHideManager.autoHideUi = !UiShowHideManager_1.UiShowHideManager.autoHideUi;
    }, []);
    const onUseProximityOpacityChange = React.useCallback(async () => {
        UiShowHideManager_1.UiShowHideManager.useProximityOpacity = !UiShowHideManager_1.UiShowHideManager.useProximityOpacity;
    }, []);
    const onSnapWidgetOpacityChange = React.useCallback(async () => {
        UiShowHideManager_1.UiShowHideManager.snapWidgetOpacity = !UiShowHideManager_1.UiShowHideManager.snapWidgetOpacity;
    }, []);
    const onWidgetOpacityChange = React.useCallback(async (values) => {
        // istanbul ignore else
        if (values.length > 0) {
            UiFramework_1.UiFramework.setWidgetOpacity(values[0]);
        }
    }, []);
    const onToggleFrameworkVersion = React.useCallback(async () => {
        UiFramework_1.UiFramework.setUiVersion(UiFramework_1.UiFramework.uiVersion === "2" ? "1" : "2");
    }, []);
    const onToggleDragInteraction = React.useCallback(async () => {
        UiFramework_1.UiFramework.setUseDragInteraction(!UiFramework_1.UiFramework.useDragInteraction);
    }, []);
    const currentTheme = UiFramework_1.UiFramework.getColorTheme();
    return (React.createElement("div", { className: "uifw-settings" },
        React.createElement(SettingsItem, { title: themeTitle.current, description: themeDescription.current, settingUi: React.createElement("div", { "data-testid": "select-theme-container", className: "select-theme-container" },
                React.createElement(ui_core_1.Select, { value: currentTheme, onChange: onThemeChange, options: themeOptions })) }),
        React.createElement(SettingsItem, { title: autoHideTitle.current, description: autoHideDescription.current, settingUi: React.createElement(ui_core_1.Toggle, { isOn: UiShowHideManager_1.UiShowHideManager.autoHideUi, showCheckmark: false, onChange: onAutoHideChange }) }),
        allowSettingUiFrameworkVersion && React.createElement(SettingsItem, { title: useNewUiTitle.current, description: useNewUiDescription.current, settingUi: React.createElement(ui_core_1.Toggle, { isOn: UiFramework_1.UiFramework.uiVersion === "2", showCheckmark: false, onChange: onToggleFrameworkVersion }) }),
        UiFramework_1.UiFramework.uiVersion === "2" && React.createElement(React.Fragment, null,
            React.createElement(SettingsItem, { title: dragInteractionTitle.current, description: dragInteractionDescription.current, settingUi: React.createElement(ui_core_1.Toggle, { isOn: UiFramework_1.UiFramework.useDragInteraction, showCheckmark: false, onChange: onToggleDragInteraction }) }),
            React.createElement(SettingsItem, { title: useProximityOpacityTitle.current, description: useProximityOpacityDescription.current, settingUi: React.createElement(ui_core_1.Toggle, { isOn: UiShowHideManager_1.UiShowHideManager.useProximityOpacity, showCheckmark: false, onChange: onUseProximityOpacityChange }) }),
            React.createElement(SettingsItem, { title: snapWidgetOpacityTitle.current, description: snapWidgetOpacityDescription.current, settingUi: React.createElement(ui_core_1.Toggle, { isOn: UiShowHideManager_1.UiShowHideManager.snapWidgetOpacity, showCheckmark: false, onChange: onSnapWidgetOpacityChange }) })),
        React.createElement(SettingsItem, { title: widgetOpacityTitle.current, description: widgetOpacityDescription.current, settingUi: React.createElement(ui_core_1.Slider, { values: [UiFramework_1.UiFramework.getWidgetOpacity()], step: 0.05, showTooltip: true, onChange: onWidgetOpacityChange, min: 0.20, max: 1.0, showMinMax: true, formatMax: (v) => v.toFixed(1), showTicks: true, getTickValues: () => [.20, .40, .60, .80, 1] }) })));
}
exports.UiSettingsPage = UiSettingsPage;
function SettingsItem(props) {
    const { title, description, settingUi } = props;
    return (React.createElement("div", { className: "uifw-settings-item" },
        React.createElement("div", { className: "panel left-panel" },
            React.createElement("span", { className: "title" }, title),
            React.createElement("span", { className: "description" }, description)),
        React.createElement("div", { className: "panel right-panel" }, settingUi)));
}
/**
 * Return a SettingsTabEntry that can be used to define the available settings that can be set for an application.
 * @param itemPriority - Used to define the order of the entry in the Settings Stage
 * @beta
 */
function getUiSettingsManagerEntry(itemPriority, allowSettingUiFrameworkVersion) {
    return {
        itemPriority, tabId: "uifw:UiSettings",
        label: UiFramework_1.UiFramework.translate("settings.uiSettingsPage.label"),
        icon: ui_abstract_1.IconSpecUtilities.createSvgIconSpec(window_settings_svg_sprite_1.default),
        page: React.createElement(UiSettingsPage, { allowSettingUiFrameworkVersion: !!allowSettingUiFrameworkVersion }),
        isDisabled: false,
        tooltip: UiFramework_1.UiFramework.translate("settings.uiSettingsPage.tooltip"),
    };
}
exports.getUiSettingsManagerEntry = getUiSettingsManagerEntry;
//# sourceMappingURL=UiSettingsPage.js.map