"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Frontstage
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.setPanelSize = exports.toPanelSide = exports.StagePanelZoneDef = exports.StagePanelZonesDef = exports.StagePanelDef = exports.PanelSizeChangedEvent = exports.PanelStateChangedEvent = exports.StagePanelState = void 0;
const immer_1 = require("immer");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const WidgetDef_1 = require("../widgets/WidgetDef");
const WidgetHost_1 = require("../widgets/WidgetHost");
const Zone_1 = require("../zones/Zone");
const UiFramework_1 = require("../UiFramework");
/** Enum for StagePanel state.
 * @public
 */
var StagePanelState;
(function (StagePanelState) {
    StagePanelState[StagePanelState["Off"] = 0] = "Off";
    StagePanelState[StagePanelState["Minimized"] = 1] = "Minimized";
    StagePanelState[StagePanelState["Open"] = 2] = "Open";
    StagePanelState[StagePanelState["Popup"] = 3] = "Popup";
})(StagePanelState = exports.StagePanelState || (exports.StagePanelState = {}));
/** Widget State Changed Event class.
 * @beta
 */
class PanelStateChangedEvent extends ui_core_1.UiEvent {
}
exports.PanelStateChangedEvent = PanelStateChangedEvent;
/** @internal */
class PanelSizeChangedEvent extends ui_core_1.UiEvent {
}
exports.PanelSizeChangedEvent = PanelSizeChangedEvent;
/**
 * A StagePanelDef represents each Stage Panel within a Frontstage.
 * @public
 */
class StagePanelDef extends WidgetHost_1.WidgetHost {
    /** Constructor for PanelDef.
     */
    constructor() {
        super();
        this._panelState = StagePanelState.Open;
        this._defaultState = StagePanelState.Open;
        this._resizable = true;
        this._pinned = true;
        this._location = ui_abstract_1.StagePanelLocation.Left;
        this._panelZones = new StagePanelZonesDef();
    }
    /** @internal */
    get maxSizeSpec() { return this._maxSizeSpec; }
    /** @internal */
    get minSize() { return this._minSize; }
    /** Default size of the panel */
    get size() { return this._size; }
    set size(size) {
        if (this._size === size)
            return;
        // istanbul ignore else
        if (UiFramework_1.UiFramework.uiVersion === "2") {
            const frontstageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
            // istanbul ignore else
            if (frontstageDef && frontstageDef.nineZoneState) {
                const side = toPanelSide(this.location);
                frontstageDef.nineZoneState = exports.setPanelSize(frontstageDef.nineZoneState, side, size);
                const panel = frontstageDef.nineZoneState.panels[side];
                if (panel.size === this._size)
                    return;
                size = panel.size;
            }
        }
        this._size = size;
        FrontstageManager_1.FrontstageManager.onPanelSizeChangedEvent.emit({
            panelDef: this,
            size,
        });
    }
    /** Indicates whether the panel is resizable. */
    // istanbul ignore next
    get resizable() { return this._resizable; }
    /** Indicates whether the panel is pinned. */
    get pinned() { return this._pinned; }
    /** Any application data to attach to this Panel. */
    get applicationData() { return this._applicationData; }
    /** Location of panel. */
    get location() { return this._location; }
    /** Panel state. Defaults to PanelState.Open. */
    get panelState() {
        return this._panelState;
    }
    set panelState(panelState) {
        if (panelState === this._panelState)
            return;
        const frontstageDef = FrontstageManager_1.FrontstageManager.activeFrontstageDef;
        if (UiFramework_1.UiFramework.uiVersion === "2" && frontstageDef && frontstageDef.nineZoneState) {
            const side = toPanelSide(this.location);
            frontstageDef.nineZoneState = immer_1.default(frontstageDef.nineZoneState, (nineZone) => {
                const panel = nineZone.panels[side];
                switch (panelState) {
                    case StagePanelState.Minimized: {
                        panel.collapsed = true;
                        break;
                    }
                    case StagePanelState.Open: {
                        panel.collapsed = false;
                        break;
                    }
                    case StagePanelState.Off: {
                        panel.collapsed = true;
                        break;
                    }
                }
            });
        }
        this._panelState = panelState;
        FrontstageManager_1.FrontstageManager.onPanelStateChangedEvent.emit({
            panelDef: this,
            panelState,
        });
    }
    /** @internal */
    get defaultState() { return this._defaultState; }
    /** @internal */
    get defaultSize() { return this._defaultSize; }
    /** Panel zones.
     * @internal
     */
    get panelZones() {
        return this._panelZones;
    }
    /** @internal */
    initializeFromProps(props, panelLocation) {
        if (panelLocation !== undefined)
            this._location = panelLocation;
        if (!props)
            return;
        this._size = props.size;
        this._defaultSize = props.size;
        this._maxSizeSpec = props.maxSize;
        this._minSize = props.minSize;
        if (panelLocation !== undefined)
            this._location = panelLocation;
        if (props.defaultState !== undefined) {
            this._panelState = props.defaultState;
            this._defaultState = props.defaultState;
        }
        this._resizable = props.resizable;
        if (props.pinned !== undefined)
            this._pinned = props.pinned;
        if (props.applicationData !== undefined)
            this._applicationData = props.applicationData;
        if (props.panelZones) {
            this._panelZones.initializeFromProps(props.panelZones, this._location);
        }
        if (props.widgets) {
            props.widgets.forEach((widgetNode, index) => {
                const stableId = `uifw-sp-${ui_abstract_1.StagePanelLocation[this._location]}-${index}`;
                const stableProps = Zone_1.getStableWidgetProps(widgetNode.props, stableId);
                const widgetDef = new WidgetDef_1.WidgetDef(stableProps);
                this.addWidgetDef(widgetDef);
            });
        }
    }
    /** Gets the list of Widgets. */
    get widgetDefs() {
        const widgetDefs = [...super.widgetDefs];
        for (const [, panelZone] of this.panelZones) {
            widgetDefs.push(...panelZone.widgetDefs);
        }
        return widgetDefs;
    }
    /** Gets the list of Widgets (w/o StagePanelZone widgets).
     * @internal
     */
    get panelWidgetDefs() {
        return super.widgetDefs;
    }
    /** @internal */
    updateDynamicWidgetDefs(stageId, stageUsage, location, _section, widgetDefs) {
        this.panelZones.start.updateDynamicWidgetDefs(stageId, stageUsage, location, ui_abstract_1.StagePanelSection.Start, widgetDefs);
        this.panelZones.middle.updateDynamicWidgetDefs(stageId, stageUsage, location, ui_abstract_1.StagePanelSection.Middle, widgetDefs);
        this.panelZones.end.updateDynamicWidgetDefs(stageId, stageUsage, location, ui_abstract_1.StagePanelSection.End, widgetDefs);
    }
}
exports.StagePanelDef = StagePanelDef;
const stagePanelZoneDefKeys = ["start", "middle", "end"];
/** @internal */
class StagePanelZonesDef {
    constructor() {
        this._start = new StagePanelZoneDef();
        this._middle = new StagePanelZoneDef();
        this._end = new StagePanelZoneDef();
    }
    get start() {
        return this._start;
    }
    get middle() {
        return this._middle;
    }
    get end() {
        return this._end;
    }
    /** @internal */
    initializeFromProps(props, panelLocation) {
        if (props.start) {
            this.start.initializeFromProps(props.start, panelLocation, "start");
        }
        if (props.middle) {
            this.middle.initializeFromProps(props.middle, panelLocation, "middle");
        }
        if (props.end) {
            this.end.initializeFromProps(props.end, panelLocation, "end");
        }
    }
    /** @internal */
    *[Symbol.iterator]() {
        for (const key of stagePanelZoneDefKeys) {
            const value = this[key];
            yield [key, value];
        }
        return undefined;
    }
}
exports.StagePanelZonesDef = StagePanelZonesDef;
/** @internal */
class StagePanelZoneDef extends WidgetHost_1.WidgetHost {
    /** @internal */
    initializeFromProps(props, panelLocation, panelZone) {
        props.widgets.forEach((widgetNode, index) => {
            const stableId = `uifw-spz-${ui_abstract_1.StagePanelLocation[panelLocation]}-${panelZone}-${index}`;
            const stableProps = Zone_1.getStableWidgetProps(widgetNode.props, stableId);
            const widgetDef = new WidgetDef_1.WidgetDef(stableProps);
            this.addWidgetDef(widgetDef);
        });
    }
}
exports.StagePanelZoneDef = StagePanelZoneDef;
/** @internal */
function toPanelSide(location) {
    switch (location) {
        case ui_abstract_1.StagePanelLocation.Bottom:
        case ui_abstract_1.StagePanelLocation.BottomMost:
            return "bottom";
        case ui_abstract_1.StagePanelLocation.Left:
            return "left";
        case ui_abstract_1.StagePanelLocation.Right:
            return "right";
        case ui_abstract_1.StagePanelLocation.Top:
        case ui_abstract_1.StagePanelLocation.TopMost:
            return "top";
    }
}
exports.toPanelSide = toPanelSide;
/** @internal */
exports.setPanelSize = immer_1.default((nineZone, side, size) => {
    const panel = nineZone.panels[side];
    panel.size = size === undefined ? size : Math.min(Math.max(size, panel.minSize), panel.maxSize);
});
//# sourceMappingURL=StagePanelDef.js.map