"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module StatusBar
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatusBarContext = exports.StatusBarRightSection = exports.StatusBarCenterSection = exports.StatusBarLeftSection = exports.StatusBarSpaceBetween = exports.StatusBar = void 0;
require("./StatusBar.scss");
const React = require("react");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const MessageManager_1 = require("../messages/MessageManager");
const SafeAreaContext_1 = require("../safearea/SafeAreaContext");
const UiShowHideManager_1 = require("../utils/UiShowHideManager");
const StatusMessagesContainer_1 = require("../messages/StatusMessagesContainer");
/** Status Bar React component.
 * @public
 */
class StatusBar extends React.Component {
    /** @internal */
    constructor(props) {
        super(props);
        this._handleMessageAddedEvent = (_args) => {
            this.setState({ messages: MessageManager_1.MessageManager.activeMessageManager.messages });
        };
        /** Respond to clearing the message list */
        this._handleMessagesUpdatedEvent = () => {
            this.setState({ messages: MessageManager_1.MessageManager.activeMessageManager.messages });
        };
        /**
         * Sets state of the status bar to updated values reflecting activity progress.
         * @param args  New values to set for ActivityMessage
         */
        this._handleActivityMessageUpdatedEvent = (args) => {
            this.setState((prevState) => ({
                activityMessageInfo: args,
                isActivityMessageVisible: args.restored ? true : prevState.isActivityMessageVisible,
            }));
        };
        /**
         * Hides ActivityMessage after cancellation
         */
        this._handleActivityMessageCancelledEvent = () => {
            this.setState({
                isActivityMessageVisible: false,
            });
        };
        /**
         * Ends canceled process and dismisses ActivityMessage
         */
        this._cancelActivityMessage = () => {
            MessageManager_1.MessageManager.endActivityMessage(false);
            this._dismissActivityMessage();
        };
        /**
         * Dismisses ActivityMessage
         */
        this._dismissActivityMessage = () => {
            this.setState({
                isActivityMessageVisible: false,
            });
        };
        this._handleOpenWidget = (openWidget) => {
            this.setState({
                openWidget,
            });
        };
        this._closeMessage = (id) => {
            MessageManager_1.MessageManager.activeMessageManager.remove(id);
            MessageManager_1.MessageManager.updateMessages();
        };
        this._handleToastTargetRef = (toastTarget) => {
            this.setState({ toastTarget });
        };
        this.state = {
            openWidget: null,
            messages: MessageManager_1.MessageManager.activeMessageManager.messages,
            activityMessageInfo: undefined,
            isActivityMessageVisible: false,
            toastTarget: null,
        };
    }
    static severityToStatus(severity) {
        let status = ui_ninezone_1.Status.Information;
        switch (severity) {
            case ui_core_1.MessageSeverity.None:
                status = ui_ninezone_1.Status.Success;
                break;
            case ui_core_1.MessageSeverity.Information:
                status = ui_ninezone_1.Status.Information;
                break;
            case ui_core_1.MessageSeverity.Warning:
                status = ui_ninezone_1.Status.Warning;
                break;
            case ui_core_1.MessageSeverity.Error:
            case ui_core_1.MessageSeverity.Fatal:
                status = ui_ninezone_1.Status.Error;
                break;
        }
        return status;
    }
    render() {
        let footerSections = null;
        const widgetControl = this.props.widgetControl;
        // istanbul ignore else
        if (widgetControl && widgetControl.getReactNode) {
            footerSections = widgetControl.getReactNode({
                isInFooterMode: this.props.isInFooterMode,
                openWidget: this.state.openWidget,
                toastTargetRef: this._handleToastTargetRef,
                onOpenWidget: this._handleOpenWidget,
            });
        }
        return (React.createElement(exports.StatusBarContext.Provider, { value: {
                isInFooterMode: this.props.isInFooterMode,
                openWidget: this.state.openWidget,
                toastTargetRef: this._handleToastTargetRef,
                onOpenWidget: this._handleOpenWidget,
            } },
            React.createElement(SafeAreaContext_1.SafeAreaContext.Consumer, null, (safeAreaInsets) => (React.createElement(ui_ninezone_1.Footer, { className: this.props.className, messages: this.getFooterMessages(), isInFooterMode: this.props.isInFooterMode, onMouseEnter: UiShowHideManager_1.UiShowHideManager.handleWidgetMouseEnter, safeAreaInsets: safeAreaInsets, style: this.props.style }, footerSections)))));
    }
    componentDidMount() {
        MessageManager_1.MessageManager.onMessageAddedEvent.addListener(this._handleMessageAddedEvent);
        MessageManager_1.MessageManager.onActivityMessageUpdatedEvent.addListener(this._handleActivityMessageUpdatedEvent);
        MessageManager_1.MessageManager.onActivityMessageCancelledEvent.addListener(this._handleActivityMessageCancelledEvent);
        MessageManager_1.MessageManager.onMessagesUpdatedEvent.addListener(this._handleMessagesUpdatedEvent);
        MessageManager_1.MessageManager.activeMessageManager.initialize();
        MessageManager_1.MessageManager.updateMessages();
    }
    componentWillUnmount() {
        MessageManager_1.MessageManager.onMessageAddedEvent.removeListener(this._handleMessageAddedEvent);
        MessageManager_1.MessageManager.onActivityMessageUpdatedEvent.removeListener(this._handleActivityMessageUpdatedEvent);
        MessageManager_1.MessageManager.onActivityMessageCancelledEvent.removeListener(this._handleActivityMessageCancelledEvent);
        MessageManager_1.MessageManager.onMessagesUpdatedEvent.removeListener(this._handleMessagesUpdatedEvent);
    }
    getFooterMessages() {
        if (!(this.state.activityMessageInfo && this.state.isActivityMessageVisible) && this.state.messages.length === 0)
            return null;
        return (React.createElement(StatusMessagesContainer_1.StatusMessagesContainer, { messages: this.state.messages, activityMessageInfo: this.state.activityMessageInfo, isActivityMessageVisible: this.state.isActivityMessageVisible, toastTarget: this.state.toastTarget, closeMessage: this._closeMessage, cancelActivityMessage: this._cancelActivityMessage, dismissActivityMessage: this._dismissActivityMessage }));
    }
}
exports.StatusBar = StatusBar;
/** StatusBar With Space Between Items React functional component
 * @public
 */
function StatusBarSpaceBetween(props) {
    const { className } = props, divProps = __rest(props, ["className"]);
    return React.createElement(ui_core_1.Div, Object.assign({}, divProps, { mainClassName: className ? className : "uifw-statusbar-space-between" }));
}
exports.StatusBarSpaceBetween = StatusBarSpaceBetween;
/** StatusBar Left Section React functional component
 * @public
 */
function StatusBarLeftSection(props) {
    const { className } = props, divProps = __rest(props, ["className"]);
    return React.createElement(ui_core_1.Div, Object.assign({}, divProps, { mainClassName: className ? className : "uifw-statusbar-left" }));
}
exports.StatusBarLeftSection = StatusBarLeftSection;
/** StatusBar Center Section React functional component
 * @public
 */
function StatusBarCenterSection(props) {
    const { className } = props, divProps = __rest(props, ["className"]);
    return React.createElement(ui_core_1.Div, Object.assign({}, divProps, { mainClassName: className ? className : "uifw-statusbar-center" }));
}
exports.StatusBarCenterSection = StatusBarCenterSection;
/** StatusBar Right Section React functional component
 * @public
 */
function StatusBarRightSection(props) {
    const { className } = props, divProps = __rest(props, ["className"]);
    return React.createElement(ui_core_1.Div, Object.assign({}, divProps, { mainClassName: className ? className : "uifw-statusbar-right" }));
}
exports.StatusBarRightSection = StatusBarRightSection;
/** Context providing values for StatusFieldProps and MessageCenterFieldProps
 *  @internal
 */
exports.StatusBarContext = React.createContext({
    isInFooterMode: true,
    onOpenWidget: /* istanbul ignore next */ () => { },
    openWidget: "",
    toastTargetRef: { current: null },
});
//# sourceMappingURL=StatusBar.js.map