"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Notification
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageCenterField = void 0;
const React = require("react");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const MessageManager_1 = require("../messages/MessageManager");
const MessageSpan_1 = require("../messages/MessageSpan");
const UiFramework_1 = require("../UiFramework");
/** Enum for the [[MessageCenterField]] active tab
 * @internal
 */
var MessageCenterActiveTab;
(function (MessageCenterActiveTab) {
    MessageCenterActiveTab[MessageCenterActiveTab["AllMessages"] = 0] = "AllMessages";
    MessageCenterActiveTab[MessageCenterActiveTab["Problems"] = 1] = "Problems";
})(MessageCenterActiveTab || (MessageCenterActiveTab = {}));
/** Message Center Field React component.
 * @public
 */
class MessageCenterField extends React.Component {
    constructor(p) {
        super(p);
        this._indicator = React.createRef();
        this._title = UiFramework_1.UiFramework.translate("messageCenter.messages");
        this.state = {
            activeTab: MessageCenterActiveTab.AllMessages,
            target: null,
            messageCount: MessageManager_1.MessageManager.messages.length,
        };
        this._handleMessagesUpdatedEvent = () => {
            // istanbul ignore else
            if (this._unloadMessagesUpdatedHandler)
                this.setState({ messageCount: MessageManager_1.MessageManager.messages.length });
        };
        this._handleOpenMessageCenterEvent = () => {
            this.setOpenWidget(this._className);
        };
        this._handleTargetRef = (target) => {
            if (typeof this.props.targetRef === "function")
                this.props.targetRef(target);
            else if (this.props.targetRef)
                this.props.targetRef.current = target;
            this.setState({ target });
        };
        this._handleClose = () => {
            this.setOpenWidget(null);
        };
        this._handleOutsideClick = (e) => {
            if (!this._indicator.current ||
                !(e.target instanceof Node) ||
                this._indicator.current.contains(e.target))
                return;
            this._handleClose();
        };
        this._handleMessageIndicatorClick = () => {
            const isOpen = this.props.openWidget === this._className;
            if (isOpen)
                this.setOpenWidget(null);
            else
                this.setOpenWidget(this._className);
        };
        this._changeActiveTab = (tab) => {
            this.setState({ activeTab: tab });
        };
        const instance = this.constructor;
        this._className = instance.name;
    }
    /** @internal */
    componentDidMount() {
        this._unloadMessagesUpdatedHandler = MessageManager_1.MessageManager.onMessagesUpdatedEvent.addListener(this._handleMessagesUpdatedEvent, this);
        this._removeOpenMessagesCenterHandler = MessageManager_1.MessageManager.onOpenMessageCenterEvent.addListener(this._handleOpenMessageCenterEvent, this);
    }
    /** @internal */
    componentWillUnmount() {
        // istanbul ignore else
        if (this._unloadMessagesUpdatedHandler) {
            this._unloadMessagesUpdatedHandler();
            this._unloadMessagesUpdatedHandler = undefined;
        }
        // istanbul ignore else
        if (this._removeOpenMessagesCenterHandler) {
            this._removeOpenMessagesCenterHandler();
            this._removeOpenMessagesCenterHandler = undefined;
        }
    }
    render() {
        const tooltip = `${this.state.messageCount} ${this._title}`;
        const footerMessages = (React.createElement(React.Fragment, null,
            React.createElement("div", { className: this.props.className, style: this.props.style, title: tooltip, ref: this._handleTargetRef },
                React.createElement(ui_ninezone_1.MessageCenter, { indicatorRef: this._indicator, isInFooterMode: this.props.isInFooterMode, label: this.props.isInFooterMode ? this._title : undefined, onClick: this._handleMessageIndicatorClick }, this.state.messageCount.toString())),
            React.createElement(ui_ninezone_1.FooterPopup, { isOpen: this.props.openWidget === this._className, onClose: this._handleClose, onOutsideClick: this._handleOutsideClick, target: this.state.target },
                React.createElement(ui_ninezone_1.MessageCenterDialog, { prompt: UiFramework_1.UiFramework.translate("messageCenter.prompt"), tabs: React.createElement(React.Fragment, null,
                        React.createElement(ui_ninezone_1.MessageCenterTab, { isActive: this.state.activeTab === MessageCenterActiveTab.AllMessages, onClick: () => this._changeActiveTab(MessageCenterActiveTab.AllMessages) }, UiFramework_1.UiFramework.translate("messageCenter.all")),
                        React.createElement(ui_ninezone_1.MessageCenterTab, { isActive: this.state.activeTab === MessageCenterActiveTab.Problems, onClick: () => this._changeActiveTab(MessageCenterActiveTab.Problems) }, UiFramework_1.UiFramework.translate("messageCenter.errors"))), title: this._title }, this.getMessages()))));
        return footerMessages;
    }
    getMessages() {
        const messages = MessageManager_1.MessageManager.messages.slice(0).reverse();
        const tabRows = new Array();
        messages.forEach((details, index) => {
            /* istanbul ignore else */
            if (this.state.activeTab === MessageCenterActiveTab.AllMessages || this.isProblemStatus(details.priority)) {
                const iconClassName = MessageManager_1.MessageManager.getIconClassName(details);
                const message = details.briefMessage;
                tabRows.push(React.createElement(ui_ninezone_1.MessageCenterMessage, { key: index.toString(), icon: React.createElement("i", { className: iconClassName }) },
                    React.createElement(MessageSpan_1.MessageSpan, { message: message }),
                    details.detailedMessage &&
                        React.createElement(React.Fragment, null,
                            React.createElement("br", null),
                            React.createElement(MessageSpan_1.MessageSpan, { className: "uicore-text-small", message: details.detailedMessage }))));
            }
        });
        return tabRows;
    }
    isProblemStatus(priority) {
        // See priority values in DgnPlatform defined in NotificationManager
        if (priority === imodeljs_frontend_1.OutputMessagePriority.Error || priority === imodeljs_frontend_1.OutputMessagePriority.Fatal)
            return true;
        return false;
    }
    setOpenWidget(openWidget) {
        // istanbul ignore else
        if (this.props.onOpenWidget)
            this.props.onOpenWidget(openWidget);
    }
}
exports.MessageCenterField = MessageCenterField;
//# sourceMappingURL=MessageCenter.js.map