"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module StatusBar
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnapModeField = void 0;
const React = require("react");
const react_redux_1 = require("react-redux");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const state_1 = require("../configurableui/state");
const UiFramework_1 = require("../UiFramework");
/**
 * Snap Mode Field React component. This component is designed to be specified in a status bar definition. It will
 * display the active snap mode that AccuSnap will use and allow the user to select a new snap mode.
 */
class SnapModeFieldComponent extends React.Component {
    constructor(props) {
        super(props);
        this._snapModeFieldArray = [
            { label: UiFramework_1.UiFramework.translate("snapModeField.keypoint"), value: imodeljs_frontend_1.SnapMode.NearestKeypoint, iconName: "snaps" },
            { label: UiFramework_1.UiFramework.translate("snapModeField.intersection"), value: imodeljs_frontend_1.SnapMode.Intersection, iconName: "snaps-intersection" },
            { label: UiFramework_1.UiFramework.translate("snapModeField.center"), value: imodeljs_frontend_1.SnapMode.Center, iconName: "snaps-center" },
            { label: UiFramework_1.UiFramework.translate("snapModeField.nearest"), value: imodeljs_frontend_1.SnapMode.Nearest, iconName: "snaps-nearest" },
            { label: UiFramework_1.UiFramework.translate("snapModeField.origin"), value: imodeljs_frontend_1.SnapMode.Origin, iconName: "snaps-origin" },
            { label: UiFramework_1.UiFramework.translate("snapModeField.midpoint"), value: imodeljs_frontend_1.SnapMode.MidPoint, iconName: "snaps-midpoint" },
            { label: UiFramework_1.UiFramework.translate("snapModeField.bisector"), value: imodeljs_frontend_1.SnapMode.Bisector, iconName: "snaps-bisector" },
        ];
        this._indicator = React.createRef();
        this._title = UiFramework_1.UiFramework.translate("snapModeField.snapMode");
        this.state = {
            target: null,
        };
        this._handleTargetRef = (target) => {
            this.setState({ target });
        };
        this._handleClose = () => {
            this.setOpenWidget(null);
        };
        this._handleOutsideClick = (e) => {
            if (!this._indicator.current ||
                !(e.target instanceof Node) ||
                this._indicator.current.contains(e.target))
                return;
            this._handleClose();
        };
        /** Called when user clicks on a Snap Mode entry in the pop-up window. */
        this._handleSnapModeFieldClick = (snapModeField) => {
            this.props.setSnapMode(snapModeField);
        };
        /** Called when user click on field in status bar which triggers the pop-up to open. */
        this._handleSnapModeIndicatorClick = () => {
            const isOpen = this.props.openWidget === this._className;
            if (isOpen)
                this.setOpenWidget(null);
            else
                this.setOpenWidget(this._className);
        };
        const instance = this.constructor;
        this._className = instance.name;
    }
    /** Return icon class name for a specific snapMode. */
    getSnapModeIconNameFromMode(snapMode) {
        for (const mode of this._snapModeFieldArray) {
            if (mode.value === snapMode)
                return mode.iconName;
        }
        /* istanbul ignore else */
        if (snapMode > 0)
            return "snaps-multione";
        /* istanbul ignore next */
        return "placeholder";
    }
    /** Standard React render method. */
    render() {
        return (React.createElement(React.Fragment, null,
            React.createElement("div", { ref: this._handleTargetRef, className: this.props.className, style: this.props.style, title: this._title },
                React.createElement(ui_ninezone_1.SnapMode, { icon: React.createElement("i", { className: `icon icon-${this.getSnapModeIconNameFromMode(this.props.snapMode)}` }), indicatorRef: this._indicator, isInFooterMode: this.props.isInFooterMode, onClick: this._handleSnapModeIndicatorClick }, this.props.isInFooterMode ? this._title : /* istanbul ignore next */ undefined)),
            React.createElement(ui_ninezone_1.FooterPopup, { contentType: ui_ninezone_1.FooterPopupContentType.Panel, isOpen: this.props.openWidget === this._className, onClose: this._handleClose, onOutsideClick: this._handleOutsideClick, target: this.state.target },
                React.createElement(ui_ninezone_1.SnapModePanel, { title: this._title }, this.getSnapEntries()))));
    }
    /** Return array of SnapRow elements, one for each support snap mode. This array will populate the pop-up used
     * to select a SnapMode.
     */
    getSnapEntries() {
        return this._snapModeFieldArray.map((item, index) => {
            return (React.createElement(ui_ninezone_1.Snap, { key: `SM_${index}`, onClick: () => this._handleSnapModeFieldClick(item.value), isActive: (this.props.snapMode & item.value) === item.value, icon: React.createElement("i", { className: `icon icon-${item.iconName}` }) }, item.label));
        });
    }
    /** Opens the pop-up window. */
    setOpenWidget(openWidget) {
        // istanbul ignore else
        if (this.props.onOpenWidget)
            this.props.onOpenWidget(openWidget);
    }
}
// Used by Redux to map dispatch functions to props entry. This requires SnapModeFieldProps interface above to include a setSnapMode entry */
const mapDispatch = {
    setSnapMode: state_1.ConfigurableUiActions.setSnapMode,
};
/** Function used by Redux to map state data in Redux store to props that are used to render this component. */
function mapStateToProps(state) {
    const frameworkState = state[UiFramework_1.UiFramework.frameworkStateKey]; // since app sets up key, don't hard-code name
    /* istanbul ignore next */
    if (!frameworkState)
        return undefined;
    return { snapMode: frameworkState.configurableUiState.snapMode };
}
// we declare the variable and export that rather than using export default.
/**
 * Snap Mode Field React component. This component is designed to be specified in a status bar definition. It will
 * display the active snap mode that AccuSnap will use and allow the user to select a new snap mode.
 * This Field React component is Redux connected.
 * @public
 */ // eslint-disable-next-line @typescript-eslint/naming-convention
exports.SnapModeField = react_redux_1.connect(mapStateToProps, mapDispatch)(SnapModeFieldComponent);
//# sourceMappingURL=SnapMode.js.map