"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module StatusBar
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewAttributesStatusField = void 0;
require("./ViewAttributes.scss");
const React = require("react");
const imodeljs_common_1 = require("@bentley/imodeljs-common");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const UiFramework_1 = require("../UiFramework");
const Indicator_1 = require("./Indicator");
/** Widget for showing Checkboxes for View Attributes
 * @beta
 */
class ViewAttributesStatusField extends React.Component {
    constructor(props) {
        super(props);
        this._title = UiFramework_1.UiFramework.translate("listTools.viewAttributes");
        /** Handle opening/closing the dialog */
        this._handleIndicatorClick = () => {
            this.updateState();
            const isOpen = this.props.openWidget === this._className;
            if (isOpen)
                this.setOpenWidget(null);
            else
                this.setOpenWidget(this._className);
        };
        // istanbul ignore next
        this._handleViewFlagClick = (flagName) => {
            if (imodeljs_frontend_1.IModelApp.viewManager.selectedView) {
                const props = imodeljs_frontend_1.IModelApp.viewManager.selectedView.viewFlags.toJSON();
                props[flagName] = props[flagName] === undefined ? true : !props[flagName];
                const viewFlags = imodeljs_common_1.ViewFlags.fromJSON(props);
                imodeljs_frontend_1.IModelApp.viewManager.selectedView.viewFlags = viewFlags;
                this.updateState();
            }
        };
        this._handleToggleCamera = () => {
            imodeljs_frontend_1.IModelApp.tools.run("View.ToggleCamera", imodeljs_frontend_1.IModelApp.viewManager.selectedView);
            this.updateState();
        };
        this._handleTargetRef = (target) => {
            this.setState({ target });
        };
        this._handleClose = () => {
            this.setOpenWidget(null);
        };
        this.state = {
            cameraOn: false,
            viewFlags: {},
            target: null,
        };
        this._className = this.constructor.name;
    }
    componentDidMount() {
        this.updateState();
    }
    // istanbul ignore next
    updateState() {
        if (imodeljs_frontend_1.IModelApp.viewManager.selectedView) {
            const viewFlags = Object.assign({}, imodeljs_frontend_1.IModelApp.viewManager.selectedView.view.viewFlags.toJSON());
            const cameraOn = imodeljs_frontend_1.IModelApp.viewManager.selectedView.isCameraOn;
            this.setState({
                viewFlags,
                cameraOn,
            });
        }
    }
    // istanbul ignore next
    stylizeName(name) {
        name = name.charAt(0).toUpperCase() + name.slice(1);
        name = name.replace(/([A-Z])/g, " $1").trim();
        return name;
    }
    getViewFlagItem(flagName, value, labelKey) {
        return React.createElement(ui_core_1.Checkbox, { key: flagName, label: labelKey ? imodeljs_frontend_1.IModelApp.i18n.translate(labelKey) : /* istanbul ignore next */ this.stylizeName(flagName), onClick: () => this._handleViewFlagClick(flagName), defaultChecked: value });
    }
    getFlagState(flagName) {
        return this.state.viewFlags.hasOwnProperty(flagName) ? /* istanbul ignore next */ this.state.viewFlags[flagName] : false;
    }
    getToggleCameraItem() {
        return React.createElement(ui_core_1.Checkbox, { key: "toggleCamera", label: imodeljs_frontend_1.IModelApp.i18n.translate("UiFramework:listTools.camera"), onClick: this._handleToggleCamera, defaultChecked: this.state.cameraOn });
    }
    getViewFlags() {
        const items = [];
        items.push(this.getViewFlagItem("acs", this.getFlagState("acs"), "UiFramework:listTools.acs"));
        items.push(this.getToggleCameraItem());
        items.push(this.getViewFlagItem("noConstruct", !this.getFlagState("noConstruct"), "UiFramework:listTools.constructions"));
        items.push(this.getViewFlagItem("hidEdges", this.getFlagState("hidEdges"), "UiFramework:listTools.hidEdges"));
        items.push(this.getViewFlagItem("monochrome", this.getFlagState("monochrome"), "UiFramework:listTools.monochrome"));
        items.push(this.getViewFlagItem("visEdges", this.getFlagState("visEdges"), "UiFramework:listTools.visEdges"));
        items.push(this.getViewFlagItem("ambientOcclusion", this.getFlagState("ambientOcclusion"), "UiFramework:listTools.ambientOcclusion"));
        items.push(this.getViewFlagItem("shadows", this.getFlagState("shadows"), "UiFramework:listTools.shadows"));
        items.push(this.getViewFlagItem("backgroundMap", this.getFlagState("backgroundMap"), "UiFramework:listTools.backgroundMap"));
        return React.createElement("div", { className: "uifw-view-attributes-contents" }, items);
    }
    render() {
        const isOpen = this.props.openWidget === this._className;
        return (React.createElement(React.Fragment, null,
            React.createElement("div", { ref: this._handleTargetRef, title: this._title },
                React.createElement(Indicator_1.Indicator, { iconName: "icon-window-settings", onClick: this._handleIndicatorClick, opened: isOpen, isInFooterMode: this.props.isInFooterMode })),
            React.createElement(ui_ninezone_1.FooterPopup, { target: this.state.target, onClose: this._handleClose, isOpen: isOpen },
                React.createElement(ui_ninezone_1.Dialog, { titleBar: React.createElement(ui_ninezone_1.TitleBar, { title: this._title }) }, this.getViewFlags()))));
    }
    /** Opens the pop-up window. */
    setOpenWidget(openWidget) {
        // istanbul ignore else
        if (this.props.onOpenWidget)
            this.props.onOpenWidget(openWidget);
    }
}
exports.ViewAttributesStatusField = ViewAttributesStatusField;
//# sourceMappingURL=ViewAttributes.js.map