"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module SyncUi
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyncUiEventDispatcher = exports.SyncUiEvent = exports.SyncUiEventId = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const presentation_common_1 = require("@bentley/presentation-common");
const presentation_frontend_1 = require("@bentley/presentation-frontend");
// cSpell:ignore configurableui
const ui_core_1 = require("@bentley/ui-core");
const Backstage_1 = require("../backstage/Backstage");
const ContentViewManager_1 = require("../content/ContentViewManager");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const SessionState_1 = require("../redux/SessionState");
const UiFramework_1 = require("../UiFramework");
const Workflow_1 = require("../workflow/Workflow");
// cSpell:ignore activecontentchanged, activitymessageupdated, activitymessagecancelled, backstagecloseevent, backstageevent, contentlayoutactivated, contentcontrolactivated,
// cSpell:ignore elementtooltipchanged, frontstageactivated, inputfieldmessageadded, inputfieldmessageremoved, modalfrontstagechanged, modaldialogchanged
// cSpell:ignore navigationaidactivated, notificationmessageadded, toolactivated, taskactivated, widgetstatechanged, workflowactivated frontstageactivating
// cSpell:ignore frontstageready activeviewportchanged selectionsetchanged presentationselectionchanged viewstatechanged
// cSpell:ignore accudrawcompassmodechanged accudrawfieldlockchanged accudrawrotationchanged uisettingschanged
/** Event Id used to sync UI components. Used to refresh visibility or enable state of control.
 * @public
 */
var SyncUiEventId;
(function (SyncUiEventId) {
    /** AccuDraw compass mode has changed. */
    SyncUiEventId["AccuDrawCompassModeChanged"] = "accudrawcompassmodechanged";
    /** AccuDraw rotation has changed. */
    SyncUiEventId["AccuDrawRotationChanged"] = "accudrawrotationchanged";
    /** The active content as maintained by the ContentViewManager has changed. */
    SyncUiEventId["ActiveContentChanged"] = "activecontentchanged";
    /** The active view maintained by the ViewManager has changed. */
    SyncUiEventId["ActiveViewportChanged"] = "activeviewportchanged";
    /** Backstage has been closed.
     * @deprecated Use BackstageEvent instead
     */
    SyncUiEventId["BackstageCloseEvent"] = "backstagecloseevent";
    /** Backstage has been closed. */
    SyncUiEventId["BackstageEvent"] = "backstageevent";
    /** A Content Layout has been activated.  */
    SyncUiEventId["ContentLayoutActivated"] = "contentlayoutactivated";
    /** A Content Control maintained by FrontstageManager has been activated. */
    SyncUiEventId["ContentControlActivated"] = "contentcontrolactivated";
    /** A Frontstage is activating. */
    SyncUiEventId["FrontstageActivating"] = "frontstageactivating";
    /** A Frontstage has been activated and the content has been assigned. */
    SyncUiEventId["FrontstageReady"] = "frontstageready";
    /** A Modal Frontstage has been opened or closed. */
    SyncUiEventId["ModalFrontstageChanged"] = "modalfrontstagechanged";
    /** A Modal Dialog has been opened or closed. */
    SyncUiEventId["ModalDialogChanged"] = "modaldialogchanged";
    /** A NavigationAid has been activated. */
    SyncUiEventId["NavigationAidActivated"] = "navigationaidactivated";
    /** An InteractiveTool has been activated via the ToolAdmin. */
    SyncUiEventId["ToolActivated"] = "toolactivated";
    /** A Task has been activated. */
    SyncUiEventId["TaskActivated"] = "taskactivated";
    /** The state of a Widget has changed. */
    SyncUiEventId["WidgetStateChanged"] = "widgetstatechanged";
    /** A Workflow has been activated. */
    SyncUiEventId["WorkflowActivated"] = "workflowactivated";
    /** The SelectionSet for the active IModelConnection has changed. */
    SyncUiEventId["SelectionSetChanged"] = "selectionsetchanged";
    /** The list of settings providers registered with SettingsManager has changed. */
    SyncUiEventId["SettingsProvidersChanged"] = "settingsproviderschanged";
    /** The current view state has changed (used by view undo/redo toolbar buttons). */
    SyncUiEventId["ViewStateChanged"] = "viewstatechanged";
    /** The current object the reads and write UI Settings has changed. */
    SyncUiEventId["UiSettingsChanged"] = "uisettingschanged";
    SyncUiEventId["ShowHideManagerSettingChange"] = "show-hide-setting-change";
})(SyncUiEventId = exports.SyncUiEventId || (exports.SyncUiEventId = {}));
/** SyncUi Event class.
 * @public
 */
class SyncUiEvent extends ui_core_1.UiEvent {
}
exports.SyncUiEvent = SyncUiEvent;
/** This class is used to send eventIds to interested UI components so the component can determine if it needs
 * to refresh its display by calling setState on itself.
 * @public
 */
class SyncUiEventDispatcher {
    /** @internal - used for testing only */
    /* istanbul ignore next */
    static setTimeoutPeriod(period) {
        SyncUiEventDispatcher._timeoutPeriod = period;
        SyncUiEventDispatcher._secondaryTimeoutPeriod = Math.floor(SyncUiEventDispatcher._timeoutPeriod / 2);
        if (SyncUiEventDispatcher._secondaryTimeoutPeriod < 1)
            SyncUiEventDispatcher._secondaryTimeoutPeriod = 1;
        if (SyncUiEventDispatcher._syncEventTimerId) {
            window.clearTimeout(SyncUiEventDispatcher._syncEventTimerId);
            SyncUiEventDispatcher._syncEventTimerId = undefined;
        }
        if (SyncUiEventDispatcher._eventIds)
            SyncUiEventDispatcher._eventIds.clear();
        SyncUiEventDispatcher._eventIdAdded = false;
    }
    /** Return set of event ids that will be sent to listeners/. */
    static get syncEventIds() {
        if (!SyncUiEventDispatcher._eventIds)
            SyncUiEventDispatcher._eventIds = new Set();
        return SyncUiEventDispatcher._eventIds;
    }
    /** Return SyncUiEvent so callers can register an event callback. */
    static get onSyncUiEvent() {
        if (!SyncUiEventDispatcher._syncUiEvent)
            SyncUiEventDispatcher._syncUiEvent = new SyncUiEvent();
        return SyncUiEventDispatcher._syncUiEvent;
    }
    /** Immediately trigger sync event processing. */
    static dispatchImmediateSyncUiEvent(eventId) {
        const eventIds = new Set();
        eventIds.add(eventId.toLowerCase());
        SyncUiEventDispatcher.onSyncUiEvent.emit({ eventIds });
    }
    /** Save eventId in Set for processing. */
    static dispatchSyncUiEvent(eventId) {
        if (0 === SyncUiEventDispatcher._timeoutPeriod) {
            bentleyjs_core_1.Logger.logInfo(UiFramework_1.UiFramework.loggerCategory(this), `[dispatchSyncUiEvent] not processed because _timeoutPeriod=0`);
            return;
        }
        SyncUiEventDispatcher.syncEventIds.add(eventId.toLowerCase());
        if (!SyncUiEventDispatcher._syncEventTimerId) { // if there is not a timer active, create one
            SyncUiEventDispatcher._syncEventTimerId = window.setTimeout(SyncUiEventDispatcher.checkForAdditionalIds, SyncUiEventDispatcher._timeoutPeriod);
        }
        else {
            SyncUiEventDispatcher._eventIdAdded = true;
        }
    }
    /** Save multiple eventIds in Set for processing. */
    static dispatchSyncUiEvents(eventIds) {
        // istanbul ignore if
        if (0 === SyncUiEventDispatcher._timeoutPeriod) {
            bentleyjs_core_1.Logger.logInfo(UiFramework_1.UiFramework.loggerCategory(this), `[dispatchSyncUiEvents] not processed because _timeoutPeriod=0`);
            return;
        }
        eventIds.forEach((id) => SyncUiEventDispatcher.syncEventIds.add(id.toLowerCase()));
        // istanbul ignore else
        if (!SyncUiEventDispatcher._syncEventTimerId) { // if there is not a timer active, create one
            SyncUiEventDispatcher._syncEventTimerId = window.setTimeout(SyncUiEventDispatcher.checkForAdditionalIds, SyncUiEventDispatcher._timeoutPeriod);
        }
        else {
            SyncUiEventDispatcher._eventIdAdded = true;
        }
    }
    /** Trigger registered event processing when timer has expired and no addition eventId are added. */
    static checkForAdditionalIds() {
        /* istanbul ignore else */
        if (!SyncUiEventDispatcher._eventIdAdded) {
            // istanbul ignore else
            if (SyncUiEventDispatcher._syncEventTimerId) {
                window.clearTimeout(SyncUiEventDispatcher._syncEventTimerId);
                SyncUiEventDispatcher._syncEventTimerId = undefined;
            }
            else {
                bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), "SyncUiEventDispatcher.checkForAdditionalIds - expected _syncEventTimerId to be defined");
            }
            SyncUiEventDispatcher._eventIdAdded = false;
            // istanbul ignore else
            if (SyncUiEventDispatcher.syncEventIds.size > 0) {
                const eventIds = new Set();
                SyncUiEventDispatcher.syncEventIds.forEach((value) => eventIds.add(value));
                SyncUiEventDispatcher._eventIds.clear();
                SyncUiEventDispatcher.onSyncUiEvent.emit({ eventIds });
            }
            return;
        }
        // istanbul ignore next
        if (SyncUiEventDispatcher._syncEventTimerId) {
            window.clearTimeout(SyncUiEventDispatcher._syncEventTimerId);
            SyncUiEventDispatcher._syncEventTimerId = undefined;
        }
        // istanbul ignore next
        SyncUiEventDispatcher._eventIdAdded = false;
        // if events have been added before the initial timer expired wait half that time to see if events are still being added.
        // istanbul ignore next
        SyncUiEventDispatcher._syncEventTimerId = window.setTimeout(SyncUiEventDispatcher.checkForAdditionalIds, SyncUiEventDispatcher._secondaryTimeoutPeriod);
    }
    /** Checks to see if an eventId of interest is contained in the set of eventIds */
    static hasEventOfInterest(eventIds, idsOfInterest) {
        /* istanbul ignore else */
        if ((idsOfInterest.length > 0) && idsOfInterest.some((value) => eventIds.has(value.toLowerCase())))
            return true;
        return false;
    }
    // istanbul ignore next
    static _dispatchViewChange() {
        SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.ViewStateChanged);
    }
    /** Initializes the Monitoring of Events that trigger dispatching sync events */
    static initialize() {
        // clear any registered listeners - this should only be encountered in unit test scenarios
        this._unregisterListenerFuncs.forEach((unregisterListenerFunc) => unregisterListenerFunc());
        this._unregisterListenerFuncs.push(FrontstageManager_1.FrontstageManager.onContentControlActivatedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.ContentControlActivated);
        }));
        this._unregisterListenerFuncs.push(FrontstageManager_1.FrontstageManager.onContentLayoutActivatedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.ContentLayoutActivated);
        }));
        this._unregisterListenerFuncs.push(FrontstageManager_1.FrontstageManager.onFrontstageActivatedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.FrontstageActivating);
        }));
        this._unregisterListenerFuncs.push(FrontstageManager_1.FrontstageManager.onFrontstageReadyEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.FrontstageReady);
        }));
        this._unregisterListenerFuncs.push(FrontstageManager_1.FrontstageManager.onModalFrontstageChangedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.ModalFrontstageChanged);
        }));
        this._unregisterListenerFuncs.push(FrontstageManager_1.FrontstageManager.onNavigationAidActivatedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.NavigationAidActivated);
        }));
        this._unregisterListenerFuncs.push(FrontstageManager_1.FrontstageManager.onToolActivatedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.ToolActivated);
        }));
        this._unregisterListenerFuncs.push(FrontstageManager_1.FrontstageManager.onWidgetStateChangedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.WidgetStateChanged);
        }));
        this._unregisterListenerFuncs.push(Backstage_1.Backstage.onBackstageEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.BackstageEvent);
        }));
        this._unregisterListenerFuncs.push(Workflow_1.WorkflowManager.onTaskActivatedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.TaskActivated);
        }));
        this._unregisterListenerFuncs.push(Workflow_1.WorkflowManager.onWorkflowActivatedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.WorkflowActivated);
        }));
        this._unregisterListenerFuncs.push(ContentViewManager_1.ContentViewManager.onActiveContentChangedEvent.addListener(() => {
            SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.ActiveContentChanged);
        }));
        // istanbul ignore else
        if (imodeljs_frontend_1.IModelApp && imodeljs_frontend_1.IModelApp.viewManager) {
            this._unregisterListenerFuncs.push(imodeljs_frontend_1.IModelApp.viewManager.onSelectedViewportChanged.addListener((args) => {
                SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.ActiveViewportChanged);
                // if this is the first view being opened up start the default tool so tool admin is happy.
                if (undefined === args.previous) {
                    imodeljs_frontend_1.IModelApp.toolAdmin.startDefaultTool();
                }
                else {
                    // istanbul ignore next
                    if (args.previous.onViewChanged && typeof args.previous.onViewChanged.removeListener === "function") // not set during unit test
                        args.previous.onViewChanged.removeListener(SyncUiEventDispatcher._dispatchViewChange);
                }
                // istanbul ignore next
                if (args.current) {
                    if (args.current.onViewChanged && typeof args.current.onViewChanged.addListener === "function") // not set during unit test
                        args.current.onViewChanged.addListener(SyncUiEventDispatcher._dispatchViewChange);
                }
            }));
        }
    }
    static selectionChangedHandler(_ev) {
        SyncUiEventDispatcher.dispatchSyncUiEvent(SyncUiEventId.SelectionSetChanged);
    }
    /** This should be called by IModelApp when the active IModelConnection is closed. */
    static clearConnectionEvents(iModelConnection) {
        iModelConnection.selectionSet.onChanged.removeListener(SyncUiEventDispatcher.selectionChangedHandler);
        SyncUiEventDispatcher._unregisterListenerFunc && SyncUiEventDispatcher._unregisterListenerFunc();
        UiFramework_1.UiFramework.setActiveIModelId("");
    }
    /** This should be called by IModelApp when the active IModelConnection is established. */
    static initializeConnectionEvents(iModelConnection) {
        var _a;
        if (SyncUiEventDispatcher._unregisterListenerFunc)
            SyncUiEventDispatcher._unregisterListenerFunc();
        if (iModelConnection.isBlankConnection()) {
            iModelConnection && UiFramework_1.UiFramework.setActiveIModelId((_a = iModelConnection.iModelId) !== null && _a !== void 0 ? _a : "");
            UiFramework_1.UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetAvailableSelectionScopes, []);
            UiFramework_1.UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetNumItemsSelected, 0);
            return;
        }
        iModelConnection.selectionSet.onChanged.removeListener(SyncUiEventDispatcher.selectionChangedHandler);
        iModelConnection.selectionSet.onChanged.addListener(SyncUiEventDispatcher.selectionChangedHandler);
        (iModelConnection.iModelId) ? UiFramework_1.UiFramework.setActiveIModelId(iModelConnection.iModelId) : /* istanbul ignore next */ "";
        // listen for changes from presentation rules selection manager (this is done once an iModelConnection is available to ensure Presentation.selection is valid)
        SyncUiEventDispatcher._unregisterListenerFunc = presentation_frontend_1.Presentation.selection.selectionChange.addListener((args, provider) => {
            // istanbul ignore if
            if (args.level !== 0) {
                // don't need to handle sub-selections
                return;
            }
            const selection = provider.getSelection(args.imodel, args.level);
            const numSelected = presentation_common_1.getInstancesCount(selection);
            UiFramework_1.UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetNumItemsSelected, numSelected);
        });
        presentation_frontend_1.Presentation.selection.scopes.getSelectionScopes(iModelConnection).then((availableScopes) => {
            // istanbul ignore else
            if (availableScopes) {
                const presentationScopes = [];
                availableScopes.map((scope) => presentationScopes.push(scope));
                UiFramework_1.UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetAvailableSelectionScopes, presentationScopes);
            }
        });
        const activeSelectionScope = presentation_frontend_1.Presentation.selection.scopes.activeScope;
        if (activeSelectionScope) {
            if (typeof (activeSelectionScope) === "object") {
                UiFramework_1.UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetSelectionScope, activeSelectionScope.id);
            }
            else {
                UiFramework_1.UiFramework.dispatchActionToStore(SessionState_1.SessionStateActionId.SetSelectionScope, activeSelectionScope);
            }
        }
    }
}
exports.SyncUiEventDispatcher = SyncUiEventDispatcher;
SyncUiEventDispatcher._eventIdAdded = false;
SyncUiEventDispatcher._timeoutPeriod = 100;
SyncUiEventDispatcher._secondaryTimeoutPeriod = SyncUiEventDispatcher._timeoutPeriod / 2;
SyncUiEventDispatcher._unregisterListenerFuncs = [];
SyncUiEventDispatcher.initialized = false;
//# sourceMappingURL=SyncUiEventDispatcher.js.map