"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Item
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolButton = void 0;
const React = require("react");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
const UiFramework_1 = require("../UiFramework");
const PropsHelper_1 = require("../utils/PropsHelper");
const useEscapeSetFocusToHome_1 = require("../hooks/useEscapeSetFocusToHome");
/** Tool Button React Component.
 * @public
 */
class ToolButton extends React.Component {
    constructor(props) {
        super(props);
        this._componentUnmounting = false;
        this._label = "";
        this._handleSyncUiEvent = (args) => {
            // istanbul ignore if
            if (this._componentUnmounting)
                return;
            let refreshState = false;
            let newState = Object.assign({}, this.state);
            // since this is a tool button automatically monitor the activation of tools so the active state of the button is updated.
            if (args.eventIds.has(SyncUiEventDispatcher_1.SyncUiEventId.ToolActivated)) {
                newState.isActive = this.props.toolId === FrontstageManager_1.FrontstageManager.activeToolId;
                refreshState = true;
            }
            if (!refreshState && this.props.stateSyncIds && this.props.stateSyncIds.length > 0) // eslint-disable-line deprecation/deprecation
                refreshState = this.props.stateSyncIds.some((value) => args.eventIds.has(value.toLowerCase())); // eslint-disable-line deprecation/deprecation
            if (refreshState) {
                if (this.props.stateFunc) // eslint-disable-line deprecation/deprecation
                    newState = this.props.stateFunc(newState); // eslint-disable-line deprecation/deprecation
                // istanbul ignore else
                if ((this.state.isActive !== newState.isActive) ||
                    /* istanbul ignore next */ (this.state.isEnabled !== newState.isEnabled) ||
                    /* istanbul ignore next */ (this.state.isVisible !== newState.isVisible)) {
                    this.setState({
                        isActive: newState.isActive,
                        isEnabled: newState.isEnabled,
                        isVisible: newState.isVisible,
                    });
                }
            }
        };
        this._execute = () => {
            if (this.props.execute) {
                this.props.execute();
            }
            else {
                const thisTool = imodeljs_frontend_1.IModelApp.tools.find(this.props.toolId);
                // istanbul ignore else
                if (thisTool)
                    (new thisTool()).run();
            }
        };
        if (props.label)
            this._label = props.label;
        else if (props.labelKey)
            this._label = UiFramework_1.UiFramework.i18n.translate(props.labelKey);
        this.state = {
            isVisible: undefined !== props.isVisible ? props.isVisible : true,
            isEnabled: undefined !== props.isEnabled ? props.isEnabled : true,
            isActive: undefined !== props.isActive ? props.isActive : false,
            isPressed: undefined !== props.isPressed ? props.isPressed : false,
        };
    }
    componentDidMount() {
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.onSyncUiEvent.addListener(this._handleSyncUiEvent);
    }
    componentWillUnmount() {
        this._componentUnmounting = true;
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.onSyncUiEvent.removeListener(this._handleSyncUiEvent);
    }
    get label() {
        return PropsHelper_1.PropsHelper.getStringFromSpec(this._label);
    }
    render() {
        if (!this.state.isVisible)
            return null;
        const icon = React.createElement(ui_core_1.Icon, { iconSpec: this.props.iconSpec });
        const toolbarItemProps = ui_ninezone_1.getToolbarItemProps(this.props);
        const badge = ui_core_1.BadgeUtilities.getComponentForBadgeType(this.props.badgeType);
        return (React.createElement(ui_ninezone_1.Item, Object.assign({}, toolbarItemProps, { className: this.props.className, style: this.props.style, isActive: this.state.isActive, isDisabled: !this.state.isEnabled, title: this.label, key: this.props.toolId, onClick: this._execute, onKeyDown: useEscapeSetFocusToHome_1.onEscapeSetFocusToHome, icon: icon, badge: badge })));
    }
}
exports.ToolButton = ToolButton;
//# sourceMappingURL=ToolButton.js.map