"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Settings
 */
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppUiSettings = void 0;
const ui_core_1 = require("@bentley/ui-core");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
const UiFramework_1 = require("../UiFramework");
/** These are the UI settings that are stored in the Redux store. They control the color theme, the UI version,
 * how toolbar group buttons work, and the opacity of widgets in 1.0 and floating widget in 2.0 when the cursor
 * is not inside them. It is expect that an IModelApp using App UI components will create and register AppUiSettings with
 * defaults specific to their application. This would be done by calling the following.
 *
 * ```ts
 * UiFramework.registerUserSettingsProvider(new AppUiSettings(defaults));
 * ```
 *
 * @beta
 */
class AppUiSettings {
    constructor(defaults) {
        this.providerId = "AppUiSettingsProvider";
        this._settings = [];
        this._applyingLocalSettings = false;
        this.setColorTheme = (theme) => {
            UiFramework_1.UiFramework.setColorTheme(theme);
            // always store to local storage to avoid flicker during startup if user is not yet logged-in
            window.localStorage.setItem("uifw:defaultTheme", theme);
        };
        this.handleSyncUiEvent = async (args) => {
            if (this._applyingLocalSettings)
                return;
            if (args.eventIds.has("configurableui:set_theme")) {
                await this.colorTheme.saveSetting(UiFramework_1.UiFramework.getUiSettingsStorage());
                // always store as default theme in local storage to avoid flicker during startup if user is not yet logged-in
                window.localStorage.setItem("uifw:defaultTheme", UiFramework_1.UiFramework.getColorTheme());
            }
            if (args.eventIds.has("configurableui:set-drag-interaction"))
                await this.dragInteraction.saveSetting(UiFramework_1.UiFramework.getUiSettingsStorage());
            if (args.eventIds.has("configurableui:set-framework-version"))
                await this.frameworkVersion.saveSetting(UiFramework_1.UiFramework.getUiSettingsStorage());
            if (args.eventIds.has("configurableui:set_widget_opacity"))
                await this.widgetOpacity.saveSetting(UiFramework_1.UiFramework.getUiSettingsStorage());
        };
        this._settings = [];
        this.colorTheme = new ui_core_1.UiSetting(AppUiSettings._settingNamespace, "ColorTheme", UiFramework_1.UiFramework.getColorTheme, this.setColorTheme, defaults.colorTheme);
        this._settings.push(this.colorTheme);
        this.dragInteraction = new ui_core_1.UiSetting(AppUiSettings._settingNamespace, "DragInteraction", () => UiFramework_1.UiFramework.useDragInteraction, (value) => UiFramework_1.UiFramework.setUseDragInteraction(value), defaults.dragInteraction);
        this._settings.push(this.dragInteraction);
        this.frameworkVersion = new ui_core_1.UiSetting(AppUiSettings._settingNamespace, "FrameworkVersion", () => UiFramework_1.UiFramework.uiVersion, (value) => UiFramework_1.UiFramework.setUiVersion(value), defaults.frameworkVersion);
        this._settings.push(this.frameworkVersion);
        this.widgetOpacity = new ui_core_1.UiSetting(AppUiSettings._settingNamespace, "WidgetOpacity", () => UiFramework_1.UiFramework.getWidgetOpacity(), (value) => UiFramework_1.UiFramework.setWidgetOpacity(value), defaults.widgetOpacity);
        this._settings.push(this.widgetOpacity);
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.onSyncUiEvent.addListener(this.handleSyncUiEvent);
    }
    async apply(storage) {
        var e_1, _a;
        this._applyingLocalSettings = true;
        try {
            for (var _b = __asyncValues(this._settings), _c; _c = await _b.next(), !_c.done;) {
                const setting = _c.value;
                await setting.getSettingAndApplyValue(storage);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        this._applyingLocalSettings = false;
    }
    async loadUserSettings(storage) {
        await this.apply(storage);
    }
}
exports.AppUiSettings = AppUiSettings;
AppUiSettings._settingNamespace = "AppUiSettings";
//# sourceMappingURL=AppUiSettings.js.map