"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module UiSettings
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelAppUiSettings = exports.settingsStatusToUiSettingsStatus = exports.UserSettingsStorage = void 0;
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const product_settings_client_1 = require("@bentley/product-settings-client");
const ui_core_1 = require("@bentley/ui-core");
/**
 * Implementation of [[UiSettings]] that uses settings admin from `IModelApp.settings`.
 * @public
 */
class UserSettingsStorage {
    async getSetting(namespace, name) {
        if (!this.isSignedIn)
            return { status: ui_core_1.UiSettingsStatus.AuthorizationError };
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const result = await imodeljs_frontend_1.IModelApp.settings.getUserSetting(requestContext, namespace, name, true);
        const status = settingsStatusToUiSettingsStatus(result.status);
        return {
            status,
            setting: result.setting,
        };
    }
    async saveSetting(namespace, name, setting) {
        if (!this.isSignedIn)
            return { status: ui_core_1.UiSettingsStatus.AuthorizationError };
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const result = await imodeljs_frontend_1.IModelApp.settings.saveUserSetting(requestContext, setting, namespace, name, true);
        const status = settingsStatusToUiSettingsStatus(result.status);
        return {
            status,
            setting: result.setting,
        };
    }
    async deleteSetting(namespace, name) {
        if (!this.isSignedIn)
            return { status: ui_core_1.UiSettingsStatus.AuthorizationError };
        const requestContext = await imodeljs_frontend_1.AuthorizedFrontendRequestContext.create();
        const result = await imodeljs_frontend_1.IModelApp.settings.deleteUserSetting(requestContext, namespace, name, true);
        const status = settingsStatusToUiSettingsStatus(result.status);
        return {
            status,
            setting: result.setting,
        };
    }
    get isSignedIn() {
        return !!imodeljs_frontend_1.IModelApp.authorizationClient && imodeljs_frontend_1.IModelApp.authorizationClient.hasSignedIn;
    }
}
exports.UserSettingsStorage = UserSettingsStorage;
/** @internal */
function settingsStatusToUiSettingsStatus(status) {
    if (status === product_settings_client_1.SettingsStatus.Success)
        return ui_core_1.UiSettingsStatus.Success;
    else if (status === product_settings_client_1.SettingsStatus.SettingNotFound)
        return ui_core_1.UiSettingsStatus.NotFound;
    else if (status === product_settings_client_1.SettingsStatus.AuthorizationError)
        return ui_core_1.UiSettingsStatus.AuthorizationError;
    return ui_core_1.UiSettingsStatus.UnknownError;
}
exports.settingsStatusToUiSettingsStatus = settingsStatusToUiSettingsStatus;
/** Alias for [[UserSettingsStorage]]
 * @beta @deprecated use UserSettingsStorage
 */
class IModelAppUiSettings extends UserSettingsStorage {
}
exports.IModelAppUiSettings = IModelAppUiSettings;
//# sourceMappingURL=UserSettingsStorage.js.map