"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.UiShowHideManager = exports.INACTIVITY_TIME_DEFAULT = exports.UiShowHideSettingsProvider = void 0;
const ui_core_1 = require("@bentley/ui-core");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
const UiFramework_1 = require("../UiFramework");
/** Class that maintain UiShowHide user settings between sessions
 * @internal
 */
class UiShowHideSettingsProvider {
    constructor() {
        this.providerId = "UiShowHideSettingsProvider";
    }
    static initialize() {
        UiFramework_1.UiFramework.registerUserSettingsProvider(new UiShowHideSettingsProvider());
    }
    async loadUserSettings(storage) {
        let result = await storage.getSetting(UiShowHideSettingsProvider._settingsNamespace, UiShowHideSettingsProvider._autoHideUiKey);
        if (result.status === ui_core_1.UiSettingsStatus.Success)
            UiShowHideManager.setAutoHideUi(result.setting);
        result = await storage.getSetting(UiShowHideSettingsProvider._settingsNamespace, UiShowHideSettingsProvider._useProximityOpacityKey);
        if (result.status === ui_core_1.UiSettingsStatus.Success)
            UiShowHideManager.setUseProximityOpacity(result.setting);
        result = await storage.getSetting(UiShowHideSettingsProvider._settingsNamespace, UiShowHideSettingsProvider._snapWidgetOpacityKey);
        if (result.status === ui_core_1.UiSettingsStatus.Success)
            UiShowHideManager.setSnapWidgetOpacity(result.setting);
    }
    static async storeAutoHideUi(v, storage) {
        void (storage !== null && storage !== void 0 ? storage : UiFramework_1.UiFramework.getUiSettingsStorage()).saveSetting(this._settingsNamespace, this._autoHideUiKey, v);
    }
    static async storeUseProximityOpacity(v, storage) {
        void (storage !== null && storage !== void 0 ? storage : UiFramework_1.UiFramework.getUiSettingsStorage()).saveSetting(this._settingsNamespace, this._useProximityOpacityKey, v);
    }
    static async storeSnapWidgetOpacity(v, storage) {
        void (storage !== null && storage !== void 0 ? storage : UiFramework_1.UiFramework.getUiSettingsStorage()).saveSetting(this._settingsNamespace, this._snapWidgetOpacityKey, v);
    }
}
exports.UiShowHideSettingsProvider = UiShowHideSettingsProvider;
UiShowHideSettingsProvider._settingsNamespace = "AppUiSettings";
UiShowHideSettingsProvider._autoHideUiKey = "AutoHideUi";
UiShowHideSettingsProvider._useProximityOpacityKey = "UseProximityOpacity";
UiShowHideSettingsProvider._snapWidgetOpacityKey = "SnapWidgetOpacity";
/** The default inactivity time.
 * @internal
 */
exports.INACTIVITY_TIME_DEFAULT = 3500; /** Wait 3.5 seconds */
/** Maintains Ui Show/Hide state. The `Ui` includes widgets, panels and the status bar.
 * @public
 */
class UiShowHideManager {
    /** Determines if the Ui is visible */
    static get isUiVisible() {
        return UiShowHideManager._isUiVisible;
    }
    static set isUiVisible(visible) {
        UiShowHideManager._isUiVisible = visible;
    }
    /** @internal */
    static setAutoHideUi(value) {
        UiShowHideManager._autoHideUi = value;
    }
    /** @internal */
    static setUseProximityOpacity(value) {
        UiShowHideManager._useProximityOpacity = value;
    }
    /** @internal */
    static setSnapWidgetOpacity(value) {
        UiShowHideManager._snapWidgetOpacity = value;
    }
    /** Determines whether the `auto-hide Ui` feature is on. Defaults to false.
     * When true, the Ui automatically hides after a few seconds of inactivity.
     */
    static get autoHideUi() {
        return UiShowHideManager._autoHideUi;
    }
    static set autoHideUi(autoHide) {
        void UiShowHideSettingsProvider.storeAutoHideUi(autoHide);
        UiShowHideManager._autoHideUi = autoHide;
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchImmediateSyncUiEvent(SyncUiEventDispatcher_1.SyncUiEventId.ShowHideManagerSettingChange);
    }
    /** Determines whether the widget panels are shown and hidden. Defaults to false. */
    static get showHidePanels() {
        return UiShowHideManager._showHidePanels;
    }
    static set showHidePanels(showHide) {
        UiShowHideManager._showHidePanels = showHide;
        UiFramework_1.UiFramework.onUiVisibilityChanged.emit({ visible: UiFramework_1.UiFramework.getIsUiVisible() });
    }
    /** Determines whether the status bar is shown and hidden. Defaults to false. */
    static get showHideFooter() {
        return UiShowHideManager._showHideFooter;
    }
    static set showHideFooter(showHide) {
        UiShowHideManager._showHideFooter = showHide;
        UiFramework_1.UiFramework.onUiVisibilityChanged.emit({ visible: UiFramework_1.UiFramework.getIsUiVisible() });
    }
    /** Determines the amount of inactivity time before the Ui is hidden. Defaults to 3.5 seconds. */
    static get inactivityTime() {
        return UiShowHideManager._inactivityTime;
    }
    static set inactivityTime(time) {
        UiShowHideManager._inactivityTime = time;
    }
    /** Determines whether the proximity of the mouse should alter the opacity of a toolbar. Defaults to true. */
    static get useProximityOpacity() {
        return UiShowHideManager._useProximityOpacity;
    }
    static set useProximityOpacity(value) {
        UiShowHideManager._useProximityOpacity = value;
        void UiShowHideSettingsProvider.storeUseProximityOpacity(value);
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchImmediateSyncUiEvent(SyncUiEventDispatcher_1.SyncUiEventId.ShowHideManagerSettingChange);
        UiFramework_1.UiFramework.onUiVisibilityChanged.emit({ visible: UiFramework_1.UiFramework.getIsUiVisible() });
    }
    /** Determines whether the opacity of a toolbar should snap. Defaults to false. */
    static get snapWidgetOpacity() {
        return UiShowHideManager._snapWidgetOpacity;
    }
    static set snapWidgetOpacity(value) {
        UiShowHideManager._snapWidgetOpacity = value;
        void UiShowHideSettingsProvider.storeSnapWidgetOpacity(value);
        SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchImmediateSyncUiEvent(SyncUiEventDispatcher_1.SyncUiEventId.ShowHideManagerSettingChange);
        UiFramework_1.UiFramework.onUiVisibilityChanged.emit({ visible: UiFramework_1.UiFramework.getIsUiVisible() });
    }
    /** Handler for when a Frontstage is ready */
    static handleFrontstageReady() {
        if (!UiShowHideManager._autoHideUi)
            return;
        UiShowHideManager.showUiAndResetTimer();
    }
    /** Handler for when the mouse moves over the content area */
    static handleContentMouseMove(_event) {
        if (!UiShowHideManager._autoHideUi)
            return;
        UiShowHideManager.showUiAndResetTimer();
    }
    /** Handler for when the mouse enters a widget */
    static handleWidgetMouseEnter(_event) {
        if (!UiShowHideManager._autoHideUi)
            return;
        UiShowHideManager.showUiAndCancelTimer();
    }
    /** Shows the Ui and resets the inactivity timer */
    static showUiAndResetTimer() {
        setTimeout(() => {
            UiShowHideManager.showUi();
            UiShowHideManager.resetTimer();
        });
    }
    /** Shows the Ui and cancels the inactivity timer */
    static showUiAndCancelTimer() {
        setTimeout(() => {
            UiShowHideManager.showUi();
            UiShowHideManager.cancelTimer();
        });
    }
    static cancelTimer() {
        clearTimeout(UiShowHideManager._timeout);
    }
    static resetTimer() {
        UiShowHideManager.cancelTimer();
        UiShowHideManager._timeout = setTimeout(UiShowHideManager.hideUi, UiShowHideManager._inactivityTime);
    }
    static showUi() {
        UiFramework_1.UiFramework.setIsUiVisible(true);
    }
    static hideUi() {
        UiFramework_1.UiFramework.setIsUiVisible(false);
    }
}
exports.UiShowHideManager = UiShowHideManager;
UiShowHideManager._isUiVisible = true;
UiShowHideManager._autoHideUi = false;
UiShowHideManager._showHidePanels = false;
UiShowHideManager._showHideFooter = false;
UiShowHideManager._inactivityTime = exports.INACTIVITY_TIME_DEFAULT;
UiShowHideManager._useProximityOpacity = true;
UiShowHideManager._snapWidgetOpacity = false;
//# sourceMappingURL=UiShowHideManager.js.map