"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Utilities
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewUtilities = void 0;
// cSpell:ignore classname
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
/**
 * Various View utility methods
 * @public
 */
class ViewUtilities {
    /**
     * Extracts the BisBaseClass from a full class name.
     * @param classFullName full class name
     * @returns BisBaseClass name
     */
    static getBisBaseClass(classFullName) {
        const bisBaseClass = classFullName.substring(classFullName.indexOf(":") + 1);
        return bisBaseClass;
    }
    /**
     * Determines if given class is a spatial view.
     * @param classname Name of class to check
     */
    static isSpatial(classname) {
        return classname === imodeljs_frontend_1.SpatialViewState.className || classname === imodeljs_frontend_1.OrthographicViewState.className;
    }
    /**
     * Determines if given class is a spatial view.
     * @param classname Name of class to check
     */
    static isOrthographic(classname) {
        return classname === imodeljs_frontend_1.OrthographicViewState.className;
    }
    /**
     * Determines if given class is a drawing view.
     * @param classname Name of class to check
     */
    static isDrawing(classname) {
        return classname === imodeljs_frontend_1.DrawingViewState.className;
    }
    /**
     * Determines if given class is a sheet view.
     * @param classname Name of class to check
     */
    static isSheet(classname) {
        return classname === imodeljs_frontend_1.SheetViewState.className;
    }
    /**
     * Determines if viewport displays a Sheet view.
     * @param viewport ScreenViewport to check
     */
    static isSheetView(viewport) {
        return ViewUtilities.isSheet(ViewUtilities.getBisBaseClass(viewport.view.classFullName));
    }
    /**
     * Determines if viewport displays a Drawing view.
     * @param viewport ScreenViewport to check
     */
    static isDrawingView(viewport) {
        return ViewUtilities.isDrawing(ViewUtilities.getBisBaseClass(viewport.view.classFullName));
    }
    /**
     * Determines if viewport displays a Orthographic view.
     * @param viewport ScreenViewport to check
     */
    static isOrthographicView(viewport) {
        return ViewUtilities.isOrthographic(ViewUtilities.getBisBaseClass(viewport.view.classFullName));
    }
    /**
     * Determines if viewport displays a Spatial view.
     * @param viewport ScreenViewport to check
     */
    static isSpatialView(viewport) {
        return ViewUtilities.isSpatial(ViewUtilities.getBisBaseClass(viewport.view.classFullName));
    }
    /**
     * Determines if viewport displays a 3d view.
     * @param viewport ScreenViewport to check
     */
    static is3dView(viewport) {
        return viewport.view.is3d();
    }
    /**
     * Determines if viewport supports use of a camera.
     * @param viewport ScreenViewport to check
     */
    static viewSupportsCamera(viewport) {
        return imodeljs_frontend_1.SpatialViewState.className === ViewUtilities.getBisBaseClass(viewport.view.classFullName);
    }
}
exports.ViewUtilities = ViewUtilities;
//# sourceMappingURL=ViewUtilities.js.map