"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ToolSettings
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolSettingsWidgetContent = exports.ToolSettingsContent = exports.useToolSettingsNode = exports.ToolSettingsGrid = exports.useHorizontalToolSettingNodes = exports.ToolSettingsDockedContent = exports.WidgetPanelsToolSettings = void 0;
require("./ToolSettings.scss");
const React = require("react");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const Frontstage_1 = require("../frontstage/Frontstage");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
/** @internal */
// istanbul ignore next - need to work on overflow unit testing
function TsLabel({ children }) {
    return React.createElement("div", { className: "uif-toolsetting-label-docked-horizontal" }, children);
}
/** @internal */
function WidgetPanelsToolSettings() {
    const frontstageDef = Frontstage_1.useActiveFrontstageDef();
    const toolSettings = React.useContext(ui_ninezone_1.ToolSettingsStateContext);
    const topCenterZone = frontstageDef === null || frontstageDef === void 0 ? void 0 : frontstageDef.topCenter;
    if (!topCenterZone || !topCenterZone.isToolSettings || toolSettings.type === "widget")
        return null;
    return (React.createElement(ToolSettingsDockedContent, null));
}
exports.WidgetPanelsToolSettings = WidgetPanelsToolSettings;
/** @internal */
function ToolSettingsDockedContent() {
    const settings = useHorizontalToolSettingNodes();
    // for the overflow to work properly each setting in the DockedToolSettings should be wrapped by a DockedToolSetting component
    return (React.createElement(ui_ninezone_1.DockedToolSettings, { key: Date.now() }, settings && settings.map((entry, index) => React.createElement(ui_ninezone_1.DockedToolSetting, { key: index },
        React.createElement(TsLabel, null, entry.labelNode),
        entry.editorNode))));
}
exports.ToolSettingsDockedContent = ToolSettingsDockedContent;
/** @internal */
function useHorizontalToolSettingNodes() {
    var _a;
    const [settings, setSettings] = React.useState((_a = FrontstageManager_1.FrontstageManager.activeToolSettingsProvider) === null || _a === void 0 ? void 0 : _a.horizontalToolSettingNodes);
    React.useEffect(() => {
        const handleToolActivatedEvent = () => {
            var _a;
            const nodes = (_a = FrontstageManager_1.FrontstageManager.activeToolSettingsProvider) === null || _a === void 0 ? void 0 : _a.horizontalToolSettingNodes;
            setSettings(nodes);
        };
        FrontstageManager_1.FrontstageManager.onToolActivatedEvent.addListener(handleToolActivatedEvent);
        return () => {
            FrontstageManager_1.FrontstageManager.onToolActivatedEvent.removeListener(handleToolActivatedEvent);
        };
    }, [setSettings]);
    React.useEffect(() => {
        const handleToolSettingsReloadEvent = () => {
            var _a;
            const nodes = (_a = FrontstageManager_1.FrontstageManager.activeToolSettingsProvider) === null || _a === void 0 ? void 0 : _a.horizontalToolSettingNodes;
            setSettings(nodes);
        };
        FrontstageManager_1.FrontstageManager.onToolSettingsReloadEvent.addListener(handleToolSettingsReloadEvent);
        return () => {
            FrontstageManager_1.FrontstageManager.onToolSettingsReloadEvent.removeListener(handleToolSettingsReloadEvent);
        };
    }, [setSettings]);
    return settings;
}
exports.useHorizontalToolSettingNodes = useHorizontalToolSettingNodes;
/** Component that arranges an array of ToolSettingsEntry items into a two column grid layout.
 * The left column is considered the label column, the right column is considered the property
 * editor column.
 * @beta
 */
function ToolSettingsGrid({ settings }) {
    return (React.createElement("div", { className: "uifw-standard-toolsettings-two-column-grid" }, settings && settings.map((setting, index) => {
        return (React.createElement(React.Fragment, { key: index },
            React.createElement("span", { className: "uifw-standard-toolsettings-label-entry" }, setting.labelNode),
            setting.editorNode));
    })));
}
exports.ToolSettingsGrid = ToolSettingsGrid;
/** @internal */
function useToolSettingsNode() {
    var _a;
    const [settings, setSettings] = React.useState((_a = FrontstageManager_1.FrontstageManager.activeToolSettingsProvider) === null || _a === void 0 ? void 0 : _a.toolSettingsNode);
    React.useEffect(() => {
        const handleToolActivatedEvent = () => {
            var _a;
            const nodes = (_a = FrontstageManager_1.FrontstageManager.activeToolSettingsProvider) === null || _a === void 0 ? void 0 : _a.toolSettingsNode;
            setSettings(nodes);
        };
        FrontstageManager_1.FrontstageManager.onToolActivatedEvent.addListener(handleToolActivatedEvent);
        return () => {
            FrontstageManager_1.FrontstageManager.onToolActivatedEvent.removeListener(handleToolActivatedEvent);
        };
    }, [setSettings]);
    React.useEffect(() => {
        const handleToolSettingsReloadEvent = () => {
            var _a;
            const nodes = (_a = FrontstageManager_1.FrontstageManager.activeToolSettingsProvider) === null || _a === void 0 ? void 0 : _a.toolSettingsNode;
            setSettings(nodes);
        };
        FrontstageManager_1.FrontstageManager.onToolSettingsReloadEvent.addListener(handleToolSettingsReloadEvent);
        return () => {
            FrontstageManager_1.FrontstageManager.onToolSettingsReloadEvent.removeListener(handleToolSettingsReloadEvent);
        };
    }, [setSettings]);
    return settings;
}
exports.useToolSettingsNode = useToolSettingsNode;
/** @internal */
function ToolSettingsContent() {
    const toolSettings = React.useContext(ui_ninezone_1.ToolSettingsStateContext);
    // This is needed to remount underlying components tree when going into widget state.
    if (toolSettings.type === "docked")
        return null;
    return React.createElement(ToolSettingsWidgetContent, null);
}
exports.ToolSettingsContent = ToolSettingsContent;
/** @internal */
function ToolSettingsWidgetContent() {
    const node = useToolSettingsNode();
    return (React.createElement(ui_ninezone_1.ScrollableWidgetContent, null, node));
}
exports.ToolSettingsWidgetContent = ToolSettingsWidgetContent;
//# sourceMappingURL=ToolSettings.js.map