"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Widget
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.NavigationWidget = exports.NavigationWidgetDef = void 0;
const React = require("react");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_components_1 = require("@bentley/ui-components");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const ConfigurableUiControl_1 = require("../configurableui/ConfigurableUiControl");
const ConfigurableUiManager_1 = require("../configurableui/ConfigurableUiManager");
const ContentViewManager_1 = require("../content/ContentViewManager");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const UiFramework_1 = require("../UiFramework");
const UiShowHideManager_1 = require("../utils/UiShowHideManager");
const ToolbarWidgetBase_1 = require("./ToolbarWidgetBase");
const WidgetDef_1 = require("./WidgetDef");
/** Definition of a Navigation Widget normally displayed in the top right zone in the 9-Zone Layout system.
 *  @public @deprecated use NavigationWidgetComposer instead
 */
class NavigationWidgetDef extends ToolbarWidgetBase_1.ToolbarWidgetDefBase {
    constructor(props) {
        super(props);
        this.widgetType = WidgetDef_1.WidgetType.Navigation;
        this.verticalDirection = (props.verticalDirection !== undefined) ? props.verticalDirection : ui_ninezone_1.Direction.Left;
        this.horizontalPanelAlignment = ui_ninezone_1.ToolbarPanelAlignment.End;
        this._navigationAidId = (props.navigationAidId !== undefined) ? props.navigationAidId : "";
        const activeStageName = FrontstageManager_1.FrontstageManager.activeFrontstageDef ? FrontstageManager_1.FrontstageManager.activeFrontstageDef.id : /* istanbul ignore next */ "";
        this.widgetBaseName = `[${activeStageName}]NavigationWidget`;
    }
    get reactNode() {
        // istanbul ignore else
        if (!this._reactNode)
            this._reactNode = React.createElement(NavigationWidgetWithDef, { navigationWidgetDef: this });
        return this._reactNode;
    }
    /** @deprecated use reactNode */
    // istanbul ignore next
    get reactElement() {
        return this.reactNode;
    }
    renderCornerItem() {
        // istanbul ignore if
        if (FrontstageManager_1.FrontstageManager.isLoading)
            return null;
        // istanbul ignore else
        if (!this._navigationAidControl && this._navigationAidId) {
            const activeContentControl = ContentViewManager_1.ContentViewManager.getActiveContentControl();
            const viewport = activeContentControl ? activeContentControl.viewport : /* istanbul ignore next */ undefined;
            this._navigationAidControl = ConfigurableUiManager_1.ConfigurableUiManager.createControl(this._navigationAidId, this._navigationAidId, { imodel: this._imodel, viewport });
            if (this._navigationAidControl.getType() !== ConfigurableUiControl_1.ConfigurableUiControlType.NavigationAid) {
                throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `renderCornerItem: navigationAidId '${this._navigationAidId}' is registered to a control that is NOT a NavigationAid`);
            }
            this._navigationAidControl.initialize();
        }
        // istanbul ignore else
        if (this._navigationAidControl) {
            const size = this._navigationAidControl.getSize() || "64px";
            const divStyle = {
                width: size,
                height: size,
            };
            return (React.createElement("div", { style: divStyle }, this._navigationAidControl.reactNode));
        }
        // istanbul ignore next
        return null;
    }
    updateNavigationAid(navigationAidId, imodel) {
        this._navigationAidId = navigationAidId;
        this._imodel = imodel;
        this._navigationAidControl = undefined;
    }
}
exports.NavigationWidgetDef = NavigationWidgetDef;
/** Navigation Widget React component.
 *  @public @deprecated use NavigationWidgetComposer instead
 */
class NavigationWidget extends React.Component {
    constructor(props) {
        super(props);
        this._handleContentControlActivated = (args) => {
            const navigationAidId = args.activeContentControl.navigationAidControl;
            setTimeout(() => {
                FrontstageManager_1.FrontstageManager.setActiveNavigationAid(navigationAidId, this.props.iModelConnection);
            });
        };
        this._handleViewClassFullNameChange = (args) => {
            setTimeout(() => {
                const activeContentControl = ContentViewManager_1.ContentViewManager.getActiveContentControl();
                // istanbul ignore else
                if (activeContentControl && args.viewport === activeContentControl.viewport) {
                    const navigationAidId = activeContentControl.navigationAidControl;
                    FrontstageManager_1.FrontstageManager.setActiveNavigationAid(navigationAidId, this.props.iModelConnection);
                }
            });
        };
        this.state = { navigationWidgetDef: new NavigationWidgetDef(props) }; // eslint-disable-line deprecation/deprecation
    }
    /** Adds listeners */
    componentDidMount() {
        FrontstageManager_1.FrontstageManager.onContentControlActivatedEvent.addListener(this._handleContentControlActivated);
        ui_components_1.ViewportComponentEvents.onViewClassFullNameChangedEvent.addListener(this._handleViewClassFullNameChange);
    }
    /** Removes listeners */
    componentWillUnmount() {
        FrontstageManager_1.FrontstageManager.onContentControlActivatedEvent.removeListener(this._handleContentControlActivated);
        ui_components_1.ViewportComponentEvents.onViewClassFullNameChangedEvent.removeListener(this._handleViewClassFullNameChange);
    }
    componentDidUpdate(prevProps, _prevState) {
        if (this.props !== prevProps) {
            this.setState((_, props) => ({ navigationWidgetDef: new NavigationWidgetDef(props) })); // eslint-disable-line deprecation/deprecation
        }
    }
    render() {
        return (React.createElement(NavigationWidgetWithDef, { className: this.props.className, style: this.props.style, navigationWidgetDef: this.state.navigationWidgetDef, horizontalToolbar: this.props.horizontalToolbar, verticalToolbar: this.props.verticalToolbar }));
    }
}
exports.NavigationWidget = NavigationWidget;
/** Navigation Widget React component that's passed a NavigationWidgetDef.
 */
class NavigationWidgetWithDef extends React.Component {
    constructor(props) {
        super(props);
        this._handleNavigationAidActivatedEvent = (args) => {
            this.props.navigationWidgetDef.updateNavigationAid(args.navigationAidId, args.iModelConnection);
            const navigationAid = this.props.navigationWidgetDef.renderCornerItem();
            this.setState({ cornerItem: navigationAid });
        };
        const horizontalToolbar = (this.props.horizontalToolbar) ? this.props.horizontalToolbar : this.props.navigationWidgetDef.renderHorizontalToolbar();
        const verticalToolbar = (this.props.verticalToolbar) ? this.props.verticalToolbar : this.props.navigationWidgetDef.renderVerticalToolbar();
        this.state = { horizontalToolbar, verticalToolbar, cornerItem: null };
    }
    reloadToolbars() {
        const horizontalToolbar = (this.props.horizontalToolbar) ? this.props.horizontalToolbar : /* istanbul ignore next */ this.props.navigationWidgetDef.renderHorizontalToolbar();
        const verticalToolbar = (this.props.verticalToolbar) ? /* istanbul ignore next */ this.props.verticalToolbar : this.props.navigationWidgetDef.renderVerticalToolbar();
        this.setState({ horizontalToolbar, verticalToolbar });
    }
    componentDidMount() {
        FrontstageManager_1.FrontstageManager.onNavigationAidActivatedEvent.addListener(this._handleNavigationAidActivatedEvent);
    }
    componentWillUnmount() {
        FrontstageManager_1.FrontstageManager.onNavigationAidActivatedEvent.removeListener(this._handleNavigationAidActivatedEvent);
    }
    componentDidUpdate(prevProps) {
        if (this.props !== prevProps)
            this.reloadToolbars();
    }
    render() {
        return (React.createElement(ui_ninezone_1.Tools, { isNavigation: true, className: this.props.className, style: this.props.style, button: this.state.cornerItem, horizontalToolbar: this.state.horizontalToolbar, verticalToolbar: this.state.verticalToolbar, preserveSpace: true, onMouseEnter: UiShowHideManager_1.UiShowHideManager.handleWidgetMouseEnter }));
    }
}
//# sourceMappingURL=NavigationWidget.js.map