"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Widget
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NavigationWidgetComposer = exports.NavigationAidHost = void 0;
const React = require("react");
const ui_components_1 = require("@bentley/ui-components");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const ConfigurableUiManager_1 = require("../configurableui/ConfigurableUiManager");
const ContentViewManager_1 = require("../content/ContentViewManager");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const useFrameworkVersion_1 = require("../hooks/useFrameworkVersion");
const UiFramework_1 = require("../UiFramework");
const UiShowHideManager_1 = require("../utils/UiShowHideManager");
function createNavigationAidControl(activeContentControl, navigationAidId, activeViewport) {
    // istanbul ignore else
    if (!activeContentControl || !navigationAidId || (activeViewport !== activeContentControl.viewport))
        return undefined;
    const viewport = activeContentControl.viewport;
    const imodel = viewport ? viewport.iModel : /* istanbul ignore next */ UiFramework_1.UiFramework.getIModelConnection();
    const navigationAidControl = ConfigurableUiManager_1.ConfigurableUiManager.createControl(navigationAidId, navigationAidId, { imodel, viewport });
    navigationAidControl.initialize();
    return navigationAidControl;
}
/**
 * NavigationAidHost is a component that hosts a NavigationAid that is specific to the active content control.
 * @public
 */
function NavigationAidHost(props) {
    const [activeContentControl, setActiveContentControl] = React.useState(() => ContentViewManager_1.ContentViewManager.getActiveContentControl());
    const [activeContentViewport, setActiveContentViewport] = React.useState(() => /* istanbul ignore next */ activeContentControl === null || activeContentControl === void 0 ? void 0 : activeContentControl.viewport);
    const [navigationAidId, setNavigationAidId] = React.useState(() => activeContentControl ? activeContentControl.navigationAidControl : /* istanbul ignore next */ "");
    React.useEffect(() => {
        // istanbul ignore next
        const handleContentControlActivated = (args) => {
            setActiveContentControl(args.activeContentControl);
            setActiveContentViewport(args.activeContentControl.viewport);
            setNavigationAidId(args.activeContentControl.navigationAidControl);
        };
        FrontstageManager_1.FrontstageManager.onContentControlActivatedEvent.addListener(handleContentControlActivated);
        return () => {
            FrontstageManager_1.FrontstageManager.onContentControlActivatedEvent.removeListener(handleContentControlActivated);
        };
    }, []);
    const [activeViewClass, setActiveViewClass] = React.useState(() => {
        const content = ContentViewManager_1.ContentViewManager.getActiveContentControl();
        if (content && content.viewport)
            return content.viewport.view.classFullName;
        return "";
    });
    React.useEffect(() => {
        // istanbul ignore next
        const handleViewClassFullNameChange = (args) => {
            setActiveViewClass(args.newName);
        };
        ui_components_1.ViewportComponentEvents.onViewClassFullNameChangedEvent.addListener(handleViewClassFullNameChange);
        return () => {
            ui_components_1.ViewportComponentEvents.onViewClassFullNameChangedEvent.removeListener(handleViewClassFullNameChange);
        };
    }, [activeViewClass]);
    const navigationAidControl = React.useMemo(() => createNavigationAidControl(activeContentControl, navigationAidId, activeContentViewport), [activeContentControl, navigationAidId, activeContentViewport]);
    const ref = React.useRef(null);
    const isInitialMount = React.useRef(true);
    const { onElementRef, proximityScale } = ui_core_1.useWidgetOpacityContext();
    React.useEffect(() => {
        // istanbul ignore else
        if (isInitialMount.current) {
            isInitialMount.current = false;
            onElementRef(ref);
        }
    }, [onElementRef]);
    const divStyle = {
        minWidth: props.minWidth ? /* istanbul ignore next */ props.minWidth : "64px",
        minHeight: props.minHeight ? /* istanbul ignore next */ props.minHeight : "64px",
    };
    if ("1" !== useFrameworkVersion_1.useFrameworkVersion() && UiShowHideManager_1.UiShowHideManager.useProximityOpacity && !UiFramework_1.UiFramework.isMobile()) {
        const navigationAidOpacity = (0.30 * proximityScale) + 0.70;
        divStyle.opacity = `${navigationAidOpacity}`;
    }
    return (React.createElement("div", { style: divStyle, ref: ref }, navigationAidControl && navigationAidControl.reactNode));
}
exports.NavigationAidHost = NavigationAidHost;
/**
 * Component that Composes a NavigationWidget typically using toolbars generated via [[ToolbarComposer]] class. The Navigation widget is shown in the top right of the content area
 * and typically holds tools to visually navigate, orient, and zoom to specific content.
 * @public
 */
function NavigationWidgetComposer(props) {
    const { navigationAidHost, horizontalToolbar, verticalToolbar } = props, otherProps = __rest(props, ["navigationAidHost", "horizontalToolbar", "verticalToolbar"]);
    const [elementSet] = React.useState(new ui_core_1.WidgetElementSet());
    const handleChildRef = React.useCallback((elementRef) => {
        elementSet.add(elementRef);
    }, [elementSet]);
    const proximityScale = ui_core_1.useProximityToMouse(elementSet, UiShowHideManager_1.UiShowHideManager.snapWidgetOpacity);
    return (React.createElement(ui_core_1.WidgetOpacityContext.Provider, { value: {
            onElementRef: handleChildRef,
            proximityScale,
        } },
        React.createElement(ui_ninezone_1.NavigationArea, Object.assign({ navigationAid: navigationAidHost ? /* istanbul ignore next */ navigationAidHost : React.createElement(NavigationAidHost, null), horizontalToolbar: horizontalToolbar, verticalToolbar: verticalToolbar }, otherProps, { onMouseEnter: UiShowHideManager_1.UiShowHideManager.handleWidgetMouseEnter }))));
}
exports.NavigationWidgetComposer = NavigationWidgetComposer;
//# sourceMappingURL=NavigationWidgetComposer.js.map