"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Widget
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.VisibilityWidget = exports.IModelConnectedVisibilityComponent = exports.VisibilityComponent = exports.VisibilityComponentHierarchy = void 0;
require("./VisibilityWidget.scss");
const React = require("react");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const hierarchy_tree_svg_sprite_1 = require("@bentley/icons-generic/icons/hierarchy-tree.svg?sprite");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_components_1 = require("@bentley/ui-components");
const ui_core_1 = require("@bentley/ui-core");
const CategoriesTreeWithSearchBox_1 = require("../imodel-components/category-tree/CategoriesTreeWithSearchBox");
const ModelsTree_1 = require("../imodel-components/models-tree/ModelsTree");
const SpatialContainmentTree_1 = require("../imodel-components/spatial-tree/SpatialContainmentTree");
const connectIModel_1 = require("../redux/connectIModel");
const UiFramework_1 = require("../UiFramework");
const WidgetControl_1 = require("../widgets/WidgetControl");
// cspell:ignore modeltree
/* eslint-disable deprecation/deprecation */
/**
 * Types of hierarchies displayed in the `VisibilityComponent`
 * @public @deprecated
 */
var VisibilityComponentHierarchy;
(function (VisibilityComponentHierarchy) {
    VisibilityComponentHierarchy["Models"] = "models";
    VisibilityComponentHierarchy["Categories"] = "categories";
    VisibilityComponentHierarchy["SpatialContainment"] = "spatial-containment";
})(VisibilityComponentHierarchy = exports.VisibilityComponentHierarchy || (exports.VisibilityComponentHierarchy = {}));
/** VisibilityComponent React component.
 * @beta @deprecated
 */
// istanbul ignore next
class VisibilityComponent extends React.Component {
    constructor(props) {
        super(props);
        this._optionsElement = null;
        this._onViewportChangedHandler = async (args) => {
            if (args.current) {
                this.setState({ viewport: args.current });
            }
        };
        this._onShowOptions = () => {
            this.setState((state) => ({
                showOptions: !state.showOptions,
            }));
        };
        this._onCloseOptions = () => {
            this.setState({ showOptions: false });
        };
        this._onShowTree = (event) => {
            const activeTree = event.target.value;
            this.setState({ activeTree, showSearchBox: false });
        };
        this._onSetEnableAll = () => {
            this._onCloseOptions();
            this.state.showAll.emit();
        };
        this._onClearAll = () => {
            this._onCloseOptions();
            this.state.hideAll.emit();
        };
        this._onToggleSearchBox = () => {
            this.setState((prevState) => ({ showSearchBox: !prevState.showSearchBox }));
        };
        this.state = {
            activeTree: VisibilityComponentHierarchy.Models, showOptions: false, showSearchBox: false,
            viewport: this.props.activeViewport, showAll: new bentleyjs_core_1.BeUiEvent(), hideAll: new bentleyjs_core_1.BeUiEvent(),
        };
    }
    async componentDidMount() {
        imodeljs_frontend_1.IModelApp.viewManager.onSelectedViewportChanged.addListener(this._onViewportChangedHandler);
    }
    /** Remove listeners */
    componentWillUnmount() {
        imodeljs_frontend_1.IModelApp.viewManager.onSelectedViewportChanged.removeListener(this._onViewportChangedHandler);
    }
    shouldEnablePreloading(hierarchy) {
        return this.props.enableHierarchiesPreloading
            && -1 !== this.props.enableHierarchiesPreloading.indexOf(hierarchy);
    }
    _renderModelsTree() {
        const { iModelConnection, config } = this.props;
        const { viewport } = this.state;
        return React.createElement(ModelsTree_1.ModelsTree, Object.assign({ iModel: iModelConnection, activeView: viewport, rootElementRef: this.props.activeTreeRef, enablePreloading: this.shouldEnablePreloading(VisibilityComponentHierarchy.Models) }, config === null || config === void 0 ? void 0 : config.modelsTree));
    }
    _renderSpatialTree() {
        const { iModelConnection, config } = this.props;
        return React.createElement(SpatialContainmentTree_1.SpatialContainmentTree, Object.assign({ iModel: iModelConnection, enablePreloading: this.shouldEnablePreloading(VisibilityComponentHierarchy.SpatialContainment) }, config === null || config === void 0 ? void 0 : config.spatialContainmentTree));
    }
    _renderCategoriesTree() {
        const { iModelConnection } = this.props;
        const { viewport, showSearchBox, showAll, hideAll } = this.state;
        return React.createElement(CategoriesTreeWithSearchBox_1.CategoryTreeWithSearchBox // eslint-disable-line deprecation/deprecation
        , { iModel: iModelConnection, activeView: viewport, showSearchBox: showSearchBox, enablePreloading: this.shouldEnablePreloading(VisibilityComponentHierarchy.Categories), showAll: showAll, hideAll: hideAll });
    }
    _renderTree() {
        const { activeTree } = this.state;
        return (React.createElement("div", { className: "uifw-visibility-tree-wrapper" },
            activeTree === VisibilityComponentHierarchy.Models && this._renderModelsTree(),
            activeTree === VisibilityComponentHierarchy.Categories && this._renderCategoriesTree(),
            activeTree === VisibilityComponentHierarchy.SpatialContainment && this._renderSpatialTree()));
    }
    render() {
        const { iModelConnection } = this.props;
        if (!iModelConnection)
            return (React.createElement("span", null, UiFramework_1.UiFramework.translate("visibilityWidget.noImodelConnection")));
        const { activeTree } = this.state;
        const showCategories = true;
        const showContainment = true;
        const searchStyle = {
            opacity: (activeTree === VisibilityComponentHierarchy.Categories) ? 1 : 0,
            visibility: (activeTree === VisibilityComponentHierarchy.Categories) ? "visible" : "hidden",
        };
        return (React.createElement("div", { className: "uifw-visibility-tree" },
            React.createElement("div", { className: "uifw-visibility-tree-header" },
                React.createElement("select", { className: "uifw-visibility-tree-select", onChange: this._onShowTree.bind(this) },
                    React.createElement("option", { value: VisibilityComponentHierarchy.Models }, UiFramework_1.UiFramework.translate("visibilityWidget.modeltree")),
                    showCategories && React.createElement("option", { value: VisibilityComponentHierarchy.Categories }, UiFramework_1.UiFramework.translate("visibilityWidget.categories")),
                    showContainment && React.createElement("option", { value: VisibilityComponentHierarchy.SpatialContainment }, UiFramework_1.UiFramework.translate("visibilityWidget.containment"))),
                React.createElement("span", { className: "icon icon-search", style: searchStyle, onClick: this._onToggleSearchBox, role: "button", tabIndex: -1 }),
                React.createElement("span", { className: "uifw-visibility-tree-options icon icon-more-vertical-2", style: searchStyle, title: UiFramework_1.UiFramework.translate("visibilityWidget.options"), ref: (element) => { this._optionsElement = element; }, onClick: this._onShowOptions.bind(this), role: "button", tabIndex: -1 }),
                React.createElement(ui_components_1.ContextMenu, { parent: this._optionsElement, isOpened: this.state.showOptions, onClickOutside: this._onCloseOptions.bind(this), position: ui_abstract_1.RelativePosition.BottomRight },
                    React.createElement(ui_components_1.ContextMenuItem, { key: 0, icon: "icon-visibility", onClick: this._onSetEnableAll, name: UiFramework_1.UiFramework.translate("pickerButtons.showAll") }),
                    React.createElement(ui_components_1.ContextMenuItem, { key: 1, icon: "icon-visibility-hide-2", onClick: this._onClearAll, name: UiFramework_1.UiFramework.translate("pickerButtons.hideAll") }))),
            this._renderTree()));
    }
}
exports.VisibilityComponent = VisibilityComponent;
/** VisibilityComponent that is connected to the IModelConnection property in the Redux store. The application must set up the Redux store and include the FrameworkReducer.
 * @beta @deprecated
 */
exports.IModelConnectedVisibilityComponent = connectIModel_1.connectIModelConnection(null, null)(VisibilityComponent); // eslint-disable-line @typescript-eslint/naming-convention
/** VisibilityWidget React component.
 * @beta @deprecated
 */
// istanbul ignore next
class VisibilityWidget extends WidgetControl_1.WidgetControl {
    constructor(info, options) {
        super(info, options);
        this._activeTreeRef = React.createRef();
        if (options && options.iModelConnection)
            this.reactNode = React.createElement(VisibilityComponent, { iModelConnection: options.iModelConnection, activeViewport: imodeljs_frontend_1.IModelApp.viewManager.selectedView, activeTreeRef: this._activeTreeRef, enableHierarchiesPreloading: options.enableHierarchiesPreloading, config: options.config });
        else // use the connection from redux
            this.reactNode = React.createElement(exports.IModelConnectedVisibilityComponent, { activeViewport: imodeljs_frontend_1.IModelApp.viewManager.selectedView, activeTreeRef: this._activeTreeRef, enableHierarchiesPreloading: options.enableHierarchiesPreloading, config: options.config });
    }
    static get iconSpec() {
        return ui_abstract_1.IconSpecUtilities.createSvgIconSpec(hierarchy_tree_svg_sprite_1.default);
    }
    static get label() {
        return UiFramework_1.UiFramework.translate("visibilityWidget.visibilityTree");
    }
    saveTransientState() {
        if (this._activeTreeRef.current)
            this._maintainScrollPosition = new ui_core_1.ScrollPositionMaintainer(this._activeTreeRef.current);
    }
    restoreTransientState() {
        if (this._maintainScrollPosition) {
            this._maintainScrollPosition.dispose();
            this._maintainScrollPosition = undefined;
        }
        return true;
    }
}
exports.VisibilityWidget = VisibilityWidget;
//# sourceMappingURL=VisibilityWidget.js.map