"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Widget
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.WidgetDef = exports.WidgetType = exports.WidgetStateChangedEvent = void 0;
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_core_1 = require("@bentley/ui-core");
const ConfigurableUiControl_1 = require("../configurableui/ConfigurableUiControl");
const ConfigurableUiManager_1 = require("../configurableui/ConfigurableUiManager");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const SyncUiEventDispatcher_1 = require("../syncui/SyncUiEventDispatcher");
const UiFramework_1 = require("../UiFramework");
const PropsHelper_1 = require("../utils/PropsHelper");
const widgetStateNameMap = new Map([
    [ui_abstract_1.WidgetState.Closed, "Closed"],
    [ui_abstract_1.WidgetState.Floating, "Floating"],
    [ui_abstract_1.WidgetState.Hidden, "Hidden"],
    [ui_abstract_1.WidgetState.Open, "Open"],
    [ui_abstract_1.WidgetState.Unloaded, "Unloaded"],
]);
/** Widget State Changed Event class.
 * @public
 */
class WidgetStateChangedEvent extends ui_core_1.UiEvent {
}
exports.WidgetStateChangedEvent = WidgetStateChangedEvent;
/** Widget type enum.
 * @public
 */
var WidgetType;
(function (WidgetType) {
    WidgetType[WidgetType["Tool"] = 0] = "Tool";
    WidgetType[WidgetType["Navigation"] = 1] = "Navigation";
    WidgetType[WidgetType["FreeFrom"] = 2] = "FreeFrom";
    WidgetType[WidgetType["Rectangular"] = 3] = "Rectangular";
    WidgetType[WidgetType["ToolSettings"] = 4] = "ToolSettings";
    WidgetType[WidgetType["StatusBar"] = 5] = "StatusBar";
})(WidgetType = exports.WidgetType || (exports.WidgetType = {}));
// -----------------------------------------------------------------------------
/** A Widget Definition in the 9-Zone Layout system.
 * @public
 */
class WidgetDef {
    constructor(widgetProps) {
        this._label = "";
        this._tooltip = "";
        this._defaultState = ui_abstract_1.WidgetState.Unloaded;
        this._state = ui_abstract_1.WidgetState.Unloaded;
        this._classId = undefined;
        this._priority = 0;
        this._isFreeform = false;
        this._isFloatingStateSupported = false;
        this._isFloatingStateWindowResizable = true;
        this._isToolSettings = false;
        this._isStatusBar = false;
        this._stateChanged = false;
        this._fillZone = false;
        this._syncEventIds = [];
        this._widgetType = WidgetType.Rectangular;
        this._tabLocation = {
            side: "left",
            tabIndex: 0,
            widgetId: "",
            widgetIndex: 0,
        };
        this._handleSyncUiEvent = (args) => {
            if ((this.syncEventIds.length > 0) && this.syncEventIds.some((value) => args.eventIds.has(value.toLowerCase()))) {
                // istanbul ignore else
                if (this.stateFunc) {
                    let newState = this.state;
                    newState = this.stateFunc(newState);
                    this.setWidgetState(newState);
                }
            }
        };
        if (widgetProps.id !== undefined)
            this._id = widgetProps.id;
        else {
            WidgetDef._sId++;
            this._id = `Widget-${WidgetDef._sId}`;
        }
        WidgetDef.initializeFromWidgetProps(widgetProps, this);
    }
    get state() { return this._state; }
    get id() { return this._id; }
    get classId() { return this._classId; }
    get priority() { return this._priority; }
    get isFreeform() { return this._isFreeform; }
    get isFloatingStateSupported() { return this._isFloatingStateSupported; }
    get isFloatingStateWindowResizable() { return this._isFloatingStateWindowResizable; }
    get isToolSettings() { return this._isToolSettings; }
    get isStatusBar() { return this._isStatusBar; }
    get stateChanged() { return this._stateChanged; }
    get fillZone() { return this._fillZone; }
    get syncEventIds() { return this._syncEventIds; }
    get stateFunc() { return this._stateFunc; }
    get applicationData() { return this._applicationData; }
    get isFloating() { return this._state === ui_abstract_1.WidgetState.Floating; }
    get iconSpec() { return this._iconSpec; }
    get badgeType() { return this._badgeType; }
    get widgetType() { return this._widgetType; }
    set widgetType(type) { this._widgetType = type; }
    /** @internal */
    get tabLocation() { return this._tabLocation; }
    set tabLocation(tabLocation) { this._tabLocation = tabLocation; }
    /** @internal */
    get defaultState() { return this._defaultState; }
    static initializeFromWidgetProps(widgetProps, me) {
        if (widgetProps.label)
            me.setLabel(widgetProps.label);
        else if (widgetProps.labelKey)
            me._label = UiFramework_1.UiFramework.i18n.translate(widgetProps.labelKey);
        me.setCanPopout(widgetProps.canPopout);
        if (widgetProps.priority !== undefined)
            me._priority = widgetProps.priority;
        if (widgetProps.tooltip)
            me.setTooltip(widgetProps.tooltip);
        else if (widgetProps.tooltipKey)
            me._tooltip = UiFramework_1.UiFramework.i18n.translate(widgetProps.tooltipKey);
        if (widgetProps.control !== undefined)
            me._classId = widgetProps.control;
        else if (widgetProps.classId !== undefined)
            me._classId = widgetProps.classId;
        if (widgetProps.defaultState !== undefined) {
            me._state = widgetProps.defaultState;
            me._defaultState = widgetProps.defaultState;
        }
        if (widgetProps.isFreeform !== undefined) {
            me._isFreeform = widgetProps.isFreeform;
            me._widgetType = me.isFreeform ? WidgetType.FreeFrom : WidgetType.Rectangular;
        }
        if (widgetProps.isFloatingStateSupported !== undefined)
            me._isFloatingStateSupported = widgetProps.isFloatingStateSupported;
        if (widgetProps.isFloatingStateWindowResizable !== undefined)
            me._isFloatingStateWindowResizable = widgetProps.isFloatingStateWindowResizable;
        if (widgetProps.isToolSettings !== undefined)
            me._isToolSettings = widgetProps.isToolSettings;
        if (widgetProps.isStatusBar !== undefined)
            me._isStatusBar = widgetProps.isStatusBar;
        if (widgetProps.fillZone !== undefined)
            me._fillZone = widgetProps.fillZone;
        if (widgetProps.applicationData !== undefined)
            me._applicationData = widgetProps.applicationData;
        if (widgetProps.element !== undefined)
            me._widgetReactNode = widgetProps.element;
        if (widgetProps.iconSpec !== undefined)
            me._iconSpec = widgetProps.iconSpec;
        // istanbul ignore if
        if (widgetProps.icon !== undefined)
            me._iconSpec = widgetProps.icon;
        if (widgetProps.badgeType !== undefined)
            me._badgeType = widgetProps.badgeType;
        me._preferredPanelSize = widgetProps.preferredPanelSize;
        me._onWidgetStateChanged = widgetProps.onWidgetStateChanged;
        me._saveTransientState = widgetProps.saveTransientState;
        me._restoreTransientState = widgetProps.restoreTransientState;
        me.setUpSyncSupport(widgetProps);
    }
    static createWidgetPropsFromAbstractProps(abstractWidgetProps) {
        const widgetProps = abstractWidgetProps;
        widgetProps.element = abstractWidgetProps.getWidgetContent();
        return widgetProps;
    }
    setUpSyncSupport(props) {
        if (props.stateFunc && props.syncEventIds && props.syncEventIds.length > 0) {
            this._syncEventIds = props.syncEventIds;
            this._stateFunc = props.stateFunc;
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.onSyncUiEvent.addListener(this._handleSyncUiEvent);
        }
    }
    /** @alpha */
    get preferredPanelSize() {
        return this._preferredPanelSize;
    }
    /** Get the label string */
    get label() {
        return PropsHelper_1.PropsHelper.getStringFromSpec(this._label);
    }
    /** Set the label.
     * @param v A string or a function to get the string.
     */
    setLabel(v) {
        if (this._label === v)
            return;
        this._label = v;
        FrontstageManager_1.FrontstageManager.onWidgetLabelChangedEvent.emit({ widgetDef: this });
    }
    /** Get the tooltip string */
    get tooltip() {
        return PropsHelper_1.PropsHelper.getStringFromSpec(this._tooltip);
    }
    /** Set the tooltip.
     * @param v A string or a function to get the string.
     */
    setTooltip(v) {
        this._tooltip = v;
    }
    get widgetControl() {
        return this._widgetControl;
    }
    getWidgetControl(type) {
        if (!this._widgetControl && this.classId) {
            let usedClassId = "";
            if (typeof this.classId === "string") {
                // istanbul ignore else
                if (this.classId)
                    this._widgetControl = ConfigurableUiManager_1.ConfigurableUiManager.createControl(this.classId, this.id, this.applicationData);
                usedClassId = this.classId;
            }
            else {
                const info = new ConfigurableUiControl_1.ConfigurableCreateInfo(this.classId.name, this.id, this.id);
                usedClassId = this.classId.name;
                this._widgetControl = new this.classId(info, this.applicationData);
            }
            // istanbul ignore else
            if (this._widgetControl) {
                if (this._widgetControl.getType() !== type) {
                    throw new ui_abstract_1.UiError(UiFramework_1.UiFramework.loggerCategory(this), `getWidgetControl: '${usedClassId}' is NOT a ${type}; it is a ${this._widgetControl.getType()}`);
                }
                this._widgetControl.widgetDef = this;
                this._widgetControl.initialize();
            }
        }
        return this._widgetControl;
    }
    get reactNode() {
        if (!this._widgetReactNode) {
            const widgetControl = this.getWidgetControl(ConfigurableUiControl_1.ConfigurableUiControlType.Widget);
            // istanbul ignore else
            if (widgetControl && widgetControl.reactNode)
                this._widgetReactNode = widgetControl.reactNode;
        }
        return this._widgetReactNode;
    }
    set reactNode(node) {
        this._widgetReactNode = node;
    }
    /** @deprecated use reactNode */
    // istanbul ignore next
    get reactElement() {
        return this.reactNode;
    }
    // istanbul ignore next
    set reactElement(node) {
        this.reactNode = node;
    }
    setWidgetState(newState) {
        if (this.state === newState)
            return;
        this._state = newState;
        this._stateChanged = true;
        FrontstageManager_1.FrontstageManager.onWidgetStateChangedEvent.emit({ widgetDef: this, widgetState: newState });
        this.onWidgetStateChanged();
    }
    setCanPopout(value) {
        this._canPopout = value;
    }
    get canPopout() {
        return (this._canPopout);
    }
    canOpen() {
        return (this.isFloating || this.isActive);
    }
    get isVisible() {
        return ui_abstract_1.WidgetState.Hidden !== this.state;
    }
    get activeState() {
        return this.state;
    }
    get isActive() {
        return ui_abstract_1.WidgetState.Open === this.activeState;
    }
    onWidgetStateChanged() {
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        this.widgetControl && UiFramework_1.UiFramework.postTelemetry(`Widget ${this.widgetControl.classId} state set to ${widgetStateNameMap.get(this._state)}`, "35402486-9839-441E-A5C7-46D546142D11");
        this.widgetControl && this.widgetControl.onWidgetStateChanged();
        this._onWidgetStateChanged && this._onWidgetStateChanged();
    }
    /** Overwrite to save transient DOM state (i.e. scroll offset). */
    saveTransientState() {
        this.widgetControl && this.widgetControl.saveTransientState();
        this._saveTransientState && this._saveTransientState();
    }
    /** Overwrite to restore transient DOM state.
     * @note Return true if the state is restored or the Widget will remount.
     */
    restoreTransientState() {
        let result = true;
        if (this.widgetControl || this._restoreTransientState) {
            let result1 = false, result2 = false;
            if (this.widgetControl)
                result1 = this.widgetControl.restoreTransientState();
            if (this._restoreTransientState)
                result2 = this._restoreTransientState();
            result = !(result1 || result2);
        }
        return result;
    }
    /** Opens the widget and makes it visible to the user.
     * I.e. opens the stage panel or brings the floating widget to front of the screen.
     * @alpha
     */
    show() {
        FrontstageManager_1.FrontstageManager.onWidgetShowEvent.emit({ widgetDef: this });
    }
    /** Opens the widget and expands it to fill full size of the stage panel.
     * @alpha
     */
    expand() {
        FrontstageManager_1.FrontstageManager.onWidgetExpandEvent.emit({ widgetDef: this });
    }
}
exports.WidgetDef = WidgetDef;
WidgetDef._sId = 0;
//# sourceMappingURL=WidgetDef.js.map