"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Widget
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.WidgetManager = exports.WidgetsChangedEvent = exports.WidgetProvidersChangedEvent = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const ui_abstract_1 = require("@bentley/ui-abstract");
const UiFramework_1 = require("../UiFramework");
const Zone_1 = require("../zones/Zone");
const WidgetDef_1 = require("./WidgetDef");
const StableWidgetDef_1 = require("./StableWidgetDef");
/** Event class for [[this.onWidgetProvidersChanged]].
 * @internal
 */
class WidgetProvidersChangedEvent extends bentleyjs_core_1.BeUiEvent {
}
exports.WidgetProvidersChangedEvent = WidgetProvidersChangedEvent;
/** Event class for [[this.onWidgetsChanged]].
 * @internal
 */
class WidgetsChangedEvent extends bentleyjs_core_1.BeUiEvent {
}
exports.WidgetsChangedEvent = WidgetsChangedEvent;
function isZoneLocation(location) {
    return location >= Zone_1.ZoneLocation.TopLeft && location <= Zone_1.ZoneLocation.BottomRight;
}
function getLocationName(location) {
    return isZoneLocation(location) ? Zone_1.ZoneLocation[location] : ui_abstract_1.StagePanelLocation[location];
}
function getWidgetManagerStableWidgetId(stageUsage, location, section, index) {
    return `uifw-wm-${stageUsage || ""}-${getLocationName(location)}-${ui_abstract_1.StagePanelSection[section]}-${index}`;
}
function getWidgetProviderStableWidgetId(providerId, stageUsage, location, section, index) {
    return `uifw-wp-${providerId}-${stageUsage}-${getLocationName(location)}-${ui_abstract_1.StagePanelSection[section]}-${index}`;
}
function getAddonStableWidgetId(stageUsage, location, section, index) {
    return `uifw-addon-${stageUsage}-${ui_abstract_1.StagePanelLocation[location]}-${ui_abstract_1.StagePanelSection[section]}-${index}`;
}
/** Widget Manager class.
 * @beta
 */
class WidgetManager {
    constructor() {
        this._widgets = [];
        this._providers = [];
        /** Event raised when Widgets are changed.
         * @internal
         */
        this.onWidgetsChanged = new WidgetsChangedEvent();
        /** Event raised when WidgetProviders are changed.
         * @internal
         */
        this.onWidgetProvidersChanged = new WidgetProvidersChangedEvent();
    }
    /** @internal */
    get widgetCount() {
        return this._widgets.length;
    }
    /** @internal */
    get widgets() { return this._widgets; }
    set widgets(w) {
        this._widgets = w;
        this.onWidgetsChanged.emit({ items: w });
    }
    /** Adds a WidgetDef for use in a Frontstage.
     * @note Added `widgetDef` must return unique id to correctly save/restore App layout.
     * Semi-stable id is generated when auto-generated `widgetDef` id is detected,
     * but correctness of such id depends on `addWidgetDef` call order and widget location.
     */
    addWidgetDef(widgetDef, stageId, stageUsage, location, section) {
        if (stageId === undefined && stageUsage === undefined) {
            bentleyjs_core_1.Logger.logError(UiFramework_1.UiFramework.loggerCategory(this), `addWidgetDef: stageId or stageUsage param must be specified`);
            return false;
        }
        section = (section !== undefined) ? section : ui_abstract_1.StagePanelSection.Start;
        const index = this._widgets.reduce((acc, info) => {
            if (info.stageId === stageId && info.stageUsage === stageUsage && info.location === location && info.section === section)
                return acc + 1;
            return acc;
        }, 0);
        const stableId = getWidgetManagerStableWidgetId(stageUsage, location, section, index);
        const stableWidget = StableWidgetDef_1.createStableWidgetDef(widgetDef, stableId);
        const newWidget = { widgetDef: stableWidget, stageId, stageUsage, location, section };
        const oldWidgets = this._widgets.filter((info) => info.widgetDef.id !== newWidget.widgetDef.id);
        const updatedWidgets = [
            ...oldWidgets,
            newWidget,
        ];
        this.widgets = updatedWidgets;
        return true;
    }
    /** Removes a WidgetDef.
     */
    removeWidgetDef(widgetId) {
        let result = false;
        const updatedWidgets = this._widgets.filter((info) => info.widgetDef.id !== widgetId);
        if (updatedWidgets.length !== this._widgets.length) {
            this.widgets = updatedWidgets;
            result = true;
        }
        return result;
    }
    /** @internal */
    get providers() { return this._providers; }
    set providers(p) {
        this._providers = p;
        this.onWidgetProvidersChanged.emit({ providers: p });
    }
    /** Adds a WidgetDef Provider
     */
    addWidgetProvider(widgetProvider) {
        const oldProviders = this._providers.filter((p) => p.id !== widgetProvider.id);
        const updatedProviders = [
            ...oldProviders,
            widgetProvider,
        ];
        this.providers = updatedProviders;
    }
    /** Removes a WidgetDef Provider
     */
    removeWidgetProvider(providerId) {
        let result = false;
        const updatedProviders = this._providers.filter((p) => p.id !== providerId);
        if (updatedProviders.length !== this._providers.length) {
            this.providers = updatedProviders;
            result = true;
        }
        return result;
    }
    // Used when WidgetDefs are requested from UiItemProviders when uiVersion="1"
    getStagePanelLocationFromZoneLocation(location) {
        switch (location) {
            case Zone_1.ZoneLocation.BottomCenter:
            case Zone_1.ZoneLocation.TopCenter:
            case Zone_1.ZoneLocation.TopRight:
            case Zone_1.ZoneLocation.TopLeft:
                return undefined; // an existing stage does not support appending widgets to these zones
            case Zone_1.ZoneLocation.BottomLeft:
            case Zone_1.ZoneLocation.CenterLeft:
                return ui_abstract_1.StagePanelLocation.Left;
            case Zone_1.ZoneLocation.BottomRight:
            case Zone_1.ZoneLocation.CenterRight:
                return ui_abstract_1.StagePanelLocation.Right;
        }
    }
    /** Gets WidgetDefs for a Frontstage location.
     */
    getWidgetDefs(stageId, stageUsage, location, section) {
        const definedSection = section === undefined ? ui_abstract_1.StagePanelSection.Start : section;
        const widgetInfos = this._widgets.filter((info) => {
            return (!info.stageId || info.stageId === stageId)
                && (!info.stageUsage || info.stageUsage === stageUsage)
                && info.location === location
                && info.section === definedSection;
        });
        let widgetDefs = widgetInfos.map((info) => info.widgetDef);
        // Consult the registered WidgetProviders
        this._providers.forEach((p, index) => {
            const wds = p.getWidgetDefs(stageId, stageUsage, location, definedSection);
            if (wds) {
                const stableWds = wds.map((wd) => {
                    const stableId = getWidgetProviderStableWidgetId(p.id, stageUsage, location, definedSection, index);
                    return StableWidgetDef_1.createStableWidgetDef(wd, stableId);
                });
                widgetDefs = widgetDefs.concat(stableWds);
            }
        });
        // Consult the UiItemsManager to get any Abstract widgets
        if (location in ui_abstract_1.StagePanelLocation) {
            const widgets = ui_abstract_1.UiItemsManager.getWidgets(stageId, stageUsage, location, definedSection);
            const updatedWidgets = ui_abstract_1.UiItemsArbiter.updateWidgets(widgets);
            updatedWidgets.forEach((abstractProps, index) => {
                const props = WidgetDef_1.WidgetDef.createWidgetPropsFromAbstractProps(abstractProps);
                const stableId = getAddonStableWidgetId(stageUsage, location, definedSection, index);
                const stableProps = Zone_1.getStableWidgetProps(props, stableId);
                const wd = new WidgetDef_1.WidgetDef(stableProps);
                widgetDefs.push(wd);
            });
        }
        else {
            // istanbul ignore else
            if (location in Zone_1.ZoneLocation) {
                const panelLocation = this.getStagePanelLocationFromZoneLocation(location);
                // istanbul ignore else
                if (panelLocation && location in ui_abstract_1.AbstractZoneLocation) {
                    const widgets = ui_abstract_1.UiItemsManager.getWidgets(stageId, stageUsage, panelLocation, undefined, location);
                    const updatedWidgets = ui_abstract_1.UiItemsArbiter.updateWidgets(widgets);
                    updatedWidgets.forEach((abstractProps, index) => {
                        const props = WidgetDef_1.WidgetDef.createWidgetPropsFromAbstractProps(abstractProps);
                        const stableId = getAddonStableWidgetId(stageUsage, location, definedSection, index);
                        const stableProps = Zone_1.getStableWidgetProps(props, stableId);
                        const wd = new WidgetDef_1.WidgetDef(stableProps);
                        widgetDefs.push(wd);
                    });
                }
            }
        }
        return widgetDefs.length > 0 ? widgetDefs : undefined;
    }
}
exports.WidgetManager = WidgetManager;
//# sourceMappingURL=WidgetManager.js.map