"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Widget
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.WidgetStackTab = exports.WidgetStackTabGroup = exports.WidgetStackTabs = exports.WidgetStack = void 0;
const React = require("react");
const ui_core_1 = require("@bentley/ui-core");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const UiShowHideManager_1 = require("../utils/UiShowHideManager");
/** Widget stack React component.
 * @internal
 */
class WidgetStack extends React.PureComponent {
    constructor() {
        super(...arguments);
        this._widgetStack = React.createRef();
        this._handleOnWidgetResize = (resizeBy, handle, filledHeightDiff) => {
            this.props.widgetChangeHandler.handleResize(this.props.widgets[0], resizeBy, handle, filledHeightDiff);
        };
        this._handleTabDragStart = (widgetId, tabIndex, initialPosition, firstTabBounds) => {
            if (!this._widgetStack.current)
                return;
            const tabBounds = ui_core_1.Rectangle.create(firstTabBounds);
            const stackedWidgetBounds = ui_core_1.Rectangle.create(this._widgetStack.current.getBounds());
            const offsetToFirstTab = stackedWidgetBounds.topLeft().getOffsetTo(tabBounds.topLeft());
            const isHorizontal = this.props.verticalAnchor === ui_ninezone_1.VerticalAnchor.BottomPanel || this.props.verticalAnchor === ui_ninezone_1.VerticalAnchor.TopPanel;
            let widgetBounds;
            if (isHorizontal)
                widgetBounds = stackedWidgetBounds.offsetX(offsetToFirstTab.x);
            else
                widgetBounds = stackedWidgetBounds.offsetY(offsetToFirstTab.y);
            this.props.widgetChangeHandler.handleTabDragStart(widgetId, tabIndex, initialPosition, widgetBounds);
        };
        this._handleTabDragEnd = () => {
            this.props.widgetChangeHandler.handleTabDragEnd();
        };
        this._handleTabClick = (widgetId, tabIndex) => {
            this.props.widgetChangeHandler.handleTabClick(widgetId, tabIndex);
        };
        this._handleTabDrag = (dragged) => {
            this.props.widgetChangeHandler.handleTabDrag(dragged);
        };
    }
    render() {
        const tabCount = this.props.widgets.reduce((acc, widgetId) => {
            const tabs = this.props.widgetTabs[widgetId];
            return acc + tabs.length;
        }, 0);
        if (tabCount === 0)
            return null;
        return (React.createElement(ui_ninezone_1.Stacked, { className: this.props.className, contentRef: this.props.openWidgetId ? this.props.getWidgetContentRef(this.props.openWidgetId) : undefined, disabledResizeHandles: this.props.disabledResizeHandles, fillZone: this.props.fillZone || this.props.isInStagePanel, horizontalAnchor: this.props.horizontalAnchor, isCollapsed: this.props.isCollapsed, isTabBarVisible: this.props.isInStagePanel, isDragged: !!this.props.draggedWidget, isFloating: this.props.isFloating, isOpen: !!this.props.openWidgetId, onMouseEnter: UiShowHideManager_1.UiShowHideManager.handleWidgetMouseEnter, onResize: this.props.isInStagePanel ? undefined : this._handleOnWidgetResize, ref: this._widgetStack, style: this.props.style, tabs: React.createElement(WidgetStackTabs, { activeTabIndex: this.props.activeTabIndex, draggedWidget: this.props.draggedWidget, horizontalAnchor: this.props.horizontalAnchor, isCollapsed: this.props.isCollapsed, isProtruding: !this.props.isInStagePanel, onTabClick: this._handleTabClick, onTabDrag: this._handleTabDrag, onTabDragEnd: this._handleTabDragEnd, onTabDragStart: this._handleTabDragStart, openWidgetId: this.props.openWidgetId, verticalAnchor: this.props.verticalAnchor, widgets: this.props.widgets, widgetTabs: this.props.widgetTabs }), verticalAnchor: this.props.verticalAnchor }));
    }
}
exports.WidgetStack = WidgetStack;
/** Tabs of [[WidgetStack]] component.
 * @internal
 */
class WidgetStackTabs extends React.PureComponent {
    render() {
        let renderIndex = -1;
        return this.props.widgets.map((widgetId) => {
            const tabs = this.props.widgetTabs[widgetId];
            if (tabs.length <= 0)
                return null;
            renderIndex++;
            return (React.createElement(React.Fragment, { key: widgetId },
                renderIndex < 1 ? undefined : React.createElement(ui_ninezone_1.TabSeparator, { isHorizontal: ui_ninezone_1.VerticalAnchorHelpers.isHorizontal(this.props.verticalAnchor) }),
                React.createElement(WidgetStackTabGroup, { activeTabIndex: this.props.activeTabIndex, draggedWidget: this.props.draggedWidget, horizontalAnchor: this.props.horizontalAnchor, isCollapsed: this.props.isCollapsed, isProtruding: this.props.isProtruding, isStacked: this.props.widgets.length > 1, onTabClick: this.props.onTabClick, onTabDrag: this.props.onTabDrag, onTabDragEnd: this.props.onTabDragEnd, onTabDragStart: this.props.onTabDragStart, openWidgetId: this.props.openWidgetId, tabs: tabs, verticalAnchor: this.props.verticalAnchor, widgetId: widgetId })));
        });
    }
}
exports.WidgetStackTabs = WidgetStackTabs;
/** Widget tab group used in [[WidgetStackTabs]] component.
 * @internal
 */
class WidgetStackTabGroup extends React.PureComponent {
    constructor() {
        super(...arguments);
        this._firstTab = React.createRef();
        this._handleTabDragStart = (tabIndex, initialPosition) => {
            if (!this._firstTab.current)
                return;
            const firstTabBounds = ui_core_1.Rectangle.create(this._firstTab.current.getBounds()).toProps();
            this.props.onTabDragStart(this.props.widgetId, tabIndex, initialPosition, firstTabBounds);
        };
        this._handleTabClick = (tabIndex) => {
            this.props.onTabClick(this.props.widgetId, tabIndex);
        };
    }
    render() {
        const lastPosition = this.props.draggedWidget ? this.props.draggedWidget.lastPosition : undefined;
        const isWidgetStackOpen = !!this.props.openWidgetId;
        const isWidgetOpen = this.props.openWidgetId === this.props.widgetId;
        const tabs = this.props.tabs.map((tab, index) => {
            const mode = !isWidgetStackOpen ? ui_ninezone_1.TabMode.Closed : isWidgetOpen && this.props.activeTabIndex === index ? ui_ninezone_1.TabMode.Active : ui_ninezone_1.TabMode.Open;
            return (React.createElement(WidgetStackTab, { horizontalAnchor: this.props.horizontalAnchor, iconSpec: tab.iconSpec, index: index, badgeType: tab.badgeType, isCollapsed: this.props.isCollapsed, isProtruding: this.props.isProtruding, key: `${this.props.widgetId}-${index}`, lastPosition: lastPosition, mode: mode, onClick: this._handleTabClick, onDrag: this.props.onTabDrag, onDragEnd: this.props.onTabDragEnd, onDragStart: this._handleTabDragStart, tabRef: index === 0 ? this._firstTab : undefined, title: tab.title, verticalAnchor: this.props.verticalAnchor }));
        });
        if (tabs.length > 1) {
            return (React.createElement(ui_ninezone_1.TabGroup, { handle: this.getTabHandleMode(), horizontalAnchor: this.props.horizontalAnchor, isCollapsed: this.props.isCollapsed, verticalAnchor: this.props.verticalAnchor }, tabs));
        }
        return tabs;
    }
    getTabHandleMode() {
        if (this.props.draggedWidget && this.props.draggedWidget.id === this.props.widgetId && this.props.draggedWidget.isUnmerge)
            return ui_ninezone_1.HandleMode.Visible;
        if (this.props.isStacked)
            return ui_ninezone_1.HandleMode.Hovered;
        return ui_ninezone_1.HandleMode.Timedout;
    }
}
exports.WidgetStackTabGroup = WidgetStackTabGroup;
/** Tab used in [[WidgetStackTabGroup]] component.
 * @internal
 */
class WidgetStackTab extends React.PureComponent {
    constructor() {
        super(...arguments);
        this._handleDragStart = (initialPosition) => {
            this.props.onDragStart(this.props.index, initialPosition);
        };
        this._handleClick = () => {
            this.props.onClick(this.props.index);
        };
    }
    render() {
        return (React.createElement(ui_ninezone_1.Tab, { badge: ui_core_1.BadgeUtilities.getComponentForBadgeType(this.props.badgeType), horizontalAnchor: this.props.horizontalAnchor, isCollapsed: this.props.isCollapsed, isProtruding: this.props.isProtruding, lastPosition: this.props.lastPosition, mode: this.props.mode, onClick: this._handleClick, onDrag: this.props.onDrag, onDragEnd: this.props.onDragEnd, onDragStart: this._handleDragStart, ref: this.props.tabRef, title: this.props.title, verticalAnchor: this.props.verticalAnchor }, ui_core_1.IconHelper.getIconReactNode(this.props.iconSpec)));
    }
}
exports.WidgetStackTab = WidgetStackTab;
//# sourceMappingURL=WidgetStack.js.map