"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module Zone
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.isToolSettingsWidgetManagerProps = exports.Zone = exports.getStableWidgetProps = exports.ZoneLocation = void 0;
const React = require("react");
const ui_abstract_1 = require("@bentley/ui-abstract");
const ui_ninezone_1 = require("@bentley/ui-ninezone");
const ConfigurableUiControl_1 = require("../configurableui/ConfigurableUiControl");
const FrontstageManager_1 = require("../frontstage/FrontstageManager");
const WidgetDef_1 = require("../widgets/WidgetDef");
const FrameworkZone_1 = require("./FrameworkZone");
const StatusBarZone_1 = require("./StatusBarZone");
const ToolSettingsZone_1 = require("./toolsettings/ToolSettingsZone");
/** Enum for [[Zone]] Location.
 * @public
 */
var ZoneLocation;
(function (ZoneLocation) {
    ZoneLocation[ZoneLocation["TopLeft"] = 1] = "TopLeft";
    ZoneLocation[ZoneLocation["TopCenter"] = 2] = "TopCenter";
    ZoneLocation[ZoneLocation["TopRight"] = 3] = "TopRight";
    ZoneLocation[ZoneLocation["CenterLeft"] = 4] = "CenterLeft";
    ZoneLocation[ZoneLocation["CenterRight"] = 6] = "CenterRight";
    ZoneLocation[ZoneLocation["BottomLeft"] = 7] = "BottomLeft";
    ZoneLocation[ZoneLocation["BottomCenter"] = 8] = "BottomCenter";
    ZoneLocation[ZoneLocation["BottomRight"] = 9] = "BottomRight";
})(ZoneLocation = exports.ZoneLocation || (exports.ZoneLocation = {}));
/** @internal */
function getStableWidgetProps(widgetProps, stableId) {
    let props = widgetProps;
    if (props.id === undefined)
        props = Object.assign(Object.assign({}, props), { id: stableId });
    return props;
}
exports.getStableWidgetProps = getStableWidgetProps;
/** Zone React component.
 * A Zone is a standard area on the screen for users to read and interact with data applicable to the current task. Each Zone has a defined purpose.
 * @public
 */
class Zone extends React.Component {
    constructor(props) {
        super(props);
        this._handleWidgetStateChangedEvent = (args) => {
            const runtimeProps = this.props.runtimeProps;
            if (!runtimeProps)
                return;
            const widgetDef = args.widgetDef;
            const id = this.getWidgetIdForDef(widgetDef);
            if (!id)
                return;
            const zoneDef = runtimeProps.zoneDefProvider.getZoneDef(id);
            // istanbul ignore if
            if (!zoneDef)
                return;
            const visibleWidgets = zoneDef.widgetDefs.filter((wd) => wd.isVisible || wd === widgetDef);
            for (let index = 0; index < visibleWidgets.length; index++) {
                const wDef = visibleWidgets[index];
                if (wDef !== widgetDef)
                    continue;
                if (widgetDef.state === ui_abstract_1.WidgetState.Hidden && index < runtimeProps.activeTabIndex && id === runtimeProps.openWidgetId) {
                    // Need to decrease active tab index, since removed tab was rendered before active tab and we want to maintain active tab.
                    runtimeProps.widgetChangeHandler.handleTabClick(id, runtimeProps.activeTabIndex - 1);
                    break;
                }
                runtimeProps.widgetChangeHandler.handleWidgetStateChange(id, index, widgetDef.state === ui_abstract_1.WidgetState.Open);
                break;
            }
        };
    }
    static initializeZoneDef(zoneDef, props) {
        zoneDef.initializeFromProps(props);
        // istanbul ignore else
        if (props.widgets) {
            props.widgets.forEach((widgetNode, index) => {
                // istanbul ignore if
                if (!React.isValidElement(widgetNode))
                    return;
                const stableId = `uifw-z-${ZoneLocation[zoneDef.zoneLocation]}-${index}`;
                const stableProps = getStableWidgetProps(widgetNode.props, stableId);
                const widgetDef = new WidgetDef_1.WidgetDef(stableProps);
                zoneDef.addWidgetDef(widgetDef);
            });
        }
    }
    componentDidMount() {
        FrontstageManager_1.FrontstageManager.onWidgetStateChangedEvent.addListener(this._handleWidgetStateChangedEvent);
    }
    componentWillUnmount() {
        FrontstageManager_1.FrontstageManager.onWidgetStateChangedEvent.removeListener(this._handleWidgetStateChangedEvent);
    }
    render() {
        const { runtimeProps } = this.props;
        if (!runtimeProps)
            return null;
        const { zoneDef } = runtimeProps;
        let widgetElement;
        // istanbul ignore else
        if (runtimeProps.zone.widgets.length === 1) {
            if (zoneDef.isToolSettings && exports.isToolSettingsWidgetManagerProps(runtimeProps.widget) && runtimeProps.widget.mode === ui_ninezone_1.ToolSettingsWidgetMode.TitleBar) {
                const widgetDef = zoneDef.getSingleWidgetDef();
                const isClosed = widgetDef ? (widgetDef.state === ui_abstract_1.WidgetState.Closed || widgetDef.state === ui_abstract_1.WidgetState.Hidden)
                    : /* istanbul ignore next */ false;
                return (React.createElement(ToolSettingsZone_1.ToolSettingsZone, { className: this.props.className, dropTarget: runtimeProps.dropTarget, getWidgetContentRef: runtimeProps.getWidgetContentRef, isClosed: isClosed, isHidden: runtimeProps.isHidden, lastPosition: runtimeProps.draggedWidget && /* istanbul ignore next */ runtimeProps.draggedWidget.lastPosition, style: this.props.style, targetChangeHandler: runtimeProps.targetChangeHandler, targetedBounds: runtimeProps.ghostOutline, widgetChangeHandler: runtimeProps.widgetChangeHandler, zone: runtimeProps.zone }));
            }
            else if (zoneDef.isStatusBar) {
                // istanbul ignore next
                if (runtimeProps.zone.id !== 8)
                    throw new TypeError();
                let widgetControl;
                const widgetDef = zoneDef.getSingleWidgetDef();
                // istanbul ignore else
                if (widgetDef)
                    widgetControl = widgetDef.getWidgetControl(ConfigurableUiControl_1.ConfigurableUiControlType.StatusBarWidget);
                return (React.createElement(StatusBarZone_1.StatusBarZone, { className: this.props.className, dropTarget: runtimeProps.dropTarget, isHidden: runtimeProps.isHidden, isInFooterMode: runtimeProps.isInFooterMode, style: this.props.style, targetChangeHandler: runtimeProps.targetChangeHandler, targetedBounds: runtimeProps.ghostOutline, widgetChangeHandler: runtimeProps.widgetChangeHandler, widgetControl: widgetControl, zoneProps: runtimeProps.zone }));
            }
            const zDef = runtimeProps.zoneDefProvider.getZoneDef(runtimeProps.zone.widgets[0]);
            // istanbul ignore if
            if (!zDef) {
                widgetElement = null;
            }
            else if (zDef.widgetCount === 1 && zDef.widgetDefs[0].widgetType !== WidgetDef_1.WidgetType.Rectangular) {
                /** Return free-form nzWidgetProps */
                const widgetDef = zDef.widgetDefs[0];
                widgetElement = (widgetDef.isVisible) ? widgetDef.reactNode : /* istanbul ignore next */ null;
            }
        }
        return (React.createElement(FrameworkZone_1.FrameworkZone, { activeTabIndex: runtimeProps.activeTabIndex, className: this.props.className, disabledResizeHandles: runtimeProps.disabledResizeHandles, draggedWidget: runtimeProps.draggedWidget, dropTarget: runtimeProps.dropTarget, fillZone: zoneDef.shouldFillZone, getWidgetContentRef: runtimeProps.getWidgetContentRef, isHidden: runtimeProps.isHidden, isInFooterMode: runtimeProps.isInFooterMode, openWidgetId: runtimeProps.openWidgetId, style: this.props.style, targetedBounds: runtimeProps.ghostOutline, targetChangeHandler: runtimeProps.targetChangeHandler, widget: runtimeProps.widget, widgetElement: widgetElement, widgetTabs: runtimeProps.widgetTabs, widgetChangeHandler: runtimeProps.widgetChangeHandler, zone: runtimeProps.zone }));
    }
    getWidgetIdForDef(widgetDef) {
        // istanbul ignore if
        if (!this.props.runtimeProps)
            return undefined;
        for (const wId of this.props.runtimeProps.zone.widgets) {
            const zoneDef = this.props.runtimeProps.zoneDefProvider.getZoneDef(wId);
            if (zoneDef && zoneDef.widgetDefs.some((wDef) => wDef === widgetDef))
                return wId;
        }
        return undefined;
    }
}
exports.Zone = Zone;
/** @internal */
const isToolSettingsWidgetManagerProps = (props) => {
    return !!props && props.id === 2;
};
exports.isToolSettingsWidgetManagerProps = isToolSettingsWidgetManagerProps;
//# sourceMappingURL=Zone.js.map