"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ToolSettings
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultToolSettingsProvider = void 0;
const React = require("react");
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
// cSpell:Ignore configurableui Fronstage
const ui_abstract_1 = require("@bentley/ui-abstract");
const ConfigurableUiManager_1 = require("../../configurableui/ConfigurableUiManager");
const ComponentGenerator_1 = require("../../uiprovider/ComponentGenerator");
const DefaultDialogGridContainer_1 = require("../../uiprovider/DefaultDialogGridContainer");
const ToolSettingsManager_1 = require("./ToolSettingsManager");
const ToolUiProvider_1 = require("./ToolUiProvider");
const FrontstageManager_1 = require("../../frontstage/FrontstageManager");
/** @internal */
/** ToolSettingsUiDataProvider keeps tool data in sync with UI display */
class ToolSettingsUiDataProvider extends ui_abstract_1.UiLayoutDataProvider {
    constructor() {
        super(...arguments);
        // send property changes from UI back to tool
        this.applyUiPropertyChange = (syncItem) => {
            // istanbul ignore next
            imodeljs_frontend_1.IModelApp.toolAdmin.activeTool && imodeljs_frontend_1.IModelApp.toolAdmin.activeTool.applyToolSettingPropertyChange(syncItem);
        };
    }
    supplyDialogItems() {
        return ToolSettingsManager_1.ToolSettingsManager.toolSettingsProperties;
    }
}
/** ToolUiProvider class that informs ConfigurableUi that Tool Settings are provided for the specified tool.
 * @internal
 */
class DefaultToolSettingsProvider extends ToolUiProvider_1.ToolUiProvider {
    constructor(info, options) {
        super(info, options);
        this._dataProvider = new ToolSettingsUiDataProvider();
    }
    get uiDataProvider() { return this._dataProvider; }
    updateToolSettingsNodes() {
        // istanbul ignore else
        if (this.uiDataProvider.layoutDialogRows()) {
            const componentGenerator = new ComponentGenerator_1.ComponentGenerator(this.uiDataProvider);
            this.toolSettingsNode = (React.createElement(DefaultDialogGridContainer_1.DefaultDialogGridContainer, { componentGenerator: componentGenerator, isToolSettings: true }));
            this.horizontalToolSettingNodes = componentGenerator.getToolSettingsEntries();
        }
        else {
            this.toolSettingsNode = null;
            this.horizontalToolSettingNodes = [];
        }
        // We emit the FrontstageManager.onToolSettingsReloadEvent event here because the Fronstage Manager is a static
        // class that UI components can easily register listeners.
        FrontstageManager_1.FrontstageManager.onToolSettingsReloadEvent.emit();
    }
    reloadPropertiesFromTool() {
        // trigger the items to be reloaded from the active tool
        this.uiDataProvider.reloadDialogItems(false);
        this.updateToolSettingsNodes();
    }
    // called when tool is activated to repopulate tool settings.
    onInitialize() {
        this.reloadPropertiesFromTool();
    }
    // called to process ToolSettingsManager.onSyncToolSettingsProperties event
    syncToolSettingsProperties(args) {
        this.uiDataProvider.fireSyncPropertiesEvent(args.syncProperties);
    }
}
exports.DefaultToolSettingsProvider = DefaultToolSettingsProvider;
ConfigurableUiManager_1.ConfigurableUiManager.registerControl("DefaultToolSettings", DefaultToolSettingsProvider);
//# sourceMappingURL=DefaultToolSettingsProvider.js.map