"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module ToolSettings
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolSettingsManager = exports.SyncToolSettingsPropertiesEvent = void 0;
const imodeljs_frontend_1 = require("@bentley/imodeljs-frontend");
const ui_core_1 = require("@bentley/ui-core");
const SyncUiEventDispatcher_1 = require("../../syncui/SyncUiEventDispatcher");
/** Sync Tool Settings Properties Event class.
 * @public
 */
class SyncToolSettingsPropertiesEvent extends ui_core_1.UiEvent {
}
exports.SyncToolSettingsPropertiesEvent = SyncToolSettingsPropertiesEvent;
// -----------------------------------------------------------------------------
// ToolSettingsManager class
// -----------------------------------------------------------------------------
/** Tool Settings Manager class. Used to generate UI components for Tool Settings.
 * @beta
 */
class ToolSettingsManager {
    // istanbul ignore next
    static syncToolSettingsProperties(toolId, syncProperties) {
        ToolSettingsManager.onSyncToolSettingsProperties.emit({ toolId, syncProperties });
    }
    // istanbul ignore next
    static reloadToolSettingsProperties() {
        ToolSettingsManager.onReloadToolSettingsProperties.emit();
    }
    static dispatchSyncUiEvent(syncEventId, useImmediateDispatch) {
        if (useImmediateDispatch)
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchImmediateSyncUiEvent(syncEventId);
        else
            SyncUiEventDispatcher_1.SyncUiEventDispatcher.dispatchSyncUiEvent(syncEventId);
    }
    /** Initializes the ToolSettingsManager */
    static initialize() {
        // istanbul ignore else
        if (imodeljs_frontend_1.IModelApp && imodeljs_frontend_1.IModelApp.toolAdmin) {
            imodeljs_frontend_1.IModelApp.toolAdmin.toolSettingsChangeHandler = ToolSettingsManager.syncToolSettingsProperties;
            imodeljs_frontend_1.IModelApp.toolAdmin.reloadToolSettingsHandler = ToolSettingsManager.reloadToolSettingsProperties;
            imodeljs_frontend_1.IModelApp.toolAdmin.toolSyncUiEventDispatcher = ToolSettingsManager.dispatchSyncUiEvent;
        }
    }
    /** clear cached Tool Settings properties. */
    static clearToolSettingsData() {
        ToolSettingsManager.useDefaultToolSettingsProvider = false;
        ToolSettingsManager._activeToolLabel = "";
        ToolSettingsManager._activeToolDescription = "";
        ToolSettingsManager._toolIdForToolSettings = "";
    }
    /** Cache Tool Settings properties */
    static initializeToolSettingsData(toolSettingsProperties, toolId, toolLabel, toolDescription) {
        ToolSettingsManager.clearToolSettingsData();
        // istanbul ignore else
        if (toolLabel)
            ToolSettingsManager._activeToolLabel = toolLabel;
        // istanbul ignore else
        if (toolDescription)
            ToolSettingsManager._activeToolDescription = toolDescription;
        /* istanbul ignore else */
        if (toolSettingsProperties && toolSettingsProperties.length > 0) {
            // istanbul ignore else
            if (toolId)
                ToolSettingsManager._toolIdForToolSettings = toolId;
            ToolSettingsManager._useDefaultToolSettingsProvider = true;
            return true;
        }
        return false;
    }
    /** Set of data used in Tool Settings for the specified tool. The tool specified should be the "active" tool.
     */
    static initializeDataForTool(tool) {
        ToolSettingsManager.initializeToolSettingsData(tool.supplyToolSettingsProperties(), tool.toolId, tool.flyover, tool.description);
    }
    /** Returns the toolSettings properties that can be used to populate the tool settings widget. */
    static get toolSettingsProperties() {
        if (imodeljs_frontend_1.IModelApp.toolAdmin && imodeljs_frontend_1.IModelApp.toolAdmin.activeTool && imodeljs_frontend_1.IModelApp.toolAdmin.activeTool.toolId === ToolSettingsManager._toolIdForToolSettings) {
            const properties = imodeljs_frontend_1.IModelApp.toolAdmin.activeTool.supplyToolSettingsProperties();
            // istanbul ignore else
            if (properties)
                return properties;
        }
        return [];
    }
    /** Returns true if the Tool Settings are to be auto populated from the toolSettingsProperties.
     * The setter is chiefly for testing.
     */
    static get useDefaultToolSettingsProvider() { return ToolSettingsManager._useDefaultToolSettingsProvider; }
    static set useDefaultToolSettingsProvider(useDefaultToolSettings) { ToolSettingsManager._useDefaultToolSettingsProvider = useDefaultToolSettings; }
    /** The name of the active tool. This is typically the flyover text specified for the tool. */
    static get activeToolLabel() { return ToolSettingsManager._activeToolLabel; }
    static set activeToolLabel(label) { ToolSettingsManager._activeToolLabel = label; }
    /** Returns the description of the active tool. */
    static get activeToolDescription() { return ToolSettingsManager._activeToolDescription; }
    /** Gets the Id of the active tool. If a tool is not active, blank is returned.
     * @return  Id of the active tool, or blank if one is not active.
     */
    static get toolIdForToolSettings() {
        return ToolSettingsManager._toolIdForToolSettings;
    }
    static focusIntoToolSettings() {
        let divElement = document.querySelector("div.nz-toolSettings-docked");
        if (divElement) {
            if (ui_core_1.focusIntoContainer(divElement))
                return true;
        }
        divElement = document.querySelector("div.uifw-tool-settings-grid-container");
        if (divElement) {
            if (ui_core_1.focusIntoContainer(divElement))
                return true;
        }
        return false;
    }
}
exports.ToolSettingsManager = ToolSettingsManager;
ToolSettingsManager._useDefaultToolSettingsProvider = false;
ToolSettingsManager._toolIdForToolSettings = "";
ToolSettingsManager._activeToolLabel = "";
ToolSettingsManager._activeToolDescription = "";
/** Get ToolSettings Properties sync event. */
ToolSettingsManager.onSyncToolSettingsProperties = new SyncToolSettingsPropertiesEvent();
ToolSettingsManager.onReloadToolSettingsProperties = new ui_core_1.UiEvent();
//# sourceMappingURL=ToolSettingsManager.js.map